Attribute VB_Name = "CrossTabFunctions"
Option Explicit


Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\az_linkages\VB_Code\CrossTabFunctions.bas"

Public Function InputsBothRaster(pParams As esriSystem.IVariantArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

  Dim pArray As esriSystem.IVariantArray
10:   Set pArray = New esriSystem.VarArray
  Dim dblPair() As Double
  ReDim dblPair(4)
  
  Dim pRastBand1 As IRasterBand
  Dim pRastBand2 As IRasterBand
  Dim strLayer1Name As String
  Dim strLayer2Name As String
  Dim pSpRef1 As ISpatialReference
  Dim pSpRef2 As ISpatialReference
  Dim booClipToPolygon As Boolean
  Dim pClipPolygon As IPolygon
  Dim pClipSpRef As ISpatialReference
  Dim psbar As IStatusBar
  Dim pPro As IStepProgressor
  
  Dim pClone As IClone
  Dim pAOI_Layer1Polygon As IPolygon
  Dim pAOI_Layer2Polygon As IPolygon
  
30:   Set pRastBand1 = pParams.Element(0)
31:   Set pSpRef1 = pParams.Element(1)
32:   strLayer1Name = pParams.Element(2)
33:   Set pRastBand2 = pParams.Element(3)
34:   Set pSpRef2 = pParams.Element(4)
35:   strLayer2Name = pParams.Element(5)
36:   booClipToPolygon = pParams.Element(6)
37:   Set pClipPolygon = pParams.Element(7)
38:   Set pClipSpRef = pParams.Element(8)
39:   Set psbar = pParams.Element(9)
40:   Set pPro = pParams.Element(10)
  
  Dim i As Long
  Dim pProgAnim As IAnimationProgressor
44:   Set pProgAnim = psbar.ProgressAnimation
45:   pProgAnim.Show
46:   psbar.Message(0) = "Unifying raster projections..."
47:   psbar.PlayProgressAnimation True
48:   DoEvents
  
  Dim dblVal1 As Double
  Dim dblVal2 As Double
  Dim dblWeight As Double
  Dim strSuggestUnits As String
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim pUnit As ILinearUnit
  Dim booIsMetric As Boolean
  Dim strCountName As String
  Dim pSpFilter As ISpatialFilter
  
60:   Set InputsBothRaster = Nothing
  
  ' DETERMINE PRIMARY SPATIAL REFERENCE FOR THIS ANALYSIS.  COULD HAVE UP TO 3 DIFFERENT SPATIAL REFERENCES
  ' COMING AT IT:
  ' 1) SPATIAL REFERENCE FOR FEATURE LAYER #1
  ' 2) SPATIAL REFERENCE FOR FEATURE LAYER #2
  ' 3) SPATIAL REFERENCE FOR AOI POLYGON
  ' PREFER PROJECTED SPATIAL REFERENCE SO OUTPUT UNITS MAKE SENSE.  LOGIC AS FOLLOWS
  ' IF USING AOI POLYGON, THEN
  ' 1) IF EITHER FEATURE LAYERS ARE PROJECTED, USE THE FIRST SPATIAL REFERENCE THAT IS PROJECTED.
  ' 2) OTHERWISE USE THE AOI POLYGON PROJECTION
  ' IF NOT USING AOI POLYGON, JUST USE THE FIRST FEATURE LAYER THAT IS PROJECTED.
  ' IF ALL ELSE FAILS, REPORT IN GEOGRAPHIC UNITS.
  
  Dim pPrimarySpRef As ISpatialReference
  
  ' COMBINE TWO RASTERS INTO A SINGLE RASTER BAND COLLECTION WITH A COMMON EXTENT, CELL SIZE AND
  ' SPATIAL REFERENCE, OPTIONALLY TRIMMED TO A MASK POLYGON
  
  ' MAKE IN-MEMORY VERSION OF CLIP POLYGON TO USE AS MASK
  Dim pPolyArray As esriSystem.IArray
81:   If booClipToPolygon Then
82:     Set pPolyArray = New esriSystem.Array
83:     pPolyArray.Add pClipPolygon
84:   Else
85:     Set pPolyArray = Nothing
86:   End If
  
  Dim pRastArray As esriSystem.IVariantArray
89:   Set pRastArray = New esriSystem.VarArray
90:   pRastArray.Add pRastBand1
91:   pRastArray.Add pRastBand2
  
  Dim pNewBandColl As IRasterBandCollection
94:   Set pNewBandColl = CombineRasterBands(pRastArray, , pPolyArray)
  
  ' FOR DEBUGGING
  Dim pNewRastLayer As IRasterLayer
  Dim pMxDoc As IMxDocument
'  Set pNewRastLayer = New RasterLayer
'  pNewRastLayer.CreateFromRaster pNewBandColl
'  Set pMxDoc = pParams.Element(11)
'  pMxDoc.FocusMap.AddLayer pNewRastLayer
'  Exit Function
  ' ============================
  
  ' GENERATE DATA ARRAY
  Dim pProps As IRasterProps
108:   Set pProps = pNewBandColl
109:   Set pPrimarySpRef = pProps.SpatialReference
  Dim dblCellSize As Double
  Dim pSizePnt As IPnt
112:   Set pSizePnt = pProps.MeanCellSize
113:   dblCellSize = pSizePnt.X * pSizePnt.Y
  ' Debug.Print "Layer..." & pProps.SpatialReference.Name & "..." & CStr(pProps.Width) & _
      " x " & CStr(pProps.Height)
  
  ' PROGRESS STUFF
  Dim booShowProgress As Boolean
119:   booShowProgress = True
  
121:   pProgAnim.Hide
122:   psbar.HideProgressAnimation
  
124:   If booShowProgress Then
125:     pPro.position = 0
126:     psbar.ShowProgressBar "Analyzing Raster", 1, pProps.Height, 1, True
127:     pPro.Message = "Preliminary Analysis:  Extracting Raster Cell Values from " & strLayer1Name & _
          " and " & strLayer2Name & "..."
129:   End If
  
  Dim pPnt As IPnt
132:   Set pPnt = New Pnt
133:   pPnt.SetCoords pProps.Width, 1

  Dim pPixBlock As IPixelBlock3

  Dim pTopLeft As IPnt

  Dim varNoDataArray As Variant
' Dim booHasData1() As Boolean
  Dim varData1 As Variant
  Dim varData2 As Variant
  
  Dim pDataArray As esriSystem.IVariantArray
  Dim lngVarDataIndex As Long
  Dim pNoDataArray As esriSystem.IVariantArray
  Dim lngNoDataIndex As Long
  Dim dblDataArray() As Double
  Dim lngBandCount As Long
  Dim lngWidth As Long
  Dim lngHeight As Long
  
153:   Set pNoDataArray = New esriSystem.VarArray
154:   Set pDataArray = New esriSystem.VarArray
155:   lngWidth = pProps.Width
156:   lngHeight = pProps.Height
  
  ' MAKE 1-ROW RASTER CURSOR
  Dim pAnalysisRaster As IRaster2
'  Dim pTempBandColl As IRasterBandCollection
'  Set pTempBandColl = New Raster
'  pTempBandColl.AppendBand pRasterBand
163:   Set pAnalysisRaster = pNewBandColl
  Dim pRastCursor As IRasterCursor
165:   Set pRastCursor = pAnalysisRaster.CreateCursorEx(pPnt)
  
167:   Set pPixBlock = pRastCursor.PixelBlock
168:   pPixBlock.PixelType(0) = PT_FLOAT
169:   pRastCursor.Reset
170:   Set pPixBlock = pRastCursor.PixelBlock
171:   varData1 = pPixBlock.PixelData(0)
  
173:   lngBandCount = pNewBandColl.Count - 1
174:   For lngNoDataIndex = 0 To lngBandCount
175:     pNoDataArray.Add ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(lngNoDataIndex))
176:     pDataArray.Add pPixBlock.PixelData(lngNoDataIndex)
177:   Next lngNoDataIndex
  ReDim dblDataArray(lngBandCount)
  
  ' BELOW IS WORKAROUND FOR PROBLEM IN WHICH PIXEL BLOCK APPEARS TO NOT BE ABLE TO CORRECTLY READ FIRST ROW.
  ' I THINK THIS HAS SOMETHING TO DO WITH SETTING THE PIXEL TYPE TO PT_FLOAT.  IT READS IT CORRECLTY AFTER
  ' THAT POINT, BUT I CANNOT SET IT BEFORE READING THE FIRST ROW FROM RASTER CURSOR, EVEN AFTER USING 'RESET'.
  ' CODE BELOW SIMPLY EXTRACTS varData1 and booHasData1 FROM A DIFFERENT PIXEL BLOCK THAT IS READ CORRECTLY.
  Dim pTempPixBlock As IPixelBlock3
  Dim pTempRaster As IRaster
186:   Set pTempRaster = pAnalysisRaster
  Dim pTempPoint As IPnt
188:   Set pTempPoint = New Pnt
189:   pTempPoint.SetCoords lngWidth, 1
190:   Set pTempPixBlock = pTempRaster.CreatePixelBlock(pTempPoint)
191:   pTempPixBlock.PixelType(0) = PT_FLOAT
  Dim pTempOrigin As IPnt
193:   Set pTempOrigin = New Pnt
194:   pTempOrigin.SetCoords 0, 0
195:   pTempRaster.Read pTempOrigin, pTempPixBlock
  
197:   varData1 = pTempPixBlock.PixelData(0)
  
199:   pNoDataArray.RemoveAll
200:   pDataArray.RemoveAll
201:   For lngNoDataIndex = 0 To lngBandCount
202:     pNoDataArray.Add ConvertNoDataMaskToSomethingUseful(pTempPixBlock.NoDataMask(lngNoDataIndex))
203:     pDataArray.Add pTempPixBlock.PixelData(lngNoDataIndex)
204:   Next lngNoDataIndex
'  booHasData1 = ConvertNoDataMaskToSomethingUseful(pTempPixBlock.NoDataMask(0))
  
  Dim lngDoEventsCounter As Long
208:   lngDoEventsCounter = 0
  
  Dim booShouldContinue As Boolean
211:   booShouldContinue = True
  Dim lngColCounter As Long
  Dim lngRowCounter As Long
214:   lngRowCounter = 1
  
  Dim pReturnArray As esriSystem.IVariantArray
217:   Set pReturnArray = New esriSystem.VarArray

  Dim dblValue As Double
  
  Dim lngValCount As Long
222:   lngValCount = -1
  Dim booHasData1() As Boolean
  Dim booHasData2() As Boolean
  Dim booData1 As Boolean
  Dim booData2 As Boolean
227:   While booShouldContinue

    ' REFRESHES THE SCREEN EVERY 5%, HOPEFULLY WILL HELP ARCGIS PROGRESS BAR TO KEEP UPDATING EVEN IF YOU CLICK ON ANOTHER PROGRAM
230:     lngDoEventsCounter = lngDoEventsCounter + 1
231:     If lngDoEventsCounter > (lngHeight / 20) Then
232:       lngDoEventsCounter = 0
233:       DoEvents
234:     End If

236:     booHasData1 = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(0))
237:     booHasData2 = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(1))
238:     varData1 = pPixBlock.PixelData(0)
239:     varData2 = pPixBlock.PixelData(1)
    
'    For lngColCounter = 0 To lngWidth - 1
'
'      If CheckNotInMask(pNoDataArray, lngColCounter) Then
'        For lngNoDataIndex = 0 To lngBandCount
'          varData1 = pDataArray.Element(lngNoDataIndex)
'          dblPair(lngNoDataIndex) = CDbl(varData1(lngColCounter, 0))
'        Next lngNoDataIndex
'        dblPair(2) = dblCellSize
'        dblPair(3) = 1
'        dblPair(4) = 1
'
'        pArray.Add dblPair
'  '      If booHasData1(lngColCounter) Then
'  '          Debug.Print varData1(lngColCounter, 0)
'  '        pReturnArray.Add varData1(lngColCounter, 0)
'  '          lngValCount = lngValCount + 1
'  '          dblVals(lngValCount) = varData1(lngColCounter, 0)
'      'End If
'    Next lngColCounter

261:     For lngColCounter = 0 To lngWidth - 1
      
263:       booData1 = booHasData1(lngColCounter)
264:       booData2 = booHasData2(lngColCounter)
      
266:       If booData1 And booData2 Then
267:         dblPair(0) = CDbl(varData1(lngColCounter, 0))
268:         dblPair(1) = CDbl(varData2(lngColCounter, 0))
269:         dblPair(2) = dblCellSize
270:         dblPair(3) = 1
271:         dblPair(4) = 1
272:         pArray.Add dblPair
        
274:       ElseIf booData1 Then        ' IMPLIES LAYER 2 WAS NULL, LAYER 1 HAD DATA
275:         dblPair(0) = CDbl(varData1(lngColCounter, 0))
276:         dblPair(1) = -1
277:         dblPair(2) = dblCellSize
278:         dblPair(3) = 1
279:         dblPair(4) = -1
280:         pArray.Add dblPair
      
282:       ElseIf booData2 Then        ' IMPLIES LAYER 1 WAS NULL, LAYER 2 HAD DATA
283:         dblPair(0) = -1
284:         dblPair(1) = CDbl(varData2(lngColCounter, 0))
285:         dblPair(2) = dblCellSize
286:         dblPair(3) = -1
287:         dblPair(4) = 1
288:         pArray.Add dblPair
        
290:       End If
  '      If booHasData1(lngColCounter) Then
  '          Debug.Print varData1(lngColCounter, 0)
  '        pReturnArray.Add varData1(lngColCounter, 0)
  '          lngValCount = lngValCount + 1
  '          dblVals(lngValCount) = varData1(lngColCounter, 0)
      'End If
297:     Next lngColCounter

299:     booShouldContinue = pRastCursor.Next
300:     If booShowProgress Then
301:       pPro.Step
302:     End If
    
304:     If booShouldContinue Then
305:       Set pPixBlock = pRastCursor.PixelBlock
306:       pPixBlock.PixelType(0) = PT_FLOAT
      
308:       booHasData1 = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(0))
309:       booHasData2 = ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(1))
310:       varData1 = pPixBlock.PixelData(0)
311:       varData2 = pPixBlock.PixelData(1)
      
'      pNoDataArray.RemoveAll
'      pDataArray.RemoveAll
'      For lngNoDataIndex = 0 To lngBandCount
'        pNoDataArray.Add ConvertNoDataMaskToSomethingUseful(pPixBlock.NoDataMask(lngNoDataIndex))
'        pDataArray.Add pPixBlock.PixelData(lngNoDataIndex)
'      Next lngNoDataIndex

'      varNoDataArray = pPixBlock.NoDataMask(0)
'      booHasData1 = ConvertNoDataMaskToSomethingUseful(varNoDataArray)
'      varData1 = pPixBlock.PixelData(0)
323:       lngRowCounter = lngRowCounter + 1
324:     End If
325:   Wend
    
  
'  ReDim dblVals(lngValCount)
'  ExtractValsFromRaster = dblVals
    
331:   If booShowProgress Then
332:     psbar.HideProgressBar
333:   End If
  
335:   Debug.Print pReturnArray.Count
  
'  If booClipToPolygon Then
'    If TypeOf pSpRef1 Is IProjectedCoordinateSystem Then
'      Set pPrimarySpRef = pSpRef1
'    ElseIf TypeOf pSpRef2 Is IProjectedCoordinateSystem Then
'      Set pPrimarySpRef = pSpRef2
'    Else
'      Set pPrimarySpRef = pClipSpRef
'    End If
'  Else
'    If TypeOf pSpRef1 Is IProjectedCoordinateSystem Then
'      Set pPrimarySpRef = pSpRef1
'    Else
'      Set pPrimarySpRef = pSpRef2
'    End If
'  End If

  ' GET MEASUREMENT UNITS FOR THIS ANALYSIS
354:   If TypeOf pPrimarySpRef Is IGeographicCoordinateSystem Then
355:     strSuggestUnits = "Pseudo-Degrees"
356:   Else
357:     Set pPrjSpRef = pPrimarySpRef
358:     Set pUnit = pPrjSpRef.CoordinateUnit
359:     booIsMetric = (pUnit.MetersPerUnit = 1)
360:     strSuggestUnits = pUnit.Name
361:     If InStr(1, strSuggestUnits, "meter", vbTextCompare) > 0 Then
362:       strSuggestUnits = "Meters"
363:     ElseIf InStr(1, strSuggestUnits, "foot", vbTextCompare) > 0 Then
364:       strSuggestUnits = "Feet"
365:     End If
366:   End If
367:   strCountName = "Square " & strSuggestUnits
  
  ' RESCALE VALUES IF UNITS ARE FEET OR METERS, AND IF TOTAL IS ABOVE THRESHOLDS
  Dim dblTotalSize As Double
  Dim lngIndex As Long
  
373:   dblTotalSize = 0
  
375:   If booShowProgress Then
376:     pPro.position = 0
377:     psbar.ShowProgressBar "Checking Data Range...", 1, pArray.Count, 1, True
378:     pPro.Message = "Checking Data Range..."
379:   End If
  
  Dim BooVar1HasOther As Boolean
  Dim booVar2HasOther As Boolean
383:   BooVar1HasOther = False
384:   booVar2HasOther = False
385:   If pArray.Count > 0 Then
386:     For lngIndex = 0 To pArray.Count - 1
387:       dblPair = pArray.Element(lngIndex)
388:       dblTotalSize = dblTotalSize + dblPair(2)
389:       If dblPair(3) < 0 Then BooVar1HasOther = True
390:       If dblPair(4) < 0 Then booVar2HasOther = True
391:       If (dblTotalSize > 100000000) And BooVar1HasOther And booVar2HasOther Then Exit For
392:       If booShowProgress Then pPro.Step
393:     Next lngIndex
394:   End If
  
  Dim pTempArray As esriSystem.IVariantArray
397:   Set pTempArray = New esriSystem.VarArray
  
  Dim dblConversion As Double
  
401:   If dblTotalSize > 0 Then
402:     dblConversion = 1
403:     If dblTotalSize > 100000000 Then
404:       If booIsMetric Then
405:         dblConversion = 1000000          ' CONVERT TO SQUARE KILOMETERS
406:         strCountName = "Sq. Km"
407:       Else
408:         dblConversion = (5280 ^ 2)       ' CONVERT TO SQUARE MILES
409:         strCountName = "Sq. Miles"
410:       End If
411:     ElseIf dblTotalSize > 1000000 Then
412:       If booIsMetric Then
413:         dblConversion = 10000            ' CONVERT TO HECTARES
414:         strCountName = "Hectares"
415:       Else
416:         dblConversion = 43560            ' CONVERT TO ACRES
417:         strCountName = "Acres"
418:       End If
419:     End If
    
 '   MsgBox "dblTotalSize = " & CStr(dblTotalSize) & vbCrLf & _
           "dblConversion = " & CStr(dblConversion) & vbCrLf & _
           "strCountName = " & CStr(strCountName)
           
425:     If booShowProgress Then
426:       pPro.position = 0
427:       psbar.ShowProgressBar "Checking Data Range (Pass 2)", 1, pArray.Count, 1, True
428:       pPro.Message = "Checking Data Range (Pass 2)..."
429:     End If
    
431:     If dblConversion > 1 Then
432:       For lngIndex = 0 To pArray.Count - 1
433:         dblPair = pArray.Element(lngIndex)
434:         dblPair(2) = dblPair(2) / dblConversion
435:         pTempArray.Add dblPair
        
437:         If booShowProgress Then pPro.Step
438:       Next lngIndex
439:       Set pArray = pTempArray
440:     End If
441:   End If
  
443:   pPro.position = 0
444:   psbar.HideProgressBar
  
446:   Set InputsBothRaster = New esriSystem.VarArray
447:   InputsBothRaster.Add pArray
448:   InputsBothRaster.Add strCountName
449:   InputsBothRaster.Add BooVar1HasOther
450:   InputsBothRaster.Add booVar2HasOther

452:   Set pArray = Nothing
453:   Set pRastBand1 = Nothing
454:   Set pRastBand2 = Nothing
455:   Set pSpRef1 = Nothing
456:   Set pSpRef2 = Nothing
457:   Set pClipPolygon = Nothing
458:   Set pClipSpRef = Nothing
459:   Set psbar = Nothing
460:   Set pPro = Nothing
461:   Set pClone = Nothing
462:   Set pAOI_Layer1Polygon = Nothing
463:   Set pAOI_Layer2Polygon = Nothing
464:   Set pMxDoc = Nothing
465:   Set pPrjSpRef = Nothing
466:   Set pUnit = Nothing
467:   Set pSpFilter = Nothing
468:   Set pPrimarySpRef = Nothing
469:   Set pPolyArray = Nothing
470:   Set pRastArray = Nothing
471:   Set pNewBandColl = Nothing
472:   Set pNewRastLayer = Nothing
473:   Set pProps = Nothing
474:   Set pSizePnt = Nothing
475:   Set pPnt = Nothing
476:   Set pPixBlock = Nothing
477:   Set pTopLeft = Nothing
478:   Set pDataArray = Nothing
479:   Set pNoDataArray = Nothing
480:   Set pAnalysisRaster = Nothing
481:   Set pRastCursor = Nothing
482:   Set pTempPixBlock = Nothing
483:   Set pTempRaster = Nothing
484:   Set pTempPoint = Nothing
485:   Set pTempOrigin = Nothing
486:   Set pReturnArray = Nothing
487:   Set pTempArray = Nothing
488:   Set pProgAnim = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "InputsBothRaster " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CombineRasterBands(pRasterBands As esriSystem.IVariantArray, _
    Optional pSpRef As ISpatialReference, _
    Optional pPolygonArray As esriSystem.IArray) As IRasterBandCollection
  On Error GoTo ErrorHandler

  Dim pRastBandColl As IRasterBandCollection
  Dim pPrimarySpRef As ISpatialReference
  Dim pRastBand As IRasterBand
  Dim lngIndex As Long
  Dim pGeoDataset As IGeoDataset
  Dim pPrimaryProps As IRasterProps
  Dim pPrimaryEnvelope As IEnvelope
  Dim pRelOp As IRelationalOperator
  Dim pPrimaryPnt As IPnt
  Dim pCheckPnt As IPnt
  
511:   For lngIndex = 0 To pRasterBands.Count - 1
512:     Set pRastBand = pRasterBands.Element(lngIndex)
513:     Set pGeoDataset = pRastBand
514:     Set pPrimaryProps = pRastBand
515:     Set pPrimaryEnvelope = pPrimaryProps.Extent
516:     Set pRelOp = pPrimaryEnvelope
517:     Set pPrimaryPnt = pPrimaryProps.MeanCellSize
518:     If TypeOf pGeoDataset.SpatialReference Is IProjectedCoordinateSystem Then
519:       Exit For
520:     End If
521:   Next lngIndex
  
  Dim booAllPropsOK As Boolean
524:   If pSpRef Is Nothing Then
525:     Set pPrimarySpRef = pGeoDataset.SpatialReference
526:     booAllPropsOK = True
527:   Else
528:     booAllPropsOK = Linkages.MyGeneralOperations.CompareSpatialReferences(pPrimarySpRef, pSpRef)
529:     Set pPrimarySpRef = pSpRef
530:   End If
  
  Dim pRasterProps As IRasterProps
  
  Dim pTempRaster As IRasterBandCollection
535:   Set pTempRaster = New Raster
  Dim pNewProps As IRasterProps
537:   Set pNewProps = pTempRaster
538:   pNewProps.Height = pPrimaryProps.Height
539:   pNewProps.Width = pPrimaryProps.Width
540:   pNewProps.Extent = pPrimaryEnvelope
541:   pNewProps.SpatialReference = pPrimarySpRef
    
543:   For lngIndex = 0 To pRasterBands.Count - 1
544:     Set pRastBand = pRasterBands.Element(lngIndex)
545:     Set pGeoDataset = pRastBand
546:     Set pRasterProps = pRastBand
    
'    If Not pRasterProps.Height = pPrimaryProps.Height Then
'      pRasterProps.Height = pPrimaryProps.Height
'    End If
'    If Not pRasterProps.Width = pPrimaryProps.Width Then
'      pRasterProps.Width = pPrimaryProps.Width
'    End If
'    If Not pRelOp.Equals(pRasterProps.Extent) Then
'      pRasterProps.Extent = pPrimaryEnvelope
'    End If
'    If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pPrimarySpRef, pGeoDataset.SpatialReference) Then
'      pRasterProps.SpatialReference = pPrimarySpRef
'    End If
    
561:     pTempRaster.AppendBand pRastBand
562:   Next lngIndex
  
  ' IF REQUESTED, MAKE IN-MEMORY MASK FEATURE CLASS FROM POLYGONS
  Dim pPolygon As IPolygon
  Dim pNewPolygonArray As esriSystem.IArray
  Dim pMathOp As IMathOp
  Dim pRastAnalysis As IRasterAnalysisEnvironment
  
570:   If Not pPolygonArray Is Nothing Then
571:     Set pNewPolygonArray = New esriSystem.Array
    
573:     For lngIndex = 0 To pPolygonArray.Count - 1
574:       Set pPolygon = pPolygonArray.Element(lngIndex)
575:       If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pPolygon.SpatialReference, pPrimarySpRef) Then
576:         pPolygon.Project pPrimarySpRef
577:       End If
578:       pNewPolygonArray.Add pPolygon
579:     Next lngIndex
    Dim pFClass As IFeatureClass
581:     Set pFClass = Linkages.MyGeneralOperations.CreateInMemoryFeatureClass(pNewPolygonArray)
582:     Debug.Print "Feature Count = " & pFClass.FeatureCount(Nothing)
    
584:     Set pMathOp = New RasterMathOps
585:     Set pRastAnalysis = pMathOp
586:     Set pRastAnalysis.OutSpatialReference = pPrimarySpRef
587:     Set pRastAnalysis.Mask = pFClass
588:     pRastAnalysis.SetExtent esriRasterEnvValue, pFClass
'    pRastAnalysis.SetExtent esriRasterEnvValue, pFClass, pTempRaster
    
591:     Set CombineRasterBands = pMathOp.Float(pTempRaster)
592:   Else
593:     Set CombineRasterBands = pTempRaster
594:   End If
  
596:   Set pRastBandColl = Nothing
597:   Set pPrimarySpRef = Nothing
598:   Set pRastBand = Nothing
599:   Set pGeoDataset = Nothing
600:   Set pPrimaryProps = Nothing
601:   Set pPrimaryEnvelope = Nothing
602:   Set pRelOp = Nothing
603:   Set pPrimaryPnt = Nothing
604:   Set pCheckPnt = Nothing
605:   Set pRasterProps = Nothing
606:   Set pTempRaster = Nothing
607:   Set pNewProps = Nothing
608:   Set pPolygon = Nothing
609:   Set pNewPolygonArray = Nothing
610:   Set pMathOp = Nothing
611:   Set pRastAnalysis = Nothing
612:   Set pFClass = Nothing

614:   Debug.Print pPrimarySpRef.Name

  Exit Function
ErrorHandler:
 ' HandleError True, "CombineRasterBands " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.Source, Err.Description, 4
End Function

Public Function CheckNotInMask(pNoDataArray As esriSystem.IVariantArray, lngColCounter As Long) As Boolean
  On Error GoTo ErrorHandler

  Dim lngIndex As Long
  Dim booHasData() As Boolean
626:   CheckNotInMask = True
627:   For lngIndex = 0 To pNoDataArray.Count - 1
628:     booHasData = pNoDataArray.Element(lngIndex)
629:     If Not booHasData(lngColCounter) Then
630:       CheckNotInMask = False
      Exit Function
632:     End If
633:   Next lngIndex

  Exit Function
ErrorHandler:
  HandleError True, "CheckNotInMask " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function InputsBothPolygon(pParams As esriSystem.IVariantArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

  Dim pArray As esriSystem.IVariantArray
644:   Set pArray = New esriSystem.VarArray
  Dim dblPair() As Double
  ReDim dblPair(2)
  
  Dim pFClass1 As IFeatureClass
  Dim pFClass2 As IFeatureClass
  Dim pFLayer1 As IFeatureLayer
  Dim pFLayer2 As IFeatureLayer
  Dim pSpRef1 As ISpatialReference
  Dim pSpRef2 As ISpatialReference
  Dim booClipToPolygon As Boolean
  Dim lngFieldIndex1 As Long
  Dim lngFieldIndex2 As Long
  Dim strFieldAlias1 As String
  Dim strFieldAlias2 As String
  Dim pClipPolygon As IPolygon
  Dim pClipSpRef As ISpatialReference
  Dim psbar As IStatusBar
  Dim pPro As IStepProgressor
  
  Dim pClone As IClone
  Dim pAOI_Layer1Polygon As IPolygon
  Dim pAOI_Layer2Polygon As IPolygon
  
668:   Set pFLayer1 = pParams.Element(0)
669:   Set pFClass1 = pFLayer1.FeatureClass
670:   Set pSpRef1 = pParams.Element(1)
671:   lngFieldIndex1 = pParams.Element(2)
672:   strFieldAlias1 = pParams.Element(3)
673:   Set pFLayer2 = pParams.Element(4)
674:   Set pFClass2 = pFLayer2.FeatureClass
675:   Set pSpRef2 = pParams.Element(5)
676:   lngFieldIndex2 = pParams.Element(6)
677:   strFieldAlias2 = pParams.Element(7)
678:   booClipToPolygon = pParams.Element(8)
679:   Set pClipPolygon = pParams.Element(9)
680:   Set pClipSpRef = pParams.Element(10)
681:   Set psbar = pParams.Element(11)
682:   Set pPro = pParams.Element(12)
  
  Dim dblVal1 As Double
  Dim dblVal2 As Double
  Dim dblWeight As Double
  Dim strSuggestUnits As String
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim pUnit As ILinearUnit
  Dim booIsMetric As Boolean
  Dim strCountName As String
  Dim pSpFilter As ISpatialFilter
  
694:   Set InputsBothPolygon = Nothing
  
  Dim BooVar1HasOther As Boolean
  Dim booVar2HasOther As Boolean
698:   BooVar1HasOther = False
699:   booVar2HasOther = False
  
  Dim booSpRefSame As Boolean
702:   booSpRefSame = Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef1, pSpRef2)
  
  ' DETERMINE PRIMARY SPATIAL REFERENCE FOR THIS ANALYSIS.  COULD HAVE UP TO 3 DIFFERENT SPATIAL REFERENCES
  ' COMING AT IT:
  ' 1) SPATIAL REFERENCE FOR FEATURE LAYER #1
  ' 2) SPATIAL REFERENCE FOR FEATURE LAYER #2
  ' 3) SPATIAL REFERENCE FOR AOI POLYGON
  ' PREFER PROJECTED SPATIAL REFERENCE SO OUTPUT UNITS MAKE SENSE.  LOGIC AS FOLLOWS
  ' IF USING AOI POLYGON, THEN
  ' 1) IF EITHER FEATURE LAYERS ARE PROJECTED, USE THE FIRST SPATIAL REFERENCE THAT IS PROJECTED.
  ' 2) OTHERWISE USE THE AOI POLYGON PROJECTION
  ' IF NOT USING AOI POLYGON, JUST USE THE FIRST FEATURE LAYER THAT IS PROJECTED.
  ' IF ALL ELSE FAILS, REPORT IN GEOGRAPHIC UNITS.
  
  Dim pPrimarySpRef As ISpatialReference
717:   If booClipToPolygon Then
718:     If TypeOf pSpRef1 Is IProjectedCoordinateSystem Then
719:       Set pPrimarySpRef = pSpRef1
720:     ElseIf TypeOf pSpRef2 Is IProjectedCoordinateSystem Then
721:       Set pPrimarySpRef = pSpRef2
722:     Else
723:       Set pPrimarySpRef = pClipSpRef
724:     End If
725:   Else
726:     If TypeOf pSpRef1 Is IProjectedCoordinateSystem Then
727:       Set pPrimarySpRef = pSpRef1
728:     Else
729:       Set pPrimarySpRef = pSpRef2
730:     End If
731:   End If
  
  ' GET MEASUREMENT UNITS FOR THIS ANALYSIS
734:   If TypeOf pPrimarySpRef Is IGeographicCoordinateSystem Then
735:     strSuggestUnits = "Pseudo-Degrees"
736:   Else
737:     Set pPrjSpRef = pPrimarySpRef
738:     Set pUnit = pPrjSpRef.CoordinateUnit
739:     booIsMetric = (pUnit.MetersPerUnit = 1)
740:     strSuggestUnits = pUnit.Name
741:     If InStr(1, strSuggestUnits, "meter", vbTextCompare) > 0 Then
742:       strSuggestUnits = "Meters"
743:     ElseIf InStr(1, strSuggestUnits, "foot", vbTextCompare) > 0 Then
744:       strSuggestUnits = "Feet"
745:     End If
746:   End If
  
  Dim booProjectLayer1 As Boolean
  Dim booProjectLayer2 As Boolean
  Dim booProjectAOI As Boolean
  Dim pPolygon As IPolygon
  Dim pEnv As IEnvelope
  Dim pRelOp As IRelationalOperator
  Dim pTopoOp As ITopologicalOperator
  Dim pTempPolygon As IPolygon
  Dim pArea As IArea
  Dim lngDataCount1 As Long
  Dim lngDataCount2 As Long
  Dim lngDataCounter As Long
  Dim strMessage As String
  Dim pFCursor As IFeatureCursor
  Dim pFeature As IFeature
  Dim dblConversion As Double
  Dim dblTotalSize As Double
  Dim lngIndex As Long
  
767:   booProjectLayer1 = Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef1, pPrimarySpRef)
768:   booProjectLayer2 = Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef2, pPrimarySpRef)
769:   If booClipToPolygon Then
770:     booProjectAOI = Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef2, pPrimarySpRef)
771:   Else
772:     booProjectAOI = False
773:   End If
  
775:   strCountName = "Square " & strSuggestUnits
    
  
'  MsgBox "booProjectLayer1 = " & CStr(booProjectLayer1)
'  MsgBox "booProjectLayer2 = " & CStr(booProjectLayer2)
  Dim pPolygonData As esriSystem.IVariantArray
  
  ' EXTRACT POLYGONS FROM FIRST LAYER
  Dim pLayer1Polygons As esriSystem.IArray
784:   Set pLayer1Polygons = New esriSystem.Array

786:   If lngFieldIndex1 = -1 Then
787:     MsgBox "PROBLEM:  Unable to find " & pFLayer1.Name & " Attribute Field [" & strFieldAlias1 & "]!" & vbCrLf & vbCrLf & _
        "Bailing out..."
    Exit Function
790:   End If

  ' IF RESTRICTING ANALYSIS TO POLYGON, THEN CREATE SPATIAL FILTER FOR CURSOR
793:   If booClipToPolygon Then
794:     Set pSpFilter = New SpatialFilter

    ' ANALYSIS AREA POLYGON MUST HAVE A SPATIAL REFERENCE
797:     If pClipSpRef Is Nothing Then
798:       MsgBox "Analysis Area polygon has no spatial reference!" & vbCrLf & vbCrLf & _
        "Please select an analysis polygon that has a known spatial reference before " & _
        "attempting this analysis..."
      Exit Function
802:     ElseIf TypeOf pClipSpRef Is IUnknownCoordinateSystem Then
803:       MsgBox "Analysis Area polygon has no spatial reference!" & vbCrLf & vbCrLf & _
        "Please select an analysis polygon that has a known spatial reference before " & _
        "attempting this analysis..."
      Exit Function
807:     End If

    ' IF USING ANALYSIS AREA POLYGON, THEN FEATURE CLASS MUST ALSO HAVE A SPATIAL REFERENCE
810:     If pSpRef1 Is Nothing Then
811:       MsgBox "Your selected Feature Class '" & pFLayer1.Name & "' has no spatial reference!" & vbCrLf & vbCrLf & _
        "When using an analysis area polygon, then both the polygon and the feature class must have " & vbCrLf & _
        "known spatial references in order to match the data up spatially." & vbCrLf & vbCrLf & _
        "Please assign the correct spatial reference to your feature class before " & _
        "attempting this analysis..."
      Exit Function
817:     ElseIf TypeOf pSpRef1 Is IUnknownCoordinateSystem Then
818:       MsgBox "Your selected Feature Class '" & pFLayer1.Name & "' has no spatial reference!" & vbCrLf & vbCrLf & _
        "When using an analysis area polygon, then both the polygon and the feature class must have " & vbCrLf & _
        "known spatial references in order to match the data up spatially." & vbCrLf & vbCrLf & _
        "Please assign the correct spatial reference to your feature class before " & _
        "attempting this analysis..."
      Exit Function
824:     End If

    ' MAKE SURE AOI POLYGON IS SAME SPATIAL REFERENCE AS FEATURE CLASS
827:     Set pClone = pClipPolygon
828:     Set pAOI_Layer1Polygon = pClone.Clone
829:     If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef1, pClipSpRef) Then
830:       pAOI_Layer1Polygon.Project pSpRef1
831:     End If
    
 '   MsgBox "Polygon projection = " & pAOI_Layer1Polygon.SpatialReference.Name
    
835:     Set pSpFilter.Geometry = pAOI_Layer1Polygon
836:     pSpFilter.SpatialRel = esriSpatialRelIntersects
837:   Else
838:     Set pSpFilter = Nothing
839:   End If
  
  ' FIRST SHOW MESSAGE STATING THAT ARCGIS IS INTERSECTING
842:   psbar.ShowProgressBar "Preliminary Analysis:  Searching " & pFLayer1.Name, 1, 100, 1, True
843:   DoEvents
  
845:   lngDataCount1 = pFClass1.FeatureCount(pSpFilter)
846:   lngDataCounter = 0
  
'  MsgBox "Feature Layer 1:  Analyzing " & CStr(lngDataCount1) & " features in " & pFLayer1.Name & "..."
 
850:   Set pFCursor = pFClass1.Search(pSpFilter, True)
851:   Set pFeature = pFCursor.NextFeature

853:   psbar.ShowProgressBar "Preliminary Analysis:  Extracting polygons from " & pFLayer1.Name, 1, lngDataCount1, 1, True
854:   pPro.position = 1
855:   DoEvents
  
  ' ROTATE THROUGH ALL RELEVANT RECORDS
858:   Do Until pFeature Is Nothing
859:     lngDataCounter = lngDataCounter + 1
860:     psbar.StepProgressBar
861:     If lngDataCounter Mod 100 = 0 Then
862:       DoEvents
863:     End If

    ' GET SIZE OF OBSERVATION:  CLIP (IF NECESSARY) TO AOI POLYGON BOUNDARY AND GET CLIPPED AREA
 
867:     Set pPolygon = pFeature.ShapeCopy
868:     If Not pPolygon.IsEmpty Then

      ' GET ATTRIBUTE VALUE
871:       dblVal1 = pFeature.Value(lngFieldIndex1)
    
873:       If booClipToPolygon Then
874:         Set pRelOp = pPolygon
875:         If Not pRelOp.Within(pAOI_Layer1Polygon) Then
876:           Set pTopoOp = pPolygon
877:           Set pTempPolygon = pTopoOp.Intersect(pAOI_Layer1Polygon, pPolygon.Dimension)
878:           Set pTopoOp = pTempPolygon
879:           pTopoOp.Simplify
880:           Set pPolygon = pTempPolygon
881:         End If
882:       End If
      
884:       If booProjectLayer1 Then
885:         pPolygon.Project pPrimarySpRef
886:       End If
      
888:       If Not pPolygon.IsEmpty Then
889:         Set pEnv = pPolygon.Envelope
890:         Set pPolygonData = New esriSystem.VarArray
        
892:         pPolygonData.Add pPolygon
893:         pPolygonData.Add dblVal1
894:         pPolygonData.Add pEnv.XMin
895:         pPolygonData.Add pEnv.XMax
896:         pPolygonData.Add pEnv.YMin
897:         pPolygonData.Add pEnv.YMax
        
899:         pLayer1Polygons.Add pPolygonData
900:       End If
901:     End If
    
903:     Set pFeature = pFCursor.NextFeature
904:   Loop

' ----------------------------------------------------------------------------------------------------------------
' /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
' ----------------------------------------------------------------------------------------------------------------

  ' EXTRACT POLYGONS FROM SECOND LAYER
  Dim pLayer2Polygons As esriSystem.IArray
912:   Set pLayer2Polygons = New esriSystem.Array

914:   If lngFieldIndex2 = -1 Then
915:     MsgBox "PROBLEM:  Unable to find " & pFLayer2.Name & " Attribute Field [" & strFieldAlias2 & "]!" & vbCrLf & vbCrLf & _
        "Bailing out..."
    Exit Function
918:   End If

  ' IF RESTRICTING ANALYSIS TO POLYGON, THEN CREATE SPATIAL FILTER FOR CURSOR
921:   If booClipToPolygon Then
922:     Set pSpFilter = New SpatialFilter

    ' ANALYSIS AREA POLYGON MUST HAVE A SPATIAL REFERENCE
925:     If pClipSpRef Is Nothing Then
926:       MsgBox "Analysis Area polygon has no spatial reference!" & vbCrLf & vbCrLf & _
        "Please select an analysis polygon that has a known spatial reference before " & _
        "attempting this analysis..."
      Exit Function
930:     ElseIf TypeOf pClipSpRef Is IUnknownCoordinateSystem Then
931:       MsgBox "Analysis Area polygon has no spatial reference!" & vbCrLf & vbCrLf & _
        "Please select an analysis polygon that has a known spatial reference before " & _
        "attempting this analysis..."
      Exit Function
935:     End If

    ' IF USING ANALYSIS AREA POLYGON, THEN FEATURE CLASS MUST ALSO HAVE A SPATIAL REFERENCE
938:     If pSpRef2 Is Nothing Then
939:       MsgBox "Your selected Feature Class '" & pFLayer2.Name & "' has no spatial reference!" & vbCrLf & vbCrLf & _
        "When using an analysis area polygon, then both the polygon and the feature class must have " & vbCrLf & _
        "known spatial references in order to match the data up spatially." & vbCrLf & vbCrLf & _
        "Please assign the correct spatial reference to your feature class before " & _
        "attempting this analysis..."
      Exit Function
945:     ElseIf TypeOf pSpRef2 Is IUnknownCoordinateSystem Then
946:       MsgBox "Your selected Feature Class '" & pFLayer2.Name & "' has no spatial reference!" & vbCrLf & vbCrLf & _
        "When using an analysis area polygon, then both the polygon and the feature class must have " & vbCrLf & _
        "known spatial references in order to match the data up spatially." & vbCrLf & vbCrLf & _
        "Please assign the correct spatial reference to your feature class before " & _
        "attempting this analysis..."
      Exit Function
952:     End If

    ' MAKE SURE AOI POLYGON IS SAME SPATIAL REFERENCE AS FEATURE CLASS
955:     Set pClone = pClipPolygon
956:     Set pAOI_Layer2Polygon = pClone.Clone
957:     If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef2, pClipSpRef) Then
958:       pAOI_Layer2Polygon.Project pSpRef2
959:     End If

961:     Set pSpFilter.Geometry = pAOI_Layer2Polygon
962:     pSpFilter.SpatialRel = esriSpatialRelIntersects
963:   Else
964:     Set pSpFilter = Nothing
965:   End If

  ' FIRST SHOW MESSAGE STATING THAT ARCGIS IS INTERSECTING
968:   psbar.ShowProgressBar "Preliminary Analysis:  Searching " & pFLayer2.Name, 1, 100, 1, True
969:   DoEvents
  
971:   lngDataCount2 = pFClass2.FeatureCount(pSpFilter)
972:   lngDataCounter = 0
  
'  MsgBox "Feature Layer 1:  Analyzing " & CStr(lngDataCount2) & " features in " & pFLayer1.Name & "..."
 
976:   Set pFCursor = pFClass2.Search(pSpFilter, True)
977:   Set pFeature = pFCursor.NextFeature

979:   psbar.ShowProgressBar "Preliminary Analysis:  Extracting polygons from " & pFLayer2.Name, 1, lngDataCount2, 1, True
980:   pPro.position = 1
981:   DoEvents
  
  ' ROTATE THROUGH ALL RELEVANT RECORDS
984:   Do Until pFeature Is Nothing
985:     lngDataCounter = lngDataCounter + 1
986:     psbar.StepProgressBar
987:     If lngDataCounter Mod 100 = 0 Then
988:       DoEvents
989:     End If

    ' GET SIZE OF OBSERVATION:  CLIP (IF NECESSARY) TO AOI POLYGON BOUNDARY AND GET CLIPPED AREA
 
993:     Set pPolygon = pFeature.ShapeCopy
994:     If Not pPolygon.IsEmpty Then

      ' GET ATTRIBUTE VALUE
997:       dblVal2 = pFeature.Value(lngFieldIndex2)
    
999:       If booClipToPolygon Then
1000:         Set pRelOp = pPolygon
1001:         If Not pRelOp.Within(pAOI_Layer2Polygon) Then
1002:           Set pTopoOp = pPolygon
1003:           Set pTempPolygon = pTopoOp.Intersect(pAOI_Layer2Polygon, pPolygon.Dimension)
1004:           Set pTopoOp = pTempPolygon
1005:           pTopoOp.Simplify
1006:           Set pPolygon = pTempPolygon
1007:         End If
1008:       End If
      
1010:       If booProjectLayer2 Then
1011:         pPolygon.Project pPrimarySpRef
1012:       End If
      
1014:       If Not pPolygon.IsEmpty Then
1015:         Set pEnv = pPolygon.Envelope
1016:         Set pPolygonData = New esriSystem.VarArray
        
1018:         pPolygonData.Add pPolygon
1019:         pPolygonData.Add dblVal2
1020:         pPolygonData.Add pEnv.XMin
1021:         pPolygonData.Add pEnv.XMax
1022:         pPolygonData.Add pEnv.YMin
1023:         pPolygonData.Add pEnv.YMax
        
1025:         pLayer2Polygons.Add pPolygonData
1026:       End If
1027:     End If
    
1029:     Set pFeature = pFCursor.NextFeature
1030:   Loop

1032:   lngDataCounter = 0
1033:   psbar.ShowProgressBar "Intersecting polygons from " & pFLayer1.Name & " and " & _
      pFLayer2.Name, 1, pLayer1Polygons.Count + pLayer2Polygons.Count, 1, True
1035:   pPro.position = 1
  
  Dim pIntersectArray As esriSystem.IVariantArray
1038:   Set pIntersectArray = New esriSystem.VarArray
  
  

'  MsgBox pFLayer1.Name & ":  " & CStr(pLayer1Polygons.Count) & vbCrLf & _
    pFLayer2.Name & ":  " & CStr(pLayer2Polygons.Count)
  
  Dim pPolygonData2 As esriSystem.IVariantArray
  Dim pNoIntersectPolygon As IPolygon
  Dim pComparePolygon As IPolygon
  Dim pIntersectPolygon As IPolygon
  Dim pTopoOp2 As ITopologicalOperator
  Dim pTopoOpClip As ITopologicalOperator
  Dim dblCount As Double
  
  Dim lngIndex2 As Long
1054:   lngDataCounter = 0
  Dim dblVals() As Double
  ReDim dblVals(4)
  
1058:   For lngIndex = 0 To pLayer1Polygons.Count - 1
1059:     Set pPolygonData = pLayer1Polygons.Element(lngIndex)
    
1061:     lngDataCounter = lngDataCounter + 1
1062:     psbar.StepProgressBar
1063:     If lngDataCounter Mod 30 = 0 Then
1064:       DoEvents
1065:     End If
    
1067:     pIntersectArray.RemoveAll
    
1069:     Set pPolygon = pPolygonData.Element(0)
1070:     Set pTopoOp = pPolygon
1071:     Set pClone = pPolygon
1072:     Set pNoIntersectPolygon = pClone.Clone
1073:     Set pTopoOpClip = pNoIntersectPolygon
    
1075:     dblVal1 = pPolygonData.Element(1)
    
1077:     For lngIndex2 = 0 To pLayer2Polygons.Count - 1
1078:       Set pPolygonData2 = pLayer2Polygons.Element(lngIndex2)
      
1080:       If CheckPolygonIntersects(pPolygon, pPolygonData.Element(2), _
          pPolygonData.Element(3), pPolygonData.Element(4), pPolygonData.Element(5), _
          pPolygonData2.Element(0), pPolygonData2.Element(2), _
          pPolygonData2.Element(3), pPolygonData2.Element(4), pPolygonData2.Element(5), _
          pRelOp) Then
          
1086:           pIntersectArray.Add pPolygonData2
1087:       End If
      
1089:     Next lngIndex2
    
    ' ACTUALLY EXTRACT DATA
1092:     For lngIndex2 = 0 To pIntersectArray.Count - 1
      
      ' CALCULATE INTERSECTING AREA
1095:       Set pPolygonData2 = pIntersectArray.Element(lngIndex2)
1096:       Set pComparePolygon = pPolygonData2.Element(0)
1097:       dblVal2 = pPolygonData2.Element(1)
1098:       Set pIntersectPolygon = pTopoOp.Intersect(pComparePolygon, pPolygon.Dimension)
1099:       Set pTopoOp2 = pIntersectPolygon
1100:       pTopoOp2.Simplify
1101:       Set pArea = pIntersectPolygon
1102:       dblCount = pArea.Area
      
      ' ADD DATA TO ARRAY
1105:       dblVals(0) = dblVal1
1106:       dblVals(1) = dblVal2
1107:       dblVals(2) = dblCount
1108:       dblVals(3) = 1
1109:       dblVals(4) = 1
1110:       pArray.Add dblVals
      
      ' TRIM AWAY ORIGINAL POLYGON IN CASE THERE IS OVERLAPPING AREA
1113:       Set pNoIntersectPolygon = pTopoOpClip.Difference(pComparePolygon)
1114:       Set pTopoOpClip = pNoIntersectPolygon
      
1116:     Next lngIndex2
    
1118:     pTopoOpClip.Simplify
1119:     If Not pNoIntersectPolygon.IsEmpty Then
1120:       Set pArea = pNoIntersectPolygon
1121:       dblCount = pArea.Area
      
      ' ADD DATA TO ARRAY
1124:       dblVals(0) = dblVal1
1125:       dblVals(1) = 0
1126:       dblVals(2) = dblCount
1127:       dblVals(3) = 1
1128:       dblVals(4) = -1       ' < 0 INDICATES THIS HAS NO CORRESPONDING VALUE FOR VARIABLE 2
1129:       pArray.Add dblVals
1130:       BooVar1HasOther = True
1131:     End If
    
1133:   Next lngIndex
  
  ' REPEAT MOST OF ANALYSIS ABOVE TO GET REGIONS WITH NO CORRESPONDING VALUE FOR VARIABLE 1
  
  
1138:   For lngIndex = 0 To pLayer2Polygons.Count - 1
1139:     Set pPolygonData = pLayer2Polygons.Element(lngIndex)
    
1141:     lngDataCounter = lngDataCounter + 1
1142:     psbar.StepProgressBar
1143:     If lngDataCounter Mod 30 = 0 Then
1144:       DoEvents
1145:     End If
    
1147:     pIntersectArray.RemoveAll
    
1149:     Set pPolygon = pPolygonData.Element(0)
1150:     Set pTopoOp = pPolygon
1151:     Set pClone = pPolygon
1152:     Set pNoIntersectPolygon = pClone.Clone
1153:     Set pTopoOpClip = pNoIntersectPolygon
    
1155:     dblVal1 = pPolygonData.Element(1)
    
1157:     For lngIndex2 = 0 To pLayer1Polygons.Count - 1
1158:       Set pPolygonData2 = pLayer1Polygons.Element(lngIndex2)
      
1160:       If CheckPolygonIntersects(pPolygon, pPolygonData.Element(2), _
          pPolygonData.Element(3), pPolygonData.Element(4), pPolygonData.Element(5), _
          pPolygonData2.Element(0), pPolygonData2.Element(2), _
          pPolygonData2.Element(3), pPolygonData2.Element(4), pPolygonData2.Element(5), _
          pRelOp) Then
          
1166:           pIntersectArray.Add pPolygonData2
1167:       End If
      
1169:     Next lngIndex2
    
    ' ACTUALLY EXTRACT DATA
1172:     For lngIndex2 = 0 To pIntersectArray.Count - 1
      
      ' CALCULATE INTERSECTING AREA
1175:       Set pPolygonData2 = pIntersectArray.Element(lngIndex2)
1176:       Set pComparePolygon = pPolygonData2.Element(0)
      
'      dblVal2 = pPolygonData2.Element(1)
'      Set pIntersectPolygon = pTopoOp.Intersect(pComparePolygon, pPolygon.Dimension)
'      Set pTopoOp2 = pIntersectPolygon
'      pTopoOp2.Simplify
'      Set pArea = pIntersectPolygon
'      dblCount = pArea.Area
'
'      ' ADD DATA TO ARRAY
'      dblVals(0) = dblVal1
'      dblVals(1) = dblVal2
'      dblVals(2) = dblCount
'      dblVals(3) = 1
'      dblVals(4) = 1
'      pArray.Add dblVals
      
      ' TRIM AWAY ORIGINAL POLYGON IN CASE THERE IS OVERLAPPING AREA
1194:       Set pNoIntersectPolygon = pTopoOpClip.Difference(pComparePolygon)
1195:       Set pTopoOpClip = pNoIntersectPolygon
      
1197:     Next lngIndex2
    
1199:     pTopoOpClip.Simplify
1200:     If Not pNoIntersectPolygon.IsEmpty Then
1201:       Set pArea = pNoIntersectPolygon
1202:       dblCount = pArea.Area
      
      ' ADD DATA TO ARRAY
1205:       dblVals(0) = 0
1206:       dblVals(1) = dblVal1
1207:       dblVals(2) = dblCount
1208:       dblVals(3) = -1       ' < 0 INDICATES THIS IS HAS NO CORRESPONDING VALUE FOR VARIABLE 1
1209:       dblVals(4) = 1
1210:       pArray.Add dblVals
1211:       booVar2HasOther = True
1212:     End If
    
1214:   Next lngIndex
1215:   psbar.HideProgressBar
  
  ' RESCALE VALUES IF UNITS ARE FEET OR METERS, AND IF TOTAL IS ABOVE THRESHOLDS
1218:   dblTotalSize = 0
1219:   If pArray.Count > 0 Then
1220:     For lngIndex = 0 To pArray.Count - 1
1221:       dblVals = pArray.Element(lngIndex)
1222:       dblTotalSize = dblTotalSize + dblVals(2)
1223:     Next lngIndex
1224:   End If
  
  Dim pTempArray As esriSystem.IVariantArray
1227:   Set pTempArray = New esriSystem.VarArray
  
1229:   If dblTotalSize > 0 Then
1230:     dblConversion = 1
1231:     If dblTotalSize > 100000000 Then
1232:       If booIsMetric Then
1233:         dblConversion = 1000000          ' CONVERT TO SQUARE KILOMETERS
1234:         strCountName = "Sq. Km"
1235:       Else
1236:         dblConversion = (5280 ^ 2)       ' CONVERT TO SQUARE MILES
1237:         strCountName = "Sq. Miles"
1238:       End If
1239:     ElseIf dblTotalSize > 1000000 Then
1240:       If booIsMetric Then
1241:         dblConversion = 10000            ' CONVERT TO HECTARES
1242:         strCountName = "Hectares"
1243:       Else
1244:         dblConversion = 43560            ' CONVERT TO ACRES
1245:         strCountName = "Acres"
1246:       End If
1247:     End If
    
 '   MsgBox "dblTotalSize = " & CStr(dblTotalSize) & vbCrLf & _
           "dblConversion = " & CStr(dblConversion) & vbCrLf & _
           "strCountName = " & CStr(strCountName)
           
1253:     If dblConversion > 1 Then
1254:       For lngIndex = 0 To pArray.Count - 1
1255:         dblPair = pArray.Element(lngIndex)
1256:         dblPair(2) = dblPair(2) / dblConversion
1257:         pTempArray.Add dblPair
1258:       Next lngIndex
1259:       Set pArray = pTempArray
1260:     End If
1261:   End If
  
1263:   Set InputsBothPolygon = New esriSystem.VarArray
1264:   InputsBothPolygon.Add pArray
1265:   InputsBothPolygon.Add strCountName
1266:   InputsBothPolygon.Add BooVar1HasOther
1267:   InputsBothPolygon.Add booVar2HasOther
  
1269:   Set pArray = Nothing
1270:   Set pFClass1 = Nothing
1271:   Set pFClass2 = Nothing
1272:   Set pFLayer1 = Nothing
1273:   Set pFLayer2 = Nothing
1274:   Set pSpRef1 = Nothing
1275:   Set pSpRef2 = Nothing
1276:   Set pClipPolygon = Nothing
1277:   Set pClipSpRef = Nothing
1278:   Set psbar = Nothing
1279:   Set pPro = Nothing
1280:   Set pClone = Nothing
1281:   Set pAOI_Layer1Polygon = Nothing
1282:   Set pAOI_Layer2Polygon = Nothing
1283:   Set pPrjSpRef = Nothing
1284:   Set pUnit = Nothing
1285:   Set pSpFilter = Nothing
1286:   Set pPrimarySpRef = Nothing
1287:   Set pPolygon = Nothing
1288:   Set pEnv = Nothing
1289:   Set pRelOp = Nothing
1290:   Set pTopoOp = Nothing
1291:   Set pTempPolygon = Nothing
1292:   Set pArea = Nothing
1293:   Set pFCursor = Nothing
1294:   Set pFeature = Nothing
1295:   Set pPolygonData = Nothing
1296:   Set pLayer1Polygons = Nothing
1297:   Set pLayer2Polygons = Nothing
1298:   Set pIntersectArray = Nothing
1299:   Set pPolygonData2 = Nothing
1300:   Set pNoIntersectPolygon = Nothing
1301:   Set pComparePolygon = Nothing
1302:   Set pIntersectPolygon = Nothing
1303:   Set pTopoOp2 = Nothing
1304:   Set pTopoOpClip = Nothing
1305:   Set pTempArray = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "InputsBothPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CheckPolygonIntersects( _
    pPolygon1 As IPolygon, _
    dblXMin1 As Double, _
    dblXMax1 As Double, _
    dblYMin1 As Double, _
    dblYMax1 As Double, _
    pPolygon2 As IPolygon, _
    dblXMin2 As Double, _
    dblXMax2 As Double, _
    dblYMin2 As Double, _
    dblYMax2 As Double, _
    pRelOp As IRelationalOperator) As Boolean
    
  On Error GoTo ErrorHandler

1327:   If dblXMin1 > dblXMax2 Or _
     dblXMin2 > dblXMax1 Or _
     dblYMin1 > dblYMax2 Or _
     dblYMin2 > dblYMax1 Then
1331:       CheckPolygonIntersects = False
1332:   Else
1333:     Set pRelOp = pPolygon1
1334:     CheckPolygonIntersects = Not pRelOp.Disjoint(pPolygon2)
1335:   End If

  Exit Function
ErrorHandler:
  HandleError True, "CheckPolygonIntersects " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function InputsRasterPolygon(pParams As esriSystem.IVariantArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

1345:   Set InputsRasterPolygon = Nothing
  
  Dim pArray As esriSystem.IVariantArray
1348:   Set pArray = New esriSystem.VarArray
  Dim dblPair() As Double
  ReDim dblPair(4)
  
  Dim pRastBand As IRasterBand
  Dim pFClass As IFeatureClass
  Dim pFLayer As IFeatureLayer
  Dim strPolygonName As String
  Dim strRasterName As String
  Dim strRasterStatistic As String
  Dim pSpRefPolygon As ISpatialReference
  Dim pSpRefRaster As ISpatialReference
  Dim booClipToPolygon As Boolean
  Dim pClipPolygon As IPolygon
  Dim pClipSpRef As ISpatialReference
  Dim psbar As IStatusBar
  Dim pPro As IStepProgressor
  Dim lngFieldIndex As Long
  Dim strFieldAlias As String
  Dim pClone As IClone
  Dim pAOI_Layer1Polygon As IPolygon
  Dim pAOI_Layer2Polygon As IPolygon
  
  Dim booFirstIsRaster As Boolean
  Dim pUnknown As IUnknown
1373:   Set pUnknown = pParams.Element(0)
1374:   booFirstIsRaster = TypeOf pUnknown Is IRasterBand
  
1376:   If booFirstIsRaster Then
1377:     Set pRastBand = pParams.Element(0)
1378:     Set pSpRefRaster = pParams.Element(1)
1379:     strRasterName = pParams.Element(2)
1380:     strRasterStatistic = pParams.Element(4)
1381:     Set pFLayer = pParams.Element(5)
1382:     Set pFClass = pFLayer.FeatureClass
1383:     Set pSpRefPolygon = pParams.Element(6)
1384:     strPolygonName = pParams.Element(7)
1385:     lngFieldIndex = pParams.Element(8)
1386:     strFieldAlias = pParams.Element(9)
1387:   Else
1388:     Set pFLayer = pParams.Element(0)
1389:     Set pFClass = pFLayer.FeatureClass
1390:     Set pSpRefPolygon = pParams.Element(1)
1391:     strPolygonName = pParams.Element(2)
1392:     lngFieldIndex = pParams.Element(3)
1393:     strFieldAlias = pParams.Element(4)
1394:     Set pRastBand = pParams.Element(5)
1395:     Set pSpRefRaster = pParams.Element(6)
1396:     strRasterName = pParams.Element(7)
1397:     strRasterStatistic = pParams.Element(9)
1398:   End If
  
1400:   booClipToPolygon = pParams.Element(10)
1401:   Set pClipPolygon = pParams.Element(11)
1402:   Set pClipSpRef = pParams.Element(12)
1403:   Set psbar = pParams.Element(13)
1404:   Set pPro = pParams.Element(14)
  
  Dim pMxDoc As IMxDocument
1407:   Set pMxDoc = pParams.Element(15)
  
1409:   pPro.position = 0
1410:   psbar.ShowProgressAnimation "Unifying projections...", ""
1411:   psbar.PlayProgressAnimation True
1412:   DoEvents
  
  Dim dblVal1 As Double
  Dim dblVal2 As Double
  Dim dblWeight As Double
  Dim strSuggestUnits As String
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim pUnit As ILinearUnit
  Dim booIsMetric As Boolean
  Dim strCountName As String
  Dim pSpFilter As ISpatialFilter

  ' ALL ANALYSIS WILL BE DONE IN PROJECTION OF RASTER
  Dim booMustProjectPolygons As Boolean
1426:   booMustProjectPolygons = Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRefRaster, pSpRefPolygon)
  Dim pInMemFC As IFeatureClass
  Dim pFLayerClip As IPolygon
  Dim pTemplateField As IField
  Dim pPolygonArray As esriSystem.IArray
  Dim pValueArray As esriSystem.IVariantArray
  
1433:   Set pTemplateField = pFClass.Fields.Field(lngFieldIndex)
1434:   Set pPolygonArray = New esriSystem.Array
1435:   Set pValueArray = New esriSystem.VarArray
  
  Dim pFCursor As IFeatureCursor
  Dim pFeature As IFeature
  Dim lngCount As Long
1440:   psbar.Message(0) = "Searching for polygons in analysis area..."
1441:   DoEvents
  Dim lngCounter As Long
  Dim pPolygon As IPolygon
  Dim varValue As Variant
  Dim pTopoOp As ITopologicalOperator
  Dim pClippedTopoOp As ITopologicalOperator
  Dim pRelOp As IRelationalOperator
  Dim pClippedPoly As IPolygon
  Dim lngDoEventsCounter As Long
  
  ' IF USING AN ANALYSIS AREA POLYGON, THEN FIRST SELECT AND CLIP ALL POLYGONS TO THIS ANALYSIS AREA
1452:   If booClipToPolygon Then
    
    ' REPROJECT CLIP POLYGON IF IT IS NOT IN THE CORRECT PROJECTION
1455:     Set pClone = pClipPolygon
1456:     Set pFLayerClip = pClone.Clone
1457:     If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pClipSpRef, pSpRefPolygon) Then
1458:       pFLayerClip.Project pSpRefPolygon
1459:     End If
    
    ' BUILD SPATIAL FILTER TO ONLY CHOOSE THE POLYGONS WITHIN THE ANALYSIS AREA.
1462:     Set pSpFilter = New SpatialFilter
1463:     Set pSpFilter.Geometry = pFLayerClip
1464:     pSpFilter.SpatialRel = esriSpatialRelIntersects
1465:     pSpFilter.AddField pTemplateField.Name
    
    ' ANALYZE ALL POLYGONS WITHIN ANALYSIS AREA, AND CLIP THEM TO ANALYSIS BOUNDARY IF NECESSARY
    ' POPULATE BOTH THE GEOMETRY AND VALUE ARRAYS, WHICH WILL BE USED TO CREATE THE IN-MEMORY VERSION
    ' OF THE FEATURE CLASS
1470:     Set pRelOp = pFLayerClip
1471:     Set pTopoOp = pFLayerClip
1472:     lngCount = pFClass.FeatureCount(pSpFilter)
1473:     Set pFCursor = pFClass.Search(pSpFilter, True)
1474:     Set pFeature = pFCursor.NextFeature

1476:     lngDoEventsCounter = 0
1477:     psbar.ShowProgressBar "Intersecting polygons from " & pFLayer.Name & " with Analysis Area polygon...", _
        1, lngCount, 1, True
1479:     pPro.position = 1
    
1481:     Do Until pFeature Is Nothing
    
      ' REFRESHES THE SCREEN EVERY 5%, HOPEFULLY WILL HELP ARCGIS PROGRESS BAR TO KEEP UPDATING EVEN IF YOU CLICK ON ANOTHER PROGRAM
1484:       lngDoEventsCounter = lngDoEventsCounter + 1
1485:       If lngDoEventsCounter > (lngCount / 20) Then
1486:         lngDoEventsCounter = 0
1487:         DoEvents
1488:       End If
    
1490:       Set pPolygon = pFeature.ShapeCopy
1491:       varValue = pFeature.Value(lngFieldIndex)
      
1493:       If pRelOp.Contains(pPolygon) Then
1494:         Set pClippedPoly = pPolygon
1495:       Else
1496:         Set pClippedPoly = pTopoOp.Intersect(pPolygon, pPolygon.Dimension)
1497:       End If
1498:       Set pClippedTopoOp = pClippedPoly
1499:       pClippedTopoOp.Simplify
      
      ' REPROJECT CLIPPED POLY TO RASTER SPATIAL REFERENCE IF NECESSARY
1502:       If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pClippedPoly.SpatialReference, pSpRefRaster) Then
1503:         pClippedPoly.Project pSpRefRaster
1504:       End If
1505:       pPolygonArray.Add pClippedPoly
      
1507:       varValue = pFeature.Value(lngFieldIndex)
1508:       pValueArray.Add varValue
            
1510:       Set pFeature = pFCursor.NextFeature
1511:     Loop
    
1513:     Set pInMemFC = Linkages.MyGeneralOperations.CreateInMemoryFeatureClass(pPolygonArray, _
        pValueArray, pTemplateField)
1515:   Else
  
    ' IF NOT CLIPPING TO ANALYSIS AREA POLYGON, THEN ONLY MAKE AN IN-MEMORY VERSION OF THE FEATURE CLASS IF
    ' THE FEATURE CLASS IS NOT IN THE CORRECT PROJECTION
    
1520:     If booMustProjectPolygons Then
    
1522:       lngCount = pFClass.FeatureCount(Nothing)
1523:       Set pFCursor = pFClass.Search(Nothing, True)
1524:       Set pFeature = pFCursor.NextFeature
  
1526:       lngDoEventsCounter = 0
1527:       psbar.ShowProgressBar "Extracting polygons from " & pFLayer.Name & "...", _
          1, lngCount, 1, True
1529:       pPro.position = 1
      
1531:       Do Until pFeature Is Nothing
      
        ' REFRESHES THE SCREEN EVERY 5%, HOPEFULLY WILL HELP ARCGIS PROGRESS BAR TO KEEP UPDATING EVEN IF YOU CLICK ON ANOTHER PROGRAM
1534:         lngDoEventsCounter = lngDoEventsCounter + 1
1535:         If lngDoEventsCounter > (lngCount / 20) Then
1536:           lngDoEventsCounter = 0
1537:           DoEvents
1538:         End If
      
1540:         Set pPolygon = pFeature.ShapeCopy
1541:         varValue = pFeature.Value(lngFieldIndex)
        
        ' PROJECT POLYGON TO RASTER SPATIAL REFERENCE
1544:         pPolygon.Project pSpRefRaster
        
        ' ADD DATA TO ARRAYS
1547:         pPolygonArray.Add pClippedPoly
        
1549:         varValue = pFeature.Value(lngFieldIndex)
1550:         pValueArray.Add varValue
              
1552:         Set pFeature = pFCursor.NextFeature
1553:       Loop
      
1555:       Set pInMemFC = Linkages.MyGeneralOperations.CreateInMemoryFeatureClass(pPolygonArray, _
          pValueArray, pTemplateField)
        
1558:     Else
1559:       Set pInMemFC = pFClass
1560:     End If
1561:   End If
  
  ' NOW HAVE FEATURE CLASS IN CORRECT PROJECTION WITH CORRECT ATTRIBUE FIELD, _
  ' AND CLIPPED TO ANALYSIS AREA IF NECESSARY
  
  ' FOR DEBUGGING
'    Dim pNewLayer As IFeatureLayer
'    Set pNewLayer = New FeatureLayer
'    Set pNewLayer.FeatureClass = pInMemFC
'    pNewLayer.Name = "test FC"
'    pMxDoc.FocusMap.AddLayer pNewLayer
  
'  Exit Function
  
  ' RASTER STATISTIC OPTIONS
  '  strRasterStatistic = "Cell Values"
  '  strRasterStatistic = "Polygon Minimum"
  '  strRasterStatistic = "Polygon Mean"
  '  strRasterStatistic = "Polygon Maximum"
  '  strRasterStatistic = "Polygon Range"
  '  strRasterStatistic = "Polygon St. Dev."
  '  strRasterStatistic = "Polygon Sum"
  
  Dim pFClassDescriptor As IFeatureClassDescriptor
  Dim pRastAnEnv As IRasterAnalysisEnvironment
  Dim pZonalOp As IZonalOp
  Dim pZonalTable As ITable
  Dim pSTColl As IStandaloneTableCollection
  Dim pStTable As IStandaloneTable
  Dim pAOIPolyForRaster As IPolygon
  Dim lngZoneField As Long
  Dim lngAreaField As Long
  Dim lngPolyField As Long
  Dim pRowCursor As ICursor
  Dim pRow As IRow
  
  Dim pFRasterBand As IRasterBand
  Dim pRastBandColl As IRasterBandCollection
  Dim pRastDataset As IRasterDataset
  Dim pConvOp As IConversionOp
  Dim pWorkspace As IWorkspace
  Dim strTempName As String
  Dim pNewParams As esriSystem.IVariantArray
  Dim pDataset As IDataset
  
  Dim dblTotalSize As Double
  Dim lngIndex As Long
  Dim BooVar1HasOther As Boolean
  Dim booVar2HasOther As Boolean
  Dim pTempArray As esriSystem.IVariantArray
  Dim dblConversion As Double
  
  Dim pZoneCursor As IFeatureCursor
  Dim pZoneFeature As IFeature
  Dim pZoneCollection As Collection
  Dim lngZoneFieldIndex As Long
  Dim lngZoneValue As Long
  Dim varPolygonValue As Variant
  Dim pArea As IArea
  Dim pSubArray As esriSystem.IVariantArray
  
  
1623:   If strRasterStatistic = "Cell Values" Then
    ' CONVERT FEATURE CLASS TO RASTER AND TREAT AS IF ANALYZING TWO RASTERS
1625:     Set pConvOp = New RasterConversionOp
1626:     Set pRastAnEnv = pConvOp
    
    ' ADJUST ANALYSIS EXTENT IF CLIPPING TO ANALYSIS AREA POLYGON
1629:     If booClipToPolygon Then
1630:       If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pClipSpRef, pSpRefRaster) Then
1631:         Set pClone = pClipPolygon
1632:         Set pAOIPolyForRaster = pClone.Clone
1633:         pAOIPolyForRaster.Project pSpRefRaster
1634:       Else
1635:         Set pAOIPolyForRaster = pClipPolygon
1636:       End If
1637:       pRastAnEnv.SetExtent esriRasterEnvValue, pAOIPolyForRaster.Envelope, pRastBand
1638:     Else
1639:       pRastAnEnv.SetExtent esriRasterEnvMinOf, , pRastBand
1640:     End If
1641:     pRastAnEnv.SetCellSize esriRasterEnvValue, pRastBand
1642:     Set pRastAnEnv.OutSpatialReference = pSpRefRaster
1643:     Set pWorkspace = Linkages.MyGeneralOperations.ReturnTempRasterWorkspace
1644:     strTempName = Linkages.MyGeneralOperations.MakeUniqueRasterName2(pWorkspace, "temp", True)
'    Set pRastDataset = pConvOp.ToRasterDataset(pInMemFC, "GRID", pWorkspace, strTempName)
    
    
1648:     Set pFClassDescriptor = New FeatureClassDescriptor
1649:     pFClassDescriptor.Create pInMemFC, Nothing, pTemplateField.Name
1650:     Set pRastDataset = pConvOp.ToRasterDataset(pFClassDescriptor, "GRID", pWorkspace, strTempName)
    
1652:     Set pRastBandColl = pRastDataset
1653:     Set pFRasterBand = pRastBandColl.Item(0)
    
1655:     Set pNewParams = New esriSystem.VarArray
1656:     If booFirstIsRaster Then
1657:       pNewParams.Add pRastBand
1658:       pNewParams.Add pSpRefRaster
1659:       pNewParams.Add strRasterName
1660:       pNewParams.Add pFRasterBand
1661:       pNewParams.Add pSpRefRaster
1662:       pNewParams.Add strPolygonName
1663:     Else
1664:       pNewParams.Add pFRasterBand
1665:       pNewParams.Add pSpRefRaster
1666:       pNewParams.Add strPolygonName
1667:       pNewParams.Add pRastBand
1668:       pNewParams.Add pSpRefRaster
1669:       pNewParams.Add strRasterName
1670:     End If
1671:     pNewParams.Add booClipToPolygon
1672:     pNewParams.Add pClipPolygon
1673:     pNewParams.Add pClipSpRef
1674:     pNewParams.Add psbar
1675:     pNewParams.Add pPro
1676:     pNewParams.Add pMxDoc
    
1678:     Set InputsRasterPolygon = Linkages.CrossTabFunctions.InputsBothRaster(pNewParams)
    
1680:     Set pNewParams = Nothing
1681:     Set pConvOp = Nothing
1682:     Set pRastAnEnv = Nothing
1683:     Set pWorkspace = Nothing
1684:     Set pRastBandColl = Nothing
1685:     Set pFRasterBand = Nothing
'    Set pDataset = pRastDataset
1687:     Set pRastDataset = Nothing
'    If pDataset.CanDelete Then
'      pDataset.DELETE
'    End If
  
1692:   Else
    ' DO ZONAL FUNCTION
    ' REMEMBER THAT ZONAL TABLE FUNCTION REQUIRES "ZONE" FIELD TO BE INTEGER, SO CANNOT ACTUALLY USE THE
    ' ATTRIBUTE FIELD SELECTED BY USER.  RATHER, CREATE A COLLECTION OBJECT THAT HAS THE ATTRIBUTE VALUE
    ' THAT CORRESPONDS WITH THE POLYGON OID FIELD (ZONES WILL BE DEFINED BY OID VALUES).
    
1698:     Set pFClassDescriptor = New FeatureClassDescriptor
1699:     pFClassDescriptor.Create pInMemFC, Nothing, pInMemFC.OIDFieldName
1700:     Set pZonalOp = New RasterZonalOp
1701:     Set pRastAnEnv = pZonalOp
    
    
    ' MAKE COLLECTION OF ZONE / ATTRIBUTE VALUES
    ' FOR DEBUGGING
    Dim strTempReport As String
    ' strTempReport = "Zone Field Index = " & CStr(lngZoneFieldIndex) & vbCrLf
    
1709:     Set pZoneCollection = New Collection
1710:     Set pZoneCursor = pInMemFC.Search(pSpFilter, True)
1711:     Set pZoneFeature = pZoneCursor.NextFeature
1712:     lngZoneFieldIndex = pInMemFC.FindField(pTemplateField.Name)
1713:     Do Until pZoneFeature Is Nothing
1714:       Set pSubArray = New esriSystem.VarArray
1715:       pSubArray.Add pZoneFeature.Value(lngZoneFieldIndex)
1716:       pSubArray.Add pZoneFeature.ShapeCopy
1717:       pZoneCollection.Add pSubArray, CStr(pZoneFeature.OID)
      
      ' FOR DEBUGGING
      ' strTempReport = strTempReport & "Key = " & CStr(pZoneFeature.OID) & ",  Value = " & CStr(pZoneFeature.Value(lngZoneFieldIndex)) & vbCrLf
      
1722:       Set pZoneFeature = pZoneCursor.NextFeature
1723:     Loop
    
    ' FOR DEBUGGING
'    Dim frmReportForm As New frmReport_modal
'    frmReportForm.txtReport.Text = strTempReport
'    frmReportForm.Show vbModal
'    Set frmReportForm = Nothing
'    Dim pTemp As IDataset
'    Dim pTemp2 As IGeoDataset
'    Set pTemp = pFClassDescriptor.FeatureClass
'    Set pTemp2 = pTemp
'    MsgBox "Using Field [" & pFClassDescriptor.FieldName & "] from Feature Class " & pTemp.BrowseName & vbCrLf & _
'        "Feature Class Spatial Reference = " & pTemp2.SpatialReference.Name & vbCrLf & _
'        "pSpRefRaster = " & pSpRefRaster.Name
    
    ' ADJUST ANALYSIS EXTENT IF CLIPPING TO ANALYSIS AREA POLYGON
1739:     If booClipToPolygon Then
1740:       If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pClipSpRef, pSpRefRaster) Then
1741:         Set pClone = pClipPolygon
1742:         Set pAOIPolyForRaster = pClone.Clone
1743:         pAOIPolyForRaster.Project pSpRefRaster
1744:       Else
1745:         Set pAOIPolyForRaster = pClipPolygon
1746:       End If
1747:       pRastAnEnv.SetExtent esriRasterEnvValue, pAOIPolyForRaster.Envelope, pRastBand
1748:     Else
1749:       pRastAnEnv.SetExtent esriRasterEnvMaxOf, , pRastBand
1750:     End If
1751:     pRastAnEnv.SetCellSize esriRasterEnvValue, pRastBand
1752:     Set pRastAnEnv.OutSpatialReference = pSpRefRaster
1753:     Set pZonalTable = pZonalOp.ZonalStatisticsAsTable(pFClassDescriptor, pRastBand, True)
    
    ' FOR DEBUGGING
'    Set pStTable = New StandaloneTable
'    Set pStTable.Table = pZonalTable
'    Set pSTColl = pMxDoc.FocusMap
'    pSTColl.AddStandaloneTable pStTable
'    pMxDoc.ActiveView.Refresh
'    pMxDoc.UpdateContents
    
1763:     lngAreaField = pZonalTable.FindField("AREA")
1764:     lngPolyField = pZonalTable.FindField("VALUE")
    Select Case strRasterStatistic
      Case "Polygon Minimum"
1767:         lngZoneField = pZonalTable.FindField("MIN")
      Case "Polygon Mean"
1769:         lngZoneField = pZonalTable.FindField("MEAN")
      Case "Polygon Maximum"
1771:         lngZoneField = pZonalTable.FindField("MAX")
      Case "Polygon Range"
1773:         lngZoneField = pZonalTable.FindField("RANGE")
      Case "Polygon St. Dev."
1775:         lngZoneField = pZonalTable.FindField("STD")
      Case "Polygon Sum"
1777:         lngZoneField = pZonalTable.FindField("SUM")
1778:     End Select
    
    
    
1782:     Set pRowCursor = pZonalTable.Search(Nothing, True)
1783:     Set pRow = pRowCursor.NextRow
    ' strTempReport = strTempReport & vbCrLf & vbCrLf & "Examining zone table..." & vbCrLf
    
1786:     Do Until pRow Is Nothing
    
      ' dblPair(0) = VALUE OF FIRST VARIABLE
      ' dblPair(1) = VALUE OF SECOND VARIABLE
      ' dblPair(2) = AREA OF OBSERVATION
      ' dblPair(3) = INDICATOR OF WHETHER FIRST VARIABLE IS NULL (<0 = TRUE)
      ' dblPair(4) = INDICATOR OF WHETHER SECOND VARIABLE IS NULL (<0 = TRUE)
    
      ' ADD DATA TO ARRAY
1795:       lngZoneValue = pRow.Value(lngPolyField)
1796:       Set pSubArray = pZoneCollection.Item(CStr(lngZoneValue))
1797:       varPolygonValue = pSubArray.Element(0)
1798:       Set pArea = pSubArray.Element(1)
      
      
1801:       If booFirstIsRaster Then
'        dblPair(1) = CDbl(pRow.Value(lngPolyField))

        
1805:         dblPair(0) = CDbl(pRow.Value(lngZoneField))        ' RASTER ZONAL STATISTIC VALUE   (1ST VARIABLE)
1806:         dblPair(3) = 1                                     ' > 0 INDICATED RASTER VALUES WERE NOT NULL
        
1808:         If IsNull(varPolygonValue) Then
1809:           dblPair(1) = 0                                   ' POLYGON ATTRIBUTE VALUE  (2ND VARIABLE)
1810:           dblPair(4) = -1                                  ' < 0 INDICATES THE POLYGON HAS NO CORRESPONDING VALUE FOR THE RASTER CELLS
1811:         Else
1812:           dblPair(1) = CDbl(varPolygonValue)               ' POLYGON ATTRIBUTE VALUE  (2ND VARIABLE)
1813:           dblPair(4) = 1                                   ' > 0 INDICATES POLYGON VALUE WAS NOT NULL
1814:         End If
        
'        dblPair(2) = CDbl(pRow.Value(lngAreaField))        ' AREA VALUE
1817:         dblPair(2) = CDbl(pArea.Area)         ' AREA VALUE
        
1819:       Else
'        dblPair(0) = CDbl(pRow.Value(lngPolyField))
        
1822:         If IsNull(varPolygonValue) Then
1823:           dblPair(0) = 0                                   ' POLYGON ATTRIBUTE VALUE  (1ST VARIABLE)
1824:           dblPair(3) = -1                                  ' < 0 INDICATES THE POLYGON HAS NO CORRESPONDING VALUE FOR THE RASTER CELLS
1825:         Else
1826:           dblPair(0) = CDbl(varPolygonValue)               ' POLYGON ATTRIBUTE VALUE  (1ST VARIABLE)
1827:           dblPair(3) = 1                                   ' > 0 INDICATES POLYGON VALUE WAS NOT NULL
1828:         End If

1830:         dblPair(1) = CDbl(pRow.Value(lngZoneField))        ' RASTER ZONAL STATISTIC VALUE   (1ST VARIABLE)
1831:         dblPair(4) = 1                                     ' > 0 INDICATED RASTER VALUES WERE NOT NULL
        
'        dblPair(2) = CDbl(pRow.Value(lngAreaField))        ' AREA VALUE
1834:         dblPair(2) = CDbl(pArea.Area)         ' AREA VALUE
1835:       End If
      
      ' FOR DEBUGGING
      ' strTempReport = strTempReport & "Row OID " & CStr(pRow.OID) & ":  Zone = " & CStr(lngZoneValue) & ".......Variable = " & _
          CStr(varPolygonValue) & ".......Var1 = " & CStr(dblPair(0)) & ".......Var2 = " & CStr(dblPair(1)) & ".......Area = " & CStr(pArea.Area) & vbCrLf
          
1841:       pArray.Add dblPair
1842:       Set pRow = pRowCursor.NextRow
1843:     Loop
  
  
'    ' FOR DEBUGGING
'    Dim frmReportForm As New frmReport_modal
'    frmReportForm.txtReport.Text = strTempReport
'    frmReportForm.Show vbModal
'    Set frmReportForm = Nothing
  
   ' GET MEASUREMENT UNITS FOR THIS ANALYSIS
1853:    If TypeOf pSpRefRaster Is IGeographicCoordinateSystem Then
1854:      strSuggestUnits = "Pseudo-Degrees"
1855:    Else
1856:      Set pPrjSpRef = pSpRefRaster
1857:      Set pUnit = pPrjSpRef.CoordinateUnit
1858:      booIsMetric = (pUnit.MetersPerUnit = 1)
1859:      strSuggestUnits = pUnit.Name
1860:      If InStr(1, strSuggestUnits, "meter", vbTextCompare) > 0 Then
1861:        strSuggestUnits = "Meters"
1862:      ElseIf InStr(1, strSuggestUnits, "foot", vbTextCompare) > 0 Then
1863:        strSuggestUnits = "Feet"
1864:      End If
1865:    End If
1866:    strCountName = "Square " & strSuggestUnits
   
   ' RESCALE VALUES IF UNITS ARE FEET OR METERS, AND IF TOTAL IS ABOVE THRESHOLDS
   
1870:    dblTotalSize = 0
   
1872:    pPro.position = 0
1873:    psbar.ShowProgressBar "Checking Data Range...", 1, pArray.Count, 1, True
1874:    pPro.Message = "Checking Data Range..."
   
1876:    BooVar1HasOther = False
1877:    booVar2HasOther = False
1878:    If pArray.Count > 0 Then
1879:      For lngIndex = 0 To pArray.Count - 1
1880:        dblPair = pArray.Element(lngIndex)
1881:        dblTotalSize = dblTotalSize + dblPair(2)
1882:        If dblPair(3) < 0 Then BooVar1HasOther = True
1883:        If dblPair(4) < 0 Then booVar2HasOther = True
1884:        If (dblTotalSize > 100000000) And BooVar1HasOther And booVar2HasOther Then Exit For
1885:        pPro.Step
1886:      Next lngIndex
1887:    End If
   
1889:    Set pTempArray = New esriSystem.VarArray
   
   
1892:    If dblTotalSize > 0 Then
1893:      dblConversion = 1
1894:      If dblTotalSize > 100000000 Then
1895:        If booIsMetric Then
1896:          dblConversion = 1000000          ' CONVERT TO SQUARE KILOMETERS
1897:          strCountName = "Sq. Km"
1898:        Else
1899:          dblConversion = (5280 ^ 2)       ' CONVERT TO SQUARE MILES
1900:          strCountName = "Sq. Miles"
1901:        End If
1902:      ElseIf dblTotalSize > 1000000 Then
1903:        If booIsMetric Then
1904:          dblConversion = 10000            ' CONVERT TO HECTARES
1905:          strCountName = "Hectares"
1906:        Else
1907:          dblConversion = 43560            ' CONVERT TO ACRES
1908:          strCountName = "Acres"
1909:        End If
1910:      End If
     
  '   MsgBox "dblTotalSize = " & CStr(dblTotalSize) & vbCrLf & _
            "dblConversion = " & CStr(dblConversion) & vbCrLf & _
            "strCountName = " & CStr(strCountName)
            
1916:      pPro.position = 0
1917:      psbar.ShowProgressBar "Checking Data Range (Pass 2)", 1, pArray.Count, 1, True
1918:      pPro.Message = "Checking Data Range (Pass 2)..."
     
     
     ' strTempReport = strTempReport & vbCrLf & vbCrLf & "After converting to " & strCountName & vbCrLf
     
1923:      If dblConversion > 1 Then
1924:        For lngIndex = 0 To pArray.Count - 1
1925:          dblPair = pArray.Element(lngIndex)
1926:          dblPair(2) = dblPair(2) / dblConversion
1927:          pTempArray.Add dblPair
        
          ' FOR DEBUGGING
          ' strTempReport = strTempReport & "Var1 = " & CStr(dblPair(0)) & ".......Var2 = " & CStr(dblPair(1)) & ".......Area = " & CStr(dblPair(2)) & vbCrLf
          

1933:          pPro.Step
1934:        Next lngIndex
1935:        Set pArray = pTempArray
1936:      End If
1937:    End If
   
1939:    pPro.position = 0
1940:    psbar.HideProgressBar
   
1942:    Set InputsRasterPolygon = New esriSystem.VarArray
1943:    InputsRasterPolygon.Add pArray
1944:    InputsRasterPolygon.Add strCountName
1945:    InputsRasterPolygon.Add BooVar1HasOther
1946:    InputsRasterPolygon.Add booVar2HasOther
    
'        ' FOR DEBUGGING
'    Dim frmReportForm As New frmReport_modal
'    frmReportForm.txtReport.Text = strTempReport
'    frmReportForm.Show vbModal
'    Set frmReportForm = Nothing
    
1954:   End If
  
1956:   Set pRastBandColl = Nothing
1957:   Set pRastDataset = Nothing
1958:   Set pArray = Nothing
1959:   Set pRastBand = Nothing
1960:   Set pFClass = Nothing
1961:   Set pFLayer = Nothing
1962:   Set pSpRefPolygon = Nothing
1963:   Set pSpRefRaster = Nothing
1964:   Set pClipPolygon = Nothing
1965:   Set pClipSpRef = Nothing
1966:   Set psbar = Nothing
1967:   Set pPro = Nothing
1968:   Set pClone = Nothing
1969:   Set pAOI_Layer1Polygon = Nothing
1970:   Set pAOI_Layer2Polygon = Nothing
1971:   Set pUnknown = Nothing
1972:   Set pMxDoc = Nothing
1973:   Set pPrjSpRef = Nothing
1974:   Set pUnit = Nothing
1975:   Set pSpFilter = Nothing
1976:   Set pInMemFC = Nothing
1977:   Set pFLayerClip = Nothing
1978:   Set pTemplateField = Nothing
1979:   Set pPolygonArray = Nothing
1980:   Set pValueArray = Nothing
1981:   Set pFCursor = Nothing
1982:   Set pFeature = Nothing
1983:   Set pPolygon = Nothing
1984:   Set pTopoOp = Nothing
1985:   Set pClippedTopoOp = Nothing
1986:   Set pRelOp = Nothing
1987:   Set pClippedPoly = Nothing
1988:   Set pFClassDescriptor = Nothing
1989:   Set pRastAnEnv = Nothing
1990:   Set pZonalOp = Nothing
1991:   Set pSTColl = Nothing
1992:   Set pStTable = Nothing
1993:   Set pAOIPolyForRaster = Nothing
1994:   Set pRowCursor = Nothing
1995:   Set pRow = Nothing
1996:   Set pTempArray = Nothing
1997:   Set pFRasterBand = Nothing
1998:   Set pConvOp = Nothing
1999:   Set pWorkspace = Nothing
2000:   Set pNewParams = Nothing
2001:   Set pZoneCursor = Nothing
2002:   Set pZoneFeature = Nothing
2003:   Set pZoneCollection = Nothing
2004:   Set pArea = Nothing
2005:   Set pSubArray = Nothing
  
2007:   If Not pZonalTable Is Nothing Then

2009:     Set pDataset = pZonalTable
2010:     Set pZonalTable = Nothing
2011:     If pDataset.CanDelete Then
2012:       pDataset.DELETE
2013:     End If
2014:     Set pDataset = Nothing
2015:   End If
  
  Exit Function
ErrorHandler:
  HandleError True, "InputsRasterPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
