Attribute VB_Name = "MyGeometricOperations"
' MyGeometricOperations
'                 ArcCosJen - Returns ArcCos (Inverse Cosine)
'                 ArcSinJen - Returns ArcSin (Inverse Sine)
'                 AsDegrees - CONVERTS RADIANS TO DEGREES
'                 AsRadians - CONVERTS DEGREES TO RADIANS
'                     atan2 - Given DeltaY and DeltaX, returns ArcTangent that is sensitive to quadrant.
'          AzimuthHaversine - GIVEN TWO GEOGRAPHIC POINTS, RETURNS STARTING AZIMUTH POINTING IN GREAT CIRCLE OVER SPHERE.
'               CalcBearing - GIVEN TWO POINTS, CALCULATES THE CARTESIAN BEARING, WHERE 0 = NORTH, 360 DEGREES GOING CLOCKWISE
'        CalcCheckClockwise - CHECKS IF 3 CONSECUTIVE POINTS ARE ARRANGED COUNTERCLOCKWISE
'         CalcClosestPoints - GIVEN TWO GEOMETRIES AND OPTIONAL NUMBER OF TIMES TO GO BACK AND FORTH BETWEEN CURVES, RETURNS
'                             IArray CONTAINING:
'                             IStringArray containing either "Intersecting Shapes" or
'                                "Empty Shapes" + two booleans indicating which geometry is empty.
'                             -- OR --
'                             3 OBJECTS:
'                             0) Connector Line AS IPOLYLINE
'                             1) Closest Point on Geometry #1         AS IPOINT
'                             2) Closest Point on Geometry #2         AS IPOINT
'            CalcDistMatrix - GIVEN AN IARRAY OF SHAPES, RETURNS A COLLECTION WHERE:
'                             INDEX = IArrayIndex1 & "_" & IArrayIndex1, and Object =
'                             IArray of {Distance, Optional Line, Optional Azimuth}
'             CalcPointLine - GIVEN POINT, DISTANCE, AZIMUTH, EMPTY ENDPOINT AND EMPTY POLYLINE, REPLACES EMPTY
'                             ENDPOINT WITH ACTUAL ENDPOINT AND OPTIONALLY RETURNS A POLYLINE CONNECTOR
'            CurveToPolygon - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolygon.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'           CurveToPolyline - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolyline.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'                  DegToRad - Given Degrees (double), returns Radians (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsRadians)
'    DistanceVincentyPoints - Given 2 points and empty double variables for azimuths, returns distance in meters (using WGS84)
'                             and start and end azimuths of geodesic curve.
'   DistanceVincentyNumbers - Given numeric values for latitude and longitude for 2 points, plus empty double variables
'                             for azimuths, returns distance in meters (using WGS84) and start and end azimuths of geodesic curve.
'      EllipticArcToPolygon - Given a segment collection and number of vertices, returns a polygon4 simulating the ellipse
'                             by generating points along the arc and then calculating a convex hull around the points.
'     EllipticArcToPolygon2 - GIVEN A SEG COLLECTION AND NUMBER OF VERTICES, RETURNS A MULTIPOINT WITH APPROXIMATELY THE
'                             REQUESTED NUMBER OF POINTS DISTRIBUTED ALONG THE ARC.
'         EnvelopeToPolygon - GIVEN AN ENVELOPE, RETURNS A POLYGON
'  Graphic_MakeFromGeometry - GIVEN A MAP DOCUMENT, GEOMETRY AND OPTIONAL NAME AND SYMBOLOGY, ADDS GRAPHIC TO MAP.
'Graphic_ReturnElementFromGeometry - GIVEN MAP DOC, GEOMETRY, OPTIONAL NAME AND OPTIONAL ADD-TO-VIEW, RETURNS THE GRAPHIC
'                                    ELEMENT
'        MultipointCentroid - GIVEN AN IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE OF POINTS
'MultipointCentroidSpheriod - GIVEN A GEOGRAPHIC IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE ON SPHERE
'                  PointAdd - ADDS TWO POINTS
'         PointLineVincenty - Given Point, Distance and Azimuth, plus empty NewPoint and End Azimuth, plus optional
'                             number of vertices and empty Polyline, returns a new point the specified distance from the
'                             origin along geodesic, plus optional polyline with specified number of vertices.
' PointLineVincentyPerPoint - Given pPoint, dblLength, dblAzimuth, empty NewPoint and empty End Azimuth,
'                             fills empty point and azimuth with correct values.
'             PointSubtract - SUBTRACTS POINT B FROM POINT A
'         PolygonToPolyline - GIVEN A POLYGON, RETURNS A POLYLINE
'                  RadToDeg - Given Radians (double), returns Degrees (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsDegrees)
'              ShowVertices - GIVEN A MAP DOC, GEOMETRY AND OPTIONAL NAME, ADDS POINT GRAPHICS TO SCREEN SHOWING WHERE
'                             VERTICES ARE
'   SphericalLatLongToCart - SUBROUTINE:  GIVEN LATITUDE AND LONGITUDE, AND OPTIONAL RADIUS, FILLS X, Y, Z VALUES
'   SphericalCartToLatLong - SUBROUTINE:  GIVEN X, Y, Z, FILLS LATITUDE AND LONGITUDE VALUES
'      SphericalPolygonArea - GIVEN A POLYGON, CALCULATES AREA USING SERIES OF SphericalTriangleArea CALLS
'  SpheroidalPolylineLength - GIVEN A GEOGRAPHIC POLYLINE, RETURNS LENGTH IN METERS BASED ON VINCENTY'S EQUATIONS
'SpheroidalPolylineMidpoint - GIVEN A GEOGRAPHIC POLYLINE, DISTANCE VALUE, booAsRatio, RETURNS POINT AND OPTIONAL POLYLINE DISTANCE
'     SphericalTriangleArea - GIVEN 3 GEOGRAPHIC POINTS, CALCULATES SPHERICAL AREA
'        TriangleAreaPoints - GIVEN THREE POINTS, RETURNS AREA IN LOCAL UNITS
'      TriangleAreaPoints3D - GIVEN THREE 3-DIMENSIONAL POINTS, RETURNS AREA IN LOCAL UNITS
'        TriangleCentroid3D - GIVEN X,Y,Z COORDINATES FOR THREE 3D POINTS, RETURNS 3D CENTROID
'     TriangleCentroidPlane - GIVEN X,Y COORDINATES FOR THREE POINTS, RETURNS CENTROID

Option Explicit

Const dblPi As Double = 3.14159265358979
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\az_linkages\VB_Code\MyGeometricOperations.bas"


Public Function MultipointCentroidSpheroid(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim dblZ As Double
  Dim dblRunningX As Double
  Dim dblRunningY As Double
  Dim dblRunningZ As Double
  Dim pPointCollection As IPointCollection
84:   Set pPointCollection = pMultipoint
  Dim lngIndex As Long
86:   dblRunningX = 0
87:   dblRunningY = 0
88:   dblRunningZ = 0
89:   Set pPoint = New Point
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  
  Dim lngCounter As Long
94:   lngCounter = pPointCollection.PointCount
95:   For lngIndex = 0 To lngCounter - 1
96:     pPointCollection.QueryPoint lngIndex, pPoint
97:     SphericalLatLongToCart pPoint.X, pPoint.Y, dblX, dblY, dblZ
98:     dblRunningX = dblRunningX + dblX
99:     dblRunningY = dblRunningY + dblY
100:     dblRunningZ = dblRunningZ + dblZ
101:   Next lngIndex
  
103:   dblX = dblRunningX / lngCounter
104:   dblY = dblRunningY / lngCounter
105:   dblZ = dblRunningZ / lngCounter
  
  ' CONVERT BACK TO GEOGRAPHIC COORDINATES
108:   SphericalCartToLatLong dblLongitude, dblLatitude, dblX, dblY, dblZ
109:   Set MultipointCentroidSpheroid = New Point
110:   MultipointCentroidSpheroid.PutCoords dblLongitude, dblLatitude


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroidSpheroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function MultipointCentroid(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim pPointCollection As IPointCollection
125:   Set pPointCollection = pMultipoint
  
  Dim lngIndex As Long
128:   dblX = 0
129:   dblY = 0
130:   Set pPoint = New Point
131:   For lngIndex = 0 To pPointCollection.PointCount - 1
132:     pPointCollection.QueryPoint lngIndex, pPoint
133:     dblX = dblX + pPoint.X
134:     dblY = dblY + pPoint.Y
135:   Next lngIndex
  
137:   Set MultipointCentroid = New Point
138:   MultipointCentroid.PutCoords dblX, dblY


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PolygonToPolyline(pPolygon As IPolygon2) As IPolyline
  On Error GoTo ErrorHandler

  
  Dim pNewGeometryCollection As IGeometryCollection
151:   Set pNewGeometryCollection = New Polyline
  Dim lngNumParts As Long
  Dim pPolyComponents() As IPolygon 'Declare an array of polygon
  Dim lngIndex As Long
  Dim pSubPolygon As IPolygon4
  Dim pTopoOp As ITopologicalOperator
  Dim pGeometryBag As IGeometryCollection
  Dim pNewSegCollection As ISegmentCollection
  Dim pOrigSegcollection As ISegmentCollection
  Dim pRing As IRing
  Dim pPath As IPath
  Dim pSpRef As ISpatialReference
  Dim lngIndex2 As Long
  Dim pPolyline As IPolyline
  
166:   Set pSpRef = pPolygon.SpatialReference
167:   Set pNewGeometryCollection = New Polyline
  
  ' GET CONNECTED COMPONENTS OF POLYGON
170:   lngNumParts = pPolygon.ExteriorRingCount
  ReDim pPolyComponents(lngNumParts - 1) 'Redimension the array of polygons with number of exterior rings
172:   pPolygon.GetConnectedComponents lngNumParts, pPolyComponents(0) 'Pass the first element of the array
  
174:   For lngIndex = 0 To lngNumParts - 1
175:     Set pSubPolygon = pPolyComponents(lngIndex)
176:     Set pTopoOp = pSubPolygon
177:     pTopoOp.Simplify
    
    ' GET SINGLE EXTERNAL RING AND ALL INTERNAL RINGS
180:     Set pGeometryBag = pSubPolygon.ExteriorRingBag
181:     Set pRing = pGeometryBag.Geometry(0)
182:     Set pOrigSegcollection = pRing
183:     Set pNewSegCollection = New Path
184:     pNewSegCollection.AddSegmentCollection pOrigSegcollection
185:     Set pPath = pNewSegCollection
186:     pNewGeometryCollection.AddGeometry pPath
    
    ' ADD ANY INTERNAL RINGS TO POLYLINE
189:     If pSubPolygon.InteriorRingCount(pRing) > 0 Then
190:       Set pGeometryBag = pSubPolygon.InteriorRingBag(pRing)
191:       For lngIndex2 = 0 To pGeometryBag.GeometryCount - 1
192:         Set pRing = pGeometryBag.Geometry(lngIndex2)
193:         Set pOrigSegcollection = pRing
194:         Set pNewSegCollection = New Path
195:         pNewSegCollection.AddSegmentCollection pOrigSegcollection
196:         Set pPath = pNewSegCollection
197:         pNewGeometryCollection.AddGeometry pPath
198:       Next lngIndex2
199:     End If
200:   Next lngIndex
  
  ' CLEAN NEW POLYLINE
203:   Set pPolyline = pNewGeometryCollection
204:   Set pTopoOp = pPolyline
205:   pTopoOp.Simplify
206:   Set pPolyline.SpatialReference = pSpRef

208:   Set PolygonToPolyline = pPolyline


  Exit Function
ErrorHandler:
  HandleError True, "PolygonToPolyline " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub TriangleCentroid3D(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double, _
                                           dblCentX As Double, dblCentY As Double, dblCentZ As Double)
  On Error GoTo ErrorHandler


223:   dblCentX = (dblPX + dblQX + dblRX) / 3
224:   dblCentY = (dblPY + dblQY + dblRY) / 3
225:   dblCentZ = (dblPZ + dblQZ + dblRZ) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroid3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub TriangleCentroidPlane(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
                                           dblRX As Double, dblRY As Double, dblCentX As Double, dblCentY As Double)
  On Error GoTo ErrorHandler


238:   dblCentX = (dblPX + dblQX + dblRX) / 3
239:   dblCentY = (dblPY + dblQY + dblRY) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroidPlane " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SphericalCartToLatLong(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
259:   dblPhi = atan2(Sqr(X ^ 2 + Y ^ 2), Z)
260:   dblTheta = atan2(Y, X)
  
262:   dblLongitude = RadToDeg(dblTheta)
263:   dblLatitude = 90 - RadToDeg(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalCartToLatLong " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub SphericalLatLongToCart(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblRadius = 1)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
282:   dblPhi = DegToRad(90 - dblLatitude)
283:   dblTheta = DegToRad(dblLongitude)
  
285:   X = dblRadius * Sin(dblPhi) * Cos(dblTheta)
286:   Y = dblRadius * Sin(dblPhi) * Sin(dblTheta)
287:   Z = dblRadius * Cos(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalLatLongToCart " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function SpheroidalPolylineMidpoint(pPolyline As IPolyline, dblDistance As Double, _
    booIsRatio As Boolean, Optional dblPolylineLength As Double) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim pVarArray As esriSystem.IVariantArray
  Dim pDblArray As esriSystem.IDoubleArray
303:   Set pVarArray = New esriSystem.VarArray
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  Dim dblX1 As Double
  Dim dblY1 As Double
  
317:   dblTotalLength = 0
318:   Set pSegCollection = pPolyline
  
  ' GET TOTAL LENGTH AND FILL ARRAY WITH SEGMENT STATISTICS
321:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
322:     Set pSeg = pSegCollection.Segment(lngIndex)
323:     Set pPoint1 = pSeg.FromPoint
324:     Set pPoint2 = pSeg.ToPoint
325:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
326:     dblTotalLength = dblTotalLength + dblLength
327:     Set pDblArray = New esriSystem.DoubleArray
328:     pDblArray.Add dblLength
329:     pDblArray.Add dblTotalLength
330:     pDblArray.Add dblAz1
331:     pDblArray.Add pPoint1.X
332:     pDblArray.Add pPoint1.Y
333:     pVarArray.Add pDblArray
334:   Next lngIndex
  
336:   dblPolylineLength = dblTotalLength
  
  Dim dblHalfLength As Double
339:   If booIsRatio Then
340:     If dblDistance > 1 Then dblDistance = 1
341:     If dblDistance < 0 Then dblDistance = 0
342:     dblHalfLength = dblPolylineLength * dblDistance
343:   Else
344:     dblHalfLength = dblDistance
345:   End If
    
347:   For lngIndex = 0 To pVarArray.Count - 1
348:     Set pDblArray = pVarArray.Element(lngIndex)
349:     dblTotalLength = pDblArray.Element(1)
350:     If dblTotalLength > dblHalfLength Then
351:       dblLength = pDblArray.Element(0)
352:       dblAz1 = pDblArray.Element(2)
353:       dblX1 = pDblArray.Element(3)
354:       dblY1 = pDblArray.Element(4)
355:       Exit For
356:     End If
357:   Next lngIndex
  
  Dim dblProperDistance As Double
360:   dblProperDistance = dblLength - (dblTotalLength - dblHalfLength)
  
362:   Set pPoint1 = New Point
363:   pPoint1.PutCoords dblX1, dblY1
364:   Set pPoint2 = New Point
    
366:   PointLineVincenty pPoint1, dblProperDistance, dblAz1, pPoint2, dblAz2
  
368:   Set SpheroidalPolylineMidpoint = pPoint2


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineMidpoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SpheroidalPolylineLength(pPolyline As IPolyline) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
392:   dblTotalLength = 0
393:   Set pSegCollection = pPolyline
  
395:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
396:     Set pSeg = pSegCollection.Segment(lngIndex)
397:     Set pPoint1 = pSeg.FromPoint
398:     Set pPoint2 = pSeg.ToPoint
399:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
400:     dblTotalLength = dblTotalLength + dblLength
401:   Next lngIndex
  
403:   SpheroidalPolylineLength = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalPolygonArea(pPolygon As IPolygon, Optional booCalcCentroid As Boolean = False, _
      Optional dblCentX As Double, Optional dblCentY As Double) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYGON IS IN GEOGRAPHIC COORDINATES
  ' BREAK UP POLYGON INTO CONNECTED COMPONENTS
  Dim pPoly4 As IPolygon4
419:   Set pPoly4 = pPolygon
  Dim pConnected As IGeometryCollection
  Dim pRingBag As IGeometryCollection
  Dim pExtRing As IRing
423:   Set pConnected = pPoly4.ConnectedComponentBag
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim pPointCollection As IPointCollection
  
  Dim pArea As IArea
  Dim pCentroid As IPoint
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  
433:   If booCalcCentroid Then
'    Dim dbl1Long As Double
'    Dim dbl1Lat As Double
    Dim dbl1X As Double
    Dim dbl1Y As Double
    Dim dbl1Z As Double
'    Dim dbl2Long As Double
'    Dim dbl2Lat As Double
    Dim dbl2X As Double
    Dim dbl2Y As Double
    Dim dbl2Z As Double
    Dim dbl3X As Double
    Dim dbl3Y As Double
    Dim dbl3Z As Double
    Dim dblTempCentX As Double
    Dim dblTempCentY As Double
    Dim dblTempCentZ As Double
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    Dim dblRunningZ As Double
453:     dblRunningX = 0
454:     dblRunningY = 0
455:     dblRunningZ = 0
    Dim dblVectLength As Double
457:   End If
  
'  Dim dbl3Long As Double
'  Dim dbl3Lat As Double
  
  Dim pSubPolygon As IPolygon4
  Dim dblArea As Double
  Dim dblTriangleArea As Double
465:   dblArea = 0
  Dim dblMultiplier As Double
  Dim lngCountPos As Long
  Dim lngCountNeg As Long
  
  ' FOR TESTING CENTROID
'  Dim dblTestArea As Double
'  Dim dblTestRunningArea As Double
'  Dim dblTestX As Double
'  Dim dblTestY As Double
'  Dim dblTestRunningX As Double
'  Dim dblTestRunningY As Double
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
480:   For lngIndex = 0 To (pConnected.GeometryCount - 1)
481:     Set pSubPolygon = pConnected.Geometry(lngIndex)
482:     Set pArea = pSubPolygon
483:     Set pCentroid = pArea.Centroid
484:     Set pSegCollection = pSubPolygon
    
'    dbl3Long = pCentroid.X
'    dbl3Lat = pCentroid.Y
    
489:     For lngIndex2 = 0 To (pSegCollection.SegmentCount - 1)
490:       Set pSeg = pSegCollection.Segment(lngIndex2)
491:       Set pPoint1 = pSeg.FromPoint
492:       Set pPoint2 = pSeg.ToPoint
      
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint1, "delete_corridors"
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint2, "delete_corridors"
496:       dblTriangleArea = SphericalTriangleArea(pPoint1, pPoint2, pCentroid, dblMultiplier)
497:       dblArea = dblArea + dblTriangleArea
      
      ' FOR TESTING CENTROID
'      dblTestArea = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, 1, pPoint2.X, pPoint2.Y, _
'                1, pCentroid.X, pCentroid.Y, 1)
'      dblTestRunningArea = dblTestRunningArea + dblTestArea
'      TriangleCentroidPlane pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, pCentroid.X, pCentroid.Y, dblTestX, dblTestY
'      dblTestRunningX = dblTestRunningX + (dblTestX * dblTestArea)
'      dblTestRunningY = dblTestRunningY + (dblTestY * dblTestArea)
      
507:       If booCalcCentroid Then
508:         SphericalLatLongToCart pPoint1.X, pPoint1.Y, dbl1X, dbl1Y, dbl1Z
509:         SphericalLatLongToCart pPoint2.X, pPoint2.Y, dbl2X, dbl2Y, dbl2Z
510:         SphericalLatLongToCart pCentroid.X, pCentroid.Y, dbl3X, dbl3Y, dbl3Z
511:         TriangleCentroid3D dbl1X, dbl1Y, dbl1Z, dbl2X, dbl2Y, dbl2Z, dbl3X, dbl3Y, dbl3Z, _
                dblTempCentX, dblTempCentY, dblTempCentZ
    
        ' NORMALIZE VECTOR
515:         dblVectLength = Sqr(dblTempCentX ^ 2 + dblTempCentY ^ 2 + dblTempCentZ ^ 2)
516:         dblTempCentX = dblTempCentX / dblVectLength
517:         dblTempCentY = dblTempCentY / dblVectLength
518:         dblTempCentZ = dblTempCentZ / dblVectLength
        
520:         dblRunningX = dblRunningX + (dblTempCentX * dblTriangleArea)
521:         dblRunningY = dblRunningY + (dblTempCentY * dblTriangleArea)
522:         dblRunningZ = dblRunningZ + (dblTempCentZ * dblTriangleArea)
523:       End If
524:     Next lngIndex2
525:   Next lngIndex
  
527:   If booCalcCentroid Then
528:     dblRunningX = dblRunningX / dblArea
529:     dblRunningY = dblRunningY / dblArea
530:     dblRunningZ = dblRunningZ / dblArea
    
532:     SphericalCartToLatLong dblCentX, dblCentY, dblRunningX, dblRunningY, dblRunningZ
533:   End If
  
  ' FOR TESTING CENTROID
'  dblTestRunningX = dblTestRunningX / dblTestRunningArea
'  dblTestRunningY = dblTestRunningY / dblTestRunningArea
'  Debug.Print "Test Centroid:  X = " & dblTestRunningX & ",  Y = " & dblTestRunningY
  
540:   SphericalPolygonArea = dblArea


  Exit Function
ErrorHandler:
  HandleError True, "SphericalPolygonArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AzimuthHaversine(pPointA As IPoint, pPointB As IPoint) As Double
  On Error GoTo ErrorHandler


  ' WITH HELP FROM http://www.movable-type.co.uk/scripts/latlong.html
  ' Formula:    Theta = atan2( sin(Deltalong)*cos(lat2), cos(lat1)*sin(lat2) - sin(lat1)*cos(lat2)*cos(DeltaLong) )
  
  Dim DeltaLong As Double
  Dim PX As Double
  Dim PY As Double
  Dim QX As Double
  Dim QY As Double
  
561:   PX = DegToRad(pPointA.X)
562:   PY = DegToRad(pPointA.Y)
563:   QX = DegToRad(pPointB.X)
564:   QY = DegToRad(pPointB.Y)
  
566:   DeltaLong = QX - PX
  Dim dblTheta As Double
568:   dblTheta = atan2(Sin(DeltaLong) * Cos(QY), Cos(PY) * Sin(QY) - Sin(PY) * Cos(QY) * Cos(DeltaLong))
569:   AzimuthHaversine = RadToDeg(dblTheta)
570:   If AzimuthHaversine < 360 Then AzimuthHaversine = AzimuthHaversine + 360


  Exit Function
ErrorHandler:
  HandleError True, "AzimuthHaversine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalTriangleArea(pPointA As IPoint, pPointB As IPoint, pPointC As IPoint, Optional dblMult As Double) As Double
  On Error GoTo ErrorHandler

  
  
  ' BASED ON GIRARD'S FORMULA:  Area = R^2 * (A + B + C - Pi)
  '                          Where A = Angle 1
  '                                B = Angle 2
  '                                C = Angle 3
  '                   A + B + C - Pi = Spherical Excess
  '                                R = Sphere Radius
  ' Trick is to get Angles A, B and C from points.
  '
  ' ANOTHER FORMULATION, BASED ON DISTANCES:
  '                       Tan(E / 4) = sqrt(Tan(S / 2) * Tan((S - A) / 2) * Tan((S - B) / 2) * Tan((S - C) / 2))
  '                 Spherical Excess = E
  '                   where  a, b, c = sides of spherical triangle
  '                                S = (A + B + C) / 2
  ' INITAL AZIMUTH = atn( sin (Lo2 - Lo1) / (cos (Lo2 - Lo1) sin L1 - cos L1 tan L2)
  '         http://fer3.com/arc/m2.aspx?i=1688&y=200111
  
  ' FOR DEBUGGING
'  Static dblMax As Double
'  Static dblMin As Double
  
  Dim dblAB As Double
  Dim dblBC As Double
  Dim dblCA As Double
  
  Dim dblR As Double         ' RADIUS
'  dblR = (6378137 + 6356752.31424518) / 2 ' AVERAGE OF WGS ELLIPSOID MAJOR AND MINOR AXES
'  dblR = Sqr(6378137 * 6356752.31424518)  ' GEOMETRIC MEAN OF WGS ELLIPSOID MAJOR AND MINOR AXES
610:   dblR = (6378137 ^ 2 * 6356752.31424518) ^ (1 / 3)   ' PROPER 3-AXIS GEOMETRIC MEAN; (a^2 * b) ^ (1/3)
  
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblC As Double
  Dim dblAzimuthAB As Double
  Dim dblAzimuthBC As Double
  Dim dblAzimuthAC As Double
  Dim dblAzimuthAA As Double
  Dim dblAzRev As Double
  Dim dblMultiplier As Double
  Dim dblLong2 As Double
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
627:   dblLat1 = DegToRad(pPointA.Y)
628:   dblLat2 = DegToRad(pPointB.Y)
629:   dblLat = dblLat1 - dblLat2
630:   dblLong = DegToRad(pPointA.X - pPointB.X)
631:   dblLong2 = -dblLong
632:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
633:   dblAB = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO B
'  DistanceVincentyNumbers pPointA.X, pPointA.Y, pPointB.X, pPointB.Y, dblAzimuthAB, dblAzRev
'  dblAB = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointB.x, pPointB.y, dblAzimuthAB, dblAzRev) / dblR
'  Debug.Print "A to B:  Vincenty = " & dblAzimuthAB
638:   dblAzimuthAB = atan2(Sin(-dblLong) * Cos(dblLat2), _
        Cos(dblLat1) * Sin(dblLat2) - Sin(dblLat1) * Cos(dblLat2) * Cos(-dblLong))
'  Debug.Print "A to B:  Simpler = " & dblAzimuthAB
'  dblAB = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE BC USING HAVERSINE FORMULA
644:   dblLat1 = DegToRad(pPointB.Y)
645:   dblLat2 = DegToRad(pPointC.Y)
646:   dblLat = dblLat1 - dblLat2
647:   dblLong = DegToRad(pPointB.X - pPointC.X)
648:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
649:   dblBC = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
655:   dblLat1 = DegToRad(pPointC.Y)
656:   dblLat2 = DegToRad(pPointA.Y)
657:   dblLat = dblLat1 - dblLat2
658:   dblLong = DegToRad(pPointC.X - pPointA.X)
  
660:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
661:   dblCA = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO C
'  DistanceVincentyNumbers pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev
'  dblCA = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev) / dblR
'  Debug.Print "A to C:  Vincenty = " & dblAzimuth
'  dblAzimuth2 = RadToDeg(atan2(Sin(pPointC.x - pPointA.x), _
                    (Cos(pPointC.x - pPointA.x) * Sin(pPointA.y) - Cos(pPointA.y) * Tan(pPointC.y))))
'  dblCA = dblR * dblC
'  Debug.Print "Az1 = " & CStr(dblAzimuth1) & ",       Az2 = " & CStr(dblAzimuth2)

  ' NOTE:  Lat1 and Lat2 flipped in equation below because variables originally defined for Line CA, not Line AC
672:   dblAzimuthAC = atan2(Sin(dblLong) * Cos(dblLat1), _
        Cos(dblLat2) * Sin(dblLat1) - Sin(dblLat2) * Cos(dblLat1) * Cos(dblLong))
'  Debug.Print "A to C:  Simpler = " & dblAzimuthAC
  
'  If dblAzimuthAB > dblMax Then dblMax = dblAzimuthAB
'  If dblAzimuthAB < dblMin Then dblMin = dblAzimuthAB
'  Debug.Print "Current Max = " & dblMax & ",   Current Min = " & dblMin
  
680:   If dblAzimuthAB < 0 Then dblAzimuthAB = dblAzimuthAB + 2 * dblPi
681:   If dblAzimuthAC < 0 Then dblAzimuthAC = dblAzimuthAC + 2 * dblPi
  
  Dim dblDiff As Double
684:   dblDiff = dblAzimuthAC - dblAzimuthAB
685:   If dblDiff > 0 Then              ' EITHER AC > AB or AC IS TO THE LEFT OF NORTH
686:     If dblDiff > dblPi Then         ' THEN AC IS TO THE LEFT OF NORTH
687:       dblMultiplier = -1           ' COUNTERCLOCKWISE
688:     Else                           ' THEN AC > AB
689:       dblMultiplier = 1            ' CLOCKWISE
690:     End If
691:   Else                             ' EITHER AC < AB or AB IS TO THE LEFT OF NORTH
692:     If Abs(dblDiff) > dblPi Then   ' THEN AB IS TO THE LEFT OF NORTH
693:       dblMultiplier = 1            ' CLOCKWISE
694:     Else                           ' THEN AC < AB
695:       dblMultiplier = -1           ' COUNTERCLOCKWISE
696:     End If
697:   End If
    
  
'  If ((dblDiff > dblPi) And (dblAzimuthAB > dblAzimuthAC)) Or ((dblDiff < dblPi) And (dblAzimuthAB < dblAzimuthAC)) Then
'    ' IS CLOCKWISE
'    dblMultiplier = 1
'  Else
'    ' IS COUNTERCLOCKWISE
'    dblMultiplier = -1
'  End If

'  If Abs(dblAzimuthAC - dblAzimuthAB) > dblPi Then dblAzimuthAB = dblAzimuth
'  dblAzimuthAB = dblAzimuthAB + 360
'  dblAzimuthAC = dblAzimuthAC + 360
'  If dblAzimuthAB > dblAzimuthAC Then
'    dblMultiplier = -1
'  Else
'    dblMultiplier = 1
'  End If
  
'  Dim booIsClockwise As Boolean
'  booIsClockwise = CalcCheckClockwise(pPointA, pPointB, pPointC)
'  If booIsClockwise Then
'    If dblMultiplier = -1 Then
'      Debug.Print "-1, 1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",   AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  Else
'    If dblMultiplier = 1 Then
'      Debug.Print "1, -1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",  AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  End If
  
  Dim dblS As Double
736:   dblS = (dblAB + dblBC + dblCA) / 2
  
  Dim dblTanEOver4 As Double
739:   dblTanEOver4 = Sqr(Tan(dblS / 2) * Tan((dblS - dblAB) / 2) * Tan((dblS - dblBC) / 2) * Tan((dblS - dblCA) / 2))
  Dim dblE As Double
741:   dblE = Atn(dblTanEOver4) * 4
  
743:   dblMult = dblMultiplier
744:   SphericalTriangleArea = dblR ^ 2 * dblE * dblMultiplier


  Exit Function
ErrorHandler:
  HandleError True, "SphericalTriangleArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ArcSinJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


756:   ArcSinJen = Atn(dblValue / Sqr(-dblValue * dblValue + 1))


  Exit Function
ErrorHandler:
  HandleError True, "ArcSinJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ArcCosJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


767:   ArcCosJen = Atn(-dblValue / Sqr(-dblValue * dblValue + 1)) + 2 * Atn(1)


  Exit Function
ErrorHandler:
  HandleError True, "ArcCosJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler


779:   DistanceVincentyPoints = DistanceVincentyNumbers(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub PointLineVincenty(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, pNewPoint As IPoint, _
      dblAz2 As Double, Optional lngNumVertices As Long, Optional pPolyline As IPolyline)
  On Error GoTo ErrorHandler

      
  Dim pWGS84 As IGeographicCoordinateSystem
  Dim pSpatRefFact As ISpatialReferenceFactory
  'Set the spatial reference factory to a new spatial reference environment
795:   Set pSpatRefFact = New SpatialReferenceEnvironment
796:   Set pWGS84 = pSpatRefFact.CreateGeographicCoordinateSystem(esriSRGeoCS_WGS1984)
  
798:   If lngNumVertices > 0 Then
799:     If lngNumVertices = 1 Then lngNumVertices = 2
    
    Dim dblShort As Double
802:     dblShort = dblLength / (lngNumVertices - 1)
    
    Dim pPointCollection As IPointCollection
805:     If pPolyline Is Nothing Then
806:       Set pPolyline = New Polyline
807:     End If
808:     Set pPointCollection = pPolyline
    
    ' ADD FIRST VERTEX
811:     pPointCollection.AddPoint pPoint
    
    ' ADD INTERNAL VERTICES
814:     If lngNumVertices > 2 Then
      Dim lngCounter As Long
      Dim dblCurrentDistance As Double
      Dim pTempPoint As IPoint
818:       For lngCounter = 1 To (lngNumVertices - 2)
819:         Set pTempPoint = New Point
820:         PointLineVincentyPerPoint pPoint, lngCounter * dblShort, dblAzimuth, pTempPoint, dblAz2
821:         pPointCollection.AddPoint pTempPoint
822:       Next lngCounter
823:     End If
    
    ' ADD LAST VERTEX AND SET FINAL POINT AND AZIMUTH VALUES
826:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
827:     pPointCollection.AddPoint pNewPoint
828:     Set pPolyline.SpatialReference = pWGS84
829:     Set pNewPoint.SpatialReference = pWGS84
830:   Else
831:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
832:     Set pNewPoint.SpatialReference = pWGS84
833:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincenty " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub PointLineVincentyPerPoint(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, _
      pNewPoint As IPoint, dblAz2 As Double)
  On Error GoTo ErrorHandler

      
  ' ASSUMES pPoint IS GEOGRAPHIC

  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  Dim dblPX As Double
856:   dblPX = pPoint.X
  Dim dblPY As Double
858:   dblPY = pPoint.Y
  
860:   If dblLength = 0 Then    ' SAME POINT
861:     pNewPoint.X = dblPX
862:     pNewPoint.Y = dblPY
863:     dblAz2 = dblAzimuth
    Exit Sub
865:   End If
  
  ' WGS84 PARAMETERS ----------------------------------------
  Dim A As Double
  Dim B As Double
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim f As Double
  Dim dblA1 As Double
  Dim dblB1 As Double
  
  Dim dblTanU1 As Double
  Dim dblSinU1 As Double
  Dim dblCosU1 As Double
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
'  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
'  U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
885:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
886:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
887:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
889:   dblTanU1 = (1 - f) * (Tan(DegToRad(dblPY)))
890:   U1 = Atn(dblTanU1)
891:   dblCosU1 = Cos(U1)
892:   dblSinU1 = Sin(U1)
  
  Dim s As Double
895:   s = dblLength
  
  Dim Sigma1 As Double
  Dim tanSigma1 As Double
  Dim cosAlpha1 As Double
  Dim sinAlpha1 As Double
'  Dim dblAlpha As Double                      ' AZIMUTH AT EQUATOR
  Dim cosAlpha As Double
  Dim cosSqAlpha As Double
  
905:   cosAlpha1 = Cos(DegToRad(dblAzimuth))
906:   sinAlpha1 = Sin(DegToRad(dblAzimuth))
907:   tanSigma1 = dblTanU1 / cosAlpha1                                                                    ' [1]
908:   Sigma1 = atan2(dblTanU1, cosAlpha1)
  Dim sinAlpha As Double
910:   sinAlpha = dblCosU1 * sinAlpha1                                                                  ' [2]
911:   cosSqAlpha = 1 - (sinAlpha ^ 2)                                                                  ' TRIG IDENTITY
912:   cosAlpha = Sqr(cosSqAlpha)
'  dblAlpha = ArcSinJen(sinAlpha)
  
915:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
916:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
917:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                                      ' [3]
918:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
919:   dblB = (uSq / 1024) * (256 + dblB1)                                                              ' [4]
  
  Dim Sigma As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim DeltaSigma As Double
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  Dim l As Double
  
  Dim lngIterations As Long
933:   lngIterations = 40
  
  Dim SigmaCompare As Double
936:   SigmaCompare = 2 * dblPi
937:   Sigma = s / (B * dblA)                  ' FIRST ESTIMATION
  
939:   Do While (Abs(Sigma - SigmaCompare) > 0.000000000001) And (lngIterations > 0)
940:     cos2SigmaM = Cos(2 * Sigma1 + Sigma)                                                             ' [5]
941:     sinSigma = Sin(Sigma)
942:     cosSigma = Cos(Sigma)
943:     DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
944:     DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
945:     DeltaSigma3 = cos2SigmaM + DeltaSigma2
946:     DeltaSigma = dblB * sinSigma * DeltaSigma3                                                       ' [6]
947:     SigmaCompare = Sigma
948:     Sigma = (s / (B * dblA)) + DeltaSigma                                                            ' [7]
    
950:     If lngIterations = 0 Then
951:       MsgBox "Vincenty Formula failed to converge!"
      Exit Sub
953:     End If
954:     lngIterations = lngIterations - 1
955:   Loop
  
957:   cos2SigmaM = Cos(2 * Sigma1 + Sigma)
958:   sinSigma = Sin(Sigma)
959:   cosSigma = Cos(Sigma)
  Dim dblLat2Denom As Double
  Dim dblLat2Temp As Double
962:   dblLat2Temp = dblSinU1 * sinSigma - dblCosU1 * cosSigma * cosAlpha1
963:   dblLat2Denom = (1 - f) * (Sqr(sinAlpha ^ 2 + dblLat2Temp ^ 2))
  
  ' CALCULATE LATITUDE FOR NEW POINT
  Dim dblLat2 As Double
967:   dblLat2 = atan2(dblSinU1 * cosSigma + dblCosU1 * sinSigma * cosAlpha1, dblLat2Denom)                ' [8]
  
  ' CALCULATE LONGITUDE FOR NEW POINT
  Dim dblLambda As Double
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
973:   dblLambda = atan2(sinSigma * sinAlpha1, dblCosU1 * cosSigma - dblSinU1 * sinSigma * cosAlpha1)      ' [9]
974:   C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                                      ' [10]
975:   dblLambda1 = cos2SigmaM + C * cosSigma * (-1# + 2# * cos2SigmaM ^ 2#)
976:   dblLambda1a = C * sinSigma * dblLambda1
  Dim dblLambda2 As Double
978:   dblLambda2 = Sigma + dblLambda1a
979:   l = dblLambda - ((1 - C) * f * sinAlpha * dblLambda2)                                               ' [11]
  
981:   pNewPoint.X = dblPX + RadToDeg(l)
982:   pNewPoint.Y = RadToDeg(dblLat2)
  
984:   dblAz2 = RadToDeg(atan2(sinAlpha, -dblLat2Temp))
985:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincentyPerPoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function DistanceVincentyNumbers(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
1006:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
1007:     DistanceVincentyNumbers = 0
1008:     dblAz1 = 0
1009:     dblAz2 = 0
    Exit Function
1011:   End If
  
  
  Dim A As Double
  Dim B As Double
1016:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
1017:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
  
  Dim f As Double
1020:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
1023:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
1028:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
1029:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
1036:   dblSinU1 = Sin(U1)
1037:   dblCosU1 = Cos(U1)
1038:   dblSinU2 = Sin(U2)
1039:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
1042:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
1043:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
1045:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
1060:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
1061:     sinLambda = Sin(dblLambda)                                                          '  |
1062:     cosLambda = Cos(dblLambda)                                                          '  |
1063:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
1065:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
1066:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
1067:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
1068:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
1069:     If cosSqAlpha = 0 Then
1070:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
1071:     Else
1072:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
1073:     End If
    
1075:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
1076:     dblLambdaComp = dblLambda
1077:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
1078:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
1080:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
1082:     If lngIterations = 0 Then
1083:       MsgBox "Vincenty Formula failed to converge!"
1084:       DistanceVincentyNumbers = -9999
      Exit Function
1086:     End If
1087:     lngIterations = lngIterations - 1
1088:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim s As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
1100:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
1105:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
1106:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
1107:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
1108:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
1110:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
1111:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
1112:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
1113:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
1114:   s = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
1128:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
1129:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
1131:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
1132:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
1133:   DistanceVincentyNumbers = s


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function RadToDeg(dblRad As Double) As Double
  On Error GoTo ErrorHandler


1145:   RadToDeg = dblRad * 180 / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "RadToDeg " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function DegToRad(dblDeg As Double) As Double
  On Error GoTo ErrorHandler


1156:   DegToRad = dblDeg * dblPi / 180


  Exit Function
ErrorHandler:
  HandleError True, "DegToRad " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function atan2(Y As Double, X As Double) As Double
  On Error GoTo ErrorHandler


1168:   If X > 0 Then
1169:     atan2 = Atn(Y / X)
1170:   ElseIf X < 0 Then
1171:     If Y = 0 Then
1172:       atan2 = (dblPi - Atn(Abs(Y / X)))
1173:     Else
1174:       atan2 = Sgn(Y) * (dblPi - Atn(Abs(Y / X)))
1175:     End If
1176:   Else    ' IF X = 0
1177:     If Y = 0 Then
1178:       atan2 = 0
1179:     Else
1180:       atan2 = Sgn(Y) * dblPi / 2
1181:     End If
1182:   End If


  Exit Function
ErrorHandler:
  HandleError True, "atan2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function TriangleAreaPoints(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler


1194:   TriangleAreaPoints = Abs(((((pPoint2.X - pPoint1.X) * (pPoint3.Y - pPoint1.Y)) - ((pPoint3.X - pPoint1.X) * (pPoint2.Y - pPoint1.Y))) / 2))


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaPoints3D(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler

  
1205:   TriangleAreaPoints3D = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, pPoint1.Z, pPoint2.X, pPoint2.Y, pPoint2.Z, _
                                                  pPoint3.X, pPoint3.Y, pPoint3.Z)


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function TriangleAreaPoints3DValues(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double) As Double
  On Error GoTo ErrorHandler


  Dim dblI As Double
  Dim dblJ As Double
  Dim dblK As Double
  
1226:   dblI = (((dblQY - dblPY) * (dblRZ - dblPZ)) - ((dblRY - dblPY) * (dblQZ - dblPZ))) ^ 2
1227:   dblJ = (((dblQX - dblPX) * (dblRZ - dblPZ)) - ((dblRX - dblPX) * (dblQZ - dblPZ))) ^ 2
1228:   dblK = (((dblQX - dblPX) * (dblRY - dblPY)) - ((dblRX - dblPX) * (dblQY - dblPY))) ^ 2

1230:   TriangleAreaPoints3DValues = (Sqr(dblI + dblJ + dblK)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3DValues " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
                                           

Public Function EnvelopeToPolygon(pEnv As IEnvelope) As IPolygon
  On Error GoTo ErrorHandler


  Dim pPtColl As IPointCollection

1245:   Set pPtColl = New Polygon
1246:   With pPtColl
1247:       .AddPoint pEnv.LowerLeft
1248:       .AddPoint pEnv.UpperLeft
1249:       .AddPoint pEnv.UpperRight
1250:       .AddPoint pEnv.LowerRight
      'Close the polygon
1252:       .AddPoint pEnv.LowerLeft
1253:   End With
  
  Dim pPolygon As IPolygon
1256:   Set pPolygon = pPtColl
1257:   Set pPolygon.SpatialReference = pEnv.SpatialReference
  Dim pTopoOp As ITopologicalOperator
1259:   Set pTopoOp = pPolygon
1260:   pTopoOp.Simplify
    
1262:   Set EnvelopeToPolygon = pPtColl


  Exit Function
ErrorHandler:
  HandleError True, "EnvelopeToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function EllipticArcToPolygon2(SegCollection As ISegmentCollection, NumVertices As Long) As IMultipoint
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
      
On Error GoTo erh

  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
1284:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
1286:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
1290:   For anIndex = 0 To lngSegCount
1291:     theTestLength = SegCollection.Segment(anIndex).length
1292:     theLength = theLength + theTestLength
1293:     lngLengths(anIndex) = theTestLength
1294:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
1300:   For anIndex = 0 To lngSegCount
1301:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
1302:     If lngNumVertices < 8 Then lngNumVertices = 8
1303:     lngVertices(anIndex) = lngNumVertices
1304:   Next anIndex
  
  Dim pMpt As IPointCollection
1307:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
1309:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
1315:   For anIndex = 0 To lngSegCount
1316:     lngNumVertices = lngVertices(anIndex)
1317:     pRatio = 1 / lngNumVertices
1318:     Set pCurve = SegCollection.Segment(anIndex)
    
1320:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
1322:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
1323:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
1327:       pMpt.AddPoint pClone.Clone
1328:     Next anIndex2
1329:   Next anIndex
  
1331:   Set EllipticArcToPolygon2 = pMpt
    Exit Function
  
erh:
1335:     MsgBox "Failed in EllipticArcToPolygon2: " & err.Description
End Function

Public Function CurveToPolygon(pOrigGeometry As IGeometry, NumVertices As Long) As IPolygon
On Error GoTo erh
  
  Dim pGeometryCollection As IGeometryCollection
1342:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
1344:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolygon As IPolycurve
1347:   Set pOrigPolygon = pOrigGeometry
  
  Dim dblFullLength As Double
1350:   dblFullLength = pOrigPolygon.length
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pPolygon As IPointCollection
  Dim pRing As IRing
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
1358:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
1360:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngRingCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
1380:   Set pNewPolyGeoColl = New Polygon
  Dim pNewRingGeometry As IGeometry
  Dim pPath As IPath
  Dim pSegmentCollection As ISegmentCollection
  Dim pNewSegCol As ISegmentCollection
  
1386:   lngRingCount = pGeometryCollection.GeometryCount - 1
1387:   For anIndex = 0 To lngRingCount
1388:     If TypeOf pOrigGeometry Is IPolyline Then
1389:       Set pPath = pGeometryCollection.Geometry(anIndex)
1390:       Set pSegmentCollection = pPath
1391:       Set pNewSegCol = New Ring
1392:       pNewSegCol.AddSegmentCollection pSegmentCollection
1393:       Set pRing = pNewSegCol
1394:       pRing.Close
1395:     Else
1396:       Set pRing = pGeometryCollection.Geometry(anIndex)
1397:     End If
1398:     Set SegCollection = pRing
1399:     Set pNewSegCollection = New Ring
1400:     lngSegCount = SegCollection.SegmentCount - 1
1401:     For anIndex2 = 0 To lngSegCount
1402:       Set pSegment = SegCollection.Segment(anIndex2)
1403:       Set pGeometry = pSegment
1404:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
1405:         booFoundCurve = True
1406:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
1407:         If lngNumVertices < 8 Then lngNumVertices = 8
1408:         pRatio = 1 / lngNumVertices
       
1410:         Set pCurve = pSegment
1411:         Set pPathSegColl = New Path
1412:         Set pNewSegment = New esriGeometry.Line
1413:         Set pStartPoint = pCurve.FromPoint
1414:         For anIndex3 = 1 To lngNumVertices
1415:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
1416:           pNewSegment.FromPoint = pStartPoint
1417:           pNewSegment.ToPoint = pEndPoint
          
1419:           Set pClone = pNewSegment
1420:           pPathSegColl.AddSegment pClone.Clone
          
'          If anIndex3 < 4 Then
'            MsgBox "Start Point:  X = " & CStr(pStartPoint.X) & ", Y = " & CStr(pStartPoint.Y) & vbCrLf & _
'              "End Point:  X = " & CStr(pEndPoint.X) & ", Y = " & CStr(pEndPoint.Y) & vbCrLf & _
'              "Segment Length = " & CStr(pNewSegment.length) & vbCrLf & _
'              "Segment Collection Count = " & CStr(pPathSegColl.SegmentCount)
'          End If
          
1429:           Set pClone = pEndPoint
1430:           Set pStartPoint = pClone.Clone
          
1432:         Next anIndex3
1433:         pNewSegCollection.AddSegmentCollection pPathSegColl

1435:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
1436:         Set pClone = pSegment
1437:         pNewSegCollection.AddSegment pClone.Clone
1438:       End If
1439:     Next anIndex2
1440:     Set pNewRingGeometry = pNewSegCollection
1441:     pNewPolyGeoColl.AddGeometry pNewRingGeometry

1443:   Next anIndex
  
  Dim pNewPolygon As IPolygon
  
1447:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolyline) Then
  
1449:     Set pNewPolygon = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
1451:     Set pTopoOp = pNewPolygon
1452:     pTopoOp.Simplify
1453:     Set pNewPolygon.SpatialReference = pSpRef

1455:   Else
1456:     Set pNewPolygon = pOrigGeometry
1457:     Set pNewPolygon.SpatialReference = pSpRef
1458:   End If
    
1460:   Set CurveToPolygon = pNewPolygon
  Exit Function
  
erh:
1464:     MsgBox "Failed in CurveToPolygon: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Function CurveToPolyline(pOrigGeometry As IGeometry, NumVertices As Long) As IPolyline
On Error GoTo erh

  Dim pGeometryCollection As IGeometryCollection
1470:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
1472:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolyline As IPolycurve
  
1476:   Set pOrigPolyline = pOrigGeometry
  
  Dim dblFullLength As Double
1479:   dblFullLength = pOrigPolyline.length

  Dim pPath As IPath
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
1487:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
1489:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngPathCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
1509:   Set pNewPolyGeoColl = New Polyline
  Dim pNewPathGeometry As IGeometry
  
  Dim pRing As IRing
  
1514:   lngPathCount = pGeometryCollection.GeometryCount - 1
1515:   For anIndex = 0 To lngPathCount
1516:     If TypeOf pOrigGeometry Is IPolygon Then
1517:       Set pRing = pGeometryCollection.Geometry(anIndex)
1518:       Set pPath = pRing
1519:     Else
1520:       Set pPath = pGeometryCollection.Geometry(anIndex)
1521:     End If
1522:     Set SegCollection = pPath
1523:     Set pNewSegCollection = New Path
1524:     lngSegCount = SegCollection.SegmentCount - 1
1525:     For anIndex2 = 0 To lngSegCount
1526:       Set pSegment = SegCollection.Segment(anIndex2)
1527:       Set pGeometry = pSegment
1528:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
1529:         booFoundCurve = True
1530:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
1531:         If lngNumVertices < 8 Then lngNumVertices = 8
1532:         pRatio = 1 / lngNumVertices
       
1534:         Set pCurve = pSegment
1535:         Set pPathSegColl = New Path
1536:         Set pNewSegment = New esriGeometry.Line
1537:         Set pStartPoint = pCurve.FromPoint
1538:         For anIndex3 = 1 To lngNumVertices
1539:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
1540:           pNewSegment.FromPoint = pStartPoint
1541:           pNewSegment.ToPoint = pEndPoint
          
1543:           Set pClone = pNewSegment
1544:           pPathSegColl.AddSegment pClone.Clone
1545:           Set pClone = pEndPoint
1546:           Set pStartPoint = pClone.Clone
1547:         Next anIndex3
1548:         pNewSegCollection.AddSegmentCollection pPathSegColl

1550:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
1551:         Set pClone = pSegment
1552:         pNewSegCollection.AddSegment pClone.Clone
1553:       End If
1554:     Next anIndex2
1555:     Set pNewPathGeometry = pNewSegCollection
1556:     pNewPolyGeoColl.AddGeometry pNewPathGeometry

1558:   Next anIndex

  
  Dim pNewPolyline As IPolyline
  
1563:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolygon) Then
  
1565:     Set pNewPolyline = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
1567:     Set pTopoOp = pNewPolyline
1568:     pTopoOp.Simplify
1569:     Set pNewPolyline.SpatialReference = pSpRef

1571:   Else
1572:     Set pNewPolyline = pOrigGeometry
1573:     Set pNewPolyline.SpatialReference = pSpRef
1574:   End If
    
1576:   Set CurveToPolyline = pNewPolyline
  Exit Function
  
erh:
1580:     MsgBox "Failed in CurveToPolyline: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Sub Graphic_MakeFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, Optional strName As String)
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
1590:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
1599:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
1601:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
1604:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
1609:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
1611:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
1613:       Set pElement = New LineElement
    Case 4, 11:
1615:       Set pElement = New PolygonElement
    Case 5:
1617:       Set pElement = New RectangleElement
    Case Else:
1619:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
1620:   End Select
    
1622:   pElement.Geometry = pNewGeometry
1623:   Set pGraphicElement = pElement
1624:   Set pSpatialReference = pGeometry.SpatialReference
1625:   Set pGraphicElement.SpatialReference = pSpatialReference
1626:   Set pElementProperties = pElement
1627:   pElementProperties.Name = strName

  ' ADD GRAPHIC TO GRAPHICS CONTAINER
1630:   pGContainer.AddElement pElement, 0

  'Draw
1633:   pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing


  Exit Sub
ErrorHandler:
  HandleError True, "Graphic_MakeFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function Graphic_ReturnElementFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, _
    Optional strName As String, Optional AddToView As Boolean) As IElement
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
1649:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
1658:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
1660:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
1663:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
1668:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
1670:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
1672:       Set pElement = New LineElement
    Case 4, 11:
1674:       Set pElement = New PolygonElement
    Case 5:
1676:       Set pElement = New RectangleElement
    Case Else:
1678:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
      Exit Function
1680:   End Select
  
1682:   pElement.Geometry = pNewGeometry
1683:   Set pGraphicElement = pElement
1684:   Set pSpatialReference = pGeometry.SpatialReference
1685:   Set pGraphicElement.SpatialReference = pSpatialReference
1686:   Set pElementProperties = pElement
1687:   pElementProperties.Name = strName
  
1689:   If AddToView Then
    ' ADD GRAPHIC TO GRAPHICS CONTAINER
1691:     pGContainer.AddElement pElement, 0
    'Draw
1693:     pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing
1694:   End If
  
1696:   Set Graphic_ReturnElementFromGeometry = pElement


  Exit Function
ErrorHandler:
  HandleError True, "Graphic_ReturnElementFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ShowVertices(pMxDoc As IMxDocument, pGeometry As IGeometry, Optional strName As String, _
        Optional DeleteCurrentGraphicsWithName As Boolean)
  On Error GoTo ErrorHandler


  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim pPoly As IPolygon
  Dim pLine As IPolyline
  Dim pOutVertex As IPoint, lOutPart As Long, lOutVertex As Long
  
1715:   If DeleteCurrentGraphicsWithName And (strName <> "") Then
1716:     Call DeleteGraphicsByName(pMxDoc, "DeleteMe")
1717:   End If
  
  Dim pArray As IArray
1720:   Set pArray = New esriSystem.Array
  
  Dim pPointCollection As IPointCollection
1723:   Set pPointCollection = pGeometry
  
  Dim pPointEnum As IEnumVertex
1726:   Set pPointEnum = pPointCollection.EnumVertices
  
1728:   pPointEnum.Reset
  
  Dim pVertex As IPoint
1731:   Set pVertex = New Point
  'Query the next vertex - have to cocreate the point
  'QueryNext is faster than Next, because the method doesn't have
  'to create the point each time
1735:   pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
  
1737:   Do While Not pVertex.IsEmpty
1738:     Graphic_MakeFromGeometry pMxDoc, pVertex, strName
1739:     pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
'    Debug.Print lOutPart & ",    " & lOutVertex & ",  " & pVertex.IsEmpty
1741:   Loop


  Exit Sub
ErrorHandler:
  HandleError True, "ShowVertices " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function CalcBearing(ByRef Point1 As IPoint, ByRef Point2 As IPoint) As Double
  On Error GoTo ErrorHandler


  Dim dblBearing As Double

  Dim xDist As Double
  Dim yDist As Double
  Dim xyTanDeg As Double
  
1758:   xDist = (Point1.X - Point2.X)
1759:   yDist = (Point1.Y - Point2.Y)
1760:   If yDist = 0 Then
1761:     If xDist < 0 Then
1762:       xyTanDeg = -90
1763:     ElseIf xDist = 0 Then
1764:       xyTanDeg = 0
1765:     Else
1766:       xyTanDeg = 90
1767:     End If
1768:   Else
1769:     xyTanDeg = AsDegrees(Atn(xDist / yDist))
1770:   End If

1772:   If (yDist >= 0) Then
1773:     dblBearing = 180 + xyTanDeg
1774:   Else
1775:     If (xDist <= 0) Then
1776:       dblBearing = xyTanDeg
1777:     Else
1778:       dblBearing = 360 + xyTanDeg
1779:     End If
1780:   End If ' END CALCULATING BEARING
  
1782:   dblBearing = Abs(dblBearing)
1783:   CalcBearing = dblBearing


  Exit Function
ErrorHandler:
  HandleError True, "CalcBearing " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function CalcDistMatrix(pArray As esriSystem.IArray, Optional IncludeLine As Boolean, _
    Optional IncludeBearing As Boolean, Optional pApp As IApplication) As Collection
  On Error GoTo ErrorHandler


1795:   Screen.MousePointer = vbHourglass
  
  ' RETURNS A COLLECTION OF IVariantArray OBJECTS
  ' EACH IVariantArray IDENTIFIED BY STRING; CONCATENATION OF [ORIGIN INDEX] & "_" & [DESTINATION INDEX]
  ' EACH IVariantArray OBJECT CONTAINS:
  '       0) ORIGIN SHAPE INDEX VALUE
  '       1) DESTINATION SHAPE INDEX VALUE
  '       2) DISTANCE
  '       3) CONNECTOR POLYLINE:  OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  '       4) BEARING:             OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  
  Dim pCollection As Collection
1807:   Set pCollection = New Collection
  
  Dim pProxOp As IProximityOperator
  
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  
  Dim lngArrayMaxIndex As Long
1815:   lngArrayMaxIndex = pArray.Count - 1
  
  Dim pGeometry1 As IGeometry
  Dim pGeometry2 As IGeometry
  
  Dim dblDistance As Double
  Dim dblBearing As Double
  Dim dblRevBearing As Double
  Dim strID As String
  Dim strRevID As String
  Dim pConnector As IPolyline
  Dim pRevConnector As IPolyline
  
  Dim pVarArray As IVariantArray
  Dim pRevVarArray As IVariantArray
  
1831:   If Not pApp Is Nothing Then
      ' PROGRESS BAR STUFF
    Dim psbar As IStatusBar
1834:     Set psbar = pApp.StatusBar
    Dim pPro As IStepProgressor
1836:     Set pPro = psbar.ProgressBar
    Dim lngCounter As Long
1838:     lngCounter = 0
    Dim lngTotalCount As Long
1840:     lngTotalCount = (((lngArrayMaxIndex + 1) * lngArrayMaxIndex) / 2)
    Dim strTotalCount As String
1842:     strTotalCount = CStr(lngTotalCount)
1843:     pPro.position = 1
1844:     psbar.ShowProgressBar "Building Preliminary Distance Matrix:  Step 1 of " & strTotalCount & "...", 1, _
            lngTotalCount, 1, True
1846:   End If
  
  Dim pOutputPointCollection As IPointCollection
  
1850:   For lngIndex = 0 To lngArrayMaxIndex
1851:     Set pGeometry1 = pArray.Element(lngIndex)
    
1853:     For lngIndex2 = lngIndex To lngArrayMaxIndex
      
1855:       strID = CStr(lngIndex) & "_" & CStr(lngIndex2)
1856:       strRevID = CStr(lngIndex2) & "_" & CStr(lngIndex)
      
1858:       Set pVarArray = New VarArray
1859:       Set pRevVarArray = New VarArray
      
      ' FIRST ELEMENT
1862:       pVarArray.Add lngIndex         ' FIRST VALUES IN THE ARRAY ARE ORIGIN NODES
1863:       pRevVarArray.Add lngIndex2
      
      ' SECOND ELEMENT
1866:       pVarArray.Add lngIndex2        ' SECOND VALUES IN THE ARRAY ARE "TO" NODES
1867:       pRevVarArray.Add lngIndex
      
1869:       If lngIndex = lngIndex2 Then   ' IF MEASURING DISTANCE TO ITSELF
1870:         Set pConnector = New Polyline
1871:         pConnector.SetEmpty
1872:         Set pRevConnector = New Polyline
1873:         pRevConnector.SetEmpty
        
        ' THIRD ELEMENT
1876:         pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
        
        ' FOURTH ELEMENT
1879:         If IncludeLine Then
1880:           pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
1881:         Else
1882:           pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
1883:         End If
        
        ' FIFTH ELEMENT
1886:         If IncludeBearing Then
1887:           pVarArray.Add -999         ' FIFTH VALUE IS BEARING
1888:         Else
1889:           pVarArray.Add False
1890:         End If
        
        ' ADD VARARRAY TO ORIGINAL COLLECTION
1893:         pCollection.Add pVarArray, strID
      
1895:       Else
        
1897:         If Not pApp Is Nothing Then
1898:           lngCounter = lngCounter + 1
1899:           pPro.Message = "Building Preliminary Distance Matrix:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
1900:           psbar.StepProgressBar
1901:         End If
        
1903:         Set pGeometry2 = pArray.Element(lngIndex2)
        
1905:         If IncludeLine Or IncludeBearing Then
          Dim pLineArray As IArray
1907:           Set pLineArray = CalcClosestPoints(pGeometry1, pGeometry2, 10)
          
1909:           If TypeOf pLineArray.Element(0) Is esriSystem.IStringArray Then      ' FUNCTION FAILED FOR SOME REASON
            Dim pStrArray As IStringArray
1911:             Set pStrArray = pLineArray.Element(0)
1912:             MsgBox "Failed to connect:" & vbCrLf & "Message = " & pStrArray.Element(0) & vbCrLf & _
                   "Index 1 = " & CStr(lngIndex) & " of " & CStr(lngArrayMaxIndex) & vbCrLf & _
                   "Index 2 = " & CStr(lngIndex2) & " of " & CStr(lngArrayMaxIndex)
1915:             Set pConnector = New Polyline
1916:             pConnector.SetEmpty
1917:             Set pRevConnector = New Polyline
1918:             pRevConnector.SetEmpty
            
            ' THIRD ELEMENT
1921:             pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
            
            ' FOURTH ELEMENT
1924:             If IncludeLine Then
1925:               pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
1926:             Else
1927:               pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
1928:             End If
            
            ' FIFTH ELEMENT
1931:             If IncludeBearing Then
1932:               pVarArray.Add -999         ' FIFTH VALUE IS BEARING
1933:             Else
1934:               pVarArray.Add False
1935:             End If
1936:           Else
          
1938:             If IncludeLine Then
1939:               Set pConnector = pLineArray.Element(0)
1940:               Set pRevConnector = New Polyline
1941:               Set pOutputPointCollection = pRevConnector
1942:               pOutputPointCollection.AddPoint pLineArray.Element(2)
1943:               pOutputPointCollection.AddPoint pLineArray.Element(1)
              
              ' THIRD ELEMENT
1946:               pVarArray.Add pConnector.length
1947:               pRevVarArray.Add pConnector.length
              
              ' FOURTH ELEMENT
1950:               pVarArray.Add pConnector
1951:               pRevVarArray.Add pRevConnector
1952:             Else
              ' FOURTH ELEMENT
1954:               pVarArray.Add False        ' JUST ADDING SMALL PLACEHOLDER ELEMENT
1955:               pRevVarArray.Add False     ' JUST ADDING SMALL PLACEHOLDER ELEMENT
1956:             End If
1957:             If IncludeBearing Then
1958:               dblBearing = CalcBearing(pLineArray.Element(1), pLineArray.Element(2))
1959:               If dblBearing < 180 Then
1960:                 dblRevBearing = dblBearing + 180
1961:               Else
1962:                 dblRevBearing = dblBearing - 180
1963:               End If
              ' FIFTH ELEMENT
1965:               pVarArray.Add dblBearing
1966:               pRevVarArray.Add dblRevBearing
1967:             Else
              ' FIFTH ELEMENT
1969:               pVarArray.Add False
1970:               pRevVarArray.Add False
1971:             End If
1972:           End If
1973:         Else
          ' THIRD ELEMENT
1975:           Set pProxOp = pGeometry1
1976:           dblDistance = pProxOp.ReturnDistance(pGeometry2)
1977:           pVarArray.Add dblDistance
1978:           pRevVarArray.Add dblDistance
          ' FOURTH ELEMENT   (DISTANCE)
1980:           pVarArray.Add False
1981:           pRevVarArray.Add False
          ' FIFTH ELEMENT   (BEARING)
1983:           pVarArray.Add False
1984:           pRevVarArray.Add False
1985:         End If
        
1987:         pCollection.Add pVarArray, strID
1988:         pCollection.Add pRevVarArray, strRevID

1990:       End If
1991:     Next lngIndex2
1992:   Next lngIndex
  
1994:   Set CalcDistMatrix = pCollection

1996:   Screen.MousePointer = vbDefault
1997:   If Not pApp Is Nothing Then
1998:     pPro.position = 1
1999:     psbar.HideProgressBar
2000:   End If


  Exit Function
ErrorHandler:
  HandleError True, "CalcDistMatrix " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function CalcClosestPoints(ByVal Shape1 As IGeometry, ByVal shape2 As IGeometry, Optional intMaxCurveRepeat As Integer) As IArray
  On Error GoTo ErrorHandler


' CalcClosestPoints
' Jenness Enterprises (www.jennessent.com)
' Given two shapes, this script returns an IARRAY object containing the line connecting the closest points on each shape, plus the connection points
' CURRENTLY DOES NOT GUARANTEE SUCCESS WITH TRUE CURVES BECAUSE VERTICES ARE NOT GOOD QUERY POINTS; ATTEMPTS SEVERAL RUNS BACK AND FORTH

' Dim pRelationalOperator As IRelationalOperator
Dim pGeometryType1 As esriGeometryType
Dim pGeometryType2 As esriGeometryType
Dim pGeometry1 As IGeometry
Dim pGeometry2 As IGeometry

2022: Set pGeometry1 = Shape1
2023: Set pGeometry2 = shape2

2025: pGeometryType2 = shape2.GeometryType

' IF SHAPE #2 HAPPENS TO BE POINT, SET THAT ONE FIRST
Dim ShouldReverse As Boolean
2029: ShouldReverse = False
2030: If pGeometryType2 = esriGeometryPoint Then
2031:   Set pGeometry1 = shape2
2032:   Set pGeometry2 = Shape1
2033:   pGeometryType2 = pGeometry2.GeometryType
2034:   ShouldReverse = True
2035: End If
  
2037: pGeometryType1 = pGeometry1.GeometryType

Dim pArray As IArray
2040: Set pArray = New esriSystem.Array
Dim pOutputLine As IPolyline
2042: Set pOutputLine = New Polyline
Dim pOutputPointCollection As IPointCollection
2044: Set pOutputPointCollection = pOutputLine

Dim pStartPoint As IPoint
Dim pEndPoint As IPoint

Dim pPoint1 As IPoint
Dim pPoint2 As IPoint
2051: Set pPoint2 = New Point

Dim pProximityOp As IProximityOperator
Dim pStringArray As IStringArray

' CHECK FOR NULL SHAPES
' CHECK FOR INTERSECTING SHAPES
' NOT SURE IF THIS WILL WORK WITH MULTIPOINTS

2060: If pGeometry1.IsEmpty Or pGeometry2.IsEmpty Then
2061:   Set pStringArray = New strArray
2062:   pStringArray.Add "Empty Shapes"
2063:   pStringArray.Add CStr(pGeometry1.IsEmpty)
2064:   pStringArray.Add CStr(pGeometry2.IsEmpty)
2065:   pArray.Add pStringArray
2066:   Set CalcClosestPoints = pArray
  Exit Function
2068: Else
2069:   Set pProximityOp = pGeometry1
2070:   If pProximityOp.ReturnDistance(pGeometry2) = 0 Then
2071:     Set pStringArray = New strArray
2072:     pStringArray.Add "Intersecting Shapes"
2073:     pArray.Add pStringArray
2074:     Set CalcClosestPoints = pArray
    Exit Function
2076:   End If
2077: End If

2079: If pGeometryType1 = esriGeometryPoint Then
2080:   Set pPoint1 = pGeometry1
  
2082:   If pGeometryType2 = esriGeometryPoint Then
2083:     Set pPoint2 = pGeometry2
    
2085:     If pPoint1.X = pPoint2.X And pPoint1.Y = pPoint2.Y Then
2086:       If ShouldReverse Then
2087:         pArray.Add pOutputLine
2088:         pArray.Add pPoint2
2089:         pArray.Add pPoint1
2090:       Else
2091:         pArray.Add pOutputLine
2092:         pArray.Add pPoint1
2093:         pArray.Add pPoint2
2094:       End If
2095:     Else
2096:       If ShouldReverse Then
2097:         pOutputPointCollection.AddPoint pPoint2
2098:         pOutputPointCollection.AddPoint pPoint1
2099:         pArray.Add pOutputLine
2100:         pArray.Add pPoint2
2101:         pArray.Add pPoint1
2102:       Else
2103:         pOutputPointCollection.AddPoint pPoint1
2104:         pOutputPointCollection.AddPoint pPoint2
2105:         pArray.Add pOutputLine
2106:         pArray.Add pPoint1
2107:         pArray.Add pPoint2
2108:       End If
2109:     End If
2110:   Else
    
2112:     Set pProximityOp = pGeometry2
    
2114:     pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
    
2116:     If ShouldReverse Then
2117:       pOutputPointCollection.AddPoint pPoint2
2118:       pOutputPointCollection.AddPoint pPoint1
2119:       pArray.Add pOutputLine
2120:       pArray.Add pPoint2
2121:       pArray.Add pPoint1
2122:     Else
2123:       pOutputPointCollection.AddPoint pPoint1
2124:       pOutputPointCollection.AddPoint pPoint2
2125:       pArray.Add pOutputLine
2126:       pArray.Add pPoint1
2127:       pArray.Add pPoint2
2128:     End If
    
    
2131:   End If
2132: Else
  Dim dblTestDistance As Double
  Dim pEnvelope As IEnvelope
  Dim pEnvelope2 As IEnvelope
2136:   Set pEnvelope = pGeometry1.Envelope
2137:   Set pEnvelope2 = pGeometry2.Envelope
2138:   pEnvelope.Union pEnvelope2
2139:   dblTestDistance = (pEnvelope.Height * pEnvelope.Width)
  Dim dblMaxDistance As Double
2141:   dblMaxDistance = dblTestDistance
  
  Dim pPointCollection1 As IPointCollection
  Dim pPointCollection2 As IPointCollection
  
2146:   If pGeometry1.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv As IEnvelope
2148:     Set pTempEnv = pGeometry1
    Dim pTempPoly1 As IPolygon
    Dim pTempPoint1 As IPoint
2151:     Set pTempPoly1 = New Polygon
2152:     Set pPointCollection1 = pTempPoly1
    Dim dXmin1 As Double
    Dim dYmin1 As Double
    Dim dXmax1 As Double
    Dim dYmax1 As Double
2157:     pTempEnv.QueryCoords dXmin1, dYmin1, dXmax1, dYmax1
2158:     Set pTempPoint1 = New Point
2159:     pTempPoint1.X = dXmin1
2160:     pTempPoint1.Y = dYmin1
2161:     pPointCollection1.AddPoint pTempPoint1
    
2163:     Set pTempPoint1 = New Point
2164:     pTempPoint1.X = dXmin1
2165:     pTempPoint1.Y = dYmax1
2166:     pPointCollection1.AddPoint pTempPoint1
    
2168:     Set pTempPoint1 = New Point
2169:     pTempPoint1.X = dXmax1
2170:     pTempPoint1.Y = dYmax1
2171:     pPointCollection1.AddPoint pTempPoint1
    
2173:     Set pTempPoint1 = New Point
2174:     pTempPoint1.X = dXmax1
2175:     pTempPoint1.Y = dYmin1
2176:     pPointCollection1.AddPoint pTempPoint1
2177:   Else
2178:     Set pPointCollection1 = pGeometry1
2179:   End If
  
2181:   If pGeometry2.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv2 As IEnvelope
2183:     Set pTempEnv2 = pGeometry2
    Dim pTempPoly2 As IPolygon
    Dim pTempPoint2 As IPoint
2186:     Set pTempPoly2 = New Polygon
2187:     Set pPointCollection2 = pTempPoly2
    Dim dXmin2 As Double
    Dim dYmin2 As Double
    Dim dXmax2 As Double
    Dim dYmax2 As Double
2192:     pTempEnv2.QueryCoords dXmin2, dYmin2, dXmax2, dYmax2
2193:     Set pTempPoint2 = New Point
2194:     pTempPoint2.X = dXmin2
2195:     pTempPoint2.Y = dYmin2
2196:     pPointCollection2.AddPoint pTempPoint2
    
2198:     Set pTempPoint2 = New Point
2199:     pTempPoint2.X = dXmin2
2200:     pTempPoint2.Y = dYmax2
2201:     pPointCollection2.AddPoint pTempPoint2
    
2203:     Set pTempPoint2 = New Point
2204:     pTempPoint2.X = dXmax2
2205:     pTempPoint2.Y = dYmax2
2206:     pPointCollection2.AddPoint pTempPoint2
    
2208:     Set pTempPoint2 = New Point
2209:     pTempPoint2.X = dXmax2
2210:     pTempPoint2.Y = dYmin2
2211:     pPointCollection2.AddPoint pTempPoint2
2212:   Else
2213:     Set pPointCollection2 = pGeometry2
2214:   End If
  
  Dim pClone As IClone
  
  Dim pVertex As IPoint
2219:   Set pVertex = New Point
  
  Dim pPointEnum As IEnumVertex
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
2225:   Set pPointEnum = pPointCollection1.EnumVertices
2226:   pPointEnum.Reset
2227:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
  ' CHECK IF CURVES; THIS CODE JUST CHECKS FIRST SEGMENT FOR CURVATURE
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollection1 As ISegmentCollection
2232:   Set pSegmentCollection1 = pGeometry1
  Dim pSegment1 As ISegment
2234:   Set pSegment1 = pSegmentCollection1.Segment(0)
  Dim pGeometryTypeA As esriGeometryType
2236:   pGeometryTypeA = pSegment1.GeometryType
  
  Dim pSegmentCollection2 As ISegmentCollection
2239:   Set pSegmentCollection2 = pGeometry2
  Dim pSegment2 As ISegment
2241:   Set pSegment2 = pSegmentCollection2.Segment(0)
  Dim pGeometryTypeB As esriGeometryType
2243:   pGeometryTypeB = pSegment2.GeometryType
  
2245:   booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
    (pGeometryTypeA = esriGeometryCircularArc) Or _
    (pGeometryTypeA = esriGeometryEllipticArc) Or _
    (pGeometryTypeB = esriGeometryBezier3Curve) Or _
    (pGeometryTypeB = esriGeometryCircularArc) Or _
    (pGeometryTypeB = esriGeometryEllipticArc)

2252:   Do While Not pVertex.IsEmpty
2253:     Set pProximityOp = pGeometry2
2254:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
2255:     If dblTestDistance < dblMaxDistance Then
2256:       dblMaxDistance = dblTestDistance
2257:       Set pClone = pVertex
2258:       Set pPoint1 = pClone.Clone
2259:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint2
2260:     End If
2261:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
2262:   Loop
  
2264:   Set pPointEnum = pPointCollection2.EnumVertices
2265:   pPointEnum.Reset
2266:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
2268:   Do While Not pVertex.IsEmpty
2269:     Set pProximityOp = pGeometry1
2270:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
2271:     If dblTestDistance < dblMaxDistance Then
2272:       dblMaxDistance = dblTestDistance
2273:       Set pClone = pVertex
2274:       Set pPoint2 = pClone.Clone
2275:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint1
2276:     End If
2277:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
2278:   Loop
  
  ' FOR DEBUGGING
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  ' IF WORKING WITH CURVES, GO BACK AND FORTH A FEW TIMES
2284:   If booWorkingWithCurves Then
    Dim intRepeat As Integer
    Dim pPoint1Temp As IPoint, pPoint2Temp As IPoint
    
2288:     Do Until (intRepeat = intMaxCurveRepeat)
 '     Graphic_MakeFromGeometry pMxDoc, pPoint1, "DeleteMe"
 '     Graphic_MakeFromGeometry pMxDoc, pPoint2, "DeleteMe"
      
2292:       Set pProximityOp = pGeometry2
2293:       pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
      
2295:       Set pProximityOp = pGeometry1
2296:       pProximityOp.QueryNearestPoint pPoint2, esriNoExtension, pPoint1
      
2298:       intRepeat = intRepeat + 1
    
2300:     Loop
  
2302:   End If
  
2304:   If ShouldReverse Then
2305:     pOutputPointCollection.AddPoint pPoint2
2306:     pOutputPointCollection.AddPoint pPoint1
2307:     pArray.Add pOutputLine
2308:     pArray.Add pPoint2
2309:     pArray.Add pPoint1
2310:   Else
2311:     pOutputPointCollection.AddPoint pPoint1
2312:     pOutputPointCollection.AddPoint pPoint2
2313:     pArray.Add pOutputLine
2314:     pArray.Add pPoint1
2315:     pArray.Add pPoint2
2316:   End If
  
2318: End If

2320: Set CalcClosestPoints = pArray


  Exit Function
ErrorHandler:
  HandleError True, "CalcClosestPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcCheckClockwise(theP As IPoint, theQ As IPoint, theR As IPoint) As Boolean
 
On Error GoTo err
' CalcCheckClockwise
' Jenness Enterprises <www.jennessent.com)>
' Given 3 consecutive points, this scripts calculates whether the third point lies to the right
' (clockwise) or to the left (counter-clockwise) of the line connecting the first point to
' the second point.

' CLOCKWISE IF TRUE
2338: CalcCheckClockwise = ((theQ.X * (theR.Y - theP.Y)) + (theQ.Y * (theP.X - theR.X)) - ((theP.X) * (theR.Y)) _
      + ((theP.Y) * (theR.X)) < 0)
    Exit Function
err:
2342:   MsgBox "Messed up CalcCheckClockwise..."
End Function

Public Function PointAdd(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


2349:   Set PointAdd = New Point
2350:   PointAdd.PutCoords pPointA.X + pPointB.X, pPointA.Y + pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointAdd " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PointSubtract(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


2362:   Set PointSubtract = New Point
2363:   PointSubtract.PutCoords pPointA.X - pPointB.X, pPointA.Y - pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointSubtract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsRadians(theDegrees As Double) As Double
  On Error GoTo ErrorHandler


2375:   AsRadians = dblPi * (theDegrees / 180)


  Exit Function
ErrorHandler:
  HandleError True, "AsRadians " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsDegrees(theRadians As Double) As Double
  On Error GoTo ErrorHandler


2387:   AsDegrees = (theRadians * 180) / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "AsDegrees " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub CalcPointLine(ptOrigin As IPoint, theLength As Double, dblAzimuth As Double, ptEndPoint As IPoint, _
    Optional pLine As IPolyline)
  On Error GoTo ErrorHandler


' Jenness Enterprises <www.jennessent.com>
' Given an origin point, distance and bearing, this script will return a new point at that distance and bearing, and a line
' connecting that new point to the origin point

'' MAKE SURE AZIMUTH IS BETWEEN 0 AND 360
Dim theAzimuth As Double
2406: theAzimuth = dblAzimuth

2408: Set ptEndPoint = New Point

2410: Do While theAzimuth < 0
2411:   theAzimuth = theAzimuth + 360
2412: Loop
2413: theAzimuth = theAzimuth Mod 360
'
'' NEW SEGMENT AND POINT DISTANCE NORTH/SOUTH AND EAST/WEST BASED ON DISTANCE AND BEARING FROM ORIGIN.
'' THERE ARE EIGHT DIFFERENT POSSIBILITIES:  THE BEARING COULD BE ONE OF THE FOUR CARDINAL DIRECTIONS OR IT
'' COULD BE IN ONE OF THE FOUR QUADRANTS.  THE BEARING IS TREATED DIFFERENTLY IN EACH OF THESE CIRCUMSTANCES.
'' THE CALCULATION TO DETERMINE THE NEW POINT LOCATION IS ESSENTIALLY A MATTER OF TAKING THE SINE OR THE
'' COSINE OF THE ANGLE (AFTER CONVERTING IT TO RADIANS), AND MULTIPLYING THAT SINE OR COSINE BY THE MEASURED
'' DISTANCE.  PLEASE CONTACT THE AUTHOR IF THIS DOESN'T MAKE SENSE, OR IF YOU WOULD LIKE FURTHER EXPLANATION.
Dim NorthSouthDistance As Double
Dim EastWestDistance As Double
Dim EastWest As Integer
Dim NorthSouth As Integer

2426: If theAzimuth = 0 Or theAzimuth = 360 Then
2427:   NorthSouthDistance = theLength
2428:   NorthSouth = 1
2429:   EastWestDistance = 0
2430:   EastWest = 1
2431: ElseIf (theAzimuth = 180) Then
2432:   NorthSouthDistance = theLength
2433:   NorthSouth = -1
2434:   EastWestDistance = 0
2435:   EastWest = 1
2436: ElseIf (theAzimuth = 90) Then
2437:   NorthSouthDistance = 0
2438:   NorthSouth = 1
2439:   EastWestDistance = theLength
2440:   EastWest = 1
2441: ElseIf (theAzimuth = 270) Then
2442:   NorthSouthDistance = 0
2443:   NorthSouth = 1
2444:   EastWestDistance = theLength
2445:   EastWest = -1
2446: ElseIf ((theAzimuth > 0) And (theAzimuth < 90)) Then
2447:   NorthSouthDistance = Cos(AsRadians(theAzimuth)) * theLength
2448:   NorthSouth = 1
2449:   EastWestDistance = Sin(AsRadians(theAzimuth)) * theLength
2450:   EastWest = 1
2451: ElseIf ((theAzimuth > 90) And (theAzimuth < 180)) Then
2452:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 90))) * theLength
2453:   NorthSouth = -1
2454:   EastWestDistance = (Cos(AsRadians(theAzimuth - 90))) * theLength
2455:   EastWest = 1
2456: ElseIf ((theAzimuth > 180) And (theAzimuth < 270)) Then
2457:   NorthSouthDistance = (Cos(AsRadians(theAzimuth - 180))) * theLength
2458:   NorthSouth = -1
2459:   EastWestDistance = (Sin(AsRadians(theAzimuth - 180))) * theLength
2460:   EastWest = -1
2461: ElseIf ((theAzimuth > 270) And (theAzimuth < 360)) Then
2462:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 270))) * theLength
2463:   NorthSouth = 1
2464:   EastWestDistance = (Cos(AsRadians(theAzimuth - 270))) * theLength
2465:   EastWest = -1
2466: End If

Dim theMovementNorth As Double
Dim theMovementWest As Double

2471: theMovementNorth = NorthSouthDistance * NorthSouth
2472: theMovementWest = EastWestDistance * EastWest

Dim startX As Double
Dim startY As Double

2477: ptOrigin.QueryCoords startX, startY
2478: ptEndPoint.PutCoords startX + theMovementWest, startY + theMovementNorth

2480: If Not pLine Is Nothing Then
  Dim pPointColl As IPointCollection
2482:   pLine.SetEmpty
2483:   Set pPointColl = pLine
2484:   pPointColl.AddPoint ptOrigin
2485:   pPointColl.AddPoint ptEndPoint
2486: End If


  Exit Sub
ErrorHandler:
  HandleError True, "CalcPointLine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function EllipticArcToPolygon(SegCollection As ISegmentCollection, NumVertices As Long) As IPolygon4
  On Error GoTo ErrorHandler


'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
2506:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
2508:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
2512:   For anIndex = 0 To lngSegCount
2513:     theTestLength = SegCollection.Segment(anIndex).length
2514:     theLength = theLength + theTestLength
2515:     lngLengths(anIndex) = theTestLength
2516:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
2522:   For anIndex = 0 To lngSegCount
2523:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
2524:     If lngNumVertices < 8 Then lngNumVertices = 8
2525:     lngVertices(anIndex) = lngNumVertices
2526:   Next anIndex
  
  Dim pMpt As IPointCollection
2529:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
2531:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
2537:   For anIndex = 0 To lngSegCount
2538:     lngNumVertices = lngVertices(anIndex)
2539:     pRatio = 1 / lngNumVertices
2540:     Set pCurve = SegCollection.Segment(anIndex)
    
2542:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
2544:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
2545:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
2549:       pMpt.AddPoint pClone.Clone
2550:     Next anIndex2
2551:   Next anIndex
  
  Dim pPoly4 As IPolygon4
  Dim pTopoOp2 As ITopologicalOperator2
  Dim pTopoOp3 As ITopologicalOperator3
2556:   Set pTopoOp2 = pMpt
2557:   Set pPoly4 = pTopoOp2.ConvexHull
2558:   Set pTopoOp3 = pPoly4
2559:   pTopoOp3.IsKnownSimple = False
2560:   pTopoOp3.Simplify
  
2562:   Set EllipticArcToPolygon = pPoly4


  Exit Function
ErrorHandler:
  HandleError True, "EllipticArcToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function SpheroidalPolylineLength2(pPolyline As IPolyline, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
2588:   dblTotalLength = 0
2589:   Set pSegCollection = pPolyline
  
2591:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
2592:     Set pSeg = pSegCollection.Segment(lngIndex)
2593:     Set pPoint1 = pSeg.FromPoint
2594:     Set pPoint2 = pSeg.ToPoint
2595:     dblLength = DistanceVincentyPoints2(pPoint1, pPoint2, dblAz1, dblAz2, dblEquatorialRadius, dblPolarRadius)
2596:     dblTotalLength = dblTotalLength + dblLength
2597:   Next lngIndex
  
2599:   SpheroidalPolylineLength2 = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints2(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double, _
  Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyPoints, TO ALLOW FOR ANY ELLIPSOID
  
2614:   DistanceVincentyPoints2 = DistanceVincentyNumbers2(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2, _
        dblEquatorialRadius, dblPolarRadius)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyNumbers2(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double, _
        Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyNumbers TO ALLOW FOR ANY ELLIPSOID
  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
2639:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
2640:     DistanceVincentyNumbers2 = 0
2641:     dblAz1 = 0
2642:     dblAz2 = 0
    Exit Function
2644:   End If
  
  
  Dim A As Double
  Dim B As Double
2649:   A = dblEquatorialRadius   ' SPHEROID; EQUATORIAL RADIUS
2650:   B = dblPolarRadius        ' SPHEROID; POLAR RADIUS
  
  Dim f As Double
2653:   f = (A - B) / A           ' FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
2656:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
2661:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
2662:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
2669:   dblSinU1 = Sin(U1)
2670:   dblCosU1 = Cos(U1)
2671:   dblSinU2 = Sin(U2)
2672:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
2675:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
2676:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
2678:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
2693:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
2694:     sinLambda = Sin(dblLambda)                                                          '  |
2695:     cosLambda = Cos(dblLambda)                                                          '  |
2696:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
2698:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
2699:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
2700:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
2701:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
2702:     If cosSqAlpha = 0 Then
2703:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
2704:     Else
2705:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
2706:     End If
    
2708:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
2709:     dblLambdaComp = dblLambda
2710:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
2711:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
2713:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
2715:     If lngIterations = 0 Then
2716:       MsgBox "Vincenty Formula failed to converge!"
2717:       DistanceVincentyNumbers2 = -9999
      Exit Function
2719:     End If
2720:     lngIterations = lngIterations - 1
2721:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim s As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
2733:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
2738:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
2739:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
2740:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
2741:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
2743:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
2744:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
2745:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
2746:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
2747:   s = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
2761:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
2762:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
2764:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
2765:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
2766:   DistanceVincentyNumbers2 = s


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function





