Attribute VB_Name = "aml_func_mod"
'             Environmental Systems Research Institute, Inc.
'Module Name: aml_func.bas
'Description: Used to perform AML-like functions for string manipipulation.
'             Also parses out tokens in a string to elements in an array.
'
'   Requires: ParseString and ParseStringR require that you Dim the named array arg in
'             the calling propgram: Must be a zero dimensioned string array:
'             Dim yourarray() as string

'    Methods: After   - Returns the substring to the right of the leftmost occurrence of
'                       searchStr
'             Before  - Returns the substring to the left of the leftmost occurrence of
'                       searchStr.
'        ExistFileDir - Returns True/False if file or Directory exists.
'             Extract - Returns an element from a list of elements
'             Index   - Returns the position of the leftmost occurrence of a specified
'                       string in a target string.
'             Keyword - Returns the position of a keyword within a list of keywords.
'             Search  - Returns the position of the first character of a search string
'                       in a target string.
'             Sort    - Returns a sorted a list of elements.
'             Subst   - Returns a string that has had one string subsituted for another
'             Substr  - Returns a substring that starts at a specified character position.
'             Token   - Allows tokens in a list to be manipulated.
'
'                       Count - the number of tokens in a list
'                       Find  - the position of a token in a list
'                       Move  - tokens in a list
'                       Insert- a new token into a list
'                       Delete- a token in a list
'                       Replace-one token in a list for another
'                       Switch -one token in a list for another
'
'        ParseString  - Populates string array with tokens in a string.
'        ParseStringR - Same as ParseString except blanks and commas are
'                       treated as delimiters.
'
'    History: DMA     - 03/04/97 - Original coding
'       Glenn Meister - 12/19/97 - Added ExistFileDir function
'
'   MODIFICATIONS:  JEFF JENNESS
'   GetFullFileString - ' CONVERTS 8.3 FILESTRING TO FULL TEXT
'           ReturnDir - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "\" CHARACTER
'      ReturnFilename - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "\" CHARACTER
'         ReturnFiles - GIVEN A STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FILES THAT MET CRITERIA
'                       AND ARRAY OF STRING FILENAMES.  OPTIONS FOR INCLUDE READ-ONLY, SYSTEM, HIDDEN FILES.
'       ReturnFolders - GIVENA STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FOLDERS THAT MET CRITERIA
'                       AND ARRAY OF FOLDER FILENAMES.
'       ClipExtension - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "." CHARACTER
'        SetExtension - GIVEN A STRING AND EXTENSION, RETURNS TEXT PRECEDING LAST "." CHARACTER, PLUS "." AND NEW EXTENSION
'    GetExtensionText - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "." CHARACTER
'      FieldIsNumeric - GIVEN A FIELD, RETURNS BOOLEAN
'       FieldIsString - GIVEN A FIELD, RETURNS BOOLEAN
'         FieldIsDate - GIVEN A FIELD, RETURNS BOOLEAN
'        FieldIsShape - GIVEN A FIELD, RETURNS BOOLEAN
'        InsertCommas - GIVEN A NUMBER OR STRING, INSERTS COMMAS AND RETURNS STRING
'     BasicTrimAvenue - GIVEN A STRING, RETURNS A STRING WITH VALUES TRIMMED OFF EACH SIDE
'  MakeUniqueFilename - GIVEN A STRING REPRESENTING A FILEPATH, RETURNS A UNIQUE FILENAME WHICH MAY HAVE NUMBERS APPENDED TO IT
'      GetTheUserName - JUST RETURNS THE USER NAME; CAN BE USED TO FIND THE "MY DOCUMENTS" FOLDER
'    TempPathLocation - RETURNS PATHNAME TO TEMP DIRECTORY
'     CreateShapefile - GIVEN PATHNAME AND SHAPE TYPE, RETURNS A pFeatureClass.  ESRI CODE, MODIFIED BY JENNESS
'    CreatedBASETable - GIVEN PATHNAME AND OPTIONAL FIELDS, RETURNS ITABLE
'        GetMxDocPath - GIVEN APPLICATION OBJECT, RETURNS PATHNAME OF MAP DOCUMENT
'         QuoteString - GIVEN A STRING, RETURNS A QUOTED VERSION OF THAT STRING
'    SubstituteString - GIVEN A STRING, A SEARCH STRING AND A SUBSTITUTE STRING, REPLACES ALL INSTANCES OF SEARCH TEXT WITH
'                       SUBSTITUTE TEXT IN THE ORIGINAL STRING
'ReturnArcGISInstallDir - RETURNS A STRING CONTAINING THE ARCGIS INSTALL LOCATION
'ReturnArcGISGeneralDir - RETURNS DIRECTORY STRING FOR INSTALL, LAST LOCATION, LAST BROWSED, LAST EXPORT AND LAST SAVE TO
' ReturnArcGISVersion - RETURNS ARCGIS VERSION NUMBER
'          FileExists - GIVEN A STRING PATH, RETURNS TRUE OR FALSE
' ReturnShapeTypeName - GIVEN A SHAPE TYPE ENUMERATION, RETURNS SHAPE NAME
' QueryForNewFilename - GIVEN A SAMPLE FILENAME, OPENS DIALOG AND QUERIES USER TO SPECIFY NEW FILENAME

Option Explicit

' FILENAME 8.3 FORMAT CONVERSION BELOW COPIED FROM http://forums.esri.com/Thread.asp?c=93&f=993&t=123512&mc=1#msgid400360
'   POSTED BY  Brett N.Meroney
'              GIS Programmer / Analyst
'              Integrated Laboratory Systems, Inc.
'              contractor to US-EPA Region VIII
'              Golden , CO
' NECESSARY TO CONVERT 8.3 FILENAMES TO USEABLE FILENAMES

Private Const strMAXPATH = 260
Private Const m_Quotation = """"

'
' Win32 Registry functions
'
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal ulOptions As Long, _
     ByVal samDesired As Long, _
     phkResult As Long) _
     As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal Reserved As Long, _
     ByVal lpClass As String, _
     ByVal dwOptions As Long, _
     ByVal samDesired As Long, _
     lpSecurityAttributes As Any, _
     phkResult As Long, _
     lpdwDisposition As Long) _
     As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" _
     (ByVal hKey As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpReserved As Long, _
     lpcSubKeys As Long, _
     lpcbMaxSubKeyLen As Long, _
     lpcbMaxClassLen As Long, _
     lpcValues As Long, _
     lpcbMaxValueNameLen As Long, _
     lpcbMaxValueLen As Long, _
     lpcbSecurityDescriptor As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" _
     (ByVal hKey As Long, _
     ByVal dwIndex As Long, _
     ByVal lpName As String, _
     lpcbName As Long, _
     lpReserved As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" _
     (ByVal hKey As Long, _
      ByVal dwIndex As Long, _
     ByVal lpValueName As String, _
     lpcbValueName As Long, _
     lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal Reserved As Long, _
     ByVal dwType As Long, _
     lpData As Any, _
     ByVal cbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" _
     (ByVal hKey As Long, _
      ByVal lpSubKey As String) _
     As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String) _
     As Long
Private Declare Function RegCloseKey Lib "advapi32.dll" _
     (ByVal hKey As Long) _
     As Long
'
' Constants for Windows 32-bit Registry API
'
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006
'
' Reg result codes
'
Private Const REG_CREATED_NEW_KEY = &H1                      ' New Registry Key created
Private Const REG_OPENED_EXISTING_KEY = &H2                      ' Existing Key opened
'
' Reg Create Type Values...
'
Private Const REG_OPTION_RESERVED = 0           ' Parameter is reserved
Private Const REG_OPTION_NON_VOLATILE = 0       ' Key is preserved when system is rebooted
Private Const REG_OPTION_VOLATILE = 1           ' Key is not preserved when system is rebooted
Private Const REG_OPTION_CREATE_LINK = 2        ' Created key is a symbolic link
Private Const REG_OPTION_BACKUP_RESTORE = 4     ' open for backup or restore
'
' Reg Key Security Options
'
Private Const DELETE = &H10000
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Private Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
Private Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Private Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_MORE_DATA = 234
Private Const ERROR_NO_MORE_ITEMS = 259

Private Const REG_SZ = 1                         ' Unicode nul terminated string
'

Private Declare Function GetLongPathName Lib "Kernel32" Alias _
    "GetLongPathNameA" (ByVal lpszShortPath As String, _
    ByVal lpszLongPath As String, ByVal cchBuffer As Long) _
    As Long

Public Enum enumArcGISFolderTypes
   enumLastBrowsedLocation
   enumLastExportToLocation
   enumLastLocation
   enumLastSaveToLocation
   enumArcGISInstallLocation
End Enum

Private Declare Function GetUserName Lib "advapi32.dll" Alias "GetUserNameA" (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function GetTempPath Lib "Kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Const Quotation = """"
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\az_linkages\VB_Code\aml_func_mod.bas"


Public Function QueryForNewFilename(strSampleName As String) As String
  On Error GoTo ErrorHandler


  


  Exit Function
ErrorHandler:
  HandleError True, "QueryForNewFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnShapeName(pEnum As esriGeometryType) As String
  On Error GoTo ErrorHandler


  Select Case pEnum
  Case 0
263:     ReturnShapeName = "Unknown Geometry"
  Case 1
265:     ReturnShapeName = "Point"
  Case 2
267:     ReturnShapeName = "Multipoint"
  Case 3
269:     ReturnShapeName = "Polyline"
  Case 4
271:     ReturnShapeName = "Polygon"
  Case 5
273:     ReturnShapeName = "Envelope"
  Case 6
275:     ReturnShapeName = "Path"
  Case 7
277:     ReturnShapeName = "Unknown Geometry"
  Case 9
279:     ReturnShapeName = "Multipatch"
  Case 11
281:     ReturnShapeName = "Ring"
  Case 13
283:     ReturnShapeName = "Line"
  Case 14
285:     ReturnShapeName = "Circular Arc"
  Case 15
287:     ReturnShapeName = "Bezier Curve"
  Case 16
289:     ReturnShapeName = "Elliptic Arc"
  Case 17
291:     ReturnShapeName = "Geometry Bag"
  Case 18
293:     ReturnShapeName = "Triangle Strip"
  Case 19
295:     ReturnShapeName = "Triangle Fan"
  Case 20
297:     ReturnShapeName = "Ray"
  Case 21
299:     ReturnShapeName = "Sphere"
  Case 22
301:     ReturnShapeName = "Triangles"
  Case Else
303:     ReturnShapeName = "Unknown Geometry"
304:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "ReturnShapeName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FileExists(strFileName As String) As Boolean
  On Error GoTo ErrorHandler


317:   FileExists = Dir(strFileName) <> ""


  Exit Function
ErrorHandler:
  HandleError True, "FileExists " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISVersion() As String
  On Error GoTo ErrorHandler

' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
337:    ReturnArcGISVersion = "Unable to determine ArcGIS Version!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
346:    strSection = ""
   Dim strKey As String
348:    strKey = "RealVersion"
   Dim strDefault As String
350:    strDefault = ""
   Dim strDir As String
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
357:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
358:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
360:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
363:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
364:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
366:          If nBytes > 0 Then
367:             Buffer = Space(nBytes)
368:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
369:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
371:                strDir = Left(Buffer, nBytes - 1)
372:                ReturnArcGISVersion = strDir
373:             End If
374:          End If
375:       End If
376:       Call RegCloseKey(hKey)
377:    End If



  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISVersion " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISInstallDir() As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
399:    ReturnArcGISInstallDir = "Unable to determine ArcGIS Install location!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
408:    strSection = ""
   Dim strKey As String
410:    strKey = "InstallDir"
   Dim strDefault As String
412:    strDefault = ""
   Dim strDir As String
   
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
420:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
421:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
423:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
426:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
427:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
429:          If nBytes > 0 Then
430:             Buffer = Space(nBytes)
431:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
432:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
434:                strDir = Left(Buffer, nBytes - 1)
435:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
436:                ReturnArcGISInstallDir = strDir
437:             End If
438:          End If
439:       End If
440:       Call RegCloseKey(hKey)
441:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISInstallDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISGeneralDir(lngArcGISFolderTypes As Long) As String
  On Error GoTo ErrorHandler
  
  ' MODIFICATION TO TRY TO WORK AROUND VB6 BUG
  ' ORIGINAL FUNCTION CALL AS FOLLOWS
  ' Public Function ReturnArcGISGeneralDir(pArcGISFolderType As enumArcGISFolderTypes) As String
  ' MODIFIED FUNCTION CALL AS FOLLOWS
  ' Public Function ReturnArcGISGeneralDir(lngArcGISFolderTypes As long) As String
  
  Dim pArcGISFolderType As enumArcGISFolderTypes
459:   pArcGISFolderType = lngArcGISFolderTypes

' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
   
   
  Dim nRet As Long
  Dim hKey As Long
  Dim nType As Long
  Dim nBytes As Long
  Dim Buffer As String
  Dim strKey As String
  Dim strDir As String
  
  Select Case pArcGISFolderType
    Case enumArcGISInstallLocation
482:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Install location!"
483:       nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\ESRI\ArcGIS", 0&, KEY_READ, hKey)
484:       strKey = "InstallDir"
    Case enumLastSaveToLocation
486:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Saved To location!"
487:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
488:       strKey = "LastSaveToLocation"
    Case enumLastBrowsedLocation
490:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Browsed To location!"
491:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
492:       strKey = "LastBrowseLocation"
    Case enumLastExportToLocation
494:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Exported To location!"
495:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
496:       strKey = "LastExportToLocation"
    Case enumLastLocation
498:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last location!"
499:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
500:       strKey = "LastLocation"
501:   End Select
   
  ' Assume failure and set return to Default
  '   GetSetting = Default

   ' Open key
507:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
509:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
512:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
513:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
515:          If nBytes > 0 Then
516:             Buffer = Space(nBytes)
517:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
518:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
520:                strDir = Left(Buffer, nBytes - 1)
521:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
522:                ReturnArcGISGeneralDir = strDir
523:             End If
524:          End If
525:       End If
526:       Call RegCloseKey(hKey)
527:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISGeneralDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function SubKey(ByVal strCompany As String, ByVal strAppName As String, Optional ByVal Section As String = "") As String
  On Error GoTo ErrorHandler


   ' Build SubKey from known values
540:    SubKey = "Software\" & strCompany & "\" & strAppName
541:    If Len(Section) Then
542:       SubKey = SubKey & "\" & Section
543:    End If


  Exit Function
ErrorHandler:
  HandleError False, "SubKey " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ReturnFolders(ByVal DirPath As String, Optional ByVal FolderName As String) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full folder filenames
  
  Dim returnCollection As Collection
559:   Set returnCollection = New Collection
  
  Dim strFolders() As String
  
  Dim strSearchString As String
564:   If FolderName = "" Then
565:     strSearchString = "*"
566:   Else
567:     strSearchString = FolderName
568:   End If
  
  Dim strSearchPath As String
  
572:   DirPath = Trim(DirPath)
573:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
574:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFolder As String
  Dim anIndex As Integer
578:   anIndex = -1
  
580:   strFoundFolder = Dir(strSearchPath, 16)
  
582:   Do While Not strFoundFolder = ""
583:     If (Not strFoundFolder = ".") And (Not strFoundFolder = "..") Then
584:       If GetAttr(DirPath & strFoundFolder) = vbDirectory Then
585:         anIndex = anIndex + 1
        ReDim Preserve strFolders(anIndex)
587:         strFolders(anIndex) = DirPath & strFoundFolder
588:       End If
589:     End If
590:     strFoundFolder = Dir
591:   Loop
  
593:   returnCollection.Add (UBound(strFolders) - LBound(strFolders) + 1)
594:   returnCollection.Add (strFolders)
  
596:   Set ReturnFolders = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFolders " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnFiles(ByVal DirPath As String, Optional ByVal FileName As String, Optional ByVal IncludeReadOnlyFiles As Boolean, _
                            Optional ByVal IncludeHiddenFiles As Boolean, Optional ByVal IncludeSystemFiles As Boolean) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full filenames
  
  Dim returnCollection As Collection
614:   Set returnCollection = New Collection
  
  Dim strFiles() As String
  Dim intOption As Integer
618:   intOption = 0
619:   If IncludeReadOnlyFiles Then intOption = intOption + 1
620:   If IncludeHiddenFiles Then intOption = intOption + 2
621:   If IncludeSystemFiles Then intOption = intOption + 4
  
  Dim strSearchString As String
624:   If FileName = "" Then
625:     strSearchString = "*"
626:   Else
627:     strSearchString = FileName
628:   End If
  
  Dim strSearchPath As String
  
632:   DirPath = Trim(DirPath)
633:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
634:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFile As String
  Dim anIndex As Integer
  
639:   strFoundFile = Dir(strSearchPath, intOption)
  
641:   Do While Not strFoundFile = ""
    ReDim Preserve strFiles(anIndex)
643:     strFiles(anIndex) = DirPath & strFoundFile
644:     strFoundFile = Dir
645:     anIndex = anIndex + 1
646:   Loop
  
648:   returnCollection.Add anIndex
'  If anIndex = 0 Then
'    returnCollection.Add 0
'  Else
'    returnCollection.Add (UBound(strFiles) - LBound(strFiles) + 1)
'  End If
654:   returnCollection.Add (strFiles)
  
656:   Set ReturnFiles = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFiles " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function QuoteString(strInput As String) As String
  On Error GoTo ErrorHandler


  Dim strQuoted As String
670:   strQuoted = m_Quotation & SubstituteString(strInput, Chr(34), m_Quotation & m_Quotation) & m_Quotation
    
672:   QuoteString = strQuoted


  Exit Function
ErrorHandler:
  HandleError True, "QuoteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ContainsString(strInText As String, strSearchText As String) As Boolean
  On Error GoTo ErrorHandler

  
684:   If strInText = "" Or strSearchText = "" Then
685:     ContainsString = False
686:   Else
687:     ContainsString = InStr(1, strInText, strSearchText, vbTextCompare) > 0
688:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ContainsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SubstituteString(strFullText As String, strSearchText As String, strSubstituteText As String)
  On Error GoTo ErrorHandler


  Dim lngIndex As Long
  Dim lngStartPos As Long
  Dim lngSearchLength As Long
  Dim lngFullLength As Long
  
705:   lngStartPos = 1
706:   lngSearchLength = Len(strSearchText)
707:   lngFullLength = Len(strFullText)
  
709:   lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
  
  Dim strNewString As String
712:   If lngIndex = 0 Then
713:     strNewString = strFullText
714:   Else
715:     strNewString = Left(strFullText, lngIndex - 1) & strSubstituteText
716:     lngStartPos = lngIndex + lngSearchLength
717:   End If
  
719:   Do While lngIndex <> 0
720:     lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
721:     If lngIndex = 0 Then
722:       strNewString = strNewString & Right(strFullText, lngFullLength - lngStartPos + 1)
723:     Else
724:       strNewString = strNewString & Mid(strFullText, lngStartPos, lngIndex - lngStartPos) & strSubstituteText
725:       lngStartPos = lngIndex + lngSearchLength
726:     End If
  
728:   Loop
729:   SubstituteString = strNewString


  Exit Function
ErrorHandler:
  HandleError True, "SubstituteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetMxDocPath(pApp As IApplication) As String
  On Error GoTo ErrorHandler


  Dim pTemplates As ITemplates
  Dim lTempCount As Long
  
744:   Set pTemplates = pApp.Templates
745:   lTempCount = pTemplates.Count
  
  ' The document is always the last item
748:   GetMxDocPath = pTemplates.Item(lTempCount - 1)


  Exit Function
ErrorHandler:
  HandleError True, "GetMxDocPath " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CreatedBASETable(strFullName As String, Optional pFields As IFields) As ITable
  On Error GoTo ErrorHandler


' createDBF: simple function to create a DBASE file.
' note: the name of the DBASE file should not contain the .dbf extension
' ESRI Sample; modified by Jenness August 20 2007
  
  Dim strName As String
  Dim strFolder As String
  
767:   strFolder = aml_func_mod.ReturnDir(strFullName)
768:   strName = aml_func_mod.ReturnFilename(strFullName)
769:   If Right(strName, 4) = ".dbf" Then strName = Left(strName, Len(strName) - 4)
  
  ' Open the Workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
  Dim fs As Object
  Dim pFieldsEdit As IFieldsEdit
  Dim pFieldEdit As IFieldEdit
  Dim pField As IField

779:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
780:   If Not aml_func_mod.ExistFileDir(strFolder) Then
781:     MsgBox "Folder does not exist: " & vbCr & strFolder
    Exit Function
783:   End If

785:   Set pFWS = pWorkspaceFactory.OpenFromFile(strFolder, 0)
  
  ' if a fields collection is not passed in then create one
788:   If pFields Is Nothing Then
    ' create the fields used by our object
790:     Set pFields = New Fields
791:     Set pFieldsEdit = pFields
792:     pFieldsEdit.FieldCount = 1
    
    'Create text Field
795:     Set pField = New Field
796:     Set pFieldEdit = pField
797:     With pFieldEdit
798:         .Precision = 8
799:         .Name = "Unique_ID"
800:         .Type = esriFieldTypeInteger
801:     End With
802:     Set pFieldsEdit.Field(0) = pField
803:   End If
  
  Dim strString As String
  Dim lngIndex As Long
  Dim pFieldInfo As IFieldInfo
  
809:   Set pField = pFields.Field(0)


'  MsgBox pField.Name
'  MsgBox pField.Scale
'  MsgBox pField.Precision
'  MsgBox pField.Type

'  For lngIndex = 0 To pFields.FieldCount - 1
'    strString = strString + "-----------------" & "  Field Name = " & pFields.Field(lngIndex).Name & vbCrLf & _
'        "  Field Scale = " & pFields.Field(lngIndex).Scale & vbCrLf & "  Precision = " & pFields.Field(lngIndex).Precision & _
'        vbCrLf & "  Field Type = " & CStr(pFields.Field(lngIndex).Type) & vbCrLf
'  Next lngIndex
'  MsgBox "Problem with workspace? " & CStr(pFWS Is Nothing) & vbCrLf & "Filename = " & strFullName & vbCrLf & _
'        "Folder = " & strFolder & vbCrLf & "strName = " & strName & vbCrLf & pFields.FieldCount & " fields..." & vbCrLf & _
'        "File already exists? " & CBool(FileExists(strFullName)) & vbCrLf & strString
  
826:   Set CreatedBASETable = pFWS.CreateTable(strName, pFields, Nothing, Nothing, "")


  Exit Function
ErrorHandler:
  HandleError True, "CreatedBASETable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function CreateShapefile(sPath As String, sName As String, pSpatialReference As ISpatialReference, strShapeType As String) As IFeatureClass   ' Don't include filename!
  On Error GoTo ErrorHandler

  
839:   If Right(sPath, 4) = ".shp" Then sPath = ReturnDir(sPath)
840:   If Right(sName, 4) = ".shp" Then sName = Left(sName, Len(sName) - 4)
  
  ' SET GEOMETRY TYPE, AND EXIT IF NOT ONE OF STANDARD OPTIONS
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
845:   Set pGeomDef = New GeometryDef
846:   Set pGeomDefEdit = pGeomDef
847:   With pGeomDefEdit
    Select Case strShapeType
      Case "Polygon", "polygon"
850:         .GeometryType = esriGeometryPolygon
      Case "Polyline", "polyline"
852:         .GeometryType = esriGeometryPolyline
      Case "Point", "point"
854:         .GeometryType = esriGeometryPoint
      Case "Multipoint", "multipoint", "MultiPoint"
856:         .GeometryType = esriGeometryMultipoint
      Case "Multipatch", "multipatch", "MultiPatch"
858:         .GeometryType = esriGeometryMultiPatch
      Case Else
860:         MsgBox "Invalid Shape Type [" & strShapeType & "]!  This function is only written to generate " & _
            "Point, Polyline, Polygon, Multipoint or Multipatch shapefiles...", vbCritical, "Invalid Shape Type:"
862:     End Select
'    Set .SpatialReference = New UnknownCoordinateSystem
864:     Set .SpatialReference = pSpatialReference
865:   End With
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
870:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  
  ' CHECK IF FOLDER EXISTS
'  If Not pWorkspaceFactory.IsWorkspace(sPath) Then
874:   If Not ExistFileDir(sPath) Then
875:     MsgBox "Unable to create Feature Class:" & vbCrLf & _
           sPath & " is not a valid workspace...", , "Failed to Create Feature Class:"
877:     Set CreateShapefile = Nothing
    Exit Function
879:   End If
  
881:   Set pFWS = pWorkspaceFactory.OpenFromFile(sPath, 0)
  
  ' Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
886:   Set pFields = New Fields
887:   Set pFieldsEdit = pFields
  
  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
894:   Set pField = New Field
895:   Set pFieldEdit = pField
896:   pFieldEdit.Name = "Shape"
897:   pFieldEdit.Type = esriFieldTypeGeometry
  
899:   Set pFieldEdit.GeometryDef = pGeomDef
900:   pFieldsEdit.AddField pField

  ' Add an ID field
903:   Set pField = New Field
904:   Set pFieldEdit = pField
905:   With pFieldEdit
906:       .length = 8
907:       .Name = "Unique_ID"
908:       .Type = esriFieldTypeInteger
909:       .Precision = 0
910:   End With
911:   pFieldsEdit.AddField pField
  
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim booFileExists As Boolean
  Dim strCheckString As String
917:   If Right(sPath, 1) = "\" Then
918:     strCheckString = sPath & sName & ".shp"
'    MsgBox sPath & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & sName & ".shp") <> "")
920:   Else
921:     strCheckString = sPath & "\" & sName & ".shp"
'    MsgBox sPath & "\" & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & "\" & sName & ".shp") <> "")
923:   End If
  
925:   booFileExists = (Dir(strCheckString) <> "")
  
927:   If booFileExists Then
928:     MsgBox "The following file already exists:" & vbCrLf & vbCrLf & strCheckString & vbCrLf & vbCrLf & _
           "Please select a new filename...", , "Bailing out of 'CreateShapefile':"
930:     Set CreateShapefile = Nothing
    Exit Function
932:   End If
  
  Dim pFeatClass As IFeatureClass
935:   Set pFeatClass = pFWS.CreateFeatureClass(sName, pFields, Nothing, _
                                           Nothing, esriFTSimple, "Shape", "")
                                           
938:   Set CreateShapefile = pFeatClass


  Exit Function
ErrorHandler:
  HandleError True, "CreateShapefile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function




Public Function TempPathLocation() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
954:   sBuffer = Space(strMAXPATH)
955:   If GetTempPath(strMAXPATH, sBuffer) <> 0 Then
956:     TempPathLocation = Left$(sBuffer, _
      InStr(sBuffer, vbNullChar) - 1)
958:   Else
959:     TempPathLocation = ""
960:   End If


  Exit Function
ErrorHandler:
  HandleError True, "TempPathLocation " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function GetTheUserName() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
  Dim sUName As String
  Dim lSize As Long
976:   sBuffer = Space$(255)
977:   lSize = Len(sBuffer)
978:   Call GetUserName(sBuffer, lSize)
979:   If lSize > 0 Then
980:     sUName = Left$(sBuffer, lSize)
981:   Else
982:     sUName = vbNullString
983:   End If
984:   GetTheUserName = BasicTrimAvenue(sUName, "", Chr(0))     ' NEED TO PEEL OFF THAT LAST ODD CHARACTER


  Exit Function
ErrorHandler:
  ' HandleError True, "GetTheUserName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
990:   GetTheUserName = "Unknown User"
  
End Function

Public Function BasicTrimAvenue(aString As String, aTrimLeft As String, aTrimRight As String) As String
  On Error GoTo ErrorHandler

  
998:   Do While (aString <> "") And (InStr(1, aTrimRight, Right(aString, 1), vbTextCompare) > 0)
999:     aString = Left(aString, Len(aString) - 1)
1000:   Loop
1001:   Do While (aString <> "") And (InStr(1, aTrimLeft, Left(aString, 1), vbTextCompare) > 0)
1002:     aString = Right(aString, Len(aString) - 1)
1003:   Loop
  
1005:   BasicTrimAvenue = aString


  Exit Function
ErrorHandler:
  HandleError True, "BasicTrimAvenue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function InsertCommas(InputValue As Variant) As String
  On Error GoTo ErrorHandler

  
  Dim theString As String
1018:   theString = CStr(InputValue)
  
  Dim theDecLocation As Long
1021:   theDecLocation = InStr(theString, ".")
  
  Dim HasDecimal As Boolean
1024:   HasDecimal = theDecLocation > 0
  
  Dim theLength As Long
1027:   theLength = Len(theString)
  
  Dim theBaseNumber As String
  Dim theRemainder As String
  
1032:   If HasDecimal Then
1033:     theRemainder = Right(theString, theLength - theDecLocation)
1034:     theBaseNumber = Left(theString, theDecLocation - 1)
1035:   Else
1036:     theRemainder = ""
1037:     theBaseNumber = theString
1038:   End If
  
  Dim theCount As Long
1041:   theCount = Len(theBaseNumber)
  
  Dim theCommaString As String
  
1045:   If theCount > 3 Then
    Dim anIndex As Long
1047:     For anIndex = (theCount - 2) To 1 Step -3
1048:       theCommaString = Mid(theBaseNumber, anIndex, 3) & "," & theCommaString
1049:       If anIndex < 4 Then
1050:         theCommaString = Left(theBaseNumber, anIndex - 1) & "," & theCommaString
1051:       End If
1052:     Next anIndex
    
1054:     Do While Right(theCommaString, 1) = ","
1055:       theCommaString = Left(theCommaString, Len(theCommaString) - 1)
1056:     Loop
1057:     Do While Left(theCommaString, 1) = ","
1058:       theCommaString = Right(theCommaString, Len(theCommaString) - 1)
1059:     Loop
1060:   Else
1061:     theCommaString = theBaseNumber
1062:   End If
  
1064:   If HasDecimal Then
1065:     theCommaString = theCommaString & "." & theRemainder
1066:   End If
  
1068:   InsertCommas = theCommaString


  Exit Function
ErrorHandler:
  HandleError True, "InsertCommas " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ClipExtension(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1083:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1084:   strDirPath = strDirTokens(0)
  
1086:   If (UBound(strDirTokens) = 0) Then
1087:     ClipExtension = strDirPath
1088:   Else
    Dim anIndex As Long
1090:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1091:       strDirPath = strDirPath & "." & strDirTokens(anIndex)
1092:     Next anIndex
1093:     ClipExtension = strDirPath
1094:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ClipExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ClipExtension2(strPathName As String) As String
  On Error GoTo ErrorHandler
  
  ' REMOVES ALL TEXT AFTER THE ".", ONLY IF THAT TEXT IS AFTER THE LAST "\" OR "/".
  ' IF GIVEN "C:\Temp\File.txt", will return "C:\Temp\File
  ' IF GIVEN "C:\Temp\File.gdb\FeatureClassName", will return "C:\Temp\File.gdb\FeatureClassName"
  
  Dim lngLastDot As Long
  Dim lngLastSlash As Long
  Dim lngLastForwardSlash As Long
  
1108:   lngLastDot = InStrRev(strPathName, ".")
1109:   lngLastSlash = InStrRev(strPathName, "\")
1110:   lngLastForwardSlash = InStrRev(strPathName, "/")
  
  Dim strSplit() As String
  Dim strFinalPath As String
  Dim lngIndex As Long
  
1116:   If lngLastDot > 0 And lngLastDot > lngLastSlash And lngLastDot > lngLastForwardSlash Then
1117:     strSplit = Split(strPathName, ".")
1118:     strFinalPath = strSplit(0)
1119:     For lngIndex = 1 To UBound(strSplit) - 1
1120:       strFinalPath = strFinalPath & "." & strSplit(lngIndex)
1121:     Next lngIndex
1122:     ClipExtension2 = strFinalPath
1123:   Else
1124:     ClipExtension2 = strPathName
1125:   End If
  
1127:   Erase strSplit

  Exit Function
ErrorHandler:
  HandleError True, "ClipExtension2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsNumeric(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1138:   theFieldType = pTheField.Type
  
1140:   FieldIsNumeric = _
    (theFieldType = esriFieldTypeSmallInteger) Or (theFieldType = esriFieldTypeDouble) Or (theFieldType = esriFieldTypeInteger) Or _
          (theFieldType = esriFieldTypeSingle)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsNumeric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsString(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1154:   theFieldType = pTheField.Type
  
1156:   FieldIsString = (theFieldType = esriFieldTypeString)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsDate(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1168:   theFieldType = pTheField.Type
  
1170:   FieldIsDate = (theFieldType = esriFieldTypeDate)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsDate " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsShape(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1182:   theFieldType = pTheField.Type
  
1184:   FieldIsShape = (theFieldType = esriFieldTypeGeometry)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsShape " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SetExtension(strPathName As String, strExtension As String) As String
  On Error GoTo ErrorHandler

  
  Dim theClippedPath As String
1197:   SetExtension = ClipExtension(strPathName) & "." & strExtension


  Exit Function
ErrorHandler:
  HandleError True, "SetExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetExtensionText(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1212:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1213:   If UBound(strDirTokens) = 0 Then
1214:     GetExtensionText = ""
1215:   Else
1216:     GetExtensionText = strDirTokens(UBound(strDirTokens))
1217:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetExtensionText " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetFullFileString(str83Type As String) As String
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM BRETT MERONEY'S POST ABOVE
  
  Dim lLen As Long
  Dim sBuffer As String
  
1234:   sBuffer = String$(strMAXPATH, 0)
1235:   lLen = GetLongPathName(str83Type, sBuffer, Len(sBuffer))
1236:   If lLen > 0 And err.Number = 0 Then
1237:     GetFullFileString = Left$(sBuffer, lLen)
1238:   Else
1239:     GetFullFileString = str83Type
1240:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetFullFileString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnDir(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1255:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1256:   strDirPath = strDirTokens(0)
  
1258:   If (UBound(strDirTokens) = 0) Then
1259:     ReturnDir = strDirPath
1260:   Else
    Dim anIndex As Long
1262:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1263:       strDirPath = strDirPath & "\" & strDirTokens(anIndex)
1264:     Next anIndex
1265:     ReturnDir = strDirPath
1266:   End If
  
1268:   ReturnDir = ReturnDir & "\"


  Exit Function
ErrorHandler:
  HandleError True, "ReturnDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnFilename(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1284:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1285:   ReturnFilename = strDirTokens(UBound(strDirTokens))


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Sub ParseString(Str As String, strArray() As String, Delim As String)
  On Error GoTo ErrorHandler


' Populates a named string array with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. NOTE: Use parseStringR if you want both commas and blanks to be
' treated as delimiting characters.
'
' Before calling this Sub you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseString (mystring), myarray, ","
' Returns:
' array(0) = ARC YES
' array(1) = POLY NO
' array(2) = TICS YES

' parseString(mystring),myarray," "
' Returns:
' array(0) = ARC
' array(1) = YES,
' array(2) = POLY
' array(3) = NO,
' array(4) = TICS
' array(5) = YES,

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"
' parsestring(mystring),myarray,","
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim tmpstr As String
Dim position As Long
Dim length As Long

'Dim variables to keep track of embedded quotes

Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long

On Error Resume Next

' If string contains no elements raise error
1346:   If Trim(Subst(Str, Delim)) = "" Then
1347:     err.Raise vbObjectError + 1, "aml_func.ParseString", _
    "StringPassed"
    Exit Sub
1350:   End If

' intialize array. Warning: This will overwrite any data elements currently
' stored in this named array

  ReDim strArray(0)

'intializer counters and tracking variables

1359:   pair = False
1360:   switch = 0
1361:   length = Len(Str)
1362:   position = 1
1363:   i = 0
1364:   tmpstr = Str

'check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = read second single quote

1371:   Do While position < length
1372:     If Mid(tmpstr, position, 1) = "'" Then
1373:       If Not (switch = 1) Then
1374:         switch = 1
1375:         position1 = position
1376:         pair = False
1377:       Else
1378:         switch = 2
1379:         position2 = position
1380:         pair = True
1381:       End If
1382:     End If

' if last char read was last in a pair of quotes, store contents between first and last
' in current array element and reset tracking variables

1387:     If pair = True Then
1388:       Mid(tmpstr, position1, 1) = " "
1389:       Mid(tmpstr, position2, 1) = " "
1390:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1391:       strArray(i) = Trim(strArray(i))
1392:       pair = False
1393:       switch = 0
      
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks

1400:     Else
1401:       If switch = 0 Then
1402:         If Mid(tmpstr, position, 1) = Delim Then
1403:           strArray(i) = Left(tmpstr, position)
1404:           tokenlen = Len(strArray(i))
1405:           strArray(i) = Trim(strArray(i))
1406:           If Not (Len(strArray(i)) = 0) Then
1407:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1409:             i = i + 1
1410:           End If
1411:         End If
1412:       End If
1413:     End If
1414:     position = position + 1
1415:   Loop
1416:   strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element

1420:   position = 1
1421:   For i = 0 To UBound(strArray)
1422:     position = Len(strArray(i))
1423:     If Mid(strArray(i), position, 1) = Delim Then
1424:       Mid(strArray(i), position, 1) = " "
1425:       strArray(i) = Trim(strArray(i))
1426:     End If
1427:   Next i


  Exit Sub
ErrorHandler:
  HandleError True, "ParseString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function After(Str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the right of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1444:   position = InStr(Str, SearchStr)
1445:   length = Len(SearchStr)
1446:   If Not (position = 0) Then
1447:    After = Mid(Str, position + length)
1448:   End If


  Exit Function
ErrorHandler:
  HandleError True, "After " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Before(Str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the left of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1465:   position = InStr(Str, SearchStr)
1466:   length = Len(SearchStr)
1467:   If Not (position = 0) Then
1468:    Before = Mid(Str, 1, position - 1)
1469:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Before " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Function ExistFileDir(sTest As String) As Boolean
  On Error GoTo ErrorHandler


'Checks for the existance of a File or Directory
  Dim af As Long
1483:   af = -1
  On Error Resume Next
1485:   af = GetAttr(sTest)
1486:   ExistFileDir = (af <> -1)


  Exit Function
ErrorHandler:
  HandleError True, "ExistFileDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function MakeUniqueFilename(strFileName As String) As String
  On Error GoTo ErrorHandler


1500:   If Not ExistFileDir(strFileName) Then
1501:     MakeUniqueFilename = strFileName
    Exit Function
1503:   Else
    
    Dim theCounter As Long
1506:     theCounter = 1
    
    Dim theFilename As String
    Dim theBaseName As String
    Dim thePointPos As Long
    Dim theExtension As String
    
1513:     If InStr(1, Right(strFileName, 5), ".", vbTextCompare) > 0 Then
1514:       thePointPos = InStrRev(strFileName, ".", -1, vbTextCompare)
1515:       theExtension = Right(strFileName, Len(strFileName) - (thePointPos - 1))
1516:       theFilename = Left(strFileName, thePointPos - 1)
1517:     Else
1518:       theExtension = ""
1519:       theFilename = strFileName
1520:     End If
    
1522:     theBaseName = theFilename
    
1524:     Do While ExistFileDir(theFilename & theExtension)
1525:       theCounter = theCounter + 1
1526:       theFilename = theBaseName & "_" & CStr(theCounter)
1527:     Loop
    
1529:     MakeUniqueFilename = theFilename & theExtension
    
1531:   End If


  Exit Function
ErrorHandler:
  HandleError True, "MakeUniqueFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Extract(ElemNum As Long, ElemList As String) As String
  On Error GoTo ErrorHandler


' extracts an element from a list of elements

Dim strArray() As String

1547:   ParseStringR (ElemList), strArray
1548:   If ElemNum > UBound(strArray) + 1 Then
   Exit Function
1550:   End If
  
1552:   If ElemNum = 0 Then
   Exit Function
1554:   End If
1555:   Extract = strArray(ElemNum - 1)


  Exit Function
ErrorHandler:
  HandleError True, "Extract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Index(Str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of the leftmost occurrence of searcStr in str.

1569:   Index = InStr(Str, SearchStr)


  Exit Function
ErrorHandler:
  HandleError True, "Index " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ParseStringR(Str As String, strArray() As String, Optional ReturnQuoted)
  On Error GoTo ErrorHandler


' Populates a named string arrary with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. Treats both blanks and commas as delimters NOTE: Use parseString
' to specify a specific delimiting character.

' ReturnQuoted - indicates if elements are to be returned quoted.
' FALSE - DEFAULT return elements unquoted
' TRUE - return elements quoted

' Before calling this function you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseStringR(mystring),myarray

' Returns:
' array(0) = ARC
' array(1) = YES
' array(2) = POLY
' array(3) = NO
' array(4) = TICS
' array(5) = YES

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"

' parsestringR(mystring),myarray
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' parsestring(mystring,myarray,TRUE)
' Returns:
' array(0) = 'Universe,Medium'
' array(1) = 'Helvetica,Bold'
' array(2) = 'Times,Medium'

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long
Dim parseAgain As Boolean
Dim tmpstr As String
Dim position As Long
Dim length As Long

On Error Resume Next

' If string contains no elements raise error
1635:   If Trim(Subst(Str, ",")) = "" Then
1636:     err.Raise vbObjectError + 1, "aml_func.ParseStringR", _
    "StringPassed"
    Exit Sub
1639:   End If

' intialize counters and tracking variables
  ReDim strArray(0)
1643:   pair = False
1644:   switch = 0
1645:   length = Len(Str)
1646:   position = 1
1647:   i = 0
1648:   tmpstr = Str

1650:   If IsMissing(ReturnQuoted) Then
1651:     ReturnQuoted = False
1652:   End If
1653:   If Not (ReturnQuoted = False) Then
1654:     ReturnQuoted = True
1655:   End If

' check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = just read second single quote

1662:   Do While position < length
1663:     If Mid(tmpstr, position, 1) = "'" Then
1664:       If Not (switch = 1) Then
1665:       switch = 1
1666:       position1 = position
1667:       pair = False
1668:     Else
1669:       switch = 2
1670:       position2 = position
1671:       pair = True
1672:       End If
1673:     End If
     
' if last char read was last in a pair of single quotes, store contents between first
' and last in current array element and reset tracking variables
 
1678:     If pair = True Then
1679:       Mid(tmpstr, position1, 1) = " "
1680:       Mid(tmpstr, position2, 1) = " "
1681:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1682:       strArray(i) = Trim(strArray(i))
1683:       pair = False
1684:       switch = 0
    
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks
  
1691:     Else
1692:       If switch = 0 Then
1693:         If Mid(tmpstr, position, 1) = "," Or Mid(tmpstr, position, 1) = " " Then
1694:           strArray(i) = Left(tmpstr, position)
1695:           tokenlen = Len(strArray(i))
1696:           strArray(i) = Trim(strArray(i))
1697:           If Not (Len(strArray(i)) = 0) Then
1698:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1700:             i = i + 1
1701:           End If
1702:         End If
1703:       End If
1704:     End If
1705:     position = position + 1
1706:   Loop
1707:     strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element
' set parseAgain flag if there are any blank elements

1712:   parseAgain = False
1713:   position = 1

1715:   For i = 0 To UBound(strArray)
1716:     position = Len(strArray(i))
1717:     If Mid(strArray(i), position, 1) = "," Then
1718:       Mid(strArray(i), position, 1) = " "
1719:       strArray(i) = Trim(strArray(i))
1720:       If Len(strArray(i)) = 0 Then
1721:         parseAgain = True
1722:       End If
1723:     End If
1724:   Next i
  
' now remove any blank elements
  
1728:   If parseAgain = True Then
1729:     tmpstr = ""
1730:     For i = 0 To UBound(strArray)
1731:       If Not (Len(strArray(i))) = 0 Then
1732:         tmpstr = tmpstr & "'" & strArray(i) & "'" & ","
1733:       End If
1734:     Next i
1735:     ParseString (tmpstr), strArray, ","
1736:   End If
      
1738:   If ReturnQuoted = True Then
1739:     For i = 0 To UBound(strArray)
1740:      strArray(i) = "'" & strArray(i) & "'"
1741:     Next i
1742:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "ParseStringR " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function Keyword(Str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of a string within a list of keywords.
' converts Str and searchStr to upper case before comparing
'  0  if keyword not found
' -1  if string is ambiguous - mutiple occurances of same keyword
'  n  position of keyword in string

Dim strArray() As String
Dim i As Long
Dim keywordCnt As Long

1764:   ParseStringR (Str), strArray
1765:   keywordCnt = 0
1766:     For i = 0 To UBound(strArray)
1767:     If UCase(SearchStr) = UCase(strArray(i)) Then
1768:       Keyword = i + 1
1769:       keywordCnt = keywordCnt + 1
1770:     End If
1771:     Next i
  
1773:   If keywordCnt > 1 Then
1774:     Keyword = -1
1775:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Keyword " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Search(Str, SearchStr) As Long
  On Error GoTo ErrorHandler


' Returns the position of the first character in Str
' which occurs in searchStr.

Dim strArray() As String
Dim i As Long
Dim Index As Long
Dim firstchar
Dim InString As Boolean

1796:   Index = 1
  ReDim strArray(Len(Str))

1799:   For i = 0 To Len(SearchStr) - 1
1800:     firstchar = Mid(SearchStr, i + 1, 1)
1801:     Index = InStr(Str, firstchar)
1802:     strArray(Index) = i + 1
1803:   Next i
  
1805:   InString = False
  
1807:   For i = 1 To UBound(strArray)
1808:     If Not (strArray(i)) = "" Then
1809:     InString = True
1810:     Exit For
1811:     Else
1812:     End If
1813:   Next i
  
1815:   If InString = False Then
1816:     Search = 0
1817:   Else
1818:     Search = i
1819:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Search " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Sort(Str As String, Optional SortOption, Optional SortType) As String
  On Error GoTo ErrorHandler


' Returns a string of sorted elements

1833:   If IsMissing(SortOption) Then
1834:     SortOption = "-ASCEND"
1835:   ElseIf Not (UCase(SortOption) = "-DESCEND") Then
1836:     SortOption = "-ASCEND"
1837:   End If
  
1839:   If IsMissing(SortType) Then
1840:     SortType = "-CHARACTER"
1841:   ElseIf Not (UCase(SortType) = "-NUMERIC") Then
1842:     SortType = "-CHARACTER"
1843:   End If
  
1845:   If (UCase(SortType)) = "-NUMERIC" Then
1846:     Call Sort_Num(Str, SortOption)
1847:   Else
1848:     Call Sort_Char(Str, SortOption, True)
1849:   End If
1850:     Sort = Str


  Exit Function
ErrorHandler:
  HandleError True, "Sort " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Num(Str As String, SortOption) As String
  On Error GoTo ErrorHandler


' Sort function - performs a numerical sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim j As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

1874:   ParseStringR (Str), sortArray
  
1876:   min = LBound(sortArray)
1877:   max = UBound(sortArray)
  
1879:   For i = min To max - 1
1880:     best_value = sortArray(i)
1881:     best_j = i
    
1883:     For j = i + 1 To max
1884:       If Val(sortArray(j)) < Val(best_value) Then
1885:         best_value = sortArray(j)
1886:         best_j = j
1887:       End If
1888:     Next j
      
1890:     sortArray(best_j) = sortArray(i)
1891:     sortArray(i) = best_value
1892:   Next i
    
1894:   If UCase(SortOption) = "-DESCEND" Then
1895:     For i = max To min Step -1
1896:       sorted = sorted & sortArray(i) & ","
1897:     Next i
1898:   Else
1899:     For i = min To max
1900:       sorted = sorted & sortArray(i) & ","
1901:     Next i
1902:   End If
  
1904:   Mid(sorted, Len(sorted), 1) = " "
1905:   Str = sorted
1906:   Sort_Num = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Num " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Char(Str As String, SortOption, Optional ReturnQuoted) As String
  On Error GoTo ErrorHandler


' Sort function - performs a character sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim j As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

1930: If IsMissing(ReturnQuoted) Then
1931:   ReturnQuoted = False
1932: End If
1933: If Not (ReturnQuoted = False) Then
1934:   ReturnQuoted = True
1935: End If

1937: ParseStringR (Str), sortArray, ReturnQuoted

1939: min = LBound(sortArray)
1940: max = UBound(sortArray)

1942:   For i = min To max - 1
1943:     best_value = sortArray(i)
1944:     best_j = i
    
1946:     For j = i + 1 To max
1947:       If sortArray(j) < best_value Then
1948:       best_value = sortArray(j)
1949:       best_j = j
1950:       End If
1951:     Next j
    
1953:     sortArray(best_j) = sortArray(i)
1954:     sortArray(i) = best_value
1955:   Next i
  
1957:   If UCase(SortOption) = "-DESCEND" Then
1958:     For i = max To min Step -1
1959:       sorted = sorted & sortArray(i) & " "
1960:     Next i
1961:   Else
1962:     For i = min To max
1963:       sorted = sorted & sortArray(i) & " "
1964:     Next i
1965:   End If
1966:   Mid(sorted, Len(sorted), 1) = " "
  
1968:   Str = sorted
1969:   Sort_Char = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Char " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Subst(Str As String, SearchChar As String, Optional ReplaceChar) As String
  On Error GoTo ErrorHandler


' Replaces all occurances of specified char in string.

Dim complete As Boolean
Dim i As Long
Dim first As String
Dim last As String
Dim tmpstr As String
Dim position As Long

1990: tmpstr = Str

1992: position = 1
1993: complete = False

1995: If IsMissing(ReplaceChar) Then
1996:   Do Until complete = True
1997:     position = InStr(position, tmpstr, SearchChar)
1998:     If position = 0 Or position > Len(tmpstr) Then
1999:       complete = True
2000:     Else
2001:       first = Before(tmpstr, SearchChar)
2002:       last = After(tmpstr, SearchChar)
2003:       tmpstr = first & last
2004:     End If
2005:   Loop
2006: End If

2008: Do Until complete = True
2009:   position = InStr(position, tmpstr, SearchChar)
2010:   If position = 0 Or position > Len(tmpstr) Then
2011:     complete = True
2012:   Else
2013:     Mid(tmpstr, position, Len(ReplaceChar)) = ReplaceChar
2014:     position = position + Len(ReplaceChar)
2015:   End If
2016: Loop

Subst = tmpstr


  Exit Function
ErrorHandler:
  HandleError True, "Subst " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Substr(Str As String, position As Long, Optional NumChars) As String
  On Error GoTo ErrorHandler


'extracts a substring starting at a specified character position.

2032: If IsMissing(NumChars) Then
2033:   If position = 0 Or position > Len(Str) Then
2034:     Substr = ""
2035:   Else
2036:     Substr = Mid(Str, position)
2037:   End If
2038: Else
2039:   If position = 0 Or position > Len(Str) Then
2040:     Substr = ""
2041:   Else
2042:    Substr = Mid(Str, position, NumChars)
2043:   End If
2044: End If


  Exit Function
ErrorHandler:
  HandleError True, "Substr " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Token(ElemList As String, Arg As String, ParamArray OtherArgs()) As Variant
  On Error GoTo ErrorHandler


' Performs various functions for string manipulation

Dim strArray() As String
Dim i As Long
Dim temp As String
Dim from_elem  As Long
Dim to_elem As Long
Dim start_elem As Long
Dim insertStr As String
Dim DELETE As Long
Dim SearchStr As String

' Parse ElemList out to strarray
' Select TOKEN argument and perform function

2071:   ParseStringR (ElemList), strArray
2072:   Arg = Subst(Arg, "-")
  
  Select Case UCase(Arg)

  ' Count - returns the number of tokens in a list
    Case "COUNT"
2078:       Token = UBound(strArray) + 1
    
  ' Find <token> - returns the position of a token in a list
    Case "FIND"
2082:       SearchStr = OtherArgs(0)
2083:       Token = 0
2084:       For i = 0 To UBound(strArray)
2085:         If UCase(SearchStr) = UCase(strArray(i)) Then
2086:           Token = i + 1
2087:         End If
2088:       Next i
    
  ' Move <from_position> <to_position> - moves a token in the list
    Case "MOVE"
2092:       from_elem = OtherArgs(0) - 1
2093:       to_elem = OtherArgs(1)
2094:       temp = strArray(to_elem)
2095:       strArray(to_elem) = strArray(from_elem)
2096:       For i = from_elem To to_elem - 1
2097:         strArray(i) = strArray(i + 1)
2098:         Next i
2099:          strArray(i) = temp
2100:       For i = 0 To UBound(strArray) - 1
2101:         Token = Token & strArray(i) & ","
2102:       Next i
    
  ' Insert <position> - inserts a new token at <position> in the list
    Case "INSERT"
      ReDim Preserve strArray(UBound(strArray) + 1)
2107:       start_elem = OtherArgs(0) - 1
2108:       insertStr = OtherArgs(1)
2109:       For i = UBound(strArray) To start_elem Step -1
2110:         strArray(i) = strArray(i - 1)
2111:         Next i
2112:         strArray(start_elem) = insertStr
2113:         For i = 0 To UBound(strArray) - 1
2114:         Token = Token & strArray(i) & ","
2115:       Next i
    
  ' Delete <position> - removes the token at <position> from the list.
    Case "DELETE"
2119:       DELETE = OtherArgs(0) - 1
2120:       For i = DELETE To UBound(strArray) - 1
2121:         strArray(i) = strArray(i + 1)
2122:       Next i
2123:       For i = 0 To UBound(strArray) - 1
2124:         Token = Token & strArray(i) & ","
2125:       Next i
    
  ' Replace <position> <new_string> - replaces the token at <position> with the
  ' <new_string>.
    Case "REPLACE"
2130:       strArray(OtherArgs(1) - 1) = OtherArgs(0)
2131:       For i = 0 To UBound(strArray)
2132:         Token = Token & strArray(i) & ","
2133:       Next i
    
  ' Switch <position_1> <position_2> - moves token at <position_1> to <position_2> and moves
  ' token at <position_2> to <position_1>.
    Case "SWITCH"
2138:       from_elem = OtherArgs(0) - 1
2139:       to_elem = OtherArgs(1) - 1
2140:       temp = strArray(to_elem)
2141:       strArray(to_elem) = strArray(from_elem)
2142:       strArray(from_elem) = temp
2143:       For i = 0 To UBound(strArray)
2144:         Token = Token & strArray(i) & ","
2145:       Next i
    
    Case Else
2148:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "Token " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function IsVersion10() As Boolean
  On Error GoTo ErrorHandler


  Dim strVersion As String
2160:   strVersion = ReturnArcGISVersion()
  
2162:   If InStr(1, strVersion, ".") Then
    Dim strTokens() As String
2164:     strTokens = Split(strVersion, ".")
2165:     strVersion = strTokens(0)
2166:   End If
  
2168:   strVersion = Trim(strVersion)
2169:   IsVersion10 = CBool(strVersion = "10")


  Exit Function
ErrorHandler:
  HandleError True, "IsVersion10 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISVersionAlt2(pMxDoc As IMxDocument, Optional strDecimalVersion As String) As Long

 '  ReturnArcGISVersionAlt = "Unable to determine ArcGIS Version!"
   ' 0 = 8.3, 1 = 9.0, 2 = 9.2, 3 = 9.3
   
   Dim pDocVersion As IDocumentVersion
2183:    Set pDocVersion = pMxDoc
   
2185:    ReturnArcGISVersionAlt2 = CLng(pDocVersion.DocumentVersion)
  
  Select Case ReturnArcGISVersionAlt2
    Case 0
2189:       strDecimalVersion = "8.3"
    Case 1
2191:       strDecimalVersion = "9.0"
    Case 2
2193:       strDecimalVersion = "9.2"
    Case 3
2195:       strDecimalVersion = "9.3"
    Case 4
2197:       strDecimalVersion = "10.0"
    Case 5
2199:       strDecimalVersion = "10.1"
    Case 6
2201:       strDecimalVersion = "10.2"
    Case Else
2203:       strDecimalVersion = "Unable to determine ArcGIS Version!"
2204:   End Select
  
End Function
