Attribute VB_Name = "BatchFunctions"
Option Explicit


Const c_sModuleFileName As String = "...\BatchFunctions.bas"

Public Function RunBottleneckInBatch(pStartPoly As IPolygon4, pEndPoly As IPolygon4, pCorPolygon As IPolygon4, _
    pMxDoc As IMxDocument, pApp As IApplication, pProgressMeterForm As Object, _
    pThisExt As Linkages.clsExtension, pSegWorkOrder As esriSystem.IVariantArray, _
    pPointWorkOrder As esriSystem.IVariantArray, pStatsWorkOrder As esriSystem.IVariantArray, _
    dblThresholdDistance As Double, pBlock1FClass As IFeatureClass, pBlock2FClass As IFeatureClass, _
    pCorridorFClass As IFeatureClass) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler
  
  Dim strOutputReport As String
15:   strOutputReport = "RunBottleneckInBatch Failed, reason unknown..."
  
  Dim pSpRef As ISpatialReference
18:   Set pSpRef = pCorPolygon.SpatialReference
  
  Dim pTopoOp As ITopologicalOperator
21:   Set pTopoOp = pCorPolygon
  Dim pRelOp As IRelationalOperator
23:   Set pRelOp = pCorPolygon
  
  Dim strMessage As String
  Dim lngCounter As Long
  Dim lngCount As Long
28:   lngCounter = 0
29:   lngCount = 18
  
31:   If Not pSegWorkOrder Is Nothing Then
32:     lngCount = lngCount + 1
33:   End If
34:   If Not pPointWorkOrder Is Nothing Then
35:     lngCount = lngCount + 1
36:   End If
37:   If Not pStatsWorkOrder Is Nothing Then
38:     lngCount = lngCount + 1
39:   End If
  
  Dim theDetailedDescription As String
'  theDetailedDescription = pProgressForm.txtDetails.Text
  
  Dim pProgressForm As Linkages.frmJenProgressMeter
45:   Set pProgressForm = pProgressMeterForm
46:   pProgressForm.ResetShortTimer
  
  ' #1
49:   lngCounter = lngCounter + 1
50:   strMessage = "Preparing habitat and corridor polygons (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
53:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
56:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
57:   pProgressForm.Refresh
58:   pProgressForm.icnProgressLine.Refresh
59:   pProgressForm.icnProgressLineShort.Refresh
60:   DoEvents
61:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
62:     strOutputReport = "RunBottleneckInBatch process terminated:"
63:     Set RunBottleneckInBatch = New esriSystem.VarArray
64:     RunBottleneckInBatch.Add False
65:     RunBottleneckInBatch.Add strOutputReport
66:     RunBottleneckInBatch.Add Nothing
67:     GoTo ClearMemory
68:   End If
  ' ---- END PROGRESS METER STUFF ---------------
  
  ' MAKE SURE ALL POLYGONS ARE IN CORRIDOR COORDINATE SYSTEM
72:   If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pStartPoly.SpatialReference, pSpRef) Then
73:     pStartPoly.Project pSpRef
74:   End If
75:   If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pEndPoly.SpatialReference, pSpRef) Then
76:     pStartPoly.Project pSpRef
77:   End If
  
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim strUnits As String
  Dim booIsGeographic As Boolean
82:   booIsGeographic = False
83:   If TypeOf pSpRef Is IGeographicCoordinateSystem Then
84:     strUnits = "Meters"
85:   ElseIf TypeOf pSpRef Is IUnknownCoordinateSystem Then
86:     strUnits = "Unknown units"
87:   Else
88:     Set pPrjSpRef = pSpRef
89:     strUnits = pPrjSpRef.CoordinateUnit.Name
90:   End If
  
  ' CREATE FINAL START POLYGON REPRESENTING ONLY OUTERMOST RINGS OF ALL SUB-PARTS OF START POLYGON THAT
  ' INTERSECT CORRIDOR POLYGON
  ' GET OUTERMOST COMPONENTS OF START POLYGON THAT INTERSECT CORRIDOR
  Dim pPolyOutStart() As IPolygon 'Declare an array of polygon
  Dim lngNumParts As Long
  Dim lngIndex As Long
98:   lngNumParts = pStartPoly.ExteriorRingCount
  ReDim pPolyOutStart(lngNumParts - 1) 'Redimension the array of polygon with number of exterior ring
100:   pStartPoly.GetConnectedComponents lngNumParts, pPolyOutStart(0) 'Pass the first element of the array
  
  ' GET OUTERMOST RING OF EACH POLYGON IN ARRAY
  Dim pClone As IClone
  Dim pTempStartA As IPolygon4
  Dim pRing() As IRing
  Dim pOuterRing As IRing
  Dim pPath As ISegmentCollection
  Dim pTopoForSimplify As ITopologicalOperator
  Dim pFinalStartPolygon As IPolygon4
110:   Set pFinalStartPolygon = New Polygon
111:   Set pFinalStartPolygon.SpatialReference = pSpRef
  Dim pOutputPolygon As IGeometryCollection
113:   Set pOutputPolygon = pFinalStartPolygon
  Dim pFinalStartLine As IPolyline
115:   Set pFinalStartLine = New Polyline
116:   Set pFinalStartLine.SpatialReference = pSpRef
  Dim pOutputPolyline As IGeometryCollection
118:   Set pOutputPolyline = pFinalStartLine
119:   For lngIndex = 0 To lngNumParts - 1
120:     Set pTempStartA = pPolyOutStart(lngIndex)
121:     Set pTopoForSimplify = pTempStartA
122:     pTopoForSimplify.Simplify
123:     If Not pRelOp.Disjoint(pTempStartA) Then  ' ONLY PICK SUB-POLYGONS THAT INTERSECT CORRIDOR
      ReDim pRing(0)
125:       pTempStartA.QueryExteriorRingsEx 1, pRing(0)   ' ONLY GET OUTERMOST RING OF POLYGON
126:       Set pOuterRing = pRing(0)
127:       Set pClone = pOuterRing
128:       pOutputPolygon.AddGeometry pOuterRing
129:       Set pPath = New Path
130:       pPath.AddSegmentCollection pClone.Clone
131:       pOutputPolyline.AddGeometry pPath
132:     End If
133:   Next
134:   Set pTopoForSimplify = pFinalStartPolygon
135:   pTopoForSimplify.Simplify
136:   Set pTopoForSimplify = pOutputPolyline
137:   pTopoForSimplify.Simplify
  
  Dim pClipStart As IPolygon
140:   Set pClipStart = pTopoOp.Intersect(pFinalStartPolygon, esriGeometry2Dimension)
  'Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pFinalStartPolygon, "delete_corridors"
  
  ' CREATE FINAL END POLYGON REPRESENTING ONLY OUTERMOST RINGS OF ALL SUB-PARTS OF END POLYGON THAT
  ' INTERSECT CORRIDOR POLYGON
  Dim pPolyOutEnd() As IPolygon 'Declare an array of polygon
146:   lngNumParts = pEndPoly.ExteriorRingCount
  ReDim pPolyOutEnd(lngNumParts - 1) 'Redimension the array of polygon with number of exterior ring
148:   pEndPoly.GetConnectedComponents lngNumParts, pPolyOutEnd(0) 'Pass the first element of the array
  
  ' GET OUTERMOST RING OF EACH POLYGON IN ARRAY
  Dim pTempEndA As IPolygon4
  Dim pFinalEndPolygon As IPolygon4
153:   Set pFinalEndPolygon = New Polygon
  Dim pOutputEndPolygon As IGeometryCollection
155:   Set pOutputEndPolygon = pFinalEndPolygon
  Dim pFinalEndPolyline As IPolyline
157:   Set pFinalEndPolyline = New Polyline
  Dim pOutputEndPolyline As IGeometryCollection
159:   Set pOutputEndPolyline = pFinalEndPolyline
160:   For lngIndex = 0 To lngNumParts - 1
161:     Set pTempEndA = pPolyOutEnd(lngIndex)
162:     Set pTopoForSimplify = pTempEndA
163:     pTopoForSimplify.Simplify
164:     If Not pRelOp.Disjoint(pTempEndA) Then  ' ONLY PICK SUB-POLYGONS THAT INTERSECT CORRIDOR
      ReDim pRing(0)
166:       pTempEndA.QueryExteriorRingsEx 1, pRing(0)   ' ONLY GET OUTERMOST RING OF POLYGON
167:       Set pOuterRing = pRing(0)
168:       Set pClone = pOuterRing
169:       pOutputEndPolygon.AddGeometry pOuterRing
170:       Set pPath = New Path
171:       pPath.AddSegmentCollection pClone.Clone
172:       pOutputEndPolyline.AddGeometry pPath
173:     End If
174:   Next
175:   Set pTopoForSimplify = pFinalEndPolygon
176:   pTopoForSimplify.Simplify
177:   Set pTopoForSimplify = pOutputEndPolyline
178:   pTopoForSimplify.Simplify

  Dim pClipEnd As IPolygon
181:   Set pClipEnd = pTopoOp.Intersect(pEndPoly, esriGeometry2Dimension)
  
  
  ' GET SET OF POLYLINES, REPRESENTING PORTION OF FINAL START/END POLYGON EDGES THAT LAY WITHIN CORRIDOR
  Dim pStartPolyline As IPolyline
186:   Set pStartPolyline = pTopoOp.Intersect(pOutputPolyline, esriGeometry1Dimension)
  Dim pEndPolyline As IPolyline
188:   Set pEndPolyline = pTopoOp.Intersect(pOutputEndPolyline, esriGeometry1Dimension)
  
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pClipStart, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pClipEnd, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pStartPolyline, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pEndPolyline, "delete_corridors"
  
'  ' MAKE DISTANCE GRID
'  ' FIRST MAKE POLYON, BASED ON BOUNDARY EXTENT WITH BOUNDARY CLIPPED OUT
  
  ' COMBINE CORRIDOR WITH START AND END HABITAT BLOCK SHELLS (NO INTERIOR HOLES, ONLY PORTIONS THAT INTERSECT CORRIDOR
  Dim pUnionPoly As IPolygon4
201:   Set pTopoOp = pCorPolygon
202:   Set pUnionPoly = pTopoOp.Union(pFinalEndPolygon)
203:   Set pTopoOp = pUnionPoly
  Dim pUnionPoly2 As IPolygon4
205:   Set pUnionPoly2 = pTopoOp.Union(pFinalStartPolygon)
206:   Set pTopoOp = pUnionPoly2
  
  ' CREATE AOI POLYGON
  Dim pAOIPoly As IPolygon4
210:   Set pAOIPoly = New Polygon
211:   Set pAOIPoly.SpatialReference = pSpRef
  Dim pIntermediatePoly As IPointCollection
213:   Set pIntermediatePoly = pAOIPoly
  
  ' CREATE AOI ENVELOPE
  Dim pAOIEnv As IEnvelope
217:   Set pAOIEnv = pCorPolygon.Envelope
218:   pAOIEnv.Expand 1.1, 1.1, True
  
  ' CONVERT AOI ENVELOPE TO AOI POLYGON
  Dim dXmin As Double
  Dim dYmin As Double
  Dim dXmax As Double
  Dim dYmax As Double
225:   pAOIEnv.QueryCoords dXmin, dYmin, dXmax, dYmax
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim pPoint3 As IPoint
  Dim pPoint4 As IPoint
230:   Set pPoint1 = New Point
231:   Set pPoint2 = New Point
232:   Set pPoint3 = New Point
233:   Set pPoint4 = New Point
234:   pPoint1.PutCoords dXmin, dYmin
235:   pPoint2.PutCoords dXmin, dYmax
236:   pPoint3.PutCoords dXmax, dYmax
237:   pPoint4.PutCoords dXmax, dYmin
238:   pIntermediatePoly.AddPoint pPoint1
239:   pIntermediatePoly.AddPoint pPoint2
240:   pIntermediatePoly.AddPoint pPoint3
241:   pIntermediatePoly.AddPoint pPoint4
242:   pIntermediatePoly.AddPoint pPoint1
243:   Set pTopoForSimplify = pAOIPoly
244:   pTopoForSimplify.Simplify
  
  ' CLIP OUT CORRIDOR UNION FROM AOI POLYGON
247:   Set pTopoOp = pAOIPoly
  Dim pNonHabitatPolygon As IPolygon4
249:   Set pNonHabitatPolygon = pTopoOp.Difference(pUnionPoly2)
250:   Set pTopoForSimplify = pNonHabitatPolygon
251:   pTopoForSimplify.Simplify
  Dim pHabitatPolygon As IPolygon4
253:   Set pHabitatPolygon = pTopoOp.Intersect(pUnionPoly2, esriGeometry2Dimension)
254:   Set pTopoForSimplify = pHabitatPolygon
255:   pTopoForSimplify.Simplify

'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pHabitatPolygon, "delete_corridors"
  
  Dim dblMaxDimension As Double
  Dim dblCellSize As Double
  
262:   dblMaxDimension = pAOIEnv.Width
263:   If pAOIEnv.Height > pAOIEnv.Width Then dblMaxDimension = pAOIEnv.Height
  
265:   dblCellSize = dblMaxDimension / 1000
  ' FOR DEBUGGING
'  dblCellSize = dblMaxDimension / 70
                    
  ' #2
270:   lngCounter = lngCounter + 1
271:   strMessage = "Preparing Raster Analysis Environment (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
274:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
277:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
278:   pProgressForm.Refresh
279:   pProgressForm.icnProgressLine.Refresh
280:   pProgressForm.icnProgressLineShort.Refresh
281:   DoEvents
282:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
283:     strOutputReport = "RunBottleneckInBatch process terminated:"
284:     Set RunBottleneckInBatch = New esriSystem.VarArray
285:     RunBottleneckInBatch.Add False
286:     RunBottleneckInBatch.Add strOutputReport
287:     RunBottleneckInBatch.Add Nothing
288:     GoTo ClearMemory
289:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Preparing Analysis Environment...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------

  
  ' MAKE CORRIDOR RASTER
  Dim pCorRaster As IRaster
  Dim pEnv As IRasterAnalysisEnvironment  'Create a RasterMakerOp operator
  Dim pRasMakerOp As IRasterMakerOp
305:   Set pRasMakerOp = New RasterMakerOp
  
  'Create an analysis environment object
'  Dim pEnv As IRasterAnalysisEnvironment
309:   Set pEnv = pRasMakerOp
  
  'Set cellsize
312:   pEnv.SetCellSize esriRasterEnvValue, dblCellSize
  
  'Set output extent
315:   pEnv.SetExtent esriRasterEnvValue, pAOIEnv
  
  'Set output spatial reference
318:   Set pEnv.OutSpatialReference = pSpRef
  
  'Create a constant raster
  Dim pBaseRaster As IRaster
322:   Set pBaseRaster = pRasMakerOp.MakeConstant(1, True)
  
'  GridFunctions.ReturnCellSize pBaseRaster
  
  ' CLIP TO CORRIDOR POLYGON
327:   Set pCorRaster = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pHabitatPolygon, True, pAOIEnv, dblCellSize)
 
  ' FOR DEBUGGING
'  Dim pRasterLayer As IRasterLayer
'  Set pRasterLayer = New RasterLayer
'  pRasterLayer.CreateFromRaster pCorRaster
'  pDoc.FocusMap.AddLayer pRasterLayer
  
  ' MAKE RASTER OPERATIONS OBJECTS
  Dim pLogOp As ILogicalOp         ' FOR CHECKING IF NULL
  Dim pDistanceOp As IDistanceOp   ' FOR CALCULATING EUCLIDIAN DISTANCE
  Dim pMathOp As IMathOp           ' FOR MULTIPLYING
  Dim pCondOp As IConditionalOp    ' FOR SETTING TO NULL
  
341:   Set pLogOp = New RasterMathOps
342:   Set pDistanceOp = New RasterDistanceOp
343:   Set pMathOp = New RasterMathOps
344:   Set pCondOp = New RasterConditionalOp
  
  ' #3
347:   lngCounter = lngCounter + 1
348:   strMessage = "Generating grid of corridor region (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '& vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
351:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
354:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
355:   pProgressForm.Refresh
356:   pProgressForm.icnProgressLine.Refresh
357:   pProgressForm.icnProgressLineShort.Refresh
358:   DoEvents
359:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
360:     strOutputReport = "RunBottleneckInBatch process terminated:"
361:     Set RunBottleneckInBatch = New esriSystem.VarArray
362:     RunBottleneckInBatch.Add False
363:     RunBottleneckInBatch.Add strOutputReport
364:     RunBottleneckInBatch.Add Nothing
365:     GoTo ClearMemory
366:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Corridor Grid...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  ' MAKE RASTER OF NULL PARTS OF pCorRaster
  Dim pOutsideRasterA As IGeoDataset
379:   Set pOutsideRasterA = pLogOp.IsNull(pCorRaster)
  Dim pOutsideRasterB As IGeoDataset
381:   Set pOutsideRasterB = pCondOp.SetNull(pLogOp.NotEqual(pOutsideRasterA, pBaseRaster), pOutsideRasterA)
  
  ' FOR DEBUGGING
'  Dim pOutsideRasterLayer As IRasterLayer
'  Set pOutsideRasterLayer = New RasterLayer
'  pOutsideRasterLayer.CreateFromRaster pOutsideRasterB
'  pDoc.FocusMap.AddLayer pOutsideRasterLayer
  
  ' #4
390:   lngCounter = lngCounter + 1
391:   strMessage = "Calculating distance to corridor edge (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
  
395:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
398:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
399:   pProgressForm.Refresh
400:   pProgressForm.icnProgressLine.Refresh
401:   pProgressForm.icnProgressLineShort.Refresh
402:   DoEvents
403:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
404:     strOutputReport = "RunBottleneckInBatch process terminated:"
405:     Set RunBottleneckInBatch = New esriSystem.VarArray
406:     RunBottleneckInBatch.Add False
407:     RunBottleneckInBatch.Add strOutputReport
408:     RunBottleneckInBatch.Add Nothing
409:     GoTo ClearMemory
410:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Distance Grid...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pOutputRaster As IGeoDataset
422:   Set pOutputRaster = pDistanceOp.EucDistance(pOutsideRasterB)
  
  ' FOR DEBUGGING
'  Dim pDistRasterLayerA As IRasterLayer
'  Set pDistRasterLayerA = New RasterLayer
'  pDistRasterLayerA.CreateFromRaster pOutputRaster
'  pDoc.FocusMap.AddLayer pDistRasterLayerA
  
  ' #5
431:   lngCounter = lngCounter + 1
432:   strMessage = "Combining distance and corridor grids (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
435:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
438:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
439:   pProgressForm.Refresh
440:   pProgressForm.icnProgressLine.Refresh
441:   pProgressForm.icnProgressLineShort.Refresh
442:   DoEvents
443:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
444:     strOutputReport = "RunBottleneckInBatch process terminated:"
445:     Set RunBottleneckInBatch = New esriSystem.VarArray
446:     RunBottleneckInBatch.Add False
447:     RunBottleneckInBatch.Add strOutputReport
448:     RunBottleneckInBatch.Add Nothing
449:     GoTo ClearMemory
450:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Combining Grids...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pDistanceRaster As IGeoDataset
462:   Set pDistanceRaster = pMathOp.Times(pCorRaster, pOutputRaster)
  
  ' FOR DEBUGGING
'  Dim pDistRasterLayer As IRasterLayer
'  Set pDistRasterLayer = New RasterLayer
'  pDistRasterLayer.CreateFromRaster pDistanceRaster
'  pDoc.FocusMap.AddLayer pDistRasterLayer

  ' MAKE DIRECTION AND FLOW ACCUMULATION GRID FROM DISTANCE GRID
  Dim pDirGrid As IGeoDataset
  Dim pFlowAccGrid As IGeoDataset
  Dim pHydOp As IHydrologyOp
474:   Set pHydOp = New RasterHydrologyOp
  
  ' #6
477:   lngCounter = lngCounter + 1
478:   strMessage = "Generating Flow Direction grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
481:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
484:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
485:   pProgressForm.Refresh
486:   pProgressForm.icnProgressLine.Refresh
487:   pProgressForm.icnProgressLineShort.Refresh
488:   DoEvents
489:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
490:     strOutputReport = "RunBottleneckInBatch process terminated:"
491:     Set RunBottleneckInBatch = New esriSystem.VarArray
492:     RunBottleneckInBatch.Add False
493:     RunBottleneckInBatch.Add strOutputReport
494:     RunBottleneckInBatch.Add Nothing
495:     GoTo ClearMemory
496:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Flow Direction Grid...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
507:   Set pDirGrid = pHydOp.FlowDirection(pDistanceRaster, False, False)
  
  ' #7
510:   lngCounter = lngCounter + 1
511:   strMessage = "Generating Flow Accumulation grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
514:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
517:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
518:   pProgressForm.Refresh
519:   pProgressForm.icnProgressLine.Refresh
520:   pProgressForm.icnProgressLineShort.Refresh
521:   DoEvents
522:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
523:     strOutputReport = "RunBottleneckInBatch process terminated:"
524:     Set RunBottleneckInBatch = New esriSystem.VarArray
525:     RunBottleneckInBatch.Add False
526:     RunBottleneckInBatch.Add strOutputReport
527:     RunBottleneckInBatch.Add Nothing
528:     GoTo ClearMemory
529:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Flow Accumulation Grid...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
540:   Set pFlowAccGrid = pHydOp.FlowAccumulation(pDirGrid)
  
  ' GET CELLS WITH 0 FLOW ACCUMULATION
  ' #8
544:   lngCounter = lngCounter + 1
545:   strMessage = "Identifying centerline cells in corridor (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
548:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
551:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
552:   pProgressForm.Refresh
553:   pProgressForm.icnProgressLine.Refresh
554:   pProgressForm.icnProgressLineShort.Refresh
555:   DoEvents
556:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
557:     strOutputReport = "RunBottleneckInBatch process terminated:"
558:     Set RunBottleneckInBatch = New esriSystem.VarArray
559:     RunBottleneckInBatch.Add False
560:     RunBottleneckInBatch.Add strOutputReport
561:     RunBottleneckInBatch.Add Nothing
562:     GoTo ClearMemory
563:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Identifying Central Cells...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pZeroGrid As IGeoDataset
575:   Set pZeroGrid = pLogOp.EqualTo(pFlowAccGrid, pRasMakerOp.MakeConstant(0, True))
  
  ' #9
578:   lngCounter = lngCounter + 1
579:   strMessage = "Clipping centerline cells (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
582:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
585:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
586:   pProgressForm.Refresh
587:   pProgressForm.icnProgressLine.Refresh
588:   pProgressForm.icnProgressLineShort.Refresh
589:   DoEvents
590:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
591:     strOutputReport = "RunBottleneckInBatch process terminated:"
592:     Set RunBottleneckInBatch = New esriSystem.VarArray
593:     RunBottleneckInBatch.Add False
594:     RunBottleneckInBatch.Add strOutputReport
595:     RunBottleneckInBatch.Add Nothing
596:     GoTo ClearMemory
597:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Clipping Central Cells...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pPossibleGrid As IGeoDataset
609:   Set pPossibleGrid = pCondOp.SetNull(pLogOp.NotEqual(pZeroGrid, pBaseRaster), pZeroGrid)
  
  ' FOR DEBUGGING
'  Dim pPossibleGridLayer As IRasterLayer
'  Set pPossibleGridLayer = New RasterLayer
'  pPossibleGridLayer.CreateFromRaster pPossibleGrid
'  pDoc.FocusMap.AddLayer pPossibleGridLayer

  ' MAKE START AND END GRIDS
  Dim pStartGrid As IGeoDataset
  Dim pEndGrid As IGeoDataset
  Dim pExtractionOp As IExtractionOp2
  
  ' #10
623:   lngCounter = lngCounter + 1
624:   strMessage = "Clipping Habitat Block #1 to analysis area (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
627:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
630:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
631:   pProgressForm.Refresh
632:   pProgressForm.icnProgressLine.Refresh
633:   pProgressForm.icnProgressLineShort.Refresh
634:   DoEvents
635:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
636:     strOutputReport = "RunBottleneckInBatch process terminated:"
637:     Set RunBottleneckInBatch = New esriSystem.VarArray
638:     RunBottleneckInBatch.Add False
639:     RunBottleneckInBatch.Add strOutputReport
640:     RunBottleneckInBatch.Add Nothing
641:     GoTo ClearMemory
642:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Clipping Habitat Block 1...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
653:   Set pStartGrid = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pFinalStartPolygon, True, pAOIEnv, dblCellSize)
  
  ' #11
656:   lngCounter = lngCounter + 1
657:   strMessage = "Clipping Habitat Block #2 to analysis area (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
660:   Debug.Print strMessage
  
'  ' ---- PROGRESS METER STUFF
663:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
664:   pProgressForm.Refresh
665:   pProgressForm.icnProgressLine.Refresh
666:   pProgressForm.icnProgressLineShort.Refresh
667:   DoEvents
668:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
669:     strOutputReport = "RunBottleneckInBatch process terminated:"
670:     Set RunBottleneckInBatch = New esriSystem.VarArray
671:     RunBottleneckInBatch.Add False
672:     RunBottleneckInBatch.Add strOutputReport
673:     RunBottleneckInBatch.Add Nothing
674:     GoTo ClearMemory
675:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Clipping Habitat Block 2...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
686:   Set pEndGrid = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pFinalEndPolygon, True, pAOIEnv, dblCellSize)
  
'  Set pStartGrid = pExtractionOp.Polygon(pBaseRaster, pFinalStartPolygon, True)
'  Set pEndGrid = pExtractionOp.Polygon(pBaseRaster, pFinalEndPolygon, True)
  
  ' FOR DEBUGGING
'  Dim pStartGridLayer As IRasterLayer
'  Set pStartGridLayer = New RasterLayer
'  pStartGridLayer.CreateFromRaster pStartGrid
'  pDoc.FocusMap.AddLayer pStartGridLayer

  ' FOR DEBUGGING
'  Dim pEndGridLayer As IRasterLayer
'  Set pEndGridLayer = New RasterLayer
'  pEndGridLayer.CreateFromRaster pEndGrid
'  pDoc.FocusMap.AddLayer pEndGridLayer
  
  Dim pRasterStats As IRasterStatistics
  Dim pRasterBandCol As IRasterBandCollection
705:   Set pRasterBandCol = pDistanceRaster
  Dim pRasterBand As IRasterBand
707:   Set pRasterBand = pRasterBandCol.Item(0)
708:   Set pRasterStats = pRasterBand.Statistics
  Dim dblMaxDist As Double
710:   dblMaxDist = pRasterStats.Maximum
  
'  Debug.Print "dblMaxDist = " & CStr(dblMaxDist)

  ' #12
715:   lngCounter = lngCounter + 1
716:   strMessage = "Generating Burn Grid of central cells (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." ' & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
719:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
722:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
723:   pProgressForm.Refresh
724:   pProgressForm.icnProgressLine.Refresh
725:   pProgressForm.icnProgressLineShort.Refresh
726:   DoEvents
727:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
728:     strOutputReport = "RunBottleneckInBatch process terminated:"
729:     Set RunBottleneckInBatch = New esriSystem.VarArray
730:     RunBottleneckInBatch.Add False
731:     RunBottleneckInBatch.Add strOutputReport
732:     RunBottleneckInBatch.Add Nothing
733:     GoTo ClearMemory
734:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Burn Grid...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  

  Dim pBurnGrid As IGeoDataset
747:   Set pBurnGrid = pCondOp.Con(pLogOp.EqualTo(pZeroGrid, pBaseRaster), _
        pZeroGrid, pRasMakerOp.MakeConstant(1000000000.001, False))
  
  ' #13
751:   lngCounter = lngCounter + 1
752:   strMessage = "Generating Resistance Grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
755:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
758:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
759:   pProgressForm.Refresh
760:   pProgressForm.icnProgressLine.Refresh
761:   pProgressForm.icnProgressLineShort.Refresh
762:   DoEvents
763:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
764:     strOutputReport = "RunBottleneckInBatch process terminated:"
765:     Set RunBottleneckInBatch = New esriSystem.VarArray
766:     RunBottleneckInBatch.Add False
767:     RunBottleneckInBatch.Add strOutputReport
768:     RunBottleneckInBatch.Add Nothing
769:     GoTo ClearMemory
770:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Resistance Grid..", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pResistanceGrid As IGeoDataset
  Dim pMathSupOp As IMathSupportOp
783:   Set pMathSupOp = New RasterMathSupportOp
  Dim pMaxDistGrid As IGeoDataset
785:   Set pMaxDistGrid = pRasMakerOp.MakeConstant(dblMaxDist, False)
786:   Set pResistanceGrid = pMathSupOp.Plus(pMathSupOp.Times( _
        pMathSupOp.Divide(pMathSupOp.Minus(pMaxDistGrid, pDistanceRaster), pMaxDistGrid), _
        pRasMakerOp.MakeConstant(100000, False)), pBurnGrid)
  
  ' FOR DEBUGGING
'  Dim pResistanceLayer As IRasterLayer
'  Set pResistanceLayer = New RasterLayer
'  pResistanceLayer.CreateFromRaster pResistanceGrid
'  pDoc.FocusMap.AddLayer pResistanceLayer
  
  
'  theLengthFactor = theOriginPoly.Distance(theEndPoly)
'
'  ' MAXIMUM GRID VALUE RANGE = 1.175494351e-38 to 3.402823466e+38
'  theMaxDist = theDistGrid.GetStatistics.Get(1)
'  theBurnGrid = (theZeroGrid=1).Con(theZeroGrid, (1000000000.001).AsGrid)
'  theResistanceGrid = ((((theMaxDist.AsGrid) - theDistGrid) / (theMaxDist.AsGrid)) * 1000) + theBurnGrid
'  theDirFN = FileName.GetCWD.MakeTmp("dir", "")
'  'theAllFN = Filename.GetCWD.MakeTmp("all", "")
'  theCostGrid = theStartGrid.CostDistance(theResistanceGrid, theDirFN, nil, nil)
'  theDirGrid = Grid.Make(Grid.MakeSrcName(theDirFN.GetFullName))
'  'theAllGrid = Grid.Make(Grid.MakeSrcName(theAllFN.GetFullName))
'
  ' MAKE FORWARD AND REVERSE COST AND DIRECTION GRIDS
  Dim pForwardBandCol As IRasterBandCollection
  Dim pRevBandCol As IRasterBandCollection
  Dim pFullForward As IGeoDataset
  Dim pFullBack As IGeoDataset
  
  ' #14
816:   lngCounter = lngCounter + 1
817:   strMessage = "Calc Cost Distance from Habitat Block #1 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
820:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
823:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
824:   pProgressForm.Refresh
825:   pProgressForm.icnProgressLine.Refresh
826:   pProgressForm.icnProgressLineShort.Refresh
827:   DoEvents
828:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
829:     strOutputReport = "RunBottleneckInBatch process terminated:"
830:     Set RunBottleneckInBatch = New esriSystem.VarArray
831:     RunBottleneckInBatch.Add False
832:     RunBottleneckInBatch.Add strOutputReport
833:     RunBottleneckInBatch.Add Nothing
834:     GoTo ClearMemory
835:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Cost Distance Grid #1...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
846:   Set pFullForward = pDistanceOp.CostDistanceFull(pStartGrid, pResistanceGrid, True, True, False)
  
  ' #15
849:   lngCounter = lngCounter + 1
850:   strMessage = "Calc Cost Distance from Habitat Block #2 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
853:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
856:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
857:   pProgressForm.Refresh
858:   pProgressForm.icnProgressLine.Refresh
859:   pProgressForm.icnProgressLineShort.Refresh
860:   DoEvents
861:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
862:     strOutputReport = "RunBottleneckInBatch process terminated:"
863:     Set RunBottleneckInBatch = New esriSystem.VarArray
864:     RunBottleneckInBatch.Add False
865:     RunBottleneckInBatch.Add strOutputReport
866:     RunBottleneckInBatch.Add Nothing
867:     GoTo ClearMemory
868:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Generating Cost Distance Grid #2...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
879:   Set pFullBack = pDistanceOp.CostDistanceFull(pEndGrid, pResistanceGrid, True, True, False)
  
881:   Set pForwardBandCol = pFullForward
882:   Set pRevBandCol = pFullBack
  
  Dim pForCostBand As IRasterBand
  Dim pForDirBand As IRasterBand
  Dim pBackCostBand As IRasterBand
  Dim pBackDirBand As IRasterBand
  
  Dim pForCostGrid As IRasterBandCollection
  Dim pForDirGrid As IRasterBandCollection
  Dim pBackCostGrid As IRasterBandCollection
  Dim pBackDirGrid As IRasterBandCollection
  
894:   Set pForCostBand = pForwardBandCol.Item(0)
895:   Set pForDirBand = pForwardBandCol.Item(1)
896:   Set pBackCostBand = pRevBandCol.Item(0)
897:   Set pBackDirBand = pRevBandCol.Item(1)
  
899:   Set pForCostGrid = New Raster
900:   pForCostGrid.Add pForCostBand, 0
901:   Set pForDirGrid = New Raster
902:   pForDirGrid.Add pForDirBand, 0
903:   Set pBackCostGrid = New Raster
904:   pBackCostGrid.Add pBackCostBand, 0
905:   Set pBackDirGrid = New Raster
906:   pBackDirGrid.Add pBackDirBand, 0
  
  ' FOR DEBUGGING
'  Dim pForCostLayer As IRasterLayer
'  Set pForCostLayer = New RasterLayer
'  pForCostLayer.CreateFromRaster pForCostGrid
'  pForCostLayer.Name = "pForCostGrid"
'  pDoc.FocusMap.AddLayer pForCostLayer
  ' FOR DEBUGGING
'  Dim pForDirLayer As IRasterLayer
'  Set pForDirLayer = New RasterLayer
'  pForDirLayer.CreateFromRaster pForDirGrid
'  pForDirLayer.Name = "pForDirGrid"
'  pDoc.FocusMap.AddLayer pForDirLayer
  ' FOR DEBUGGING
'  Dim pBackCostLayer As IRasterLayer
'  Set pBackCostLayer = New RasterLayer
'  pBackCostLayer.CreateFromRaster pBackCostGrid
'  pBackCostLayer.Name = "pBackCostGrid"
'  pDoc.FocusMap.AddLayer pBackCostLayer
  ' FOR DEBUGGING
'  Dim pBackDirLayer As IRasterLayer
'  Set pBackDirLayer = New RasterLayer
'  pBackDirLayer.CreateFromRaster pBackDirGrid
'  pBackDirLayer.Name = "pBackDirGrid"
'  pDoc.FocusMap.AddLayer pBackDirLayer
      
  ' #16
934:   lngCounter = lngCounter + 1
935:   strMessage = "Examining potential routes from HB #1 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
938:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
941:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
942:   pProgressForm.Refresh
943:   pProgressForm.icnProgressLine.Refresh
944:   pProgressForm.icnProgressLineShort.Refresh
945:   DoEvents
946:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
947:     strOutputReport = "RunBottleneckInBatch process terminated:"
948:     Set RunBottleneckInBatch = New esriSystem.VarArray
949:     RunBottleneckInBatch.Add False
950:     RunBottleneckInBatch.Add strOutputReport
951:     RunBottleneckInBatch.Add Nothing
952:     GoTo ClearMemory
953:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Examining Potential Routes: #1...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  
  Dim pMultPt As IPointCollection
966:   Set pMultPt = New Multipoint
  Dim dblIndex As Double
  Dim pStepPoint As IPoint
969:   Set pStepPoint = New Point
970:   For dblIndex = 0 To pStartPolyline.length Step dblCellSize
971:     pStartPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
972:     Set pClone = pStepPoint
973:     pMultPt.AddPoint pClone.Clone
974:   Next dblIndex
975:   Set pClone = pStartPolyline.ToPoint
976:   pMultPt.AddPoint pClone.Clone
  
  Dim pStartPtCollection As IPointCollection
979:   Set pStartPtCollection = New Multipoint
   
  ' MAKE ARRAY OF CELL VALUES AT POINTS
  Dim pValArray As esriSystem.IVariantArray
  Dim vVal As Variant
  
985:   Set pValArray = Linkages.GridFunctions.CellValues(pMultPt, pZeroGrid)
986:   For lngIndex = 0 To pValArray.Count - 1
987:     vVal = pValArray.Element(lngIndex)
988:     If Not IsNull(vVal) Then
989:       If vVal = 1 Then
'        Set pTestPoint = pMultPt.Point(lngIndex)
991:         pStartPtCollection.AddPoint pMultPt.Point(lngIndex)
'        Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pMultPt.Point(lngIndex), "delete_corridors"
993:       End If
994:     End If
995:   Next lngIndex
  
997:   If pStartPtCollection.PointCount = 0 Then Set pStartPtCollection = pMultPt
  Dim pStartPolylines As IGeometryCollection
999:   Set pStartPolylines = pDistanceOp.CostPathAsPolyline(pStartPtCollection, pBackCostGrid, pBackDirGrid)
  
  ' START EXAMINING ACTUAL LINES
  ' LOOKING FOR BEST LINE BASED ON 4 CRITERIA:
  ' 1) MOST IMPORTANT:  SELECT LINE WITH HIGHEST MINIMUM DISTANCE VALUE
  ' 2) IF > 1, SUB-SELECT LINE WITH LOWEST CUMULATIVE COST
  ' 3) IF STILL > 1, SUB-SELECT LINE WITH HIGHEST MEAN DISTANCE VALUE
  ' 4) IF STILL > 1, SUB-SELECT SHORTEST LINE
  ' 5) IF STILL > 1, THEN ALMOST CERTAINLY SAME LINE.  JUST TAKE THE FIRST ONE.
  
  Dim pFinalLineArray As esriSystem.IArray
1010:   Set pFinalLineArray = New esriSystem.Array
  Dim pLngKeys As esriSystem.ILongArray
1012:   Set pLngKeys = New esriSystem.LongArray
  
  Dim pLineDataArray As esriSystem.IVariantArray
  Dim pTestPolyline As IPolyline
  Dim pTestMultipoint As IPointCollection
  Dim dblLength As Double
  Dim pTestStatsArray As esriSystem.IVariantArray
  Dim lngIndex2 As Long
  Dim vValue As Variant
  Dim dblValue As Double
  Dim dblMin As Double
  Dim dblMean As Double
  Dim vCost As Variant
  Dim dblCost As Double
  Dim booFoundValue As Boolean
  Dim pFinalDataArray As esriSystem.IVariantArray
  Dim pPointDblArray As esriSystem.IDoubleArray
  Dim lngMeanCounter As Long
  
1031:   If pStartPolylines.GeometryCount > 0 Then
1032:     For lngIndex = 0 To pStartPolylines.GeometryCount - 1
      
1034:       strMessage = "  --> From HB #1:  Calculating Stats for Route #" & CStr(lngIndex + 1) _
        & " of " & CStr(pStartPolylines.GeometryCount) & "..." & vbCrLf
1036:       Debug.Print strMessage
      
'      ' ---- PROGRESS METER STUFF
'      pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'      theDetailedDescription = theDetailedDescription & strMessage
'      pProgressForm.txtDetails.Text = theDetailedDescription
'      pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'      pProgressForm.Est_Time_Left lngCounter, "Examining Potential Routes: #1...", "Step "
'      pProgressForm.Refresh
'      pProgressForm.icnProgressLine.Refresh
'      DoEvents
'      ' ---- END PROGRESS METER STUFF ---------------
      
1049:       Set pTestPolyline = pStartPolylines.Geometry(lngIndex)
1050:       Set pTopoOp = pTestPolyline
1051:       pTopoOp.Simplify
      
1053:       dblLength = pTestPolyline.length
      
1055:       If dblLength > 0 Then
1056:         Set pTestMultipoint = New Multipoint
1057:         Set pFinalDataArray = New esriSystem.VarArray
1058:         lngMeanCounter = 0
        
        ' GENERATE POINTS ALONG ROUTE
1061:         For dblIndex = 0 To dblLength Step dblCellSize
1062:           pTestPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
1063:           Set pClone = pStepPoint
1064:           pTestMultipoint.AddPoint pClone.Clone
1065:         Next dblIndex
1066:         Set pStepPoint = pTestPolyline.ToPoint
1067:         Set pClone = pStepPoint
1068:         pTestMultipoint.AddPoint pClone.Clone
        
        ' CALCULATE MAXIMUM COST OF ROUTE
1071:         vCost = Linkages.GridFunctions.CellValue(pStepPoint, pForCostGrid)
1072:         If IsNull(vCost) Then
1073:           dblCost = 2 ^ 30
1074:         Else
1075:           dblCost = CDbl(vCost)
1076:         End If
        
        ' GET CELL VALUES OF ROUTE
1079:         Set pTestStatsArray = Linkages.GridFunctions.CellValues(pTestMultipoint, pDistanceRaster)
1080:         booFoundValue = False
        
        ' CALCULATE STATISTICS OF CELL VALUES
1083:         dblMean = 0
1084:         For lngIndex2 = 0 To pTestStatsArray.Count - 1
1085:           vValue = pTestStatsArray.Element(lngIndex2)
1086:           If Not IsNull(vValue) Then
1087:             dblValue = CDbl(vValue) * 2    ' (Width = distance to edge x 2)
1088:             If Not booFoundValue Then
1089:               dblMin = dblValue
1090:             Else
1091:               If dblValue < dblMin Then dblMin = dblValue
1092:             End If
1093:             dblMean = dblMean + dblValue
1094:             booFoundValue = True
1095:             lngMeanCounter = lngMeanCounter + 1
1096:           Else
1097:             dblValue = -99
1098:           End If
          
          ' MAKE FINAL OUTPUT ARRAY, IN CASE THIS IS THE BEST LINE
1101:           Set pPointDblArray = New esriSystem.DoubleArray
1102:           Set pStepPoint = pTestMultipoint.Point(lngIndex2)
1103:           pPointDblArray.Add pStepPoint.X
1104:           pPointDblArray.Add pStepPoint.Y
1105:           pPointDblArray.Add dblValue
1106:           If lngIndex2 = pTestStatsArray.Count - 1 Then
1107:             pPointDblArray.Add pTestPolyline.length
1108:           Else
1109:             pPointDblArray.Add lngIndex2 * dblCellSize
1110:           End If
1111:           pFinalDataArray.Add pPointDblArray
1112:         Next lngIndex2
        
1114:         If booFoundValue Then
          ' ADD DATA TO OVERALL ARRAY, TO ANALYZE LATER
1116:           Set pLineDataArray = New esriSystem.VarArray
          
1118:           dblMean = dblMean / lngMeanCounter
1119:           pLineDataArray.Add pTestPolyline
1120:           pLineDataArray.Add dblMin
1121:           pLineDataArray.Add dblMean
1122:           pLineDataArray.Add dblCost
1123:           pLineDataArray.Add pFinalDataArray
1124:           pLngKeys.Add lngIndex
1125:           pFinalLineArray.Add pLineDataArray
1126:         End If
1127:       End If
1128:     Next lngIndex
1129:   End If
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pStartPolylines.Geometry(3), "delete_corridors"
  
  ' #17
1134:   lngCounter = lngCounter + 1
1135:   strMessage = "Examining potential routes from HB #2 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
1138:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
1141:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
1142:   pProgressForm.Refresh
1143:   pProgressForm.icnProgressLine.Refresh
1144:   pProgressForm.icnProgressLineShort.Refresh
1145:   DoEvents
1146:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
1147:     strOutputReport = "RunBottleneckInBatch process terminated:"
1148:     Set RunBottleneckInBatch = New esriSystem.VarArray
1149:     RunBottleneckInBatch.Add False
1150:     RunBottleneckInBatch.Add strOutputReport
1151:     RunBottleneckInBatch.Add Nothing
1152:     GoTo ClearMemory
1153:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Examining Potential Routes: #2...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pMultPt2 As IPointCollection
1165:   Set pMultPt2 = New Multipoint
  Dim pEndPtColl As IPointCollection
1167:   Set pEndPtColl = New Multipoint
1168:   Set pStepPoint = New Point
1169:   For dblIndex = 0 To pEndPolyline.length Step dblCellSize
1170:     pEndPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
1171:     Set pClone = pStepPoint
1172:     pMultPt2.AddPoint pClone.Clone
1173:   Next dblIndex
1174:   Set pClone = pEndPolyline.ToPoint
1175:   pMultPt2.AddPoint pClone.Clone
  
1177:   Set pValArray = Linkages.GridFunctions.CellValues(pMultPt2, pZeroGrid)
1178:   For lngIndex = 0 To pValArray.Count - 1
1179:     vVal = pValArray.Element(lngIndex)
1180:     If Not IsNull(vVal) Then
1181:       If vVal = 1 Then
1182:         pEndPtColl.AddPoint pMultPt2.Point(lngIndex)
'        Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pMultPt2.Point(lngIndex), "delete_corridors"
1184:       End If
1185:     End If
1186:   Next lngIndex
  
1188:   If pEndPtColl.PointCount = 0 Then Set pEndPtColl = pMultPt2
  Dim pEndPolylines As IGeometryCollection
1190:   Set pEndPolylines = pDistanceOp.CostPathAsPolyline(pEndPtColl, pForCostGrid, pForDirGrid)
  
  ' REMEMBER TO FLIP THESE LINES!
  
1194:   If pEndPolylines.GeometryCount > 0 Then
1195:     For lngIndex = 0 To pEndPolylines.GeometryCount - 1
      
1197:       strMessage = "  --> From HB #2:  Calculating Stats for Route #" & CStr(lngIndex + 1) _
        & " of " & CStr(pEndPolylines.GeometryCount) & "..." & vbCrLf
1199:       Debug.Print strMessage
      
      ' ---- PROGRESS METER STUFF
'      pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'      theDetailedDescription = theDetailedDescription & strMessage
'      pProgressForm.txtDetails.Text = theDetailedDescription
'      pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'      pProgressForm.Est_Time_Left lngCounter, "Examining Potential Routes: #2...", "Step "
'      pProgressForm.Refresh
'      pProgressForm.icnProgressLine.Refresh
'      DoEvents
'      ' ---- END PROGRESS METER STUFF ---------------
  
1212:       Set pTestPolyline = pEndPolylines.Geometry(lngIndex)
1213:       Set pTopoOp = pTestPolyline
1214:       pTopoOp.Simplify
1215:       pTestPolyline.ReverseOrientation
      
1217:       dblLength = pTestPolyline.length
      
1219:       If dblLength > 0 Then
1220:         Set pTestMultipoint = New Multipoint
1221:         Set pFinalDataArray = New esriSystem.VarArray
1222:         lngMeanCounter = 0
        
        ' GENERATE POINTS ALONG ROUTE
1225:         For dblIndex = 0 To dblLength Step dblCellSize
1226:           pTestPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
1227:           Set pClone = pStepPoint
1228:           pTestMultipoint.AddPoint pClone.Clone
1229:         Next dblIndex
1230:         Set pStepPoint = pTestPolyline.ToPoint
1231:         Set pClone = pStepPoint
1232:         pTestMultipoint.AddPoint pClone.Clone
        
        ' CALCULATE MAXIMUM COST OF ROUTE
1235:         vCost = Linkages.GridFunctions.CellValue(pTestPolyline.FromPoint, pBackCostGrid)
1236:         If IsNull(vCost) Then
1237:           dblCost = 2 ^ 30
1238:         Else
1239:           dblCost = CDbl(vCost)
1240:         End If
        
        ' GET CELL VALUES OF ROUTE
1243:         Set pTestStatsArray = Linkages.GridFunctions.CellValues(pTestMultipoint, pDistanceRaster)
1244:         booFoundValue = False
        
        ' CALCULATE STATISTICS OF CELL VALUES
1247:         dblMean = 0
1248:         For lngIndex2 = 0 To pTestStatsArray.Count - 1
1249:           vValue = pTestStatsArray.Element(lngIndex2)
1250:           If Not IsNull(vValue) Then
1251:             dblValue = CDbl(vValue) * 2    ' (Width = distance to edge x 2)
1252:             If Not booFoundValue Then
1253:               dblMin = dblValue
1254:             Else
1255:               If dblValue < dblMin Then dblMin = dblValue
1256:             End If
1257:             dblMean = dblMean + dblValue
1258:             booFoundValue = True
1259:             lngMeanCounter = lngMeanCounter + 1
1260:           Else
1261:             dblValue = -99
1262:           End If
          
          ' MAKE FINAL OUTPUT ARRAY, IN CASE THIS IS THE BEST LINE
1265:           Set pPointDblArray = New esriSystem.DoubleArray
1266:           Set pStepPoint = pTestMultipoint.Point(lngIndex2)
1267:           pPointDblArray.Add pStepPoint.X
1268:           pPointDblArray.Add pStepPoint.Y
1269:           pPointDblArray.Add dblValue
1270:           If lngIndex2 = pTestStatsArray.Count - 1 Then
1271:             pPointDblArray.Add pTestPolyline.length
1272:           Else
1273:             pPointDblArray.Add lngIndex2 * dblCellSize
1274:           End If
1275:           pFinalDataArray.Add pPointDblArray
1276:         Next lngIndex2
        
1278:         If booFoundValue Then
          ' ADD DATA TO OVERALL ARRAY, TO ANALYZE LATER
1280:           Set pLineDataArray = New esriSystem.VarArray
          
1282:           dblMean = dblMean / lngMeanCounter
1283:           pLineDataArray.Add pTestPolyline
1284:           pLineDataArray.Add dblMin
1285:           pLineDataArray.Add dblMean
1286:           pLineDataArray.Add dblCost
1287:           pLineDataArray.Add pFinalDataArray
1288:           pLngKeys.Add lngIndex
1289:           pFinalLineArray.Add pLineDataArray
1290:         End If
1291:       End If
1292:     Next lngIndex
1293:   End If
  
  ' BAIL OUT IF DIDN'T FIND ANY POSSIBLE LINES
1296:   If pFinalLineArray.Count = 0 Then
1297:     MsgBox "Unable to calculate connector line!  Reasons unknown." & vbCrLf & "Bailing out...", , "Operation Failed:"
    Exit Function
1299:   End If
  
  ' FIND BEST LINE
  Dim pEligibleIndices As esriSystem.ILongArray
1303:   Set pEligibleIndices = New esriSystem.LongArray
  Dim pTempIndices As esriSystem.ILongArray
  
  ' FIND LINES WITH LARGEST MINIMUM DISTANCES
  Dim dblMinDistValue As Double
1308:   For lngIndex = 0 To pFinalLineArray.Count - 1
1309:     Set pLineDataArray = pFinalLineArray.Element(lngIndex)
1310:     Set pTestPolyline = pLineDataArray.Element(0)
1311:     dblMin = pLineDataArray.Element(1)
1312:     If dblMin > dblMinDistValue Then
1313:       dblMinDistValue = dblMin
1314:       pEligibleIndices.RemoveAll
1315:       pEligibleIndices.Add lngIndex
1316:     ElseIf dblMinDistValue = dblMin Then
1317:       pEligibleIndices.Add lngIndex
1318:     End If
1319:   Next lngIndex
  
  ' IF > 1, SUB-SELECT LINES WITH LOWEST TOTAL COST
  Dim lngFinalIndex As Long
1323:   lngFinalIndex = -999
  Dim dblLowestCost As Double
1325:   If pEligibleIndices.Count > 1 Then
1326:     Set pTempIndices = New esriSystem.LongArray
1327:     For lngIndex = 0 To pEligibleIndices.Count - 1
1328:       lngIndex2 = pEligibleIndices.Element(lngIndex)
1329:       Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
1330:       Set pTestPolyline = pLineDataArray.Element(0)
1331:       dblCost = pLineDataArray.Element(3)
1332:       If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
1333:         dblLowestCost = dblCost
1334:         pTempIndices.RemoveAll
1335:         pTempIndices.Add lngIndex2
1336:       Else
1337:         If dblCost < dblLowestCost Then
1338:           dblLowestCost = dblCost
1339:           pTempIndices.RemoveAll
1340:           pTempIndices.Add lngIndex2
1341:         ElseIf dblCost = dblLowestCost Then
1342:           pTempIndices.Add lngIndex2
1343:         End If
1344:       End If
1345:     Next lngIndex
    
1347:     Set pEligibleIndices = pTempIndices
1348:   Else
1349:     lngFinalIndex = pEligibleIndices.Element(0)
1350:   End If
  
  ' IF > 1, SUB-SELECT LINES WITH HIGHEST MEAN DISTANCE
'  Dim dblHighestMeanDist As Double
'  If pEligibleIndices.Count > 1 Then
'    Set pTempIndices = New esriSystem.LongArray
'    For lngIndex = 0 To pEligibleIndices.Count - 1
'      lngIndex2 = pEligibleIndices.Element(lngIndex)
'      Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
'      Set pTestPolyline = pLineDataArray.Element(0)
'      dblMean = pLineDataArray.Element(2)
'      If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
'        dblHighestMeanDist = dblMean
'        pTempIndices.RemoveAll
'        pTempIndices.Add lngIndex2
'      Else
'        If dblMean > dblHighestMeanDist Then
'          dblHighestMeanDist = dblMean
'          pTempIndices.RemoveAll
'          pTempIndices.Add lngIndex2
'        ElseIf dblCost = dblLowestCost Then
'          pTempIndices.Add lngIndex2
'        End If
'      End If
'    Next lngIndex
'
'    Set pEligibleIndices = pTempIndices
'  Else
'    lngFinalIndex = pEligibleIndices.Element(0)
'  End If
  
  ' IF > 1, SUB-SELECT SHORTEST LINE
  Dim dblShortestLength As Double
1383:   If pEligibleIndices.Count > 1 Then
1384:     Set pTempIndices = New esriSystem.LongArray
1385:     For lngIndex = 0 To pEligibleIndices.Count - 1
1386:       lngIndex2 = pEligibleIndices.Element(lngIndex)
1387:       Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
1388:       Set pTestPolyline = pLineDataArray.Element(0)
1389:       dblLength = pTestPolyline.length
1390:       If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
1391:         dblShortestLength = dblLength
1392:         pTempIndices.RemoveAll
1393:         pTempIndices.Add lngIndex2
1394:       Else
1395:         If dblLength < dblShortestLength Then
1396:           dblShortestLength = dblLength
1397:           pTempIndices.RemoveAll
1398:           pTempIndices.Add lngIndex2
1399:         ElseIf dblCost = dblLowestCost Then
1400:           pTempIndices.Add lngIndex2
1401:         End If
1402:       End If
1403:     Next lngIndex
    
1405:     Set pEligibleIndices = pTempIndices
1406:   Else
1407:     lngFinalIndex = pEligibleIndices.Element(0)
1408:   End If
  
  ' IF > 1, JUST TAKE THE FIRST ONE
1411:   If lngFinalIndex < 0 Then lngFinalIndex = pEligibleIndices.Element(0)
  
  ' GET OUTPUT ARRAY
  Dim pFinalStatsArray As esriSystem.IVariantArray
1415:   Set pLineDataArray = pFinalLineArray.Element(lngFinalIndex)
1416:   Set pFinalStatsArray = pLineDataArray.Element(4)
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pLineDataArray.Element(0), "delete_corridors"
  
  ' #18
1421:   lngCounter = lngCounter + 1
1422:   strMessage = "Clearing memory and temporary datasets (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1425:   Debug.Print strMessage

'  ' ---- PROGRESS METER STUFF
1428:   pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
1429:   pProgressForm.Refresh
1430:   pProgressForm.icnProgressLine.Refresh
1431:   pProgressForm.icnProgressLineShort.Refresh
1432:   DoEvents
1433:   If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
1434:     strOutputReport = "RunBottleneckInBatch process terminated:"
1435:     Set RunBottleneckInBatch = New esriSystem.VarArray
1436:     RunBottleneckInBatch.Add False
1437:     RunBottleneckInBatch.Add strOutputReport
1438:     RunBottleneckInBatch.Add Nothing
1439:     GoTo ClearMemory
1440:   End If
'  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
'  theDetailedDescription = theDetailedDescription & strMessage
'  pProgressForm.txtDetails.Text = theDetailedDescription
'  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
'  pProgressForm.Est_Time_Left lngCounter, "Clearing temporary datasets...", "Step "
'  pProgressForm.Refresh
'  pProgressForm.icnProgressLine.Refresh
'  DoEvents
'  ' ---- END PROGRESS METER STUFF ---------------
  
  ' OPTIONAL OUTPUTS
  Dim pSegWS As IWorkspace
  Dim pPointWS As IWorkspace
  Dim pStatsWS As IWorkspace
  Dim strSegName As String
  Dim strPointName As String
  Dim strBottleneckStatsName As String
  Dim strSegStatsName As String
  Dim booCalcBottleneckStats As Boolean
  Dim booCalcSegStats As Boolean
  
  Dim booSegOutputWorked As Boolean
  Dim strSegReport As String
  Dim booPointOutputWorked As Boolean
  Dim strPointReport As String
  Dim booStatsOutputWorked As Boolean
  Dim strStatsReport As String
  
1469:   booSegOutputWorked = True
1470:   strSegReport = ""
1471:   booPointOutputWorked = True
1472:   strPointReport = ""
1473:   booStatsOutputWorked = True
1474:   strStatsReport = ""
  
  Dim pSegmentArray As esriSystem.IArray
  
1478:   If Not pSegWorkOrder Is Nothing Then
    
1480:     lngCounter = lngCounter + 1
1481:     strMessage = "Creating Segment Feature Class (Step " & CStr(lngCounter) _
        & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
        "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1484:     Debug.Print strMessage
  
  '  ' ---- PROGRESS METER STUFF
1487:     pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
1488:     pProgressForm.Refresh
1489:     pProgressForm.icnProgressLine.Refresh
1490:     pProgressForm.icnProgressLineShort.Refresh
1491:     DoEvents
1492:     If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
1493:       strOutputReport = "RunBottleneckInBatch process terminated:"
1494:       Set RunBottleneckInBatch = New esriSystem.VarArray
1495:       RunBottleneckInBatch.Add False
1496:       RunBottleneckInBatch.Add strOutputReport
1497:       RunBottleneckInBatch.Add Nothing
1498:       GoTo ClearMemory
1499:     End If
  '  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
  '  theDetailedDescription = theDetailedDescription & strMessage
  '  pProgressForm.txtDetails.Text = theDetailedDescription
  '  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
  '  pProgressForm.Est_Time_Left lngCounter, "Clearing temporary datasets...", "Step "
  '  pProgressForm.Refresh
  '  pProgressForm.icnProgressLine.Refresh
  '  DoEvents
  '  ' ---- END PROGRESS METER STUFF ---------------
1509:     Set pSegWS = pSegWorkOrder.Element(0)
1510:     strSegName = pSegWorkOrder.Element(1)
    
1512:     strSegReport = MakeSegFeatureClassBatch(pSegWS, strSegName, pFinalStatsArray, pSpRef, _
      booSegOutputWorked, pCorPolygon, dblThresholdDistance, pSegmentArray, pBlock1FClass, pBlock2FClass, _
      pCorridorFClass, strUnits, pMxDoc)
      
1516:   End If
  
  
1519:   If Not pPointWorkOrder Is Nothing Then
  
1521:     lngCounter = lngCounter + 1
1522:     strMessage = "Creating Point Feature Class (Step " & CStr(lngCounter) _
        & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
        "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1525:     Debug.Print strMessage
  
  '  ' ---- PROGRESS METER STUFF
1528:     pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
1529:     pProgressForm.Refresh
1530:     pProgressForm.icnProgressLine.Refresh
1531:     pProgressForm.icnProgressLineShort.Refresh
1532:     DoEvents
1533:     If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
1534:       strOutputReport = "RunBottleneckInBatch process terminated:"
1535:       Set RunBottleneckInBatch = New esriSystem.VarArray
1536:       RunBottleneckInBatch.Add False
1537:       RunBottleneckInBatch.Add strOutputReport
1538:       RunBottleneckInBatch.Add Nothing
1539:       GoTo ClearMemory
1540:     End If
  '  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
  '  theDetailedDescription = theDetailedDescription & strMessage
  '  pProgressForm.txtDetails.Text = theDetailedDescription
  '  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
  '  pProgressForm.Est_Time_Left lngCounter, "Clearing temporary datasets...", "Step "
  '  pProgressForm.Refresh
  '  pProgressForm.icnProgressLine.Refresh
  '  DoEvents
  '  ' ---- END PROGRESS METER STUFF ---------------
  
1551:     Set pPointWS = pPointWorkOrder.Element(0)
1552:     strPointName = pPointWorkOrder.Element(1)
    
1554:     strPointReport = MakePointFeatureClassBatch(pPointWS, strPointName, pFinalStatsArray, pSpRef, _
      booPointOutputWorked, pCorPolygon, dblThresholdDistance, pBlock1FClass, pBlock2FClass, pCorridorFClass, _
      strUnits, pMxDoc)
    
1558:   End If
  
1560:   If Not pStatsWorkOrder Is Nothing Then
  
1562:     lngCounter = lngCounter + 1
1563:     strMessage = "Calculating Statistical Tables (Step " & CStr(lngCounter) _
        & " of " & CStr(lngCount) & ")..." '  & vbCrLf & _
        "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1566:     Debug.Print strMessage
  
  '  ' ---- PROGRESS METER STUFF
1569:     pProgressForm.Est_Time_Left_Short lngCounter, strMessage, lngCount
1570:     pProgressForm.Refresh
1571:     pProgressForm.icnProgressLine.Refresh
1572:     pProgressForm.icnProgressLineShort.Refresh
1573:     DoEvents
1574:     If BatchBailOutOfProgress(pProgressMeterForm, pThisExt) Then
1575:       strOutputReport = "RunBottleneckInBatch process terminated:"
1576:       Set RunBottleneckInBatch = New esriSystem.VarArray
1577:       RunBottleneckInBatch.Add False
1578:       RunBottleneckInBatch.Add strOutputReport
1579:       RunBottleneckInBatch.Add Nothing
1580:       GoTo ClearMemory
1581:     End If
  '  pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
  '  theDetailedDescription = theDetailedDescription & strMessage
  '  pProgressForm.txtDetails.Text = theDetailedDescription
  '  pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
  '  pProgressForm.Est_Time_Left lngCounter, "Clearing temporary datasets...", "Step "
  '  pProgressForm.Refresh
  '  pProgressForm.icnProgressLine.Refresh
  '  DoEvents
  '  ' ---- END PROGRESS METER STUFF ---------------
  
1592:     Set pStatsWS = pStatsWorkOrder.Element(0)
1593:     booCalcBottleneckStats = pStatsWorkOrder.Element(1)
1594:     strBottleneckStatsName = pStatsWorkOrder.Element(2)
1595:     booCalcSegStats = pStatsWorkOrder.Element(3)
1596:     strSegStatsName = pStatsWorkOrder.Element(4)
        
1598:     strStatsReport = MakeStatsTablesBatch(pStatsWS, booCalcBottleneckStats, strBottleneckStatsName, _
      booCalcSegStats, strSegStatsName, pFinalStatsArray, booStatsOutputWorked, pSpRef, _
      dblThresholdDistance, pSegmentArray, pBlock1FClass, pBlock2FClass, pCorridorFClass, strUnits, pMxDoc)
      
1602:   End If
  
  
1605:   Debug.Print "Finished"
  
  Dim lngProblemCounter As Long
1608:   lngProblemCounter = 0
1609:   If booSegOutputWorked And booPointOutputWorked And booStatsOutputWorked Then
1610:     strOutputReport = "Bottleneck Analysis successfully completed..."
1611:   Else
1612:     strOutputReport = "Bottleneck Analysis completed, but problems encountered:   "
1613:     If Not booSegOutputWorked Then
1614:       lngProblemCounter = lngProblemCounter + 1
1615:       strOutputReport = strOutputReport & CStr(lngProblemCounter) & "] " & strSegReport & "   "
1616:     End If
1617:     If Not booPointOutputWorked Then
1618:       lngProblemCounter = lngProblemCounter + 1
1619:       strOutputReport = strOutputReport & CStr(lngProblemCounter) & "] " & strPointReport & "   "
1620:     End If
1621:     If Not booStatsOutputWorked Then
1622:       lngProblemCounter = lngProblemCounter + 1
1623:       strOutputReport = strOutputReport & CStr(lngProblemCounter) & "] " & strStatsReport & "   "
1624:     End If
1625:     strOutputReport = Trim(strOutputReport)
1626:   End If
  
1628:   Set RunBottleneckInBatch = New esriSystem.VarArray
1629:   RunBottleneckInBatch.Add booSegOutputWorked And booPointOutputWorked And booStatsOutputWorked
1630:   RunBottleneckInBatch.Add strOutputReport
1631:   RunBottleneckInBatch.Add pFinalStatsArray
  
'  Set RunBottleneckInBatch = pFinalStatsArray
  
1635:   GoTo ClearMemory


  Exit Function
ErrorHandler:
'  HandleError True, "RunBottleneckInBatch Failed " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
1641:   strOutputReport = "RunBottleneckInBatch Failed:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"
  
1644:   Set RunBottleneckInBatch = New esriSystem.VarArray
1645:   RunBottleneckInBatch.Add False
1646:   RunBottleneckInBatch.Add strOutputReport
1647:   RunBottleneckInBatch.Add Nothing


ClearMemory:
1651:   Set pSpRef = Nothing
1652:   Set pTopoOp = Nothing
1653:   Set pRelOp = Nothing
1654:   Set pProgressForm = Nothing
1655:   Set pPrjSpRef = Nothing
1656:   Erase pPolyOutStart
1657:   Set pClone = Nothing
1658:   Set pTempStartA = Nothing
1659:   Erase pRing
1660:   Set pOuterRing = Nothing
1661:   Set pPath = Nothing
1662:   Set pTopoForSimplify = Nothing
1663:   Set pFinalStartPolygon = Nothing
1664:   Set pOutputPolygon = Nothing
1665:   Set pFinalStartLine = Nothing
1666:   Set pOutputPolyline = Nothing
1667:   Set pClipStart = Nothing
1668:   Erase pPolyOutEnd
1669:   Set pTempEndA = Nothing
1670:   Set pFinalEndPolygon = Nothing
1671:   Set pOutputEndPolygon = Nothing
1672:   Set pFinalEndPolyline = Nothing
1673:   Set pOutputEndPolyline = Nothing
1674:   Set pClipEnd = Nothing
1675:   Set pStartPolyline = Nothing
1676:   Set pEndPolyline = Nothing
1677:   Set pUnionPoly = Nothing
1678:   Set pUnionPoly2 = Nothing
1679:   Set pAOIPoly = Nothing
1680:   Set pIntermediatePoly = Nothing
1681:   Set pAOIEnv = Nothing
1682:   Set pPoint1 = Nothing
1683:   Set pPoint2 = Nothing
1684:   Set pPoint3 = Nothing
1685:   Set pPoint4 = Nothing
1686:   Set pNonHabitatPolygon = Nothing
1687:   Set pHabitatPolygon = Nothing
1688:   Set pCorRaster = Nothing
1689:   Set pEnv = Nothing
1690:   Set pRasMakerOp = Nothing
1691:   Set pBaseRaster = Nothing
1692:   Set pLogOp = Nothing
1693:   Set pDistanceOp = Nothing
1694:   Set pMathOp = Nothing
1695:   Set pCondOp = Nothing
1696:   Set pOutsideRasterA = Nothing
1697:   Set pOutsideRasterB = Nothing
1698:   Set pOutputRaster = Nothing
1699:   Set pDistanceRaster = Nothing
1700:   Set pDirGrid = Nothing
1701:   Set pFlowAccGrid = Nothing
1702:   Set pHydOp = Nothing
1703:   Set pZeroGrid = Nothing
1704:   Set pPossibleGrid = Nothing
1705:   Set pStartGrid = Nothing
1706:   Set pEndGrid = Nothing
1707:   Set pExtractionOp = Nothing
1708:   Set pRasterStats = Nothing
1709:   Set pRasterBandCol = Nothing
1710:   Set pRasterBand = Nothing
1711:   Set pBurnGrid = Nothing
1712:   Set pResistanceGrid = Nothing
1713:   Set pMathSupOp = Nothing
1714:   Set pMaxDistGrid = Nothing
1715:   Set pForwardBandCol = Nothing
1716:   Set pRevBandCol = Nothing
1717:   Set pFullForward = Nothing
1718:   Set pFullBack = Nothing
1719:   Set pForCostBand = Nothing
1720:   Set pForDirBand = Nothing
1721:   Set pBackCostBand = Nothing
1722:   Set pBackDirBand = Nothing
1723:   Set pForCostGrid = Nothing
1724:   Set pForDirGrid = Nothing
1725:   Set pBackCostGrid = Nothing
1726:   Set pBackDirGrid = Nothing
1727:   Set pMultPt = Nothing
1728:   Set pStepPoint = Nothing
1729:   Set pStartPtCollection = Nothing
1730:   Set pValArray = Nothing
1731:   vVal = Null
1732:   Set pStartPolylines = Nothing
1733:   Set pFinalLineArray = Nothing
1734:   Set pLngKeys = Nothing
1735:   Set pLineDataArray = Nothing
1736:   Set pTestPolyline = Nothing
1737:   Set pTestMultipoint = Nothing
1738:   Set pTestStatsArray = Nothing
1739:   vValue = Null
1740:   vCost = Null
1741:   Set pFinalDataArray = Nothing
1742:   Set pPointDblArray = Nothing
1743:   Set pMultPt2 = Nothing
1744:   Set pEndPtColl = Nothing
1745:   Set pEndPolylines = Nothing
1746:   Set pEligibleIndices = Nothing
1747:   Set pTempIndices = Nothing
1748:   Set pFinalStatsArray = Nothing
1749:   Set pSegWS = Nothing
1750:   Set pPointWS = Nothing
1751:   Set pStatsWS = Nothing
1752:   Set pSegmentArray = Nothing



End Function


Public Function PerformBatchBottleneckAnalysis(pDocParams As esriSystem.IVariantArray, _
    pParams As esriSystem.IVariantArray) As String
  On Error GoTo ErrorHandler

  Dim lngMilliseconds As Long
1764:   lngMilliseconds = Linkages.MyGeneralOperations.GetTickCount
  Dim datTimeBegan As Date
1766:   datTimeBegan = Now
  
  Dim pApp As IApplication
  Dim pMxDoc As IMxDocument
  Dim pThisExt As Linkages.clsExtension
  
1772:   Set pApp = pDocParams.Element(0)
1773:   Set pThisExt = pDocParams.Element(1)
1774:   Set pMxDoc = pDocParams.Element(2)
  
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
1778:   Set psbar = pApp.StatusBar
  Dim pProg As IStepProgressor
1780:   Set pProg = psbar.ProgressBar
  
  ' RETURN ARRAY CONTAINS 4 ITEMS
  ' 0) IStandaloneTable; selected table
  ' 1) IArray; containing selected fields
  '    a) Wildland Block 1 Source Field
  '    b) Wildland Block 2 Source Field
  '    c) Corridor Source Field
  '    d) Threshold Distance FIeld
  '    e) Segment Name Field
  '    f) Segment Workspace Field
  '    g) Point Name Field
  '    h) Point Workspace Field
  '    i) Bottleneck Stats Name Field
  '    j) Segment Stats Name Field
  '    k) Stats Workspace Field
  ' 2) LONG ARRAY; containing index values for 11 attribute fields above
  ' 3) IVariantArray (pFClassArray) CONTAINING:
  '    a) STRING Output Workspace Path
  '    b) STRING Output Table Name
  '    c) JenWorkspaceType
  '    d) New Fields for Table, OUTPUT TABLE WILL HAVE 11 FIELDS:
'         i) CLONE OF WILDLAND BLOCK 1 SOURCE FIELD, ADJUSTED TO dBASE IF NECESSARY
'        ii) CLONE OF WILDLAND BLOCK 2 SOURCE FIELD, ADJUSTED TO dBASE IF NECESSARY
'       iii) CLONE OF CORRIDOR SOURCE FIELD, ADJUSTED TO dBASE IF NECESSARY
'        iv) CLONE OF THRESHOLD DISTANCE FIELD, ADJUSTED TO dBASE IF NECESSARY
'         v) CLONE OF SEGMENT NAME FIELD, ADJUSTED TO dBASE IF NECESSARY
'        vi) CLONE OF SEGMENT WORKSPACE FIELD, ADJUSTED TO dBASE IF NECESSARY
'       vii) CLONE OF POINT NAME FIELD, ADJUSTED TO dBASE IF NECESSARY
'      viii) CLONE OF POINT WORKSPACE FIELD, ADJUSTED TO dBASE IF NECESSARY
'        ix) CLONE OF BOTTLENECK STATS NAME FIELD, ADJUSTED TO dBASE IF NECESSARY
'         x) CLONE OF SEGMENT STATS NAME FIELD, ADJUSTED TO dBASE IF NECESSARY
'        xi) CLONE OF STATS WORKSPACE FIELD, ADJUSTED TO dBASE IF NECESSARY
'      xiii) STRING; TIME BEGAN
'       xiv) STRING; TIME ENDED
'        xv) STRING; TIME ELAPSED
'       xvi) STRING; RESULTS
  '    e) Dialog Title if necessary
  '    f) IVariantArray of Field Name Pairs:
  '       i) Field Name Pair is IStringArray containing:
  '          1. Field name from original table
  '          2. Field name in new output table

 ' MsgBox "Batch Function:  pParams is nothing = " & CStr(pParams Is Nothing)
  
  Dim pStTable As IStandaloneTable
1826:   Set pStTable = pParams.Element(0)
  Dim pSourceTable As ITable
1828:   Set pSourceTable = pStTable.Table
  
  Dim pSelFields As esriSystem.IArray
1831:   Set pSelFields = pParams.Element(1)
  Dim pField As IField
  
  Dim lngFieldIndices() As Long
1835:   lngFieldIndices = pParams.Element(2)
  Dim lngBlock1Index As Long
  Dim lngBlock2index As Long
  Dim lngCorridorIndex As Long
  Dim lngDistIndex As Long
  Dim lngSegNameIndex As Long
  Dim lngSegWSIndex As Long
  Dim lngPointNameIndex As Long
  Dim lngPointWSIndex As Long
  Dim lngBottleneckStatsNameIndex As Long
  Dim lngSegStatsNameIndex As Long
  Dim lngStatsWSIndex As Long
  
1848:   lngBlock1Index = lngFieldIndices(0)
1849:   lngBlock2index = lngFieldIndices(1)
1850:   lngCorridorIndex = lngFieldIndices(2)
1851:   lngDistIndex = lngFieldIndices(3)
1852:   lngSegNameIndex = lngFieldIndices(4)
1853:   lngSegWSIndex = lngFieldIndices(5)
1854:   lngPointNameIndex = lngFieldIndices(6)
1855:   lngPointWSIndex = lngFieldIndices(7)
1856:   lngBottleneckStatsNameIndex = lngFieldIndices(8)
1857:   lngSegStatsNameIndex = lngFieldIndices(9)
1858:   lngStatsWSIndex = lngFieldIndices(10)
  
  ' OUTPUT FEATURE CLASS / TABLE DETAILS
  Dim pOutputArray As esriSystem.IVariantArray
1862:   Set pOutputArray = pParams.Element(3)
  
  ' CREATE OUTPUT TABLE
'  strReport = strReport & "Output Details:" & vbCrLf & _
            "  --> strWsPathName = " & pOutputArray.Element(0) & vbCrLf & _
            "  --> strBaseName = " & pOutputArray.Element(1) & vbCrLf & _
            "  --> pCategory = " & CStr(pOutputArray.Element(2)) & vbCrLf & _
            "  --> Fields Added = " & CStr(Not pOutputArray.Element(3) Is Nothing) & vbCrLf & _
            "  --> GxDialogCaption = " & pOutputArray.Element(4) & vbCrLf
  Dim pTable As ITable
1872:     Set pTable = Linkages.MyGeneralOperations.CreateGeneralTable(pOutputArray.Element(0), pOutputArray.Element(1), _
          pOutputArray.Element(2), pOutputArray.Element(3), pOutputArray.Element(4))
1874:   If pTable Is Nothing Then
1875:     MsgBox "No Table Created:" & vbCrLf & "Bailing out..."
1876:     GoTo ClearMemory
1877:   End If
  
  Dim lngIndex As Long
  Dim lngNewBlock1Index As Long
  Dim lngNewBlock2index As Long
  Dim lngNewCorridorIndex As Long
  Dim lngNewDistIndex As Long
  Dim lngNewSegNameIndex As Long
  Dim lngNewSegWSIndex As Long
  Dim lngNewPointNameIndex As Long
  Dim lngNewPointWSIndex As Long
  Dim lngNewBottleneckStatsNameIndex As Long
  Dim lngNewSegStatsNameIndex As Long
  Dim lngNewStatsWSIndex As Long
  Dim lngNewTimeBeganIndex As Long
  Dim lngNewTimeEndIndex As Long
  Dim lngNewTimeElapsedIndex As Long
  Dim lngNewResultsIndex As Long
  
  Dim pFieldNamePairs As IVariantArray
1897:   Set pFieldNamePairs = pOutputArray.Element(5)
  Dim pSubArray As esriSystem.IStringArray
  Dim lngNewUniqueIDIndex As Long
1900:   For lngIndex = 0 To pFieldNamePairs.Count - 1
1901:     Set pSubArray = pFieldNamePairs.Element(lngIndex)
      Select Case lngIndex
        Case 0
1904:           lngNewBlock1Index = pTable.FindField(pSubArray.Element(1))
        Case 1
1906:           lngNewBlock2index = pTable.FindField(pSubArray.Element(1))
        Case 2
1908:           lngNewCorridorIndex = pTable.FindField(pSubArray.Element(1))
        Case 3
1910:           lngNewDistIndex = pTable.FindField(pSubArray.Element(1))
        Case 4
1912:           lngNewSegNameIndex = pTable.FindField(pSubArray.Element(1))
        Case 5
1914:           lngNewSegWSIndex = pTable.FindField(pSubArray.Element(1))
        Case 6
1916:           lngNewPointNameIndex = pTable.FindField(pSubArray.Element(1))
        Case 7
1918:           lngNewPointWSIndex = pTable.FindField(pSubArray.Element(1))
        Case 8
1920:           lngNewBottleneckStatsNameIndex = pTable.FindField(pSubArray.Element(1))
        Case 9
1922:           lngNewSegStatsNameIndex = pTable.FindField(pSubArray.Element(1))
        Case 10
1924:           lngNewStatsWSIndex = pTable.FindField(pSubArray.Element(1))
1925:       End Select
1926:   Next lngIndex
  
1928:   lngNewTimeBeganIndex = pTable.FindField("Time_Began")
1929:   lngNewTimeEndIndex = pTable.FindField("Time_Done")
1930:   lngNewTimeElapsedIndex = pTable.FindField("Time_Elaps")
1931:   lngNewResultsIndex = pTable.FindField("Results")
1932:   lngNewUniqueIDIndex = pTable.FindField("Unique_ID")
  
  Dim pBlock1FClass As IFeatureClass
  Dim pBlock2FClass As IFeatureClass
  Dim pCorridorFClass As IFeatureClass
  Dim strPointName As String
  Dim strPointWS As String
  Dim strSegName As String
  Dim strSegWS As String
  Dim strValue As String
  Dim varValue As Variant
  Dim strFailureReason As String
  Dim lngAnalysisCounter As Long
  Dim strFullAnalysisReport As String
  Dim strAnalysisReport As String
  Dim dblThresholdValue As Double
  Dim pSegWS As IWorkspace
  Dim pPointWS As IWorkspace
  Dim pStatsWS As IWorkspace
  Dim strWSFail As String
  Dim strNameFailReason As String
  Dim pCursor As ICursor
  Dim lngFeatureCount As Long
  Dim pRow As IRow
1956:   lngFeatureCount = pSourceTable.RowCount(Nothing)
  Dim datSubTimeBegan As Date
  Dim datSubTimeEnd As Date
  Dim pBlock1Polygon As IPolygon
  Dim pBlock2Polygon As IPolygon
  Dim pCorridorPolygon As IPolygon
  Dim lngPolygonCount As Long
  Dim strStatsWS As String
  Dim strBottleneckStatsName As String
  Dim strSegStatsName As String
  
  Dim strSubBaseString As String
1968:   strSubBaseString = strSubBaseString & "\par \pard\b Analyis #zzzLongAnalysisNumber (of zzzLongAnalysisCount):\b0 " & vbNewLine
1969:   strSubBaseString = strSubBaseString & "\par \pard{\pntext\f2\'B7\tab}{\*\pn\pnlvlblt\pnf2\pnindent0{\pntxtb\'B7}}\fi-360\li720\sl276\slmult1\b Block 1 Feature Class: \b0  \cf1 zzzBlock1Name\cf0 " & vbNewLine
1970:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Block 2 Feature Class:\b0   \cf1 zzzBlock2Name\cf0 " & vbNewLine
1971:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Corridor Feature Class:\b0   \cf1 zzzCorridorName\cf0 " & vbNewLine
1972:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Threshold Distance:\b0   \cf1 zzzDistance\cf0 " & vbNewLine
1973:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Point Name:\b0   \cf1 zzzPointName\cf0 " & vbNewLine
1974:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Point Workspace:\b0   \cf1 zzzPointWS\cf0 " & vbNewLine
1975:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segment Name:\b0   \cf1 zzzSegNameValue\cf0 " & vbNewLine
1976:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segment Workspace:\b0   \cf1 zzzSegWS\cf0 " & vbNewLine
1977:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Bottleneck Stats Name:\b0   \cf1 zzzBottleneckStatsName\cf0 " & vbNewLine
1978:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segment Stats Name:\b0   \cf1 zzzSegmentStatsName\cf0 " & vbNewLine
1979:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Output Statistics Workspace:\b0   \cf1 zzzStatsWS\cf0 " & vbNewLine
1980:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Time Began:\b0   \cf1 zzzTimeBegan\cf0 " & vbNewLine
1981:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Time Finished:\b0   \cf1 zzzTimeFinished\cf0 " & vbNewLine
1982:   strSubBaseString = strSubBaseString & "\par \b{\pntext\f2\'B7\tab}Time Elapsed:\b0   \cf1 zzzTimeElapsed\cf0 " & vbNewLine
1983:   strSubBaseString = strSubBaseString & "\par \pard{\pntext\f2\'B7\tab}{\*\pn\pnlvlblt\pnf2\pnindent0{\pntxtb\'B7}}\fi-360\li720\sa200\sl276\slmult1 \b Results:  \cf2 zzzResultsReport\cf0 \b0" & vbNewLine

  Dim strPrefix As String
  Dim strSuffix As String
1987:   Linkages.MyGeneralOperations.ReturnQuerySpecialCharacters pSourceTable, strPrefix, strSuffix

'  ' SAMPLE CODE SNIPPET:  ADD RECORDS TO TABLE USING BUFFER
  Dim pBuffer As IRowBuffer
1991:   Set pBuffer = pTable.CreateRowBuffer
  Dim pTableCursor As ICursor
1993:   Set pTableCursor = pTable.Insert(True)
  
1995:   Set psbar = pApp.StatusBar
1996:   Set pProg = psbar.ProgressBar

1998:   psbar.ShowProgressBar "Verifying batch analysis table...", 0, lngFeatureCount, 1, True
1999:   pProg.position = 0
2000:   lngAnalysisCounter = 0

  ReDim booBatchErrors(lngFeatureCount - 1)
2003:   Set pCursor = pSourceTable.Search(Nothing, True)
2004:   Set pRow = pCursor.NextRow
  Dim pDataset As IDataset
  Dim pTempFCursor As IFeatureCursor
  Dim pTempFeature As IFeature
  Dim strDatasetName As String
  Dim booSegmentWS As Boolean
  Dim booSegmentName As Boolean
  Dim booPointWS As Boolean
  Dim booPointName As Boolean
  Dim booSegmentWSEmpty As Boolean
  Dim booPointWSEmpty As Boolean
  Dim booBottleneckStatsName As Boolean
  Dim booSegStatsName As Boolean
  Dim booStatsWSEmpty As Boolean
  Dim booStatsWS As Boolean
  
  Dim strResults As String
  Dim strTimeElapsed As String
  
  Dim lngMillisecondsEnd As Long
  Dim pBottleneckData As esriSystem.IVariantArray
  Dim pFinalStatsArray As esriSystem.IVariantArray
  Dim booBottleneckWorked As Boolean
  
  
    ' FOR PROGRESS METER --------------------------------------------------------------
  Dim pProgressForm As Linkages.frmJenProgressMeter
2031:   Set pProgressForm = New Linkages.frmJenProgressMeter
  
  Dim booShowProgress As Boolean
  
  Dim booFoundProblems As Boolean
  Dim strProblemReport As String
  Dim strRunningProblems As String
  Dim lngTimer As Long
  Dim theTimeBegan As Date
  Dim dateRunningTime As Date
  Dim strHeader As String
  Dim strMessage As String
  
  Dim theDetailedDescription As String
  Dim theProgressSpecific As String
  Dim pSegWorkOrder As esriSystem.IVariantArray
  Dim pPointWorkOrder As esriSystem.IVariantArray
  Dim pStatsWorkOrder As esriSystem.IVariantArray
  
      
  ' FOLLOWING CODE SHOULD BE INSERTED INTO A FUNCTION.  NEEDS MINOR EDITS IF GOING INTO A SUBROUTINE
  ' SPECIFICALLY, CHANGE "goto cleanup" to "goto cleanup" IN BAILOUT SECTION
  ' PROGRESS BAR STUFF'
  
  ' FOLLOWING VARIABLES USED IF CHECKING FOR ERRORS DURING CODE OPERATION
2056:   strProblemReport = "-----------------------------------------\par" & vbCrLf & _
                     "NOTE:  The following problems were found in the dataset:\par" & vbCrLf & _
                     "zzzRunningProblemszzz"
                     
2060:   theTimeBegan = Now
2061:   dateRunningTime = Now
'  Set psbar = pApp.StatusBar
'  Set pPro = psbar.ProgressBar
2064:   strHeader = "Calculating General Progress"
2065:   strMessage = strHeader
  
  ' INSERT FOLLOWING WHERE YOU WANT TO CREATE THE PROGRESS FORM
'  If booShowProgress Then
      ' PROGRESS METER STUFF ------------------------------------------------------------------
  
  '  pprogressform.SetExpanded = ext.ProgressDialogSetExpanded
2072:     pProgressForm.SetExpanded = True
2073:     pProgressForm.SetAutoClose = pThisExt.ProgressDialogAutoClose
  
2075:     theTimeBegan = Now
2076:     pProgressForm.ShouldContinue = True
2077:     pProgressForm.ProgBeginTime = Now
2078:     pProgressForm.ProgRecCount = lngFeatureCount
2079:     pProgressForm.lblCurrentTime.Caption = Format(Now, "ttttt")
2080:     pProgressForm.lblBeginTime.Caption = "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd")
  
2082:     theDetailedDescription = "Calculating..." & vbCrLf & _
        "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd") & vbCrLf & _
        "-------------------------------------------" & vbCrLf
2085:     pProgressForm.txtDetails.Text = theDetailedDescription
2086:     pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)
  
2088:     pProgressForm.Frame.Caption = " Current Status:"
2089:     pProgressForm.Frame.Visible = True
2090:     pProgressForm.cmdDetails.Visible = True
2091:     pProgressForm.cmdStop.Visible = True
'  End If
  
2094:   Do Until pRow Is Nothing

2096:     lngAnalysisCounter = lngAnalysisCounter + 1
2097:     strMessage = "Batch Analysis Step " & Format(lngAnalysisCounter, "#,##0") & " of " & Format(lngFeatureCount, "#,##0") & "..."
2098:     pProg.Message = strMessage
2099:     DoEvents
        
    
    
    
2104:     pBuffer.Value(lngNewUniqueIDIndex) = lngAnalysisCounter
2105:     pBuffer.Value(lngNewBlock1Index) = ""
2106:     pBuffer.Value(lngNewBlock2index) = ""
2107:     pBuffer.Value(lngNewCorridorIndex) = ""
2108:     If Linkages.aml_func_mod.FieldIsNumeric(pBuffer.Fields.Field(lngNewDistIndex)) Then
2109:       pBuffer.Value(lngNewDistIndex) = -999
2110:     Else
2111:       pBuffer.Value(lngNewDistIndex) = ""
2112:     End If
2113:     pBuffer.Value(lngNewSegWSIndex) = ""
2114:     pBuffer.Value(lngNewSegNameIndex) = ""
2115:     pBuffer.Value(lngNewPointWSIndex) = ""
2116:     pBuffer.Value(lngNewPointNameIndex) = ""
2117:     pBuffer.Value(lngNewBottleneckStatsNameIndex) = ""
2118:     pBuffer.Value(lngNewSegStatsNameIndex) = ""
2119:     pBuffer.Value(lngNewStatsWSIndex) = ""
2120:     pBuffer.Value(lngNewTimeBeganIndex) = ""
2121:     pBuffer.Value(lngNewTimeEndIndex) = ""
2122:     pBuffer.Value(lngNewResultsIndex) = ""
    
2124:     strAnalysisReport = strSubBaseString
2125:     datSubTimeBegan = Now
2126:     lngMilliseconds = Linkages.MyGeneralOperations.GetTickCount
2127:     pBuffer.Value(lngNewTimeBeganIndex) = Format(datSubTimeBegan, "ttttt, dddddd")
2128:     strAnalysisReport = Replace(strAnalysisReport, "zzzTimeBegan", Format(datSubTimeBegan, "ttttt, dddddd"), , , vbTextCompare)
  
2130:     strResults = "Results Undetermined..."
    
2132:     lngPolygonCount = -999
2133:     Set pBlock1FClass = Nothing
2134:     Set pBlock2FClass = Nothing
2135:     Set pCorridorFClass = Nothing
    
2137:     strAnalysisReport = Replace(strAnalysisReport, "zzzLongAnalysisNumber", Format(lngAnalysisCounter, "#,##0"), , , vbTextCompare)
2138:     strAnalysisReport = Replace(strAnalysisReport, "zzzLongAnalysisCount", Format(lngFeatureCount, "#,##0"), , , vbTextCompare)
    
  ' LOOP THROUGH WHATEVER STEPS ARE REQUIRED TO GATHER DATA FOR A SINGLE ROW
    'pBuffer.Value(lngFieldIndex) = aValue
          
    ' BLOCK 1 DATA
2144:     varValue = pRow.Value(lngBlock1Index)
2145:     strValue = CStr(varValue)
2146:     strValue = Trim(strValue)
2147:     strDatasetName = aml_func_mod.ReturnFilename(strValue)
2148:     strAnalysisReport = Replace(strAnalysisReport, "zzzBlock1Name", strDatasetName, , , vbTextCompare)
2149:     Set pBlock1FClass = Linkages.BatchFunctions.ReturnFeatureClassFromString(strValue, strFailureReason, pMxDoc, lngPolygonCount)
2150:     Set pDataset = pBlock1FClass
2151:     pBuffer.Value(lngNewBlock1Index) = pDataset.BrowseName & ", in " & pDataset.Workspace.PathName
2152:     Set pTempFCursor = pBlock1FClass.Search(Nothing, False)
2153:     Set pTempFeature = pTempFCursor.NextFeature
2154:     Set pBlock1Polygon = pTempFeature.ShapeCopy
          
    ' BLOCK 2 DATA
2157:     varValue = pRow.Value(lngBlock2index)
2158:     strValue = CStr(varValue)
2159:     strValue = Trim(strValue)
2160:     strDatasetName = aml_func_mod.ReturnFilename(strValue)
2161:     strAnalysisReport = Replace(strAnalysisReport, "zzzBlock2Name", strDatasetName, , , vbTextCompare)
2162:     Set pBlock2FClass = Linkages.BatchFunctions.ReturnFeatureClassFromString(strValue, strFailureReason, pMxDoc, lngPolygonCount)
2163:     Set pDataset = pBlock2FClass
2164:     pBuffer.Value(lngNewBlock2index) = pDataset.BrowseName & ", in " & pDataset.Workspace.PathName
2165:     Set pTempFCursor = pBlock2FClass.Search(Nothing, False)
2166:     Set pTempFeature = pTempFCursor.NextFeature
2167:     Set pBlock2Polygon = pTempFeature.ShapeCopy
          
    ' CORRIDOR DATA
2170:     varValue = pRow.Value(lngCorridorIndex)
2171:     strValue = CStr(varValue)
2172:     strValue = Trim(strValue)
2173:     strDatasetName = aml_func_mod.ReturnFilename(strValue)
2174:     strAnalysisReport = Replace(strAnalysisReport, "zzzCorridorName", strDatasetName, , , vbTextCompare)
2175:     Set pCorridorFClass = Linkages.BatchFunctions.ReturnFeatureClassFromString(strValue, strFailureReason, pMxDoc, lngPolygonCount)
2176:     Set pDataset = pCorridorFClass
2177:     pBuffer.Value(lngNewCorridorIndex) = pDataset.BrowseName & ", in " & pDataset.Workspace.PathName
2178:     Set pTempFCursor = pCorridorFClass.Search(Nothing, False)
2179:     Set pTempFeature = pTempFCursor.NextFeature
2180:     Set pCorridorPolygon = pTempFeature.ShapeCopy

    ' THRESHOLD DISTANCE
2183:     varValue = pRow.Value(lngDistIndex)
2184:     If IsNull(varValue) Then
2185:       strValue = ""
2186:     Else
2187:       strValue = CStr(varValue)
2188:     End If
2189:     strValue = Trim(strValue)
2190:     If strValue = "" Then
2191:       dblThresholdValue = -999
2192:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzDistance", "4 No Value, Skipping Threshold Distance analysis", , , vbTextCompare)
2193:     ElseIf IsNumeric(strValue) Then
2194:       dblThresholdValue = CDbl(strValue)
2195:       strAnalysisReport = Replace(strAnalysisReport, "zzzDistance", Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdValue), , , vbTextCompare)
2196:     End If
2197:     If pRow.Fields.Field(lngDistIndex).Type = esriFieldTypeString Then
2198:       pBuffer.Value(lngNewDistIndex) = strValue
2199:     ElseIf pRow.Fields.Field(lngDistIndex).Type = esriFieldTypeDouble Then
2200:       pBuffer.Value(lngNewDistIndex) = dblThresholdValue
2201:     ElseIf pRow.Fields.Field(lngDistIndex).Type = esriFieldTypeInteger Then
2202:       pBuffer.Value(lngNewDistIndex) = CLng(dblThresholdValue)
2203:     ElseIf pRow.Fields.Field(lngDistIndex).Type = esriFieldTypeSmallInteger Then
2204:       pBuffer.Value(lngNewDistIndex) = CLng(dblThresholdValue)
2205:     Else
2206:       pBuffer.Value(lngNewDistIndex) = dblThresholdValue
2207:     End If
  
    ' SEGMENT WORKSPACE
2210:     Set pSegWS = Nothing
2211:     booSegmentWS = True
2212:     varValue = pRow.Value(lngSegWSIndex)
2213:     If IsNull(varValue) Then
2214:       strValue = ""
2215:     Else
2216:       strValue = CStr(varValue)
2217:     End If
2218:     strValue = Trim(strValue)
2219:     If strValue = "" Then
2220:       booSegmentWSEmpty = True
2221:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegWS", "4 No workspace specified:  Not generating output segments...", , , vbTextCompare)
2222:       booSegmentWS = False
2223:       pBuffer.Value(lngNewSegWSIndex) = ""
2224:     Else
2225:       booSegmentWSEmpty = False
2226:       Set pSegWS = Linkages.BatchFunctions.ReturnWorkspaceFromPath(strValue, strWSFail)
2227:       If pSegWS Is Nothing Then
2228:         booSegmentWS = False
2229:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegWS", _
            "3 Failed to create workspace at " & Replace(strValue, "\", "\\") & "..." & strWSFail, , , vbTextCompare)
2231:         pBuffer.Value(lngNewSegWSIndex) = "Failed to create workspace at " & strValue & "..." & strWSFail
2232:       Else
2233:         booSegmentWS = True
2234:         strAnalysisReport = Replace(strAnalysisReport, "zzzSegWS", Replace(strValue, "\", "\\"), , , vbTextCompare)
2235:         pBuffer.Value(lngNewSegWSIndex) = strValue
2236:       End If
2237:     End If
  
    ' SEGMENT NAME
2240:     booSegmentName = True
2241:     varValue = pRow.Value(lngSegNameIndex)
2242:     If IsNull(varValue) Then
2243:       strValue = ""
2244:     Else
2245:       strValue = CStr(varValue)
2246:     End If
2247:     strValue = Trim(strValue)
2248:     strSegName = strValue
2249:     If booSegmentWS Then
2250:       If strValue <> "" Then
2251:         booSegmentName = True
2252:         strAnalysisReport = Replace(strAnalysisReport, "zzzSegNameValue", strValue, , , vbTextCompare)
2253:         pBuffer.Value(lngNewSegNameIndex) = strValue
2254:       Else
2255:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegNameValue", "4 Not generating output segments...", , , vbTextCompare)
2256:         booSegmentName = False
2257:         pBuffer.Value(lngNewSegNameIndex) = ""
2258:       End If
2259:     Else
2260:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegNameValue", "4 Not generating output segments...", , , vbTextCompare)
2261:       booSegmentName = False
2262:       pBuffer.Value(lngNewSegNameIndex) = ""
2263:     End If
    
2265:     If booSegmentName And booSegmentWS Then
2266:       Set pSegWorkOrder = New esriSystem.VarArray
2267:       pSegWorkOrder.Add pSegWS
2268:       pSegWorkOrder.Add strSegName
2269:     Else
2270:       Set pSegWorkOrder = Nothing
2271:     End If
  
    ' POINT WORKSPACE
2274:     Set pPointWS = Nothing
2275:     booPointWS = True
2276:     varValue = pRow.Value(lngPointWSIndex)
2277:     If IsNull(varValue) Then
2278:       strValue = ""
2279:     Else
2280:       strValue = CStr(varValue)
2281:     End If
2282:     strValue = Trim(strValue)
2283:     If strValue = "" Then
2284:       booPointWSEmpty = True
2285:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzPointWS", "4 No workspace specified: Not generating output points...", , , vbTextCompare)
2286:       booPointWS = False
2287:       pBuffer.Value(lngNewPointWSIndex) = ""
2288:     Else
2289:       booPointWSEmpty = False
2290:       Set pPointWS = Linkages.BatchFunctions.ReturnWorkspaceFromPath(strValue, strWSFail)
2291:       If pPointWS Is Nothing Then
2292:         booPointWS = False
2293:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzPointWS", _
            "3 Failed to create workspace at " & Replace(strValue, "\", "\\") & "..." & strWSFail, , , vbTextCompare)
2295:         pBuffer.Value(lngNewPointWSIndex) = "Failed to create workspace at " & strValue & "..." & strWSFail
2296:       Else
2297:         booPointWS = True
2298:         strAnalysisReport = Replace(strAnalysisReport, "zzzPointWS", Replace(strValue, "\", "\\"), , , vbTextCompare)
2299:         pBuffer.Value(lngNewPointWSIndex) = strValue
2300:       End If
2301:     End If
  
    ' POINT NAME
2304:     booPointName = True
2305:     varValue = pRow.Value(lngPointNameIndex)
2306:     If IsNull(varValue) Then
2307:       strValue = ""
2308:     Else
2309:       strValue = CStr(varValue)
2310:     End If
2311:     strValue = Trim(strValue)
2312:     strPointName = strValue
2313:     If booPointWS Then
2314:       If strValue <> "" Then
2315:         booPointName = True
2316:         strAnalysisReport = Replace(strAnalysisReport, "zzzPointName", strValue, , , vbTextCompare)
2317:         pBuffer.Value(lngNewPointNameIndex) = strValue
2318:       Else
2319:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzPointName", "4 Not generating output points...", , , vbTextCompare)
2320:         booPointName = False
2321:         pBuffer.Value(lngNewPointNameIndex) = ""
2322:       End If
2323:     Else
2324:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzPointName", "4 Not generating output points...", , , vbTextCompare)
2325:       booPointName = False
2326:       pBuffer.Value(lngNewPointNameIndex) = ""
2327:     End If
    
2329:     If booPointName And booPointWS Then
2330:       Set pPointWorkOrder = New esriSystem.VarArray
2331:       pPointWorkOrder.Add pPointWS
2332:       pPointWorkOrder.Add strPointName
2333:     Else
2334:       Set pPointWorkOrder = Nothing
2335:     End If
  
    ' STATS WORKSPACE
2338:     Set pStatsWS = Nothing
2339:     booStatsWS = True
2340:     varValue = pRow.Value(lngStatsWSIndex)
2341:     If IsNull(varValue) Then
2342:       strValue = ""
2343:     Else
2344:       strValue = CStr(varValue)
2345:     End If
2346:     strValue = Trim(strValue)
2347:     If strValue = "" Then
2348:       booStatsWSEmpty = True
2349:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzStatsWS", "4 No workspace specified: Not generating statistical tables...", , , vbTextCompare)
2350:       booStatsWS = False
2351:       pBuffer.Value(lngNewStatsWSIndex) = ""
2352:     Else
2353:       booStatsWSEmpty = False
2354:       Set pStatsWS = Linkages.BatchFunctions.ReturnWorkspaceFromPath(strValue, strWSFail)
2355:       If pStatsWS Is Nothing Then
2356:         booStatsWS = False
2357:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzStatsWS", _
            "3 Failed to create workspace at " & Replace(strValue, "\", "\\") & "..." & strWSFail, , , vbTextCompare)
2359:         pBuffer.Value(lngNewStatsWSIndex) = "Failed to create workspace at " & strValue & "..." & strWSFail
2360:       Else
2361:         booStatsWS = True
2362:         strAnalysisReport = Replace(strAnalysisReport, "zzzStatsWS", Replace(strValue, "\", "\\"), , , vbTextCompare)
2363:         pBuffer.Value(lngNewStatsWSIndex) = strValue
2364:       End If
2365:     End If
    
    
    ' BOTTLENECK STATS NAME
2369:     booBottleneckStatsName = True
2370:     varValue = pRow.Value(lngBottleneckStatsNameIndex)
2371:     If IsNull(varValue) Then
2372:       strValue = ""
2373:     Else
2374:       strValue = CStr(varValue)
2375:     End If
2376:     strValue = Trim(strValue)
2377:     strBottleneckStatsName = strValue
2378:     If booStatsWS Then
2379:       If strValue <> "" Then
2380:         booBottleneckStatsName = True
2381:         strAnalysisReport = Replace(strAnalysisReport, "zzzBottleneckStatsName", strValue, , , vbTextCompare)
2382:         pBuffer.Value(lngNewBottleneckStatsNameIndex) = strValue
2383:       Else
2384:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzBottleneckStatsName", "4 Not generating bottleneck statistics...", , , vbTextCompare)
2385:         booBottleneckStatsName = False
2386:         pBuffer.Value(lngNewBottleneckStatsNameIndex) = ""
2387:       End If
2388:     Else
2389:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzBottleneckStatsName", "4 Not generating bottleneck statistics...", , , vbTextCompare)
2390:       booBottleneckStatsName = False
2391:       pBuffer.Value(lngNewBottleneckStatsNameIndex) = ""
2392:     End If
    
'    not saving to the correct stats folder!
    
    ' SEGMENT STATS NAME
2397:     booSegStatsName = True
2398:     varValue = pRow.Value(lngSegStatsNameIndex)
2399:     If IsNull(varValue) Then
2400:       strValue = ""
2401:     Else
2402:       strValue = CStr(varValue)
2403:     End If
2404:     strValue = Trim(strValue)
2405:     strSegStatsName = strValue
2406:     If booStatsWS Then
2407:       If strValue <> "" Then
2408:         booSegStatsName = True
2409:         strAnalysisReport = Replace(strAnalysisReport, "zzzSegmentStatsName", strValue, , , vbTextCompare)
2410:         pBuffer.Value(lngNewSegStatsNameIndex) = strValue
2411:       Else
2412:         strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegmentStatsName", "4 Not generating segment statistics...", , , vbTextCompare)
2413:         booSegStatsName = False
2414:         pBuffer.Value(lngNewSegStatsNameIndex) = ""
2415:       End If
2416:     Else
2417:       strAnalysisReport = Replace(strAnalysisReport, "1 zzzSegmentStatsName", "4 Not generating segment statistics...", , , vbTextCompare)
2418:       booSegStatsName = False
2419:       pBuffer.Value(lngNewSegStatsNameIndex) = ""
2420:     End If
    
2422:     If (booSegStatsName Or booBottleneckStatsName) And booStatsWS Then
2423:       Set pStatsWorkOrder = New esriSystem.VarArray
2424:       pStatsWorkOrder.Add pStatsWS
2425:       pStatsWorkOrder.Add booBottleneckStatsName
2426:       pStatsWorkOrder.Add strBottleneckStatsName
2427:       pStatsWorkOrder.Add booSegStatsName
2428:       pStatsWorkOrder.Add strSegStatsName
2429:     Else
2430:       Set pStatsWorkOrder = Nothing
2431:     End If
    
    ' UPDATE PROGRESS METER ==================================================================
2434:     lngTimer = lngAnalysisCounter
        
    ' ---- PROGRESS METER STUFF
2437:     DoEvents
2438:     theProgressSpecific = strMessage
2439:     theDetailedDescription = theDetailedDescription & _
      " --> " & strMessage & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2442:     pProgressForm.ResetShortTimer
2443:     pProgressForm.txtDetails.Text = theDetailedDescription
2444:     pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)

2446:     pProgressForm.Est_Time_Left lngTimer - 1, "Batch Bottleneck Process...", "Calculating Data...", lngTimer > 1
    ' pprogressform.lblCurrentTime.Caption = Format(Now, "ttttt")
2448:     pProgressForm.Refresh
2449:     pProgressForm.icnProgressLine.Refresh
2450:     DoEvents
    
2452:     If BatchBailOutOfProgress(pProgressForm, pThisExt) Then
  
     ' ********************************************************************************************
      ' ALTERNATIVE CODE THAT SAVES PARTIAL TABLE, CHECKS IF ERRORS FOUND AND OUTPUTS RTF INSTEAD OF TEXT
'      strTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedRTF(theTimeBegan, Now, 7)
2457:       strResults = "Exited Function Prematurely at Step " & CStr(lngTimer)
      
2459:       strAnalysisReport = Replace(strAnalysisReport, "2 zzzResultsReport", "3 " & strResults, , , vbTextCompare)
      
2461:       datSubTimeEnd = Now
2462:       lngMillisecondsEnd = Linkages.MyGeneralOperations.GetTickCount
2463:       pBuffer.Value(lngNewTimeEndIndex) = Format(datSubTimeEnd, "ttttt, dddddd")
2464:       strAnalysisReport = Replace(strAnalysisReport, "zzzTimeFinished", Format(datSubTimeEnd, "ttttt, dddddd"), , , vbTextCompare)
    
2466:       strTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedFromMilliseconds(lngMillisecondsEnd - lngMilliseconds)
2467:       strAnalysisReport = Replace(strAnalysisReport, "zzzTimeElapsed", strTimeElapsed, , , vbTextCompare)
      
2469:       strTimeElapsed = Replace(strTimeElapsed, "...", "")
2470:       pBuffer.Value(lngNewTimeElapsedIndex) = strTimeElapsed
2471:       pBuffer.Value(lngNewResultsIndex) = strResults
      
2473:       pTableCursor.InsertRow pBuffer
2474:       strFullAnalysisReport = strFullAnalysisReport & strAnalysisReport
      
'      If booFoundProblems Then
'
''        StatMatricesFromGeoDatasets = strInputParamsReport & "-----------------------------------------\par" & vbCrLf & _
'          "NOTE:  Exited Function Prematurely at Step " & CStr(lngTimer) & "\par" & vbCrLf & _
'            Replace(strProblemReport, "zzzRunningProblemszzz", strRunningProblems) & vbCrLf & _
'          "-----------------------------------------\par" & vbCrLf & _
'          "\pard\qc " & strTimeElapsed & vbCrLf & "}"
'      Else
'        StatMatricesFromGeoDatasets = strInputParamsReport & "-----------------------------------------\par" & vbCrLf & _
'          "NOTE:  Exited Function Prematurely at Step " & CStr(lngTimer) & "\par" & vbCrLf & _
'          "-----------------------------------------\par" & vbCrLf & _
'          "\pard\qc " & strTimeElapsed & vbCrLf & "}"
'      End If
  
     ' ********************************************************************************************
  
2492:       pProgressForm.cmdStop.Caption = "Close"
2493:       pProg.position = 0
2494:       psbar.HideProgressBar
2495:       Exit Do
2496:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
  
        
    
2501:     Set pBottleneckData = RunBottleneckInBatch(pBlock1Polygon, pBlock2Polygon, pCorridorPolygon, pMxDoc, _
        pApp, pProgressForm, pThisExt, pSegWorkOrder, pPointWorkOrder, pStatsWorkOrder, dblThresholdValue, pBlock1FClass, _
        pBlock2FClass, pCorridorFClass)
    
2505:     If BatchBailOutOfProgress(pProgressForm, pThisExt) Then
  
     ' ********************************************************************************************
      ' ALTERNATIVE CODE THAT SAVES PARTIAL TABLE, CHECKS IF ERRORS FOUND AND OUTPUTS RTF INSTEAD OF TEXT
'      strTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedRTF(theTimeBegan, Now, 7)
2510:       strResults = "Exited Function Prematurely at Step " & CStr(lngTimer)
      
2512:       strAnalysisReport = Replace(strAnalysisReport, "2 zzzResultsReport", "3 " & strResults, , , vbTextCompare)
      
2514:       datSubTimeEnd = Now
2515:       lngMillisecondsEnd = Linkages.MyGeneralOperations.GetTickCount
2516:       pBuffer.Value(lngNewTimeEndIndex) = Format(datSubTimeEnd, "ttttt, dddddd")
2517:       strAnalysisReport = Replace(strAnalysisReport, "zzzTimeFinished", Format(datSubTimeEnd, "ttttt, dddddd"), , , vbTextCompare)
    
2519:       strTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedFromMilliseconds(lngMillisecondsEnd - lngMilliseconds)
2520:       strAnalysisReport = Replace(strAnalysisReport, "zzzTimeElapsed", strTimeElapsed, , , vbTextCompare)
      
2522:       strTimeElapsed = Replace(strTimeElapsed, "...", "")
2523:       pBuffer.Value(lngNewTimeElapsedIndex) = strTimeElapsed
2524:       pBuffer.Value(lngNewResultsIndex) = strResults
      
2526:       pTableCursor.InsertRow pBuffer
2527:       strFullAnalysisReport = strFullAnalysisReport & strAnalysisReport
      
'      If booFoundProblems Then
'
''        StatMatricesFromGeoDatasets = strInputParamsReport & "-----------------------------------------\par" & vbCrLf & _
'          "NOTE:  Exited Function Prematurely at Step " & CStr(lngTimer) & "\par" & vbCrLf & _
'            Replace(strProblemReport, "zzzRunningProblemszzz", strRunningProblems) & vbCrLf & _
'          "-----------------------------------------\par" & vbCrLf & _
'          "\pard\qc " & strTimeElapsed & vbCrLf & "}"
'      Else
'        StatMatricesFromGeoDatasets = strInputParamsReport & "-----------------------------------------\par" & vbCrLf & _
'          "NOTE:  Exited Function Prematurely at Step " & CStr(lngTimer) & "\par" & vbCrLf & _
'          "-----------------------------------------\par" & vbCrLf & _
'          "\pard\qc " & strTimeElapsed & vbCrLf & "}"
'      End If
  
     ' ********************************************************************************************
  
2545:       pProgressForm.cmdStop.Caption = "Close"
2546:       pProg.position = 0
2547:       psbar.HideProgressBar
2548:       Exit Do
2549:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
    
    
    
2554:     booBottleneckWorked = pBottleneckData.Element(0)
2555:     strResults = pBottleneckData.Element(1)
2556:     Set pFinalStatsArray = pBottleneckData.Element(2)
    
2558:     If booBottleneckWorked Then
2559:       strAnalysisReport = Replace(strAnalysisReport, "zzzResultsReport", strResults, , , vbTextCompare)
2560:     Else
2561:       strAnalysisReport = Replace(strAnalysisReport, "2 zzzResultsReport", "3 " & strResults, , , vbTextCompare)
2562:     End If
    
2564:     datSubTimeEnd = Now
2565:     lngMillisecondsEnd = Linkages.MyGeneralOperations.GetTickCount
2566:     pBuffer.Value(lngNewTimeEndIndex) = Format(datSubTimeEnd, "ttttt, dddddd")
2567:     strAnalysisReport = Replace(strAnalysisReport, "zzzTimeFinished", Format(datSubTimeEnd, "ttttt, dddddd"), , , vbTextCompare)
  
2569:     strTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedFromMilliseconds(lngMillisecondsEnd - lngMilliseconds)
2570:     strAnalysisReport = Replace(strAnalysisReport, "zzzTimeElapsed", strTimeElapsed, , , vbTextCompare)
    
2572:     strTimeElapsed = Replace(strTimeElapsed, "...", "")
2573:     pBuffer.Value(lngNewTimeElapsedIndex) = strTimeElapsed
2574:     pBuffer.Value(lngNewResultsIndex) = strResults
    
2576:     pTableCursor.InsertRow pBuffer
2577:     strFullAnalysisReport = strFullAnalysisReport & strAnalysisReport
2578:     Set pRow = pCursor.NextRow
    
    
    ' UPDATE PROGRESS METER ==================================================================
2582:     DoEvents
2583:     theProgressSpecific = strMessage
2584:     theDetailedDescription = theDetailedDescription & _
      Replace(" --> Results of Step " & Format(lngAnalysisCounter, "#,##0") & ": " & strResults & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]", "Bottleneck Analysis successfully", _
      "Step successfully", , , vbTextCompare) & vbCrLf & vbCrLf
2588:     pProgressForm.ResetShortTimer
2589:     pProgressForm.txtDetails.Text = theDetailedDescription
2590:     pProgressForm.txtDetails.SelStart = Len(theDetailedDescription)

2592:     pProgressForm.Refresh
2593:     pProgressForm.icnProgressLine.Refresh
2594:     DoEvents

    ' ---- END PROGRESS METER STUFF --------------------------------
  
2598:     pProg.Step
2599:     DoEvents
2600:     pTableCursor.Flush
    
2602:   Loop
  
2604:   pTableCursor.Flush
  
  ' SNIPPET TO MAKE TABLE AND ADD TO MAP DOCUMENT
  Dim pNewStandaloneTable As IStandaloneTable
  Dim pTableWindow2 As ITableWindow2
  Dim pStandaloneTableCollection As IStandaloneTableCollection

2611:   Set pNewStandaloneTable = New StandaloneTable
2612:   Set pNewStandaloneTable.Table = pTable

2614:   Set pTableWindow2 = New TableWindow

2616:   With pTableWindow2
2617:     Set .StandaloneTable = pNewStandaloneTable
2618:     Set .Application = pApp
2619:     .TableSelectionAction = esriSelectFeatures
2620:     .ShowAliasNamesInColumnHeadings = True
2621:     .ShowSelected = False
2622:     .Show True
2623:   End With
2624:   Set pStandaloneTableCollection = pMxDoc.FocusMap
2625:   pStandaloneTableCollection.AddStandaloneTable pNewStandaloneTable

2627:   pMxDoc.UpdateContents
  
  Dim strBaseString As String
2630:   strBaseString = strBaseString & "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fnil\fcharset0 Arial;}{\f1\fnil\fcharset0 MS Sans Serif;}{\f2\fnil\fcharset2 Symbol;}}" & vbNewLine
2631:   strBaseString = strBaseString & "{\colortbl ;\red128\green0\blue0;\red0\green77\blue187;\red255\green0\blue0;\red165\green165\blue165;}" & vbNewLine
2632:   strBaseString = strBaseString & "\viewkind4\uc1\pard\sl276\slmult1\qc\lang9\b\fs16 Batch Analysis Report" & vbNewLine
2633:   strBaseString = strBaseString & "\par -------------------------------------------------------\b0 " & vbNewLine
2634:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\b Batch Parameters Table:\b0   zzzTableName (n = zzzCount records)" & vbNewLine
2635:   strBaseString = strBaseString & "\par \pard{\pntext\f2\'B7\tab}{\*\pn\pnlvlblt\pnf2\pnindent0{\pntxtb\'B7}}\fi-360\li720\sl276\slmult1\b Wildland Block 1 Source Field:  \cf1\b0 zzzBlock1Field\cf0 " & vbNewLine
2636:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Wildland Block 2 Source Field:\b0   \cf1 zzzBlock2Field\cf0 " & vbNewLine
2637:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Corridor Source Field:  \cf1\b0 zzzCorridorField\cf0 " & vbNewLine
2638:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Threshold Distance Field:\b0   \cf1 zzzDistField\cf0 " & vbNewLine
2639:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segments Name Field:\b0  \cf1 zzzSegNameField\cf0 " & vbNewLine
2640:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segments Workspace Field: \b0  \cf1 zzzSegWSField\cf0 " & vbNewLine
2641:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Points Name Field:   \cf1\b0 zzzPointsNameField\cf0 " & vbNewLine
2642:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Points Workspace Field:   \cf1\b0 zzzPointsWSField\cf0 " & vbNewLine
2643:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Bottleneck Statistics Name Field:   \cf1\b0 zzzBottleneckStatsNameField\cf0 " & vbNewLine
2644:   strBaseString = strBaseString & "\par \b{\pntext\f2\'B7\tab}Output Segment Statistics Name Field:   \cf1\b0 zzzSegmentStatsNameField\cf0 " & vbNewLine
2645:   strBaseString = strBaseString & "\par \pard{\pntext\f2\'B7\tab}{\*\pn\pnlvlblt\pnf2\pnindent0{\pntxtb\'B7}}\fi-360\li720\sa200\sl276\slmult1\b Output Statistics Workspace Field:  \cf1\b0 zzzStatsWSField\cf0 " & vbNewLine

2647:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\qc ----------------------------------------------------------" & vbNewLine
2648:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\b Analysis Results Table:\b0 " & vbNewLine
2649:   strBaseString = strBaseString & "\par \pard{\pntext\f2\'B7\tab}{\*\pn\pnlvlblt\pnf2\pnindent0{\pntxtb\'B7}}\fi-360\li720\sl276\slmult1\cf1 zzzOutputTable...\cf0 " & vbNewLine
2650:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\qc ----------------------------------------------------------" & vbNewLine

2652:   strBaseString = strBaseString & "\par \pard\sa200\sl276\slmult1\ul\b Individual Analyses....\ulnone\b0 " & vbNewLine
2653:   strBaseString = strBaseString & "zzzAnalysisReport" & vbNewLine
2654:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\qc ----------------------------------------------------------" & vbNewLine
2655:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\qc zzzTimeElapsed" & vbNewLine
2656:   strBaseString = strBaseString & "\par \pard\sl276\slmult1\qc\i\fs14 Tool Version " & CStr(App.Major) & "." & CStr(App.Minor) & "." & CStr(App.Revision) & vbNewLine
2657:   strBaseString = strBaseString & "\par }" & vbNewLine

2659:   strBaseString = Replace(strBaseString, "zzzTableName", pStTable.Name, , , vbTextCompare)
2660:   Set pField = pSelFields.Element(0)
2661:   strBaseString = Replace(strBaseString, "zzzBlock1Field", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2662:   Set pField = pSelFields.Element(1)
2663:   strBaseString = Replace(strBaseString, "zzzBlock2Field", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2664:   Set pField = pSelFields.Element(2)
2665:   strBaseString = Replace(strBaseString, "zzzCorridorField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2666:   Set pField = pSelFields.Element(3)
2667:   strBaseString = Replace(strBaseString, "zzzDistField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2668:   Set pField = pSelFields.Element(4)
2669:   strBaseString = Replace(strBaseString, "zzzSegNameField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2670:   Set pField = pSelFields.Element(5)
2671:   strBaseString = Replace(strBaseString, "zzzSegWSField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2672:   Set pField = pSelFields.Element(6)
2673:   strBaseString = Replace(strBaseString, "zzzPointsNameField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2674:   Set pField = pSelFields.Element(7)
2675:   strBaseString = Replace(strBaseString, "zzzPointsWSField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2676:   Set pField = pSelFields.Element(8)
2677:   strBaseString = Replace(strBaseString, "zzzBottleneckStatsNameField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2678:   Set pField = pSelFields.Element(9)
2679:   strBaseString = Replace(strBaseString, "zzzSegmentStatsNameField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
2680:   Set pField = pSelFields.Element(10)
2681:   strBaseString = Replace(strBaseString, "zzzStatsWSField", strPrefix & pField.Name & strSuffix, , , vbTextCompare)
    
2683:   strBaseString = Replace(strBaseString, "zzzCount", Format(lngFeatureCount, "#,##0"), , , vbTextCompare)

2685:   strBaseString = Replace(strBaseString, "zzzAnalysisReport", strFullAnalysisReport, , , vbTextCompare)
  
2687:   Set pDataset = pTable
2688:   strBaseString = Replace(strBaseString, "zzzOutputTable", pDataset.BrowseName & ", \cf0\b in \b0\cf1 " & _
        Replace(pDataset.Workspace.PathName, "\", "\\"), , , vbTextCompare)

  Dim strFullTimeElapsed As String
2692:   strFullTimeElapsed = Linkages.MyGeneralOperations.ReturnTimeElapsedRTF(datTimeBegan, Now, 7)
2693:   strBaseString = Replace(strBaseString, "zzzTimeElapsed", strFullTimeElapsed, , , vbTextCompare)

2695:   Unload pProgressForm
2696:   Set pProgressForm = Nothing

2698:   PerformBatchBottleneckAnalysis = strBaseString

2700:   psbar.HideProgressBar
2701:   pProg.position = 0

2703:   GoTo ClearMemory
  Exit Function
ErrorHandler:
  HandleError True, "PerformBatchBottleneckAnalysis " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

2708:   PerformBatchBottleneckAnalysis = "Error in analysis..."


ClearMemory:
2712:   pProg.position = 0
2713:   psbar.HideProgressBar
  
2715:   Set pApp = Nothing
2716:   Set pMxDoc = Nothing
2717:   Set pThisExt = Nothing
2718:   Set psbar = Nothing
2719:   Set pProg = Nothing
2720:   Set pStTable = Nothing
2721:   Set pSourceTable = Nothing
2722:   Set pSelFields = Nothing
2723:   Set pField = Nothing
2724:   Erase lngFieldIndices
2725:   Set pOutputArray = Nothing
2726:   Set pTable = Nothing
2727:   Set pFieldNamePairs = Nothing
2728:   Set pSubArray = Nothing
2729:   Set pBlock1FClass = Nothing
2730:   Set pBlock2FClass = Nothing
2731:   Set pCorridorFClass = Nothing
2732:   Set pSegWS = Nothing
2733:   Set pPointWS = Nothing
2734:   Set pStatsWS = Nothing
2735:   Set pCursor = Nothing
2736:   Set pRow = Nothing
2737:   Set pBlock1Polygon = Nothing
2738:   Set pBlock2Polygon = Nothing
2739:   Set pCorridorPolygon = Nothing
2740:   Set pBuffer = Nothing
2741:   Set pTableCursor = Nothing
2742:   Set pDataset = Nothing
2743:   Set pTempFCursor = Nothing
2744:   Set pTempFeature = Nothing
2745:   Set pBottleneckData = Nothing
2746:   Set pFinalStatsArray = Nothing
2747:   Set pProgressForm = Nothing
2748:   Set pSegWorkOrder = Nothing
2749:   Set pPointWorkOrder = Nothing
2750:   Set pStatsWorkOrder = Nothing
2751:   Set pNewStandaloneTable = Nothing
2752:   Set pTableWindow2 = Nothing
2753:   Set pStandaloneTableCollection = Nothing




End Function

Public Function CheckIfFeatureClassesIdentical(pFClass1 As IFeatureClass, pFClass2 As IFeatureClass) As Boolean
  On Error GoTo ErrorHandler
  
2763:   If pFClass1 Is Nothing Or pFClass2 Is Nothing Then
2764:     CheckIfFeatureClassesIdentical = False
2765:   Else
2766:     CheckIfFeatureClassesIdentical = CBool(pFClass1 Is pFClass2)
2767:   End If

  Exit Function
ErrorHandler:
  HandleError True, "CheckIfFeatureClassesIdentical " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CheckIfFeatureClassesIntersect(pFClass1 As IFeatureClass, pFClass2 As IFeatureClass) As Boolean
  On Error GoTo ErrorHandler
  
  Dim pPolygon1 As IPolygon
  Dim pPolygon2 As IPolygon
  Dim pTopoOp As ITopologicalOperator
  Dim pRelOp As IRelationalOperator
  Dim pFCursor As IFeatureCursor
  Dim pFeature As IFeature
  
2784:   CheckIfFeatureClassesIntersect = False
  
  Dim pSpRef1 As ISpatialReference
  Dim pSpRef2 As ISpatialReference
  
2789:   If Not pFClass1 Is Nothing And Not pFClass2 Is Nothing Then
2790:     If pFClass1.FeatureCount(Nothing) > 0 And pFClass2.FeatureCount(Nothing) > 0 Then
2791:       Set pFCursor = pFClass1.Search(Nothing, False)
2792:       Set pFeature = pFCursor.NextFeature
2793:       Set pPolygon1 = pFeature.ShapeCopy
2794:       Set pSpRef1 = pPolygon1.SpatialReference
      
2796:       Set pFCursor = pFClass2.Search(Nothing, False)
2797:       Set pFeature = pFCursor.NextFeature
2798:       Set pPolygon2 = pFeature.ShapeCopy
2799:       Set pSpRef2 = pPolygon2.SpatialReference
      
2801:       If Not pPolygon1.IsEmpty And Not pPolygon2.IsEmpty Then
        
2803:         If Not Linkages.MyGeneralOperations.CompareSpatialReferences(pSpRef1, pSpRef2) Then
2804:           pPolygon2.Project pSpRef1
2805:         End If
        
2807:         Set pTopoOp = pPolygon1
2808:         pTopoOp.Simplify
2809:         Set pTopoOp = pPolygon2
2810:         pTopoOp.Simplify
            
2812:         Set pRelOp = pPolygon1
2813:         CheckIfFeatureClassesIntersect = Not pRelOp.Disjoint(pPolygon2)
2814:       End If
2815:     End If
2816:   End If
  
2818:   GoTo ClearMemory
  Exit Function
ErrorHandler:
  HandleError True, "CheckIfFeatureClassesIntersect " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


ClearMemory:
2825:   Set pPolygon1 = Nothing
2826:   Set pPolygon2 = Nothing
2827:   Set pTopoOp = Nothing
2828:   Set pRelOp = Nothing
2829:   Set pFCursor = Nothing
2830:   Set pFeature = Nothing



End Function

Public Function CheckIfDuplicateName(pCurrentNames As Collection, strName As String) As Boolean
  On Error GoTo ErrorHandler

  
2840:   If Linkages.MyGeneralOperations.CheckCollectionForKey(pCurrentNames, strName) Then
2841:     CheckIfDuplicateName = True
2842:   Else
2843:     pCurrentNames.Add True, strName
2844:     CheckIfDuplicateName = False
2845:   End If
  

  Exit Function
ErrorHandler:
  HandleError True, "CheckIfDuplicateName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CheckIfNameInUse(pWS As IFeatureWorkspace, strName As String, booIsFeatureClass As Boolean) As Boolean
  On Error GoTo ErrorHandler
  
'  Dim pFClass As IFeatureClass
'  Dim pTable As ITable
  
'  If booIsFeatureClass Then
'    Set pFClass = pWS.OpenFeatureClass(strName)
'    If pFClass Is Nothing Then
'      CheckIfNameInUse = False
'    Else
'      CheckIfNameInUse = True
'    End If
'  Else
'    Set pTable = pWS.OpenTable(strName)
'    If pTable Is Nothing Then
'      CheckIfNameInUse = False
'    Else
'      CheckIfNameInUse = True
'    End If
'  End If
  
  Dim pGenWS As IWorkspace
2876:   Set pGenWS = pWS
  Dim pWS2 As IWorkspace2
  
  Dim strFullPath As String
2880:   If pGenWS.Type = esriFileSystemWorkspace Then

2882:     strFullPath = pGenWS.PathName
2883:     If Right(strFullPath, 1) <> "\" Then strFullPath = strFullPath & "\"
2884:     strFullPath = strFullPath & strName
2885:     strFullPath = Linkages.aml_func_mod.ClipExtension(strFullPath)
    
2887:     CheckIfNameInUse = Linkages.aml_func_mod.ExistFileDir(strFullPath)
2888:     If Not CheckIfNameInUse Then CheckIfNameInUse = Linkages.aml_func_mod.ExistFileDir(strFullPath & ".shp")
2889:     If Not CheckIfNameInUse Then CheckIfNameInUse = Linkages.aml_func_mod.ExistFileDir(strFullPath & ".dbf")
      
2891:   Else
    
2893:     Set pWS2 = pWS
2894:     CheckIfNameInUse = pWS2.NameExists(esriDTFeatureClass, strName)
2895:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTTable, strName)
2896:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTFeatureDataset, strName)
2897:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTGeometricNetwork, strName)
2898:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTRasterDataset, strName)
2899:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTRasterCatalog, strName)
2900:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTTerrain, strName)
2901:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTRelationshipClass, strName)
2902:     If Not CheckIfNameInUse Then CheckIfNameInUse = pWS2.NameExists(esriDTTopology, strName)
    
2904:   End If
  
2906:   GoTo ClearMemory
  Exit Function
ErrorHandler:
'  HandleError True, "CheckIfNameInUse " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl) & vbCrLf & _
      "strName = " & strName & vbCrLf & _
      "pGenWS.Pathname = " & pGenWS.PathName & vbCrLf & _
      "pGenWS.Type = esriFileSystemWorkspace = " & CBool(pGenWS.Type = esriFileSystemWorkspace), err.Number, err.Source, err.Description, 4
2913:   CheckIfNameInUse = False
  
ClearMemory:
2916:   Set pGenWS = Nothing
2917:   Set pWS2 = Nothing
  
End Function

Public Function CheckIfNameValid(pWS As IWorkspace, strName As String, strReason As String) As Boolean
  On Error GoTo ErrorHandler
  
  Dim strLetters As String
2925:   strLetters = "abcdefghijklmnopqrstuvwxyz"
  Dim strFullCharacters As String
2927:   strFullCharacters = strLetters & "0123456789_"
  Dim lngIndex As Long
  Dim strChar As String
  Dim strFirstCharacter As String
  
2932:   strName = Trim(strName)
2933:   If strName = "" Then
2934:     CheckIfNameValid = False
2935:     strReason = "No Name..."
2936:   ElseIf pWS.Type = esriFileSystemWorkspace Then
    ' SHAPEFILES CAN BE NAMED ANYTHING
2938:     CheckIfNameValid = True
2939:     strReason = ""
2940:   Else
    ' BEGIN WITH ASSUMPTION NAME IS VALID, THEN SET TO FALSE IF ANY PROBLEMS FOUND
2942:     CheckIfNameValid = True
2943:     strReason = ""
2944:     If StrComp(Left(strName, 4), "gdb_") = 0 Then
2945:       CheckIfNameValid = False
2946:       strReason = "Geodatabase Feature class name cannot start with ""gdb_""..."
2947:     ElseIf StrComp(Left(strName, 4), "sde_") = 0 Then
2948:       CheckIfNameValid = False
2949:       strReason = "Geodatabase Feature class name cannot start with ""sde_""..."
2950:     ElseIf StrComp(Left(strName, 6), "delta_") = 0 Then
2951:       CheckIfNameValid = False
2952:       strReason = "Geodatabase Feature class name cannot start with ""delta_""..."
2953:     Else
2954:       strFirstCharacter = Left(strName, 1)
2955:       If InStr(1, strLetters, strFirstCharacter, vbTextCompare) = 0 Then
2956:         CheckIfNameValid = False
2957:         strReason = "Geodatabase Feature class names must start with a letter..."
2958:       Else
2959:         For lngIndex = 1 To Len(strName)
2960:           strChar = Mid(strName, lngIndex, 1)
2961:           If InStr(1, strFullCharacters, strChar, vbTextCompare) = 0 Then
2962:             CheckIfNameValid = False
2963:             strReason = "Geodatabase Feature class names must contain only letters, numbers or the '_' (underscore) character..."
2964:             Exit For
2965:           End If
2966:         Next lngIndex
2967:       End If
2968:     End If
2969:   End If

  Exit Function
ErrorHandler:
'  HandleError True, "CheckIfNameValid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

2975:   CheckIfNameValid = False
2976:   strReason = "CheckIfNameValid Failed:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"
End Function

Public Function ReturnFeatureClassFromString(strPath As String, strFailureReason As String, _
  pMxDoc As IMxDocument, lngFeatureCount As Long) As IFeatureClass
  On Error GoTo ErrorHandler


2985:   Set ReturnFeatureClassFromString = Nothing
2986:   strFailureReason = "Unknown reason"
  
  Dim strAdjustedPath As String
  Dim strFCLassName As String
  Dim strWSPath As String
  
  Dim pFClass As IFeatureClass
  Dim pFLayer As IFeatureLayer
  Dim pFeatWS As IFeatureWorkspace
  Dim pWS As IWorkspace
  Dim pWsFact As IWorkspaceFactory
  Dim strWSError As String
  
  '  return failure if empty string
3000:   If Trim(strPath) = "<-- No Value -->" Or Trim(strPath) = "" Then
3001:     Set ReturnFeatureClassFromString = Nothing
3002:     strFailureReason = "No Name Specified"
3003:     GoTo ClearMemory
3004:   End If
  
  ' FIRST JUST RETURN LAYER IF THERE IS ONE WITH THE SPECIFIED NAME
3007:   Set pFLayer = Linkages.MyGeneralOperations.ReturnLayerByName(strPath, pMxDoc.FocusMap)
3008:   If Not pFLayer Is Nothing Then
3009:     If Not pFLayer.Valid Then
3010:       strFailureReason = "Invalid Feature Layer"
3011:       Set ReturnFeatureClassFromString = Nothing
3012:       lngFeatureCount = -999
3013:       GoTo ClearMemory
3014:     Else
3015:       Set pFClass = pFLayer.FeatureClass
3016:       lngFeatureCount = pFClass.FeatureCount(Nothing)
3017:       Set ReturnFeatureClassFromString = pFClass
3018:       strFailureReason = ""
3019:       GoTo ClearMemory
3020:     End If
    
3022:   Else
    
    ' JUST RETURN FAILURE REASON IF THERE IS NO SOURCE PATH
3025:     If InStr(1, strPath, "\", vbTextCompare) = 0 And InStr(1, strPath, "/", vbTextCompare) = 0 Then
3026:       strFailureReason = "Unable to find layer '" & strPath & "' in focal map!"
3027:       Set ReturnFeatureClassFromString = Nothing
3028:       lngFeatureCount = -999
3029:       GoTo ClearMemory
3030:     Else
      
      ' CLIP EXTENSION
3033:       strAdjustedPath = Linkages.aml_func_mod.ClipExtension(strPath)
      
      ' CONVERT FORWARD SLASHES TO BACKWARD SLASHES
3036:       strAdjustedPath = Replace(strAdjustedPath, "/", "\")
      
      ' REPLACE DOUBLE SLASHES WITH SINGLE, SUCH AS MIGHT GET WITH A PYTHON PATHNAME
3039:       Do Until InStr(1, strAdjustedPath, "\\", vbTextCompare) = 0
3040:         strAdjustedPath = Replace(strAdjustedPath, "\\", "\")
3041:       Loop
      
      ' GET FEATURE CLASS NAME AND WORKSPACE
3044:       strFCLassName = Linkages.aml_func_mod.ReturnFilename(strAdjustedPath)
3045:       strWSPath = Linkages.aml_func_mod.ReturnDir(strAdjustedPath)
      
3047:       Set pWS = ReturnWorkspaceFromPath(strWSPath, strWSError)
3048:       If pWS Is Nothing Then
3049:         strFailureReason = strWSError
3050:         Set ReturnFeatureClassFromString = Nothing
3051:         lngFeatureCount = -999
3052:         GoTo ClearMemory
3053:       Else
3054:         Set pFeatWS = pWS
        
3056:         Set ReturnFeatureClassFromString = ReturnFeatureClassInWS(pFeatWS, strFCLassName, strWSPath, strFailureReason)
        
        ' MAKE SURE POLYGON FEATURE CLASS!
3059:         If Not ReturnFeatureClassFromString.ShapeType = esriGeometryPolygon Then
3060:           Set ReturnFeatureClassFromString = Nothing
3061:           strFailureReason = "Not a polygon feature class..."
3062:         End If
        
3064:       End If      ' END CHECKING IF WORKSPACE IS VALID
3065:     End If        ' END CHECKING IF PATH IS HARD DRIVE LOCATION (I.E. IF SLASHES EXIST IN PATH NAME)
3066:   End If          ' END CHECKING IF FEATURE LAYER EXISTS
 
3068:   If ReturnFeatureClassFromString Is Nothing Then
3069:     lngFeatureCount = -999
3070:   Else
3071:     strFailureReason = ""
3072:     lngFeatureCount = ReturnFeatureClassFromString.FeatureCount(Nothing)
3073:   End If
  
3075:   GoTo ClearMemory

  Exit Function
ErrorHandler:
  HandleError True, "ReturnFeatureClassFromString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
3080:   Set ReturnFeatureClassFromString = Nothing
3081:   strFailureReason = "ReturnFeatureClassFromString Failed:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"
      
ClearMemory:
3085:   Set pFClass = Nothing
3086:   Set pFLayer = Nothing
3087:   Set pFeatWS = Nothing
3088:   Set pWS = Nothing
3089:   Set pWsFact = Nothing
  
End Function

Public Function ReturnFeatureClassInWS(pFeatWS As IFeatureWorkspace, strFCLassName As String, _
  strWsName As String, strFailureReason As String) As IFeatureClass
  On Error GoTo ErrorHandler
  
3097:   Set ReturnFeatureClassInWS = pFeatWS.OpenFeatureClass(strFCLassName)
  
3099:   GoTo ClearMemory
  Exit Function
  
ErrorHandler:
3103:   Set ReturnFeatureClassInWS = Nothing
3104:   strFailureReason = "Unable to open feature class '" & strFCLassName & "'" & vbCrLf & _
                     "    ...Searching workspace '" & strWsName & "'..."
ClearMemory:
  

End Function

Public Function ReturnWorkspaceFromPath(strPath As String, strErrorReason As String) As IWorkspace
  On Error GoTo ErrorHandler
  
  Dim pWsFact As IWorkspaceFactory
3115:   Set pWsFact = New ShapefileWorkspaceFactory
  
3117:   If Right(strPath, 1) = "\" Or Right(strPath, 1) = "/" Then
3118:     strPath = Left(strPath, Len(strPath) - 1)
3119:   End If
  
  ' CHECK FOR DIFFERENT WORKSPACE TYPES
3122:   If InStr(1, strPath, ".sdc", vbTextCompare) > 0 Then
3123:     If Linkages.aml_func_mod.ExistFileDir(strPath) Then
3124:       Set pWsFact = New SDCWorkspaceFactory
'      strPath = strPath & "\"
'      strPath = Replace(strPath, ".sdc", "", , , vbTextCompare)
3127:       strErrorReason = "Unable to open SDC Workspace '" & Replace(strPath, "\", "\\") & "'..."
3128:     Else
3129:       strErrorReason = "Unable to find SDC Workspace '" & Replace(strPath, "\", "\\") & "'..."
3130:       Set ReturnWorkspaceFromPath = Nothing
3131:       GoTo ClearMemory
3132:     End If
3133:   ElseIf InStr(1, strPath, ".gdb", vbTextCompare) > 0 Then
3134:     If Linkages.aml_func_mod.ExistFileDir(strPath) Then
3135:       Set pWsFact = New FileGDBWorkspaceFactory
3136:       strErrorReason = "Unable to open File Geodatabase '" & Replace(strPath, "\", "\\") & "'..."
3137:     Else
3138:       strErrorReason = "Unable to find File Geodatabase '" & Replace(strPath, "\", "\\") & "'..."
3139:       Set ReturnWorkspaceFromPath = Nothing
3140:       GoTo ClearMemory
3141:     End If
3142:   ElseIf InStr(1, strPath, ".mdb", vbTextCompare) > 0 Then
3143:     If Linkages.aml_func_mod.ExistFileDir(strPath) Then
3144:       Set pWsFact = New AccessWorkspaceFactory
3145:       strErrorReason = "Unable to open Personal Geodatabase '" & Replace(strPath, "\", "\\") & "'..."
3146:     Else
3147:       strErrorReason = "Unable to find Personal Geodatabase '" & Replace(strPath, "\", "\\") & "'..."
3148:       Set ReturnWorkspaceFromPath = Nothing
3149:       GoTo ClearMemory
3150:     End If
3151:   Else
3152:     If Linkages.aml_func_mod.ExistFileDir(strPath) Then
3153:       Set pWsFact = New ShapefileWorkspaceFactory
3154:       strErrorReason = "Unable to open Folder '" & Replace(strPath, "\", "\\") & "'..."
3155:     Else
3156:       strErrorReason = "Unable to find Folder '" & Replace(strPath, "\", "\\") & "'..."
3157:       Set ReturnWorkspaceFromPath = Nothing
3158:       GoTo ClearMemory
3159:     End If
3160:   End If
  
3162:   Set ReturnWorkspaceFromPath = pWsFact.OpenFromFile(strPath, 0)
3163:   strErrorReason = ""
  
3165:   GoTo ClearMemory
  Exit Function

ErrorHandler:
3169:   Set ReturnWorkspaceFromPath = Nothing
  
ClearMemory:
3172:   Set pWsFact = Nothing
  

End Function

Public Function BatchBailOutOfProgress(frmProgressDialog As Object, pThisExt As Linkages.clsExtension) As Boolean
  On Error GoTo ErrorHandler
    
  Dim frmProgress As Linkages.frmJenProgressMeter
3181:   Set frmProgress = frmProgressDialog
  
3183:   pThisExt.ProgressDialogAutoClose = (frmProgress.chkClose.Value = 1)
3184:   pThisExt.ProgressDialogSetExpanded = (frmProgress.SetExpanded)
  
'  MsgBox "Checking Autoclose Stuff: " & vbCrLf & _
      "Check-Box checked? " & CStr(frmProgress.chkClose.Value = 1) & vbCrLf & _
      "Extension Property Checked? " & CStr(pExt.ProgressDialogAutoClose)
      
  
  ' PROGRESS METER STUFF --------------------------------------------------------
3192:   BatchBailOutOfProgress = Not frmProgress.ShouldContinue
3193:   If BatchBailOutOfProgress Then
    Dim theDateString As String
    Dim theElapsedTimeString As String
    
3197:     theDateString = Format(Now, "long date") & ";  " & Format(Now, "long time")
3198:     Screen.MousePointer = vbDefault
    Dim theTimeBegan As Date
    Dim theTimeEnd As Date
3201:     theTimeBegan = frmProgress.ProgBeginTime
3202:     theTimeEnd = Now

3204:     theElapsedTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)
  
    Dim theReport As String
3207:     theReport = "Operation cancelled prematurely at " & theDateString & vbCrLf & _
        "---------------------------------------------------------" & vbCrLf & theElapsedTimeString
    
    Dim strCurrentReport As String
3211:     strCurrentReport = frmProgress.txtDetails.Text
3212:     frmProgress.txtDetails.Text = strCurrentReport & vbCrLf & theReport
    
3214:     If frmProgress.chkClose.Value = 1 Then
    '  Unload frmProgress
    '  Set frmProgress = Nothing
3217:       frmProgress.Frame.Visible = False
3218:     End If
    
    Exit Function
3221:   End If

3223:   Set frmProgress = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "BatchBailOutOfProgress " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

3229:   Set frmProgress = Nothing
  
End Function

Public Function MakePointFeatureClassBatch(pWS As IWorkspace, strName As String, _
  pDataArray As esriSystem.IVariantArray, pSpRef As ISpatialReference, booSucceeded As Boolean, _
  pCorridorPolygon As IPolygon, dblThresholdDistance As Double, _
  pBlock1FClass As IFeatureClass, _
  pBlock2FClass As IFeatureClass, _
  pCorridorFClass As IFeatureClass, _
  strUnits As String, pMxDoc As IMxDocument) As String
  
  On Error GoTo ErrorHandler
  
3243:   MakePointFeatureClassBatch = "Failed, Unknown Reason..."
3244:   booSucceeded = False
  
  Dim strReport As String
  Dim strFName As String
  Dim lngIndex As Long
  
3250:   strName = Linkages.aml_func_mod.ClipExtension2(strName)
  
3252:   If Linkages.MyGeneralOperations.CheckIfFeatureClassExists(pWS, strName) Then
3253:     MakePointFeatureClassBatch = "Feature class '" & strName & "' already exists.  No new point feature class created..."
3254:     booSucceeded = False
3255:     GoTo ClearMemory
3256:   End If
  
  Dim pFieldArray As esriSystem.IVariantArray
3259:   Set pFieldArray = New esriSystem.VarArray
  
  Dim lngDataType As JenDatasetTypes
  Dim strWSPath As String
3263:   strWSPath = pWS.PathName
3264:   If pWS.Type = esriFileSystemWorkspace Then
3265:     lngDataType = ENUM_Shapefile
3266:   Else
3267:     If InStr(1, strWSPath, ".mdb", vbTextCompare) > 0 Then
3268:       lngDataType = ENUM_PersonalGDB
3269:     ElseIf InStr(1, strWSPath, ".gdb", vbTextCompare) > 0 Then
3270:       lngDataType = ENUM_FileGDB
3271:     Else
3272:       MakePointFeatureClassBatch = "Invalid Workspace Type:  This function can only create shapefiles, " & _
          "file geodatabase feature classes and personal geodatabase feature classes.  No new point feature class created..."
3274:       booSucceeded = False
3275:       GoTo ClearMemory
3276:     End If
3277:   End If
  
  Dim booDoThreshold As Boolean
3280:   booDoThreshold = dblThresholdDistance >= 0
  
    ' MAKE FIELDS
    ' MAKE X_Coord FIELD
  Dim pUniqueIDField As IField
  Dim pUniqueIDieldEdit As IFieldEdit
3286:   Set pUniqueIDField = New Field
3287:   Set pUniqueIDieldEdit = pUniqueIDField
3288:   With pUniqueIDieldEdit
3289:     .Name = "Unique_ID"
3290:     .Type = esriFieldTypeInteger
3291:   End With
3292:   pFieldArray.Add pUniqueIDField
  
    ' MAKE X_Coord FIELD
  Dim pPointsXField As IField
  Dim pPointsXFieldEdit As IFieldEdit
3297:   Set pPointsXField = New Field
3298:   Set pPointsXFieldEdit = pPointsXField
3299:   With pPointsXFieldEdit
3300:     .Name = "X_Coord"
3301:     .Type = esriFieldTypeDouble
3302:     .Precision = 16
3303:     .Scale = 8
3304:   End With
3305:   pFieldArray.Add pPointsXField
  
    ' MAKE Y_Coord FIELD
  Dim pPointsYField As IField
  Dim pPointsYFieldEdit As IFieldEdit
3310:   Set pPointsYField = New Field
3311:   Set pPointsYFieldEdit = pPointsYField
3312:   With pPointsYFieldEdit
3313:     .Name = "Y_Coord"
3314:     .Type = esriFieldTypeDouble
3315:     .Precision = 16
3316:     .Scale = 8
3317:   End With
3318:   pFieldArray.Add pPointsYField
  
    ' MAKE WIDTH FIELD
  Dim pPointsWidthField As IField
  Dim pPointsWidthFieldEdit As IFieldEdit
3323:   Set pPointsWidthField = New Field
3324:   Set pPointsWidthFieldEdit = pPointsWidthField
3325:   With pPointsWidthFieldEdit
3326:     .Name = "Width"
3327:     .Type = esriFieldTypeDouble
3328:     .Precision = 16
3329:     .Scale = 8
3330:   End With
3331:   pFieldArray.Add pPointsWidthField
  
    ' MAKE THRESHOLD FIELD
  Dim pPointsThresholdField As IField
  Dim pPointsThresholdFieldEdit As IFieldEdit
3336:   Set pPointsThresholdField = New Field
3337:   Set pPointsThresholdFieldEdit = pPointsThresholdField
3338:   With pPointsThresholdFieldEdit
3339:     .Name = "Threshold"
3340:     .Type = esriFieldTypeString
3341:     .length = 10
3342:   End With
3343:   pFieldArray.Add pPointsThresholdField
  
  Dim pPointsFClass As IFeatureClass
  
3347:   Set pPointsFClass = Linkages.MyGeneralOperations.CreateGeneralFeatureClass(esriGeometryPoint, pSpRef, _
      pWS.PathName, strName, lngDataType, pFieldArray, , , , , False, pCorridorPolygon.Envelope, _
      pDataArray.Count)
  
  ' FIND VARIABLE INDEX VALUES
  Dim lngPointsID As Long
  Dim lngPointsX As Long
  Dim lngPointsY As Long
  Dim lngPointsWidth As Long
  Dim lngThreshold As Long
3357:   lngPointsID = pPointsFClass.FindField("Unique_ID")
3358:   lngPointsX = pPointsFClass.FindField("X_Coord")
3359:   lngPointsY = pPointsFClass.FindField("Y_Coord")
3360:   lngPointsWidth = pPointsFClass.FindField("Width")
3361:   lngThreshold = pPointsFClass.FindField("Threshold")
  
  ' MAKE INSERT CURSOR
  Dim pPointsCur As IFeatureCursor
3365:   Set pPointsCur = pPointsFClass.Insert(True)
  Dim pPointsBuf As IFeatureBuffer
3367:   Set pPointsBuf = pPointsFClass.CreateFeatureBuffer
  
  Dim pPoint As IPoint
  Dim pValArray  As esriSystem.IDoubleArray
  Dim dblX As Double
  Dim dblY As Double
  Dim dblWidth As Double
  
  ' ADD DATA TO FEATURE CLASS
3376:   For lngIndex = 0 To pDataArray.Count - 1
3377:     Set pValArray = pDataArray.Element(lngIndex)
3378:     dblX = pValArray.Element(0)
3379:     dblY = pValArray.Element(1)
3380:     dblWidth = pValArray.Element(2)
    
3382:     Set pPoint = New Point
3383:     pPoint.PutCoords dblX, dblY
3384:     Set pPoint.SpatialReference = pSpRef
3385:     With pPointsBuf
3386:       Set .Shape = pPoint
3387:       .Value(lngPointsID) = lngIndex + 1
3388:       .Value(lngPointsX) = dblX
3389:       .Value(lngPointsY) = dblY
3390:       .Value(lngPointsWidth) = dblWidth
3391:       If booDoThreshold Then
3392:         If dblWidth > dblThresholdDistance Then
3393:           .Value(lngThreshold) = "ABOVE"
3394:         Else
3395:           .Value(lngThreshold) = "BELOW"
3396:         End If
3397:       Else
3398:         .Value(lngThreshold) = "N/A"
3399:       End If
3400:     End With
3401:     pPointsCur.InsertFeature pPointsBuf
3402:   Next lngIndex
  
3404:   pPointsCur.Flush
  
  '-----------------------------------------------------------------------------------------------------------------------
  ' WRITE METADATA =======================================================================================================
  '-----------------------------------------------------------------------------------------------------------------------
  Dim pDataset As IDataset
3410:   Set pDataset = pPointsFClass
  
  Dim pBlock1Dataset As IDataset
  Dim pBlock2Dataset As IDataset
  Dim pCorridorDataset As IDataset
  
3416:   Set pBlock1Dataset = pBlock1FClass
3417:   Set pBlock2Dataset = pBlock2FClass
3418:   Set pCorridorDataset = pCorridorFClass
  
  Dim pPropSet As IPropertySet
3421:   Set pPropSet = Linkages.Metadata_Functions.ReturnMetadataPropSetFromDataset(pDataset)
  
  ' SYNCHRONIZE METADATA
  Dim strResponse As String
3425:   strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
3426:   Debug.Print "Synchronization: " & strResponse
  
  ' WRITE ABSTRACT
  Dim strAbstract As String
3430:   strAbstract = "This dataset represents points along the bottleneck route.  This bottleneck route " & _
    "describes the path between the two habitat blocks '" & pBlock1Dataset.BrowseName & "' and '" & _
    pBlock2Dataset.BrowseName & "', within the corridor polygon '" & pCorridorDataset.BrowseName & "'., " & _
    "which follows the route with the widest narrow point."
3434:   strResponse = Linkages.Metadata_Functions.SetMetadataAbstract(pDataset, strAbstract)
3435:   Debug.Print "Saving Abstract: " & strResponse
  
  ' WRITE PURPOSE
  Dim strPurpose As String
3439:   strPurpose = "Point dataset of points along route with widest bottleneck, with corridor width values at each point."
3440:   strResponse = Linkages.Metadata_Functions.SetMetadataPurpose(pDataset, strPurpose)
3441:   Debug.Print "Saving Purpose: " & strResponse
  
  ' ADD KEYWORDS
  Dim pKeyWords As esriSystem.IStringArray
  Dim pIncludeThemeKeys As esriSystem.IStringArray
  Dim pIncludeSearchKeys As esriSystem.IStringArray
  Dim pIncludeDescKeys As esriSystem.IStringArray
  Dim pIncludeStratKeys As esriSystem.IStringArray
  Dim pIncludeThemeSlashThemekeys As esriSystem.IStringArray
  Dim pIncludePlaceKeys As esriSystem.IStringArray
  Dim pIncludeTemporalKeys As esriSystem.IStringArray
  
3453:   Set pIncludeThemeKeys = New esriSystem.strArray
3454:   Set pIncludeSearchKeys = New esriSystem.strArray
3455:   Set pIncludeDescKeys = New esriSystem.strArray
3456:   Set pIncludeStratKeys = New esriSystem.strArray
3457:   Set pIncludeThemeSlashThemekeys = New esriSystem.strArray
3458:   Set pIncludePlaceKeys = New esriSystem.strArray
3459:   Set pIncludeTemporalKeys = New esriSystem.strArray
    
  Dim pCombinedKeyWords As esriSystem.IStringArray
3462:   pIncludeThemeKeys.Add "Bottleneck"
3463:   pIncludeThemeKeys.Add "Width"
3464:   pIncludeThemeKeys.Add "Corridor"
3465:   pIncludeThemeKeys.Add pBlock1Dataset.BrowseName
3466:   pIncludeThemeKeys.Add pBlock2Dataset.BrowseName
3467:   pIncludeThemeKeys.Add pCorridorDataset.BrowseName
  
3469:   For lngIndex = 0 To pIncludeThemeKeys.Count - 1
3470:     pIncludeSearchKeys.Add pIncludeThemeKeys.Element(lngIndex)
3471:     pIncludeDescKeys.Add pIncludeThemeKeys.Element(lngIndex)
3472:   Next lngIndex
  
' EXTRACT ANY EXISTING KEYWORDS FROM CURRENT BLOCK OR CORRIDOR FCLASSES
3475:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock1FClass, _
      pKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
3479:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock2FClass, _
      pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
3483:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pCorridorFClass, _
      pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
  
3488:   strResponse = Linkages.Metadata_Functions.SetMetadataKeyWords(pDataset, pIncludeThemeKeys, pIncludeSearchKeys, _
        pIncludeDescKeys, pIncludeStratKeys, pIncludeThemeSlashThemekeys, pIncludePlaceKeys, pIncludeTemporalKeys)
  
  ' SET PROCESS STEP
  Dim strDescription As String
3493:   strDescription = "Generated points along bottleneck route, connecting '" & pBlock1Dataset.BrowseName & "' to '" & _
      pBlock1Dataset.BrowseName & "' along corridor '" & pCorridorDataset.BrowseName & "' following the path with the widest possible " & _
      "pinch points."
3496:   strResponse = Linkages.Metadata_Functions.AddNewLineageStep(pDataset, strDescription, Now, JenMetadata_Processor, _
      "", True)
  
  ' ADD NEW GEOPROCESSING EVENT
3500:   strResponse = Linkages.Metadata_Functions.AddNewGeoProcStep(pDataset, _
      "[!!!NOTE - This command is not Python code!!!]:  Generate Bottleneck Point Feature Class in " & _
      "Batch Bottleneck Analysis (" & _
      "[Wildland Block 1 = " & pBlock1Dataset.Workspace.PathName & "\" & pBlock1Dataset.BrowseName & "], " & _
      "[Wildland Block 2 = " & pBlock2Dataset.Workspace.PathName & "\" & pBlock2Dataset.BrowseName & "], " & _
      "[Corridor = " & pCorridorDataset.Workspace.PathName & "\" & pCorridorDataset.BrowseName & "], " & _
      "[Threshold Distance = " & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdDistance, True, 8) & _
      " " & strUnits & "], " & _
      "[Output Point Feature Class Name = " & pDataset.BrowseName & "], " & _
      "[Output Point Feature Class Workspace = " & pDataset.Workspace.PathName & "]", App.Path & "\Linkages.dll\BatchBottleneckTool", _
      Now, "Corridor Designer Extension Batch Bottleneck Tool", False)
  
  ' ADD NEW METADATA CONTACTS
3513:   strResponse = Linkages.Metadata_Functions.AddContact_Metadata(pDataset, JenMetadata_Processor, "", True)
  
  ' ADD NEW CITATION CONTACTS; SHOULD HAVE ORIGINATOR
3516:   strResponse = Linkages.Metadata_Functions.AddContact_CitationResponsibleParty(pDataset, JenMetadata_Originator, "", True)
    
  ' ADD NEW RESOURCE CONTACTS
3519:   strResponse = Linkages.Metadata_Functions.AddContact_ResourcePointOfContact(pDataset, JenMetadata_Processor, "", True)
  
  ' ADD CITATION DATES
3522:   strResponse = Linkages.Metadata_Functions.AddCitationDates(pDataset, Now, Now)
  
  ' SET RESOURCE STATUS
3525:   strResponse = Linkages.Metadata_Functions.AddResourceDetailsStatus(pDataset, JenMetadata_Completed)
  
  ' SET MAINTENANCE STATUS
3528:   strResponse = Linkages.Metadata_Functions.AddResourceMaintenance(pDataset, JenMetadata_Maint_Unknown)
  
  ' ADD FIELD INFO
3531:   strResponse = Linkages.Metadata_Functions.AddDetailsForObjectDefinition(pDataset, _
      "Points representing corridor midpoints", "Corridor Designer")
      
'  lngPointsID = pPointsFClass.FindField("Unique_ID")
'  lngPointsX = pPointsFClass.FindField("X_Coord")
'  lngPointsY = pPointsFClass.FindField("Y_Coord")
'  lngPointsWidth = pPointsFClass.FindField("Width")
'  lngThreshold = pPointsFClass.FindField("Threshold")
  
3540:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Unique_ID", _
      "Unique numeric ID values for each point in feature class...", "Corridor Designer ArcGIS Tools", _
      "1", "99999999", , , , , , "Sequential Unique ID Values")
3543:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Width", _
      "Minimum width of the corridor at this point, where width is defined as the distance between the two closest " & _
      "points on the corridor polygon boundary to this point.  This point should represent the midpoint of the corridor.", _
      "Corridor Designer ArcGIS Tools", _
      "0", "99999999", , strUnits, , , , "Threshold values are specified by the user")
  Dim varData() As Variant
  ReDim varData(0)
  Dim strList() As String
  Dim strThresholdAndUnits As String
3552:   strThresholdAndUnits = "[" & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdDistance, True, 8) & " " & strUnits & "]"
  ReDim strList(2, 2)
3554:   strList(0, 0) = "ABOVE"
3555:   strList(1, 0) = "Corridor width at this point is above the user-specified threshold corridor width " & strThresholdAndUnits & "."
3556:   strList(2, 0) = "Corridor Designer ArcGIS Tools"
3557:   strList(0, 1) = "BELOW"
3558:   strList(1, 1) = "Corridor width at this point is below the user-specified threshold corridor width " & strThresholdAndUnits & "."
3559:   strList(2, 1) = "Corridor Designer ArcGIS Tools"
3560:   strList(0, 2) = "N/A"
3561:   strList(1, 2) = "No threshold distance was specified for this analysis."
3562:   strList(2, 2) = "Corridor Designer ArcGIS Tools"
3563:   varData(0) = strList
3564:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Threshold", _
      "Indicated whether the corridor is above or below the user-specified threshold distance at this point " & _
      strThresholdAndUnits & ". " & _
      "Points below the threshold value indicate possible bottlenecks in the corridor.", "Corridor Designer ArcGIS Tools", _
      , , , , , , varData, "Threshold values are specified by the user")
      
  Dim pNewSpRef As ISpatialReference
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim pGeoDataset As IGeoDataset
  Dim booAddDomainUnits As Boolean
  Dim strDomainUnits As String
  Dim dblMinX As Double
  Dim dblMaxX As Double
  Dim dblMinY As Double
  Dim dblMaxY As Double
3579:   If TypeOf pDataset Is IGeoDataset Then
3580:     Set pGeoDataset = pDataset
3581:     Set pNewSpRef = pGeoDataset.SpatialReference
3582:     booAddDomainUnits = True
3583:     If TypeOf pNewSpRef Is IProjectedCoordinateSystem Then
3584:       Set pPrjSpRef = pNewSpRef
3585:       pPrjSpRef.GetDomain dblMinX, dblMaxX, dblMinY, dblMaxY
3586:       strDomainUnits = "Projected Units (" & pPrjSpRef.CoordinateUnit.Name & ")"
3587:       strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "X_Coord", _
          "Projected X-coordinate value of point (" & pPrjSpRef.CoordinateUnit.Name & ")", "Corridor Designer ArcGIS Tools", _
          Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblMinX, False, 8), _
          Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblMaxX, False, 8), , strDomainUnits, , , , , "Point Coordinates")
3591:       strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Y_Coord", _
          "Projected Y-coordinate value of point (" & pPrjSpRef.CoordinateUnit.Name & ")", "Corridor Designer ArcGIS Tools", _
          Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblMinY, False, 8), _
          Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblMaxY, False, 8), , strDomainUnits, , , , , "Point Coordinates")
3595:     Else
3596:       strDomainUnits = "Degrees "
3597:       strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "X_Coord", _
          "Longitude value of point...", "Corridor Designer ArcGIS Tools", _
          "-180", "180", , strDomainUnits & "Longitude", , , , "Point Coordinates")
3600:       strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Y_Coord", _
          "Latitude value of point...", "Corridor Designer ArcGIS Tools", _
          "-90", "90", , strDomainUnits & "Latitude", , , , "Point Coordinates")
3603:     End If
3604:   End If
  
  ' SET METADATA FORMAT VERSION
3607:   strResponse = Linkages.Metadata_Functions.SetMetadataFormatVersion(pDataset, , True, pMxDoc)
    
  ' RESYNCHRONIZE METADATA
3610:   strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
    
3612:   MakePointFeatureClassBatch = "Succeeded"
3613:   booSucceeded = True
  
3615:   GoTo ClearMemory
  Exit Function
ErrorHandler:
  ' HandleError True, "MakePointFeatureClassBatch " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

3620:   MakePointFeatureClassBatch = "MakePointFeatureClassBatch Failed [ in " & c_sModuleFileName & "]:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"
  
3623:   booSucceeded = False

ClearMemory:
3626:   Set pFieldArray = Nothing
3627:   Set pUniqueIDField = Nothing
3628:   Set pUniqueIDieldEdit = Nothing
3629:   Set pPointsXField = Nothing
3630:   Set pPointsXFieldEdit = Nothing
3631:   Set pPointsYField = Nothing
3632:   Set pPointsYFieldEdit = Nothing
3633:   Set pPointsWidthField = Nothing
3634:   Set pPointsWidthFieldEdit = Nothing
3635:   Set pPointsThresholdField = Nothing
3636:   Set pPointsThresholdFieldEdit = Nothing
3637:   Set pPointsFClass = Nothing
3638:   Set pPointsCur = Nothing
3639:   Set pPointsBuf = Nothing
3640:   Set pPoint = Nothing
3641:   Set pValArray = Nothing
3642:   Set pDataset = Nothing
3643:   Set pPropSet = Nothing
3644:   Set pKeyWords = Nothing
3645:   Set pIncludeThemeKeys = Nothing
3646:   Set pIncludeSearchKeys = Nothing
3647:   Set pIncludeDescKeys = Nothing
3648:   Set pIncludeStratKeys = Nothing
3649:   Set pIncludeThemeSlashThemekeys = Nothing
3650:   Set pIncludePlaceKeys = Nothing
3651:   Set pIncludeTemporalKeys = Nothing
3652:   Set pCombinedKeyWords = Nothing
3653:   Set pBlock1Dataset = Nothing
3654:   Set pBlock2Dataset = Nothing
3655:   Set pCorridorDataset = Nothing
3656:   Erase varData
3657:   Erase strList
3658:   Set pPrjSpRef = Nothing
3659:   Set pGeoDataset = Nothing
3660:   Set pNewSpRef = Nothing


  
End Function

Public Function MakeSegFeatureClassBatch(pWS As IWorkspace, strName As String, _
  pDataArray As esriSystem.IVariantArray, pSpRef As ISpatialReference, booSucceeded As Boolean, _
  pCorridorPolygon As IPolygon, dblThresholdValue As Double, pSegmentArray As esriSystem.IArray, _
  pBlock1FClass As IFeatureClass, _
  pBlock2FClass As IFeatureClass, _
  pCorridorFClass As IFeatureClass, _
  strUnits As String, pMxDoc As IMxDocument) As String

  On Error GoTo ErrorHandler
  
3676:   MakeSegFeatureClassBatch = "Failed, Unknown Reason..."
3677:   booSucceeded = False
  
3679:   strName = Linkages.aml_func_mod.ClipExtension2(strName)
  
'  Dim strReport As String
'  Dim strFName As String
  Dim lngIndex As Long

3685:   If Linkages.MyGeneralOperations.CheckIfFeatureClassExists(pWS, strName) Then
3686:     MakeSegFeatureClassBatch = "Feature class '" & strName & "' already exists.  No new segment feature class created..."
3687:     booSucceeded = False
3688:     GoTo ClearMemory
3689:   End If

3691:   If pDataArray.Count <= 1 Then
3692:     MakeSegFeatureClassBatch = "Too few points to make segments!  No new segment feature class created..."
3693:     booSucceeded = False
3694:     GoTo ClearMemory
3695:   End If

  ' MAKE SEG ARRAY
  Dim pSegArray As esriSystem.IArray
  Dim strSegArrayError As String
3700:   Set pSegmentArray = Nothing
3701:   Set pSegArray = ReturnSegmentArrayFromDataArray(pDataArray, pSpRef, dblThresholdValue, strSegArrayError)
3702:   If pSegArray Is Nothing Then
3703:     MakeSegFeatureClassBatch = strSegArrayError
3704:     booSucceeded = False
3705:     GoTo ClearMemory
3706:   End If
3707:   Set pSegmentArray = pSegArray
  
  Dim lngDataType As JenDatasetTypes
  Dim strWSPath As String
3711:   strWSPath = pWS.PathName
3712:   If pWS.Type = esriFileSystemWorkspace Then
3713:     lngDataType = ENUM_Shapefile
3714:   Else
3715:     If InStr(1, strWSPath, ".mdb", vbTextCompare) > 0 Then
3716:       lngDataType = ENUM_PersonalGDB
3717:     ElseIf InStr(1, strWSPath, ".gdb", vbTextCompare) > 0 Then
3718:       lngDataType = ENUM_FileGDB
3719:     Else
3720:       MakeSegFeatureClassBatch = "Invalid Workspace Type:  This function can only create shapefiles, " & _
          "file geodatabase feature classes and personal geodatabase feature classes.  No new segment feature class created..."
3722:       booSucceeded = False
3723:       GoTo ClearMemory
3724:     End If
3725:   End If

    ' MAKE FIELDS

  Dim pFieldArray As esriSystem.IVariantArray
3730:   Set pFieldArray = New esriSystem.VarArray
    ' MAKE ID FIELD
  Dim pUniqueIDField As IField
  Dim pUniqueIDieldEdit As IFieldEdit
3734:   Set pUniqueIDField = New Field
3735:   Set pUniqueIDieldEdit = pUniqueIDField
3736:   With pUniqueIDieldEdit
3737:     .Name = "Unique_ID"
3738:     .Type = esriFieldTypeInteger
3739:   End With
3740:   pFieldArray.Add pUniqueIDField

    ' MAKE NAME FIELD
  Dim pSegsNameField As IField
  Dim pSegsNameFieldEdit As IFieldEdit
3745:   Set pSegsNameField = New Field
3746:   Set pSegsNameFieldEdit = pSegsNameField
3747:   With pSegsNameFieldEdit
3748:     .Name = "Name"
3749:     .Type = esriFieldTypeString
3750:     .length = 50
3751:   End With
3752:   pFieldArray.Add pSegsNameField

    ' MAKE LENGTH FIELD
  Dim pSegsLengthField As IField
  Dim pSegsLengthFieldEdit As IFieldEdit
3757:   Set pSegsLengthField = New Field
3758:   Set pSegsLengthFieldEdit = pSegsLengthField
3759:   With pSegsLengthFieldEdit
3760:     .Name = "Length"
3761:     .Type = esriFieldTypeDouble
3762:     .Precision = 16
3763:     .Scale = 8
3764:   End With
3765:   pFieldArray.Add pSegsLengthField

  Dim pSegsFClass As IFeatureClass

3769:   Set pSegsFClass = Linkages.MyGeneralOperations.CreateGeneralFeatureClass(esriGeometryPolyline, pSpRef, _
      pWS.PathName, strName, lngDataType, pFieldArray, , , , , False, pCorridorPolygon.Envelope, _
      pDataArray.Count)

  Dim pEnvelope As IEnvelope
3774:   Set pEnvelope = New Envelope

  ' FIND VARIABLE INDEX VALUES
  Dim lngSegsID As Long
  Dim lngSegsName As Long
  Dim lngSegsLength As Long
'  Dim lngSegsMinWidthField As Long
'  Dim lngSegsMaxWidthField As Long
3782:   lngSegsID = pSegsFClass.FindField("Unique_ID")
3783:   lngSegsName = pSegsFClass.FindField("Name")
3784:   lngSegsLength = pSegsFClass.FindField("Length")

  ' MAKE INSERT CURSOR
  Dim pSegsCur As IFeatureCursor
3788:   Set pSegsCur = pSegsFClass.Insert(True)
  Dim pSegsBuf As IFeatureBuffer
3790:   Set pSegsBuf = pSegsFClass.CreateFeatureBuffer

  Dim pPolyline As IPolyline
  Dim pSegAbovePair As esriSystem.IVariantArray
  Dim booIsAbove As Boolean
  
  Dim strThresholdForTable As String
3797:   If dblThresholdValue <= 0 Then
3798:     strThresholdForTable = "0"
3799:   Else
3800:     strThresholdForTable = CStr(dblThresholdValue)
3801:   End If
  
  ' ADD DATA TO SHAPEFILE
3804:   For lngIndex = 0 To pSegArray.Count - 1
3805:     Set pSegAbovePair = pSegArray.Element(lngIndex)
3806:     Set pPolyline = pSegAbovePair.Element(0)
3807:     If pPolyline.SpatialReference Is Nothing Then Set pPolyline.SpatialReference = pSpRef
3808:     If (pEnvelope Is Nothing) Then
3809:       Set pEnvelope = pPolyline.Envelope
3810:     Else
3811:       pEnvelope.Union pPolyline.Envelope
3812:     End If
3813:     booIsAbove = pSegAbovePair.Element(1)
3814:     If booIsAbove Then
3815:       strName = "Above Threshold (>" & strThresholdForTable & ")"
3816:     Else
3817:       strName = "Below Threshold (<=" & strThresholdForTable & ")"
3818:     End If

3820:     With pSegsBuf
3821:       Set .Shape = pPolyline
3822:       .Value(lngSegsID) = lngIndex + 1
3823:       .Value(lngSegsName) = strName
3824:       .Value(lngSegsLength) = pPolyline.length
3825:     End With
3826:     pSegsCur.InsertFeature pSegsBuf
3827:   Next lngIndex

3829:   pSegsCur.Flush
    
    '-----------------------------------------------------------------------------------------------------------------------
  ' WRITE METADATA =======================================================================================================
  '-----------------------------------------------------------------------------------------------------------------------
  Dim pDataset As IDataset
3835:   Set pDataset = pSegsFClass
  
  Dim pBlock1Dataset As IDataset
  Dim pBlock2Dataset As IDataset
  Dim pCorridorDataset As IDataset
  
3841:   Set pBlock1Dataset = pBlock1FClass
3842:   Set pBlock2Dataset = pBlock2FClass
3843:   Set pCorridorDataset = pCorridorFClass
  
  Dim pPropSet As IPropertySet
3846:   Set pPropSet = Linkages.Metadata_Functions.ReturnMetadataPropSetFromDataset(pDataset)
  
  ' SYNCHRONIZE METADATA
  Dim strResponse As String
3850:   strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
3851:   Debug.Print "Synchronization: " & strResponse
  
  ' WRITE ABSTRACT
  Dim strAbstract As String
3855:   strAbstract = "This dataset represents the path between the two habitat blocks '" & pBlock1Dataset.BrowseName & "' and '" & _
    pBlock2Dataset.BrowseName & "', within the corridor polygon '" & pCorridorDataset.BrowseName & "'., " & _
    "which follows the route with the widest narrow point.  If a threshold distance has been specified, then " & _
    "this route will be split at points where the corridor width crosses that threshold value."
3859:   strResponse = Linkages.Metadata_Functions.SetMetadataAbstract(pDataset, strAbstract)
  
  ' WRITE PURPOSE
  Dim strPurpose As String
3863:   strPurpose = "This dataset finds the midpoint route through a corridor, which in turn can be used to identify " & _
      "narrow points or bottlenecks within the corridor."
3865:   strResponse = Linkages.Metadata_Functions.SetMetadataPurpose(pDataset, strPurpose)
  
  ' ADD KEYWORDS
  Dim pKeyWords As esriSystem.IStringArray
  Dim pIncludeThemeKeys As esriSystem.IStringArray
  Dim pIncludeSearchKeys As esriSystem.IStringArray
  Dim pIncludeDescKeys As esriSystem.IStringArray
  Dim pIncludeStratKeys As esriSystem.IStringArray
  Dim pIncludeThemeSlashThemekeys As esriSystem.IStringArray
  Dim pIncludePlaceKeys As esriSystem.IStringArray
  Dim pIncludeTemporalKeys As esriSystem.IStringArray
  
3877:   Set pIncludeThemeKeys = New esriSystem.strArray
3878:   Set pIncludeSearchKeys = New esriSystem.strArray
3879:   Set pIncludeDescKeys = New esriSystem.strArray
3880:   Set pIncludeStratKeys = New esriSystem.strArray
3881:   Set pIncludeThemeSlashThemekeys = New esriSystem.strArray
3882:   Set pIncludePlaceKeys = New esriSystem.strArray
3883:   Set pIncludeTemporalKeys = New esriSystem.strArray
    
  Dim pCombinedKeyWords As esriSystem.IStringArray
3886:   pIncludeThemeKeys.Add "Bottleneck"
3887:   pIncludeThemeKeys.Add "Width"
3888:   pIncludeThemeKeys.Add "Corridor"
3889:   pIncludeThemeKeys.Add pBlock1Dataset.BrowseName
3890:   pIncludeThemeKeys.Add pBlock2Dataset.BrowseName
3891:   pIncludeThemeKeys.Add pCorridorDataset.BrowseName
  
3893:   For lngIndex = 0 To pIncludeThemeKeys.Count - 1
3894:     pIncludeSearchKeys.Add pIncludeThemeKeys.Element(lngIndex)
3895:     pIncludeDescKeys.Add pIncludeThemeKeys.Element(lngIndex)
3896:   Next lngIndex
  
' EXTRACT ANY EXISTING KEYWORDS FROM CURRENT BLOCK OR CORRIDOR FCLASSES
3899:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock1FClass, _
      pKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
3903:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock2FClass, _
      pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
3907:   Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pCorridorFClass, _
      pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
      pIncludeThemeSlashThemekeys, _
      pIncludePlaceKeys, pIncludeTemporalKeys)
  
3912:   strResponse = Linkages.Metadata_Functions.SetMetadataKeyWords(pDataset, pIncludeThemeKeys, pIncludeSearchKeys, _
        pIncludeDescKeys, pIncludeStratKeys, pIncludeThemeSlashThemekeys, pIncludePlaceKeys, pIncludeTemporalKeys)
  
  ' SET PROCESS STEP
  Dim strDescription As String
3917:   strDescription = "Generated polyline bottleneck route, connecting '" & pBlock1Dataset.BrowseName & "' to '" & _
      pBlock1Dataset.BrowseName & "' along corridor '" & pCorridorDataset.BrowseName & "' following the path with the widest possible " & _
      "pinch points."
3920:   strResponse = Linkages.Metadata_Functions.AddNewLineageStep(pDataset, strDescription, Now, JenMetadata_Processor, _
      "", True)
  
  ' ADD NEW GEOPROCESSING EVENT
3924:   strResponse = Linkages.Metadata_Functions.AddNewGeoProcStep(pDataset, _
      "[!!!NOTE - This command is not Python code!!!]:  Generate Bottleneck Polyline Feature Class in " & _
      "Batch Bottleneck Analysis (" & _
      "[Wildland Block 1 = " & pBlock1Dataset.Workspace.PathName & "\" & pBlock1Dataset.BrowseName & "], " & _
      "[Wildland Block 2 = " & pBlock2Dataset.Workspace.PathName & "\" & pBlock2Dataset.BrowseName & "], " & _
      "[Corridor = " & pCorridorDataset.Workspace.PathName & "\" & pCorridorDataset.BrowseName & "], " & _
      "[Threshold Distance = " & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdValue, True, 8) & _
      " " & strUnits & "], " & _
      "[Output Polyline Feature Class Name = " & pDataset.BrowseName & "], " & _
      "[Output Polyline Feature Class Workspace = " & pDataset.Workspace.PathName & "]", App.Path & "\Linkages.dll\BatchBottleneckTool", _
      Now, "Corridor Designer Extension Batch Bottleneck Tool", False)
  
  ' ADD NEW METADATA CONTACTS
3937:   strResponse = Linkages.Metadata_Functions.AddContact_Metadata(pDataset, JenMetadata_Processor, "", True)
  
  ' ADD NEW CITATION CONTACTS; SHOULD HAVE ORIGINATOR
3940:   strResponse = Linkages.Metadata_Functions.AddContact_CitationResponsibleParty(pDataset, JenMetadata_Originator, "", True)
    
  ' ADD NEW RESOURCE CONTACTS
3943:   strResponse = Linkages.Metadata_Functions.AddContact_ResourcePointOfContact(pDataset, JenMetadata_Processor, "", True)
  
  ' ADD CITATION DATES
3946:   strResponse = Linkages.Metadata_Functions.AddCitationDates(pDataset, Now, Now)
  
  ' SET RESOURCE STATUS
3949:   strResponse = Linkages.Metadata_Functions.AddResourceDetailsStatus(pDataset, JenMetadata_Completed)
  
  ' SET MAINTENANCE STATUS
3952:   strResponse = Linkages.Metadata_Functions.AddResourceMaintenance(pDataset, JenMetadata_Maint_Unknown)
  
  ' ADD FIELD INFO
3955:   strResponse = Linkages.Metadata_Functions.AddDetailsForObjectDefinition(pDataset, _
      "Polylines representing corridor midpoints", "Corridor Designer")
      
'  lngSegsID = pSegsFClass.FindField("Unique_ID")
'  lngSegsName = pSegsFClass.FindField("Name")
'  lngSegsLength = pSegsFClass.FindField("Length")
  
3962:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Unique_ID", _
      "Unique numeric ID values for each polyline in feature class...", "Corridor Designer ArcGIS Tools", _
      "1", "99999999", , , , , , "Sequential Unique ID Values")
3965:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Length", _
      "Length of polyline, in polyline coordinate system units", _
      "Corridor Designer ArcGIS Tools", _
      "0", "99999999", , strUnits, , , , "Length units are invalid if polyline is in latitude/longitude coordinates!")
  Dim varData() As Variant
  ReDim varData(0)
  Dim strList() As String
  Dim strThresholdAndUnits As String
3973:   strThresholdAndUnits = "[" & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdValue, True, 8) & " " & strUnits & "]"
  ReDim strList(2, 1)
3975:   strList(0, 0) = "Above Threshold (>" & strThresholdForTable & ")"
3976:   strList(1, 0) = "Corridor width along this polyline is above the user-specified threshold corridor width " & strThresholdAndUnits & "."
3977:   strList(2, 0) = "Corridor Designer ArcGIS Tools"
3978:   strList(0, 1) = "Below Threshold (<=" & strThresholdForTable & ")"
3979:   strList(1, 1) = "Corridor width along this polyline is below the user-specified threshold corridor width " & strThresholdAndUnits & "."
3980:   strList(2, 1) = "Corridor Designer ArcGIS Tools"
3981:   varData(0) = strList
3982:   strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Name", _
      "Indicated whether the corridor is above or below the user-specified threshold distance at this point " & _
      strThresholdAndUnits & ". " & _
      "Points below the threshold value indicate possible bottlenecks in the corridor.", "Corridor Designer ArcGIS Tools", _
      , , , , , , varData, "Threshold values are specified by the user")
      
      
  ' SET METADATA FORMAT VERSION
3990:   strResponse = Linkages.Metadata_Functions.SetMetadataFormatVersion(pDataset, , True, pMxDoc)
    
  ' RESYNCHRONIZE METADATA
3993:   strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
    
    
3996:   MakeSegFeatureClassBatch = "Succeeded"
3997:   booSucceeded = True

3999:   GoTo ClearMemory
  Exit Function
ErrorHandler:
'  HandleError True, "MakeSegFeatureClassBatch " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

4004:   MakeSegFeatureClassBatch = "MakeSegFeatureClassBatch Failed [ in " & c_sModuleFileName & "]:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"

4007:   booSucceeded = False


ClearMemory:
4011:   Set pSegArray = Nothing
4012:   Set pFieldArray = Nothing
4013:   Set pUniqueIDField = Nothing
4014:   Set pUniqueIDieldEdit = Nothing
4015:   Set pSegsNameField = Nothing
4016:   Set pSegsNameFieldEdit = Nothing
4017:   Set pSegsLengthField = Nothing
4018:   Set pSegsLengthFieldEdit = Nothing
4019:   Set pSegsFClass = Nothing
4020:   Set pEnvelope = Nothing
4021:   Set pSegsCur = Nothing
4022:   Set pSegsBuf = Nothing
4023:   Set pPolyline = Nothing
4024:   Set pSegAbovePair = Nothing
4025:   Set pDataset = Nothing
4026:   Set pBlock1Dataset = Nothing
4027:   Set pBlock2Dataset = Nothing
4028:   Set pCorridorDataset = Nothing
4029:   Set pPropSet = Nothing
4030:   Set pKeyWords = Nothing
4031:   Set pIncludeThemeKeys = Nothing
4032:   Set pIncludeSearchKeys = Nothing
4033:   Set pIncludeDescKeys = Nothing
4034:   Set pIncludeStratKeys = Nothing
4035:   Set pIncludeThemeSlashThemekeys = Nothing
4036:   Set pIncludePlaceKeys = Nothing
4037:   Set pIncludeTemporalKeys = Nothing
4038:   Set pCombinedKeyWords = Nothing
4039:   Erase varData
4040:   Erase strList



End Function

Public Function ReturnSegmentArrayFromDataArray(pDataArray As esriSystem.IVariantArray, _
    pSpRef As ISpatialReference, dblThresholdValue As Double, strSegArrayError As String) As esriSystem.IArray
  On Error GoTo ErrorHandler



'  Dim pPolyline As IPolyline
  Dim pSegAbovePair As esriSystem.IVariantArray
'  Dim booIsAbove As Boolean
'  Dim strName As String
  
  ' MAKE SEG ARRAY
  Dim pSegArray As esriSystem.IArray
  Dim pValueArray As esriSystem.IDoubleArray
  Dim dblMapX As Double
  Dim dblMapY As Double
  Dim dblMapShortX As Double
  Dim dblMapShortY As Double
  Dim dblMapPrevX As Double
  Dim dblMapPrevY As Double
  Dim pAboveSegment As IPointCollection
  Dim pBelowSegment As IPointCollection
  Dim pPoint As IPoint
'  Dim pPrevPoint As IPoint
  Dim pShortPoint As IPoint
'  Dim pClone As IClone
  Dim pTopoOp As ITopologicalOperator
  Dim dblWidth As Double
  Dim booCurrentThresh As Boolean
  Dim pGeom As IGeometry
  Dim dblPreviousWidth As Double
  Dim booPreviousThresh As Boolean
  Dim lngIndex As Long
  Dim dblProportion As Double
  
  Dim lngLastPointIndex As Long
4082:   lngLastPointIndex = pDataArray.Count - 1
  
4084:   Set pSegArray = New esriSystem.Array
    
4086:   Set pValueArray = pDataArray.Element(0)
4087:   dblMapX = pValueArray.Element(0)
4088:   dblMapY = pValueArray.Element(1)
4089:   dblWidth = pValueArray.Element(2)
4090:   dblPreviousWidth = dblWidth
4091:   booCurrentThresh = dblWidth > dblThresholdValue
4092:   booPreviousThresh = booCurrentThresh
4093:   Set pPoint = New Point
4094:   Set pPoint.SpatialReference = pSpRef
4095:   pPoint.PutCoords dblMapX, dblMapY
4096:   Set pSegArray = New esriSystem.Array
4097:   dblMapPrevX = dblMapX
4098:   dblMapPrevY = dblMapY
4099:   If booCurrentThresh Then
4100:     Set pAboveSegment = New Polyline
4101:     Set pGeom = pAboveSegment
4102:     Set pGeom.SpatialReference = pSpRef
4103:     pAboveSegment.AddPoint pPoint
4104:   Else
4105:     Set pBelowSegment = New Polyline
4106:     Set pGeom = pBelowSegment
4107:     Set pGeom.SpatialReference = pSpRef
4108:     pBelowSegment.AddPoint pPoint
4109:   End If

'    For lngIndex = 0 To UBound(m_dblGraphVals, 1)
'      dblTempX = m_dblGraphVals(lngIndex, 0)
'      dblTempY = m_dblGraphVals(lngIndex, 1)
  
4115:   For lngIndex = 1 To lngLastPointIndex
4116:     Set pValueArray = pDataArray.Element(lngIndex)
4117:     dblMapX = pValueArray.Element(0)
4118:     dblMapY = pValueArray.Element(1)
4119:     dblWidth = pValueArray.Element(2)
    
4121:     booCurrentThresh = (dblWidth > dblThresholdValue)
      
4123:     If booCurrentThresh = booPreviousThresh Then    ' THEN HAS NOT CROSSED THRESHOLD

4125:       Set pPoint = New Point
4126:       Set pPoint.SpatialReference = pSpRef
4127:       pPoint.PutCoords dblMapX, dblMapY

4129:       If booCurrentThresh Then
4130:         pAboveSegment.AddPoint pPoint
        
        ' ONLY ADD THIS SEGMENT TO LIST IF WE ARE ON LAST POINT
4133:         If lngIndex = lngLastPointIndex Then
4134:           Set pSegAbovePair = New esriSystem.VarArray
4135:           Set pTopoOp = pAboveSegment
4136:           pTopoOp.Simplify
4137:           pSegAbovePair.Add pAboveSegment
4138:           pSegAbovePair.Add True
4139:           pSegArray.Add pSegAbovePair
4140:         End If
4141:       Else
4142:         pBelowSegment.AddPoint pPoint
        ' ONLY ADD THIS SEGMENT TO LIST IF WE ARE ON LAST POINT
4144:         If lngIndex = lngLastPointIndex Then
4145:           Set pSegAbovePair = New esriSystem.VarArray
4146:           Set pTopoOp = pBelowSegment
4147:           pTopoOp.Simplify
4148:           pSegAbovePair.Add pBelowSegment
4149:           pSegAbovePair.Add False
4150:           pSegArray.Add pSegAbovePair
4151:         End If
4152:       End If
          
'      Set pClone = pPoint
'      Set pPrevPoint = pClone.Clone
4156:       dblMapPrevX = dblMapX
4157:       dblMapPrevY = dblMapY
4158:       dblPreviousWidth = dblWidth
              
4160:     Else    ' HAS CROSSED THRESHOLD; NEED TO MAKE TWO LINES
'      dblProportion = (dblGraphThreshold - dblPrevY) / (dblTempY - dblPrevY)
4162:       dblProportion = (dblThresholdValue - dblPreviousWidth) / (dblWidth - dblPreviousWidth)
     
4164:       dblMapShortX = dblMapPrevX + ((dblMapX - dblMapPrevX) * dblProportion)
4165:       dblMapShortY = dblMapPrevY + ((dblMapY - dblMapPrevY) * dblProportion)
               
4167:       Set pShortPoint = New Point
4168:       Set pShortPoint.SpatialReference = pSpRef
4169:       pShortPoint.PutCoords dblMapShortX, dblMapShortY
4170:       Set pPoint = New Point
4171:       Set pPoint.SpatialReference = pSpRef
4172:       pPoint.PutCoords dblMapX, dblMapY
      
'      Set pClone = pPoint
'      Set pPrevPoint = pClone.Clone
4176:       dblMapPrevX = dblMapX
4177:       dblMapPrevY = dblMapY
4178:       dblPreviousWidth = dblWidth
    
4180:       If booCurrentThresh Then ' THEN PREVIOUSLY WAS BELOW THRESHOLD.  NOW IS ABOVE THRESHOLD.
          
        ' ADD SEGMENT BEFORE CROSSING ABOVE THRESHOLD
4183:         pBelowSegment.AddPoint pShortPoint
        
        'ADD THIS SEGMENT TO LIST
4186:         Set pSegAbovePair = New esriSystem.VarArray
4187:         Set pTopoOp = pBelowSegment
4188:         pTopoOp.Simplify
4189:         pSegAbovePair.Add pBelowSegment
4190:         pSegAbovePair.Add False
4191:         pSegArray.Add pSegAbovePair
        
        ' ADD SEGMENT AFTER CROSSING ABOVE THRESHOLD
4194:         Set pAboveSegment = New Polyline
4195:         pAboveSegment.AddPoint pShortPoint
4196:         pAboveSegment.AddPoint pPoint
        ' ONLY ADD THIS SEGMENT TO LIST IF WE ARE ON LAST POINT
4198:         If lngIndex = lngLastPointIndex Then
4199:           Set pSegAbovePair = New esriSystem.VarArray
4200:           Set pTopoOp = pAboveSegment
4201:           pTopoOp.Simplify
4202:           pSegAbovePair.Add pAboveSegment
4203:           pSegAbovePair.Add True
4204:           pSegArray.Add pSegAbovePair
4205:         End If
        
4207:       Else                     ' THEN PREVIOUSLY WAS ABOVE THRESHOLD.  NOW IS BELOW THRESHOLD.
                  
        ' ADD SEGMENT BEFORE CROSSING BELOW THRESHOLD
4210:         pAboveSegment.AddPoint pShortPoint
        
        'ADD THIS SEGMENT TO LIST
4213:         Set pSegAbovePair = New esriSystem.VarArray
4214:         Set pTopoOp = pAboveSegment
4215:         pTopoOp.Simplify
4216:         pSegAbovePair.Add pAboveSegment
4217:         pSegAbovePair.Add True
4218:         pSegArray.Add pSegAbovePair
        
        ' ADD SEGMENT AFTER CROSSING ABOVE THRESHOLD
4221:         Set pBelowSegment = New Polyline
4222:         pBelowSegment.AddPoint pShortPoint
4223:         pBelowSegment.AddPoint pPoint
        ' ONLY ADD THIS SEGMENT TO LIST IF WE ARE ON LAST POINT
4225:         If lngIndex = lngLastPointIndex Then
4226:           Set pSegAbovePair = New esriSystem.VarArray
4227:           Set pTopoOp = pBelowSegment
4228:           pTopoOp.Simplify
4229:           pSegAbovePair.Add pBelowSegment
4230:           pSegAbovePair.Add False
4231:           pSegArray.Add pSegAbovePair
4232:         End If
4233:       End If
4234:     End If
      
4236:     booPreviousThresh = booCurrentThresh
    
4238:   Next lngIndex
  
4240:   strSegArrayError = ""
4241:   Set ReturnSegmentArrayFromDataArray = pSegArray
  
4243:   GoTo ClearMemory
  
  Exit Function
  
ErrorHandler:
'  HandleError True, "ReturnSegmentArrayFromDataArray " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

4250:   Set ReturnSegmentArrayFromDataArray = Nothing
4251:   strSegArrayError = "ReturnSegmentArrayFromDataArray Failed [ in " & c_sModuleFileName & "]:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"

ClearMemory:
4255:   Set pSegAbovePair = Nothing
4256:   Set pSegArray = Nothing
4257:   Set pValueArray = Nothing
4258:   Set pAboveSegment = Nothing
4259:   Set pBelowSegment = Nothing
4260:   Set pPoint = Nothing
4261:   Set pShortPoint = Nothing
4262:   Set pTopoOp = Nothing
4263:   Set pGeom = Nothing



End Function

Public Function MakeStatsTablesBatch(pWS As IWorkspace, booCalcBottleneck As Boolean, _
    strBottleneckName As String, booCalcSegStats As Boolean, strSegName As String, _
    pDataArray As esriSystem.IVariantArray, booSucceeded As Boolean, pSpRef As ISpatialReference, _
    dblThresholdDistance As Double, pSegmentArray As esriSystem.IArray, _
  pBlock1FClass As IFeatureClass, _
  pBlock2FClass As IFeatureClass, _
  pCorridorFClass As IFeatureClass, _
  strUnits As String, pMxDoc As IMxDocument) As String
  
  On Error GoTo ErrorHandler
    
4280:   strBottleneckName = Linkages.aml_func_mod.ClipExtension2(strBottleneckName)
4281:   strSegName = Linkages.aml_func_mod.ClipExtension2(strSegName)
  
  Dim strThresholdForTable As String
4284:   If dblThresholdDistance <= 0 Then
4285:     strThresholdForTable = "0"
4286:   Else
4287:     strThresholdForTable = CStr(dblThresholdDistance)
4288:   End If

4290:   MakeStatsTablesBatch = "Failed, Unknown Reason..."
4291:   booSucceeded = False
  
  Dim strReport As String
'  Dim strFName As String
  Dim lngIndex As Long
    
  ' MAKE SEG ARRAY
  Dim pSegArray As esriSystem.IArray
  Dim strSegArrayError As String
4300:   If pSegmentArray Is Nothing Then
4301:     Set pSegArray = ReturnSegmentArrayFromDataArray(pDataArray, pSpRef, dblThresholdDistance, strSegArrayError)
4302:   Else
4303:     Set pSegArray = pSegmentArray
4304:     If pSegArray Is Nothing Then
4305:       strSegArrayError = "No Segment Data Array..."
4306:     End If
4307:   End If
4308:   If pSegArray Is Nothing Then
4309:     booSucceeded = False
4310:     MakeStatsTablesBatch = strSegArrayError
4311:     GoTo ClearMemory
4312:   End If
  
  Dim pSegAbovePair As esriSystem.IVariantArray
  
  ' PRECALCULATE STATISTICS
  Dim dblMinWidth As Double
  Dim dblMaxWidth As Double
  Dim dblRange As Double
  Dim dblMean As Double
  Dim dblMedian As Double
  Dim dblStDev As Double
  Dim dblLength As Double
  Dim dblThreshold As Double
  Dim dblLengthBelowThreshold As Double
  Dim dblLengthAboveThreshold As Double
  Dim dblProportionBelowThreshold As Double
  Dim dblProportionAboveThreshold As Double
  
  Dim booIsAbove As Boolean
  Dim dblWidths() As Double
  ReDim dblWidths(pDataArray.Count - 1)
  Dim dblWidth As Double
  Dim pValueArray As esriSystem.IDoubleArray
  
4336:   For lngIndex = 0 To pDataArray.Count - 1
4337:     Set pValueArray = pDataArray.Element(lngIndex)
4338:     dblWidth = pValueArray.Element(2)
4339:     dblWidths(lngIndex) = dblWidth
4340:   Next lngIndex
  
4342:   Linkages.MyGeneralOperations.BasicStatsFromArraySimpleFast2 dblWidths, True, , , dblMinWidth, _
      dblMaxWidth, dblMean, dblMedian, , dblStDev, , dblRange, True
  
4345:   dblLength = 0
4346:   dblLengthBelowThreshold = 0
4347:   dblLengthAboveThreshold = 0
  
  Dim pPrjSpRef As IProjectedCoordinateSystem
  Dim pGeoSpRef As IGeographicCoordinateSystem
  Dim pDatum As IDatum
  Dim dblSemiMinorAxis As Double
  Dim dblSemiMajorAxis As Double
  
  Dim booIsGeographic As Boolean
4356:   booIsGeographic = False
4357:   If TypeOf pSpRef Is IGeographicCoordinateSystem Then
4358:     strUnits = "Meters"
4359:     Set pGeoSpRef = pSpRef
4360:     Set pDatum = pGeoSpRef.Datum
4361:     booIsGeographic = True
4362:     dblSemiMinorAxis = pDatum.Spheroid.SemiMinorAxis
4363:     dblSemiMajorAxis = pDatum.Spheroid.SemiMajorAxis
4364:   ElseIf TypeOf pSpRef Is IUnknownCoordinateSystem Then
4365:     strUnits = "Unknown units"
4366:   Else
4367:     Set pPrjSpRef = pSpRef
4368:     strUnits = pPrjSpRef.CoordinateUnit.Name
4369:   End If
  
  Dim dblPolylineLength As Double
  Dim pPolyline As IPolyline
  Dim pAboveSegmentData As esriSystem.IArray
4374:   Set pAboveSegmentData = New esriSystem.Array
  Dim pBelowSegmentData As esriSystem.IArray
4376:   Set pBelowSegmentData = New esriSystem.Array
  Dim pSubArray As esriSystem.IVariantArray
  Dim lngAboveCounter As Long
  Dim lngBelowCounter As Long
  Dim strName As String
  
4382:   lngAboveCounter = 0
4383:   lngBelowCounter = 0
  
4385:   For lngIndex = 0 To pSegArray.Count - 1
4386:     Set pSegAbovePair = pSegArray.Element(lngIndex)
4387:     Set pPolyline = pSegAbovePair.Element(0)
4388:     If booIsGeographic Then
4389:       dblPolylineLength = Linkages.MyGeometricOperations.SpheroidalPolylineLength2(pPolyline, _
          dblSemiMajorAxis, dblSemiMinorAxis)
4391:     Else
4392:       dblPolylineLength = pPolyline.length
4393:     End If
4394:     booIsAbove = pSegAbovePair.Element(1)
    
4396:     dblLength = dblLength + dblPolylineLength
    
4398:     Set pSubArray = New esriSystem.VarArray
    
4400:     If booIsAbove Then
4401:       lngAboveCounter = lngAboveCounter + 1
4402:       strName = "Above Threshold (>" & strThresholdForTable & "), #" & Format(lngAboveCounter, "#,##0")
4403:       dblLengthAboveThreshold = dblLengthAboveThreshold + dblPolylineLength
4404:       pSubArray.Add dblPolylineLength
4405:       pSubArray.Add strName
4406:       pSubArray.Add strUnits
4407:       pAboveSegmentData.Add pSubArray
4408:     Else
4409:       lngBelowCounter = lngBelowCounter + 1
4410:       strName = "Below Threshold (<=" & strThresholdForTable & "), #" & Format(lngAboveCounter, "#,##0")
4411:       dblLengthBelowThreshold = dblLengthBelowThreshold + dblPolylineLength
4412:       pSubArray.Add dblPolylineLength
4413:       pSubArray.Add strName
4414:       pSubArray.Add strUnits
4415:       pBelowSegmentData.Add pSubArray
4416:     End If
        
4418:   Next lngIndex
4419:   If dblLength = 0 Then
4420:     dblProportionAboveThreshold = 0
4421:     dblProportionBelowThreshold = 0
4422:   Else
4423:     dblProportionAboveThreshold = dblLengthAboveThreshold / dblLength
4424:     dblProportionBelowThreshold = dblLengthBelowThreshold / dblLength
4425:   End If
  
  Dim lngDataType As JenDatasetTypes
  Dim strWSPath As String
4429:   strWSPath = pWS.PathName
4430:   If pWS.Type = esriFileSystemWorkspace Then
4431:     lngDataType = ENUM_Shapefile
4432:   Else
4433:     If InStr(1, strWSPath, ".mdb", vbTextCompare) > 0 Then
4434:       lngDataType = ENUM_PersonalGDB
4435:     ElseIf InStr(1, strWSPath, ".gdb", vbTextCompare) > 0 Then
4436:       lngDataType = ENUM_FileGDB
4437:     Else
4438:       strReport = strReport & "Invalid Workspace Type:  This function can only create dBASE tables, " & _
          "file geodatabase tables and personal geodatabase tables.  No new bottleneck statistics table created..."
4440:       booSucceeded = False
4441:       GoTo ClearMemory
4442:     End If
4443:   End If
  
4445:   If booCalcBottleneck Then
4446:     If Linkages.MyGeneralOperations.CheckIfTableExists(pWS, strBottleneckName) Then
4447:       strReport = strReport & "Table '" & strName & "' already exists.  No bottleneck statistics table created..."
4448:       booSucceeded = False
4449:       GoTo ClearMemory
4450:     End If
  
4452:     If pDataArray.Count < 1 Then
4453:       strReport = strReport & "No points!  No bottleneck statistics table created..."
4454:       booSucceeded = False
4455:       GoTo ClearMemory
4456:     End If
  
    ' GENERAL STATISTICS TABLE -------------------------------------------------------------------------------

    ' MAKE FIELDS
    Dim pStatsFields As esriSystem.IVariantArray
4462:     Set pStatsFields = New esriSystem.VarArray
      
      ' MAKE UNIQUE ID FIELD
    Dim pStatsUniqueIDField As IField
    Dim pStatsUniqueIDFieldEdit As IFieldEdit
4467:     Set pStatsUniqueIDField = New Field
4468:     Set pStatsUniqueIDFieldEdit = pStatsUniqueIDField
4469:     With pStatsUniqueIDFieldEdit
4470:       .Name = "Unique_ID"
4471:       .Type = esriFieldTypeInteger
4472:       .Precision = 8
4473:     End With
4474:     pStatsFields.Add pStatsUniqueIDField
    
      ' MAKE Statistics FIELD
    Dim pStatsStatisticsField As IField
    Dim pStatsStatisticsFieldEdit As IFieldEdit
4479:     Set pStatsStatisticsField = New Field
4480:     Set pStatsStatisticsFieldEdit = pStatsStatisticsField
4481:     With pStatsStatisticsFieldEdit
4482:       .Name = "Statistic"
4483:       .Type = esriFieldTypeString
4484:       .length = 30
4485:     End With
4486:     pStatsFields.Add pStatsStatisticsField
    
      ' MAKE Value FIELD
    Dim pStatsValueField As IField
    Dim pStatsValueFieldEdit As IFieldEdit
4491:     Set pStatsValueField = New Field
4492:     Set pStatsValueFieldEdit = pStatsValueField
4493:     With pStatsValueFieldEdit
4494:       .Name = "Value"
4495:       .Type = esriFieldTypeDouble
4496:       .Precision = 16
4497:       .Scale = 8
4498:     End With
4499:     pStatsFields.Add pStatsValueField
      
    ' MAKE dBASE TABLE
    Dim pStatsTable As ITable
4503:     Set pStatsTable = Linkages.MyGeneralOperations.CreateGeneralTable(pWS.PathName, strBottleneckName, lngDataType, _
        pStatsFields)
  '  Set pStatsTable = aml_func_mod.CreatedBASETable(strFName, pStatsFields)
    
    ' MAKE INSERT CURSOR
    Dim pStatsCur As ICursor
4509:     Set pStatsCur = pStatsTable.Insert(True)
    Dim pStatsBuf As IRowBuffer
4511:     Set pStatsBuf = pStatsTable.CreateRowBuffer
    
    ' FIND VARIABLE INDEX VALUES
    Dim lngStatsID As Long
    Dim lngStatsStatistics As Long
    Dim lngStatsValue As Long
    Dim lngCounter As Long
    
4519:     lngCounter = 0
    
4521:     lngStatsID = pStatsTable.FindField("Unique_ID")
4522:     lngStatsStatistics = pStatsTable.FindField("Statistic")
4523:     lngStatsValue = pStatsTable.FindField("Value")
    
    ' ADD DATA TO TABLE
    ' MINIMUM
4527:     lngCounter = lngCounter + 1
4528:     pStatsBuf.Value(lngStatsID) = lngCounter
4529:     pStatsBuf.Value(lngStatsStatistics) = "Minimum Width [" & strUnits & "]"
4530:     pStatsBuf.Value(lngStatsValue) = dblMinWidth
4531:     pStatsCur.InsertRow pStatsBuf
    
    ' MAXIMUM
4534:     lngCounter = lngCounter + 1
4535:     pStatsBuf.Value(lngStatsID) = lngCounter
4536:     pStatsBuf.Value(lngStatsStatistics) = "Maximum Width [" & strUnits & "]"
4537:     pStatsBuf.Value(lngStatsValue) = dblMaxWidth
4538:     pStatsCur.InsertRow pStatsBuf
    
    ' RANGE
4541:     lngCounter = lngCounter + 1
4542:     pStatsBuf.Value(lngStatsID) = lngCounter
4543:     pStatsBuf.Value(lngStatsStatistics) = "Range of Width Values"
4544:     pStatsBuf.Value(lngStatsValue) = dblRange
4545:     pStatsCur.InsertRow pStatsBuf
    
    ' MEAN
4548:     lngCounter = lngCounter + 1
4549:     pStatsBuf.Value(lngStatsID) = lngCounter
4550:     pStatsBuf.Value(lngStatsStatistics) = "Mean Width [" & strUnits & "]"
4551:     pStatsBuf.Value(lngStatsValue) = dblMean
4552:     pStatsCur.InsertRow pStatsBuf
    
    ' MEDIAN
4555:     lngCounter = lngCounter + 1
4556:     pStatsBuf.Value(lngStatsID) = lngCounter
4557:     pStatsBuf.Value(lngStatsStatistics) = "Median Width [" & strUnits & "]"
4558:     pStatsBuf.Value(lngStatsValue) = dblMedian
4559:     pStatsCur.InsertRow pStatsBuf
    
    ' STANDARD DEVIATION
4562:     lngCounter = lngCounter + 1
4563:     pStatsBuf.Value(lngStatsID) = lngCounter
4564:     pStatsBuf.Value(lngStatsStatistics) = "Standard Deviation of Width"
4565:     pStatsBuf.Value(lngStatsValue) = dblStDev
4566:     pStatsCur.InsertRow pStatsBuf
    
    ' CENTERLINE LENGTH
4569:     lngCounter = lngCounter + 1
4570:     pStatsBuf.Value(lngStatsID) = lngCounter
4571:     pStatsBuf.Value(lngStatsStatistics) = "Centerline Length [" & strUnits & "]"
4572:     pStatsBuf.Value(lngStatsValue) = dblLength
4573:     pStatsCur.InsertRow pStatsBuf
    
    ' THRESHOLD VALUE
4576:     lngCounter = lngCounter + 1
4577:     pStatsBuf.Value(lngStatsID) = lngCounter
4578:     pStatsBuf.Value(lngStatsStatistics) = "Width Threshold Value"
4579:     pStatsBuf.Value(lngStatsValue) = dblThresholdDistance
4580:     pStatsCur.InsertRow pStatsBuf
    
    ' LENGTH BELOW THRESHOLD
4583:     lngCounter = lngCounter + 1
4584:     pStatsBuf.Value(lngStatsID) = lngCounter
4585:     pStatsBuf.Value(lngStatsStatistics) = "Length Below Threshold [" & strUnits & "]"
4586:     pStatsBuf.Value(lngStatsValue) = dblLengthBelowThreshold
4587:     pStatsCur.InsertRow pStatsBuf
    
    ' PROPORTION BELOW THRESHOLD
4590:     lngCounter = lngCounter + 1
4591:     pStatsBuf.Value(lngStatsID) = lngCounter
4592:     pStatsBuf.Value(lngStatsStatistics) = "Proportion Below Threshold"
4593:     pStatsBuf.Value(lngStatsValue) = dblProportionBelowThreshold
4594:     pStatsCur.InsertRow pStatsBuf
  
    ' LENGTH ABOVE THRESHOLD
4597:     lngCounter = lngCounter + 1
4598:     pStatsBuf.Value(lngStatsID) = lngCounter
4599:     pStatsBuf.Value(lngStatsStatistics) = "Length Above Threshold [" & strUnits & "]"
4600:     pStatsBuf.Value(lngStatsValue) = dblLengthAboveThreshold
4601:     pStatsCur.InsertRow pStatsBuf
    
    ' PROPORTION ABOVE THRESHOLD
4604:     lngCounter = lngCounter + 1
4605:     pStatsBuf.Value(lngStatsID) = lngCounter
4606:     pStatsBuf.Value(lngStatsStatistics) = "Proportion Above Threshold"
4607:     pStatsBuf.Value(lngStatsValue) = dblProportionAboveThreshold
4608:     pStatsCur.InsertRow pStatsBuf
    
4610:     pStatsCur.Flush
     
4612:   End If
  
4614:   If booCalcSegStats Then
    
4616:     Set pStatsFields = New esriSystem.VarArray
      
      ' MAKE UNIQUE ID FIELD
4619:     Set pStatsUniqueIDField = New Field
4620:     Set pStatsUniqueIDFieldEdit = pStatsUniqueIDField
4621:     With pStatsUniqueIDFieldEdit
4622:       .Name = "Unique_ID"
4623:       .Type = esriFieldTypeInteger
4624:       .Precision = 8
4625:     End With
4626:     pStatsFields.Add pStatsUniqueIDField
  
      ' MAKE NAME FIELD
    Dim pSegsNameField As IField
    Dim pSegsNameFieldEdit As IFieldEdit
4631:     Set pSegsNameField = New Field
4632:     Set pSegsNameFieldEdit = pSegsNameField
4633:     With pSegsNameFieldEdit
4634:       .Name = "Name"
4635:       .Type = esriFieldTypeString
4636:       .length = 50
4637:     End With
4638:     pStatsFields.Add pSegsNameField
    
      ' MAKE LENGTH FIELD
    Dim pSegsLengthField As IField
    Dim pSegsLengthFieldEdit As IFieldEdit
4643:     Set pSegsLengthField = New Field
4644:     Set pSegsLengthFieldEdit = pSegsLengthField
4645:     With pSegsLengthFieldEdit
4646:       .Name = "Length"
4647:       .Type = esriFieldTypeDouble
4648:       .Precision = 16
4649:       .Scale = 8
4650:     End With
4651:     pStatsFields.Add pSegsLengthField
    
      ' MAKE UNITS FIELD
    Dim pSegsUnitsField As IField
    Dim pSegsUnitsFieldEdit As IFieldEdit
4656:     Set pSegsUnitsField = New Field
4657:     Set pSegsUnitsFieldEdit = pSegsUnitsField
4658:     With pSegsUnitsFieldEdit
4659:       .Name = "Units"
4660:       .Type = esriFieldTypeString
4661:       .length = 50
4662:     End With
4663:     pStatsFields.Add pSegsUnitsField
      
    ' MAKE dBASE TABLE
    Dim pSegsTable As ITable
4667:     Set pSegsTable = Linkages.MyGeneralOperations.CreateGeneralTable(pWS.PathName, strSegName, lngDataType, _
        pStatsFields)
    
    ' MAKE INSERT CURSOR
    Dim pSegsCur As ICursor
4672:     Set pSegsCur = pSegsTable.Insert(True)
    Dim pSegsBuf As IRowBuffer
4674:     Set pSegsBuf = pSegsTable.CreateRowBuffer
    
    ' FIND VARIABLE INDEX VALUES
    Dim lngSegsID As Long
    Dim lngSegsName As Long
    Dim lngSegsLength As Long
    Dim lngUnitsIndex As Long
4681:     lngSegsID = pSegsTable.FindField("Unique_ID")
4682:     lngSegsName = pSegsTable.FindField("Name")
4683:     lngSegsLength = pSegsTable.FindField("Length")
4684:     lngUnitsIndex = pSegsTable.FindField("Units")
        
4686:     lngCounter = 0
    
    ' ADD DATA TO TABLE
4689:     lngCounter = lngCounter + 1
4690:     pSegsBuf.Value(lngSegsID) = lngCounter
4691:     pSegsBuf.Value(lngSegsName) = "Total Length"
4692:     pSegsBuf.Value(lngSegsLength) = dblLength
4693:     pSegsBuf.Value(lngUnitsIndex) = strUnits
4694:     pSegsCur.InsertRow pSegsBuf
    
    ' THRESHOLD VALUE
4697:     lngCounter = lngCounter + 1
4698:     pSegsBuf.Value(lngSegsID) = lngCounter
4699:     pSegsBuf.Value(lngSegsName) = "Threshold Value"
4700:     pSegsBuf.Value(lngSegsLength) = dblThresholdDistance
4701:     pSegsBuf.Value(lngUnitsIndex) = strUnits
4702:     pSegsCur.InsertRow pSegsBuf
    
    ' LENGTH BELOW THRESHOLD
4705:     lngCounter = lngCounter + 1
4706:     pSegsBuf.Value(lngSegsID) = lngCounter
4707:     pSegsBuf.Value(lngSegsName) = "Length Below Threshold [" & strUnits & "]"
4708:     pSegsBuf.Value(lngSegsLength) = dblLengthBelowThreshold
4709:     pSegsBuf.Value(lngUnitsIndex) = strUnits
4710:     pSegsCur.InsertRow pSegsBuf
    
    ' PROPORTION BELOW THRESHOLD
4713:     lngCounter = lngCounter + 1
4714:     pSegsBuf.Value(lngSegsID) = lngCounter
4715:     pSegsBuf.Value(lngSegsName) = "Proportion Below Threshold"
4716:     pSegsBuf.Value(lngSegsLength) = dblProportionBelowThreshold
4717:     pSegsBuf.Value(lngUnitsIndex) = "Proportion"
4718:     pSegsCur.InsertRow pSegsBuf
  
    ' LENGTH ABOVE THRESHOLD
4721:     lngCounter = lngCounter + 1
4722:     pSegsBuf.Value(lngSegsID) = lngCounter
4723:     pSegsBuf.Value(lngSegsName) = "Length Above Threshold [" & strUnits & "]"
4724:     pSegsBuf.Value(lngSegsLength) = dblLengthAboveThreshold
4725:     pSegsBuf.Value(lngUnitsIndex) = strUnits
4726:     pSegsCur.InsertRow pSegsBuf
    
    ' PROPORTION ABOVE THRESHOLD
4729:     lngCounter = lngCounter + 1
4730:     pSegsBuf.Value(lngSegsID) = lngCounter
4731:     pSegsBuf.Value(lngSegsName) = "Proportion Above Threshold"
4732:     pSegsBuf.Value(lngSegsLength) = dblProportionAboveThreshold
4733:     pSegsBuf.Value(lngUnitsIndex) = "Proportion"
4734:     pSegsCur.InsertRow pSegsBuf
  
4736:     If pAboveSegmentData.Count > 0 Then
4737:       For lngIndex = 0 To pAboveSegmentData.Count - 1
4738:         Set pSubArray = pAboveSegmentData.Element(lngIndex)
4739:         dblPolylineLength = pSubArray.Element(0)
4740:         strName = pSubArray.Element(1)
4741:         strUnits = pSubArray.Element(2)
                
4743:         lngCounter = lngCounter + 1
4744:         pSegsBuf.Value(lngSegsID) = lngCounter
4745:         pSegsBuf.Value(lngSegsName) = strName
4746:         pSegsBuf.Value(lngSegsLength) = dblPolylineLength
4747:         pSegsBuf.Value(lngUnitsIndex) = strUnits
4748:         pSegsCur.InsertRow pSegsBuf
4749:       Next lngIndex
4750:     End If
  
4752:     If pBelowSegmentData.Count > 0 Then
4753:       For lngIndex = 0 To pBelowSegmentData.Count - 1
4754:         Set pSubArray = pBelowSegmentData.Element(lngIndex)
4755:         dblPolylineLength = pSubArray.Element(0)
4756:         strName = pSubArray.Element(1)
4757:         strUnits = pSubArray.Element(2)
                
4759:         lngCounter = lngCounter + 1
4760:         pSegsBuf.Value(lngSegsID) = lngCounter
4761:         pSegsBuf.Value(lngSegsName) = strName
4762:         pSegsBuf.Value(lngSegsLength) = dblPolylineLength
4763:         pSegsBuf.Value(lngUnitsIndex) = strUnits
4764:         pSegsCur.InsertRow pSegsBuf
4765:       Next lngIndex
4766:     End If
       
4768:     pSegsCur.Flush
4769:   End If
  
  
  '-----------------------------------------------------------------------------------------------------------------------
  ' WRITE BOTTLENECK STATISTICS METADATA =================================================================================
  '-----------------------------------------------------------------------------------------------------------------------
  
  Dim pBlock1Dataset As IDataset
  Dim pBlock2Dataset As IDataset
  Dim pCorridorDataset As IDataset
  
4780:   Set pBlock1Dataset = pBlock1FClass
4781:   Set pBlock2Dataset = pBlock2FClass
4782:   Set pCorridorDataset = pCorridorFClass
  
  Dim pPropSet As IPropertySet
  Dim strResponse As String
  Dim strAbstract As String
  Dim strPurpose As String
  Dim pKeyWords As esriSystem.IStringArray
  Dim pIncludeThemeKeys As esriSystem.IStringArray
  Dim pIncludeSearchKeys As esriSystem.IStringArray
  Dim pIncludeDescKeys As esriSystem.IStringArray
  Dim pIncludeStratKeys As esriSystem.IStringArray
  Dim pIncludeThemeSlashThemekeys As esriSystem.IStringArray
  Dim pIncludePlaceKeys As esriSystem.IStringArray
  Dim pIncludeTemporalKeys As esriSystem.IStringArray
  Dim strDescription As String
  Dim varData() As Variant
  Dim strList() As String
  Dim strThresholdAndUnits As String
  
  Dim pDataset As IDataset
  
4803:   If booCalcBottleneck Then
4804:     Set pDataset = pStatsTable
    
4806:     Set pPropSet = Linkages.Metadata_Functions.ReturnMetadataPropSetFromDataset(pDataset)
    
    ' SYNCHRONIZE METADATA
4809:     strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
    
    ' WRITE ABSTRACT
4812:     strAbstract = "This table contains statistics describing the bottleneck route.  This bottleneck route " & _
      "describes the path between the two habitat blocks '" & pBlock1Dataset.BrowseName & "' and '" & _
      pBlock2Dataset.BrowseName & "', within the corridor polygon '" & pCorridorDataset.BrowseName & "', " & _
      "which follows the route with the widest narrow point."
4816:     strResponse = Linkages.Metadata_Functions.SetMetadataAbstract(pDataset, strAbstract)
    
    ' WRITE PURPOSE
4819:     strPurpose = "Statistics describing route with widest bottleneck."
4820:     strResponse = Linkages.Metadata_Functions.SetMetadataPurpose(pDataset, strPurpose)
    
    ' ADD KEYWORDS
4823:     Set pIncludeThemeKeys = New esriSystem.strArray
4824:     Set pIncludeSearchKeys = New esriSystem.strArray
4825:     Set pIncludeDescKeys = New esriSystem.strArray
4826:     Set pIncludeStratKeys = New esriSystem.strArray
4827:     Set pIncludeThemeSlashThemekeys = New esriSystem.strArray
4828:     Set pIncludePlaceKeys = New esriSystem.strArray
4829:     Set pIncludeTemporalKeys = New esriSystem.strArray
      
    Dim pCombinedKeyWords As esriSystem.IStringArray
4832:     pIncludeThemeKeys.Add "Bottleneck"
4833:     pIncludeThemeKeys.Add "Width"
4834:     pIncludeThemeKeys.Add "Corridor"
4835:     pIncludeThemeKeys.Add "Statistics"
4836:     pIncludeThemeKeys.Add pBlock1Dataset.BrowseName
4837:     pIncludeThemeKeys.Add pBlock2Dataset.BrowseName
4838:     pIncludeThemeKeys.Add pCorridorDataset.BrowseName
    
4840:     For lngIndex = 0 To pIncludeThemeKeys.Count - 1
4841:       pIncludeSearchKeys.Add pIncludeThemeKeys.Element(lngIndex)
4842:       pIncludeDescKeys.Add pIncludeThemeKeys.Element(lngIndex)
4843:     Next lngIndex
    
  ' EXTRACT ANY EXISTING KEYWORDS FROM CURRENT BLOCK OR CORRIDOR FCLASSES
4846:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock1FClass, _
        pKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
4850:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock2FClass, _
        pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
4854:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pCorridorFClass, _
        pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
    
4859:     strResponse = Linkages.Metadata_Functions.SetMetadataKeyWords(pDataset, pIncludeThemeKeys, pIncludeSearchKeys, _
          pIncludeDescKeys, pIncludeStratKeys, pIncludeThemeSlashThemekeys, pIncludePlaceKeys, pIncludeTemporalKeys)
    
    ' SET PROCESS STEP
4863:     strDescription = "Generated bottleneck statistics describing route connecting '" & pBlock1Dataset.BrowseName & "' to '" & _
        pBlock1Dataset.BrowseName & "' along corridor '" & pCorridorDataset.BrowseName & "' following the path with the widest possible " & _
        "pinch points."
4866:     strResponse = Linkages.Metadata_Functions.AddNewLineageStep(pDataset, strDescription, Now, JenMetadata_Processor, _
        "", True)
    
    ' ADD NEW GEOPROCESSING EVENT
4870:     strResponse = Linkages.Metadata_Functions.AddNewGeoProcStep(pDataset, _
        "[!!!NOTE - This command is not Python code!!!]:  Generate Bottleneck Statistics Table in " & _
        "Batch Bottleneck Analysis (" & _
        "[Wildland Block 1 = " & pBlock1Dataset.Workspace.PathName & "\" & pBlock1Dataset.BrowseName & "], " & _
        "[Wildland Block 2 = " & pBlock2Dataset.Workspace.PathName & "\" & pBlock2Dataset.BrowseName & "], " & _
        "[Corridor = " & pCorridorDataset.Workspace.PathName & "\" & pCorridorDataset.BrowseName & "], " & _
        "[Threshold Distance = " & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdDistance, True, 8) & _
        " " & strUnits & "], " & _
        "[Output Bottleneck Statistics Table Name = " & pDataset.BrowseName & "], " & _
        "[Output Bottleneck Statistics Workspace = " & pDataset.Workspace.PathName & "]", App.Path & "\Linkages.dll\BatchBottleneckTool", _
        Now, "Corridor Designer Extension Batch Bottleneck Tool", False)
    
    ' ADD NEW METADATA CONTACTS
4883:     strResponse = Linkages.Metadata_Functions.AddContact_Metadata(pDataset, JenMetadata_Processor, "", True)
    
    ' ADD NEW CITATION CONTACTS; SHOULD HAVE ORIGINATOR
4886:     strResponse = Linkages.Metadata_Functions.AddContact_CitationResponsibleParty(pDataset, JenMetadata_Originator, "", True)
      
    ' ADD NEW RESOURCE CONTACTS
4889:     strResponse = Linkages.Metadata_Functions.AddContact_ResourcePointOfContact(pDataset, JenMetadata_Processor, "", True)
    
    ' ADD CITATION DATES
4892:     strResponse = Linkages.Metadata_Functions.AddCitationDates(pDataset, Now, Now)
    
    ' SET RESOURCE STATUS
4895:     strResponse = Linkages.Metadata_Functions.AddResourceDetailsStatus(pDataset, JenMetadata_Completed)
    
    ' SET MAINTENANCE STATUS
4898:     strResponse = Linkages.Metadata_Functions.AddResourceMaintenance(pDataset, JenMetadata_Maint_Unknown)
    
    ' ADD FIELD INFO
4901:     strResponse = Linkages.Metadata_Functions.AddDetailsForObjectDefinition(pDataset, _
        "Statistics describing bottleneck route.", "Corridor Designer")
  
  '    lngStatsID = pStatsTable.FindField("Unique_ID")
  '    lngStatsStatistics = pStatsTable.FindField("Statistic")
  '    lngStatsValue = pStatsTable.FindField("Value")
  '
  '    lngSegsID = pSegsTable.FindField("Unique_ID")
  '    lngSegsName = pSegsTable.FindField("Name")
  '    lngSegsLength = pSegsTable.FindField("Length")
  '    lngUnitsIndex = pSegsTable.FindField("Units")
    
4913:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Unique_ID", _
        "Unique numeric ID values for each record in the Bottleneck Statistics table..", "Corridor Designer ArcGIS Tools", _
        "1", "99999999", , , , , , "Sequential Unique ID Values")
4916:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Value", _
        "Statistics describing the bottleneck route.", "Corridor Designer ArcGIS Tools", _
        , , , , , , , "General Descriptive Statistics")
    
    ReDim varData(0)
4921:     strThresholdAndUnits = "[Bottleneck = " & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdDistance, True, 8) & " " & strUnits & "]"
    ReDim strList(2, 11)
4923:     strList(0, 0) = "Minimum Width [" & strUnits & "]"
4924:     strList(1, 0) = "Width of the corridor at the narrowest point."
4925:     strList(2, 0) = "Corridor Designer ArcGIS Tools"
4926:     strList(0, 1) = "Maximum Width [" & strUnits & "]"
4927:     strList(1, 1) = "Width of the corridor at the widest point [" & strUnits & "]."
4928:     strList(2, 1) = "Corridor Designer ArcGIS Tools"
4929:     strList(0, 2) = "Range of Width Values"
4930:     strList(1, 2) = "Range of width values in corridor [" & strUnits & "]."
4931:     strList(2, 2) = "Corridor Designer ArcGIS Tools"
4932:     strList(0, 3) = "Mean Width [" & strUnits & "]"
4933:     strList(1, 3) = "Average width of corridor [" & strUnits & "]."
4934:     strList(2, 3) = "Corridor Designer ArcGIS Tools"
4935:     strList(0, 4) = "Median Width [" & strUnits & "]"
4936:     strList(1, 4) = "Median width in corridor [" & strUnits & "]."
4937:     strList(2, 4) = "Corridor Designer ArcGIS Tools"
4938:     strList(0, 5) = "Standard Deviation of Width"
4939:     strList(1, 5) = "Standard Deviation of corridor width."
4940:     strList(2, 5) = "Corridor Designer ArcGIS Tools"
4941:     strList(0, 6) = "Centerline Length [" & strUnits & "]."
4942:     strList(1, 6) = "Length of centerline through corridor, following the path with the widest narrow point [" & strUnits & "]."
4943:     strList(2, 6) = "Corridor Designer ArcGIS Tools"
4944:     strList(0, 7) = "Width Threshold Value"
4945:     strList(1, 7) = "Threshold value used to define bottlenecks [" & strUnits & "]."
4946:     strList(2, 7) = "Corridor Designer ArcGIS Tools"
4947:     strList(0, 8) = "Length Below Threshold [" & strUnits & "]."
4948:     strList(1, 8) = "Length of centerline that exists in a bottleneck " & strThresholdAndUnits & "."
4949:     strList(2, 8) = "Corridor Designer ArcGIS Tools"
4950:     strList(0, 9) = "Proportion Below Threshold"
4951:     strList(1, 9) = "Proportion of centerline that exists in a bottleneck (range = 0 to 1)."
4952:     strList(2, 9) = "Corridor Designer ArcGIS Tools"
4953:     strList(0, 10) = "Length Above Threshold [" & strUnits & "]."
4954:     strList(1, 10) = "Length of centerline that exists in a region where the corridor is sufficiently wide not to be " & _
        "considered a bottleneck " & strThresholdAndUnits & "."
4956:     strList(2, 10) = "Corridor Designer ArcGIS Tools"
4957:     strList(0, 11) = "Proportion Above Threshold"
4958:     strList(1, 11) = "Proportion of centerline that exists in a region where the corridor is sufficiently wide not to be " & _
        "considered a bottleneck (range = 0 to 1)."
4960:     strList(2, 11) = "Corridor Designer ArcGIS Tools"
4961:     varData(0) = strList
4962:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Statistic", _
        "Description of bottleneck statistics", "Corridor Designer ArcGIS Tools", _
        , , , , , , varData)
          
    ' SET METADATA FORMAT VERSION
4967:     strResponse = Linkages.Metadata_Functions.SetMetadataFormatVersion(pDataset, , True, pMxDoc)
      
    ' RESYNCHRONIZE METADATA
4970:     strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
4971:   End If
    
  '-----------------------------------------------------------------------------------------------------------------------
  ' WRITE SEGMENT STATISTICS METADATA =================================================================================
  '-----------------------------------------------------------------------------------------------------------------------
4976:   If booCalcSegStats Then
4977:     Set pDataset = pSegsTable
4978:     Set pPropSet = Linkages.Metadata_Functions.ReturnMetadataPropSetFromDataset(pDataset)
    
    ' SYNCHRONIZE METADATA
4981:     strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
    
    ' WRITE ABSTRACT
4984:     strAbstract = "This table contains statistics describing the portions of the bottleneck route inside and " & _
      "outside of bottlenecks.  This bottleneck route " & _
      "describes the path between the two habitat blocks '" & pBlock1Dataset.BrowseName & "' and '" & _
      pBlock2Dataset.BrowseName & "', within the corridor polygon '" & pCorridorDataset.BrowseName & "'., " & _
      "which follows the route with the widest narrow point."
4989:     strResponse = Linkages.Metadata_Functions.SetMetadataAbstract(pDataset, strAbstract)
    
    ' WRITE PURPOSE
4992:     strPurpose = "Statistics describing regions of route with widest bottleneck inside and outside of bottlenecks."
4993:     strResponse = Linkages.Metadata_Functions.SetMetadataPurpose(pDataset, strPurpose)
    
    ' ADD KEYWORDS
4996:     Set pIncludeThemeKeys = New esriSystem.strArray
4997:     Set pIncludeSearchKeys = New esriSystem.strArray
4998:     Set pIncludeDescKeys = New esriSystem.strArray
4999:     Set pIncludeStratKeys = New esriSystem.strArray
5000:     Set pIncludeThemeSlashThemekeys = New esriSystem.strArray
5001:     Set pIncludePlaceKeys = New esriSystem.strArray
5002:     Set pIncludeTemporalKeys = New esriSystem.strArray
      
5004:     pIncludeThemeKeys.Add "Bottleneck"
5005:     pIncludeThemeKeys.Add "Width"
5006:     pIncludeThemeKeys.Add "Corridor"
5007:     pIncludeThemeKeys.Add "Statistics"
5008:     pIncludeThemeKeys.Add pBlock1Dataset.BrowseName
5009:     pIncludeThemeKeys.Add pBlock2Dataset.BrowseName
5010:     pIncludeThemeKeys.Add pCorridorDataset.BrowseName
    
5012:     For lngIndex = 0 To pIncludeThemeKeys.Count - 1
5013:       pIncludeSearchKeys.Add pIncludeThemeKeys.Element(lngIndex)
5014:       pIncludeDescKeys.Add pIncludeThemeKeys.Element(lngIndex)
5015:     Next lngIndex
    
  ' EXTRACT ANY EXISTING KEYWORDS FROM CURRENT BLOCK OR CORRIDOR FCLASSES
5018:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock1FClass, _
        pKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
5022:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pBlock2FClass, _
        pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
5026:     Set pCombinedKeyWords = Linkages.Metadata_Functions.ReturnExistingMetadataKeyWords(pCorridorFClass, _
        pCombinedKeyWords, booSucceeded, pIncludeThemeKeys, pIncludeSearchKeys, pIncludeDescKeys, pIncludeStratKeys, _
        pIncludeThemeSlashThemekeys, _
        pIncludePlaceKeys, pIncludeTemporalKeys)
    
5031:     strResponse = Linkages.Metadata_Functions.SetMetadataKeyWords(pDataset, pIncludeThemeKeys, pIncludeSearchKeys, _
          pIncludeDescKeys, pIncludeStratKeys, pIncludeThemeSlashThemekeys, pIncludePlaceKeys, pIncludeTemporalKeys)
    
    ' SET PROCESS STEP
5035:     strDescription = "Generated bottleneck statistics describing portions of route inside and outside " & _
        "of bottlenecks.  This route connects '" & pBlock1Dataset.BrowseName & "' to '" & _
        pBlock1Dataset.BrowseName & "' along corridor '" & pCorridorDataset.BrowseName & "' following the path with the widest possible " & _
        "pinch points."
5039:     strResponse = Linkages.Metadata_Functions.AddNewLineageStep(pDataset, strDescription, Now, JenMetadata_Processor, _
        "", True)
    
    ' ADD NEW GEOPROCESSING EVENT
5043:     strResponse = Linkages.Metadata_Functions.AddNewGeoProcStep(pDataset, _
        "[!!!NOTE - This command is not Python code!!!]:  Generate Bottleneck Segment Statistics Table in " & _
        "Batch Bottleneck Analysis (" & _
        "[Wildland Block 1 = " & pBlock1Dataset.Workspace.PathName & "\" & pBlock1Dataset.BrowseName & "], " & _
        "[Wildland Block 2 = " & pBlock2Dataset.Workspace.PathName & "\" & pBlock2Dataset.BrowseName & "], " & _
        "[Corridor = " & pCorridorDataset.Workspace.PathName & "\" & pCorridorDataset.BrowseName & "], " & _
        "[Threshold Distance = " & Linkages.MyGeneralOperations.TrimZerosAndDecimals(dblThresholdDistance, True, 8) & _
        " " & strUnits & "], " & _
        "[Output Segment Statistics Table Name = " & pDataset.BrowseName & "], " & _
        "[Output Segment Statistics Workspace = " & pDataset.Workspace.PathName & "]", App.Path & "\Linkages.dll\BatchBottleneckTool", _
        Now, "Corridor Designer Extension Batch Bottleneck Tool", False)
    
    ' ADD NEW METADATA CONTACTS
5056:     strResponse = Linkages.Metadata_Functions.AddContact_Metadata(pDataset, JenMetadata_Processor, "", True)
    
    ' ADD NEW CITATION CONTACTS; SHOULD HAVE ORIGINATOR
5059:     strResponse = Linkages.Metadata_Functions.AddContact_CitationResponsibleParty(pDataset, JenMetadata_Originator, "", True)
      
    ' ADD NEW RESOURCE CONTACTS
5062:     strResponse = Linkages.Metadata_Functions.AddContact_ResourcePointOfContact(pDataset, JenMetadata_Processor, "", True)
    
    ' ADD CITATION DATES
5065:     strResponse = Linkages.Metadata_Functions.AddCitationDates(pDataset, Now, Now)
    
    ' SET RESOURCE STATUS
5068:     strResponse = Linkages.Metadata_Functions.AddResourceDetailsStatus(pDataset, JenMetadata_Completed)
    
    ' SET MAINTENANCE STATUS
5071:     strResponse = Linkages.Metadata_Functions.AddResourceMaintenance(pDataset, JenMetadata_Maint_Unknown)
    
    ' ADD FIELD INFO
5074:     strResponse = Linkages.Metadata_Functions.AddDetailsForObjectDefinition(pDataset, _
        "Statistics describing segments along bottleneck route.", "Corridor Designer")
  
  '    lngStatsID = pStatsTable.FindField("Unique_ID")
  '    lngStatsStatistics = pStatsTable.FindField("Statistic")
  '    lngStatsValue = pStatsTable.FindField("Value")
  '
  '    lngSegsID = pSegsTable.FindField("Unique_ID")
  '    lngSegsName = pSegsTable.FindField("Name")
  '    lngSegsLength = pSegsTable.FindField("Length")
  '    lngUnitsIndex = pSegsTable.FindField("Units")
    
5086:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Unique_ID", _
        "Unique numeric ID values for each record in the Bottleneck Statistics table..", "Corridor Designer ArcGIS Tools", _
        "1", "99999999", , , , , , "Sequential Unique ID Values")
5089:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Length", _
        "Length [" & strUnits & "] of each segment.", "Corridor Designer ArcGIS Tools", _
        , , , , , , , "General Descriptive Statistics")
        
    ReDim varData(0)
    ReDim strList(2, 7)

5096:     strList(0, 0) = "Total Length"
5097:     strList(1, 0) = "Length of centerline through corridor, following the path with the widest narrow point [" & strUnits & "]."
5098:     strList(2, 0) = "Corridor Designer ArcGIS Tools"
5099:     strList(0, 1) = "Threshold Value"
5100:     strList(1, 1) = "Threshold value used to define bottlenecks [" & strUnits & "]."
5101:     strList(2, 1) = "Corridor Designer ArcGIS Tools"
5102:     strList(0, 2) = "Length Below Threshold [" & strUnits & "]."
5103:     strList(1, 2) = "Length of centerline that exists in a bottleneck " & strThresholdAndUnits & "."
5104:     strList(2, 2) = "Corridor Designer ArcGIS Tools"
5105:     strList(0, 3) = "Proportion Below Threshold"
5106:     strList(1, 3) = "Proportion of centerline that exists in a bottleneck (range = 0 to 1)."
5107:     strList(2, 3) = "Corridor Designer ArcGIS Tools"
5108:     strList(0, 4) = "Length Above Threshold [" & strUnits & "]."
5109:     strList(1, 4) = "Length of centerline that exists in a region where the corridor is sufficiently wide not to be " & _
        "considered a bottleneck " & strThresholdAndUnits & "."
5111:     strList(2, 4) = "Corridor Designer ArcGIS Tools"
5112:     strList(0, 5) = "Proportion Above Threshold"
5113:     strList(1, 5) = "Proportion of centerline that exists in a region where the corridor is sufficiently wide not to be " & _
        "considered a bottleneck (range = 0 to 1)."
5115:     strList(2, 5) = "Corridor Designer ArcGIS Tools"
5116:     strList(0, 6) = "Above Threshold (>" & CStr(dblThresholdDistance) & ")"
5117:     strList(1, 6) = "Description of single stretch of centerline that is outside of a bottleneck."
5118:     strList(2, 6) = "Corridor Designer ArcGIS Tools"
5119:     strList(0, 7) = "Below Threshold (<" & CStr(dblThresholdDistance) & ")"
5120:     strList(1, 7) = "Description of single stretch of centerline that is inside of a bottleneck."
5121:     strList(2, 7) = "Corridor Designer ArcGIS Tools"
    
5123:     varData(0) = strList
5124:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Name", _
        "Description of bottleneck statistics", "Corridor Designer ArcGIS Tools", _
        , , , , , , varData)
          
5128:     strResponse = Linkages.Metadata_Functions.AddFieldAttributes(pDataset, "Units", _
        "Measurement units of statistics in 'Length' field.", "Corridor Designer ArcGIS Tools", _
        , , , , , , , "Typically 'Meters', 'Feet' or 'Proportion'")
        
    ' SET METADATA FORMAT VERSION
5133:     strResponse = Linkages.Metadata_Functions.SetMetadataFormatVersion(pDataset, , True, pMxDoc)
      
    ' RESYNCHRONIZE METADATA
5136:     strResponse = Linkages.Metadata_Functions.SynchronizeMetadataPropSet(pDataset)
  
5138:   End If
  
5140:   MakeStatsTablesBatch = "Succeeded"
5141:   booSucceeded = True


5144:   GoTo ClearMemory
  Exit Function
ErrorHandler:
'  HandleError True, "MakeStatsTablesBatch " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
  
5149:   MakeStatsTablesBatch = "MakeStatsTablesBatch Failed [ in " & c_sModuleFileName & "]:  " & err.Description & ", Error #" & _
      CStr(err.Number) & " [" & GetErrorLineNumberString(Erl) & "]"
  
5152:   booSucceeded = False


ClearMemory:
5156:   Set pSegArray = Nothing
5157:   Erase dblWidths
5158:   Set pPrjSpRef = Nothing
5159:   Set pGeoSpRef = Nothing
5160:   Set pDatum = Nothing
5161:   Set pPolyline = Nothing
5162:   Set pAboveSegmentData = Nothing
5163:   Set pBelowSegmentData = Nothing
5164:   Set pSubArray = Nothing
5165:   Set pStatsFields = Nothing
5166:   Set pStatsUniqueIDField = Nothing
5167:   Set pStatsUniqueIDFieldEdit = Nothing
5168:   Set pStatsStatisticsField = Nothing
5169:   Set pStatsStatisticsFieldEdit = Nothing
5170:   Set pStatsValueField = Nothing
5171:   Set pStatsValueFieldEdit = Nothing
5172:   Set pStatsTable = Nothing
5173:   Set pStatsCur = Nothing
5174:   Set pStatsBuf = Nothing
5175:   Set pSegsNameField = Nothing
5176:   Set pSegsNameFieldEdit = Nothing
5177:   Set pSegsLengthField = Nothing
5178:   Set pSegsLengthFieldEdit = Nothing
5179:   Set pSegsUnitsField = Nothing
5180:   Set pSegsUnitsFieldEdit = Nothing
5181:   Set pSegsTable = Nothing
5182:   Set pSegsCur = Nothing
5183:   Set pSegsBuf = Nothing



End Function
Public Sub AddStringToEndOfArray(varStringArray As Variant, strInsertValue As String)
  On Error GoTo ErrorHandler

  Dim strArray() As String
  Dim strNewArray() As String
  Dim lngIndex As Long
  Dim lngCounter As Long
  Dim strTestValue As String
  Dim booValueAlreadyThere As Boolean
  
5198:   booValueAlreadyThere = False
  
5200:   If IsNull(varStringArray) Or IsEmpty(varStringArray) Then
    ReDim strNewArray(0)
5202:     strNewArray(0) = strInsertValue
5203:   Else
5204:     strNewArray = varStringArray
    
5206:     For lngIndex = 0 To UBound(strNewArray)
5207:       strTestValue = strNewArray(lngIndex)
5208:       If StrComp(strTestValue, strInsertValue, vbTextCompare) = 0 Then
5209:         booValueAlreadyThere = True
5210:       End If
5211:     Next lngIndex
    
5213:     If Not booValueAlreadyThere Then
      ReDim Preserve strNewArray(UBound(strNewArray) + 1)
5215:       strNewArray(UBound(strNewArray)) = strInsertValue
5216:     End If
    
5218:   End If
  
5220:   varStringArray = strNewArray

5222:   GoTo ClearMemory
  Exit Sub
ErrorHandler:
  HandleError True, "AddStringToEndOfArray " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

ClearMemory:
5228:   Erase strArray
5229:   Erase strNewArray
End Sub
