Attribute VB_Name = "CorridorAnalysisFunctions"
Option Explicit
Const c_sModuleFileName As String = "D:\arcGIS_stuff\consultation\az_linkages\VB_Code\CorridorAnalyses.bas"


' ApplyCorridorSymbol - GIVEN POLYGON ELEMENT, RETURNS AN IFillShapeElement
' StatsForStrings - GIVEN IStringArray AND OPTIONAL IDblArray, RETURNS ARRAY OF COUNTS, PROPORTIONS AND SIZES OF EACH UNIQUE ID
' CheckCollectionForKey - GIVEN pCollection and STRING, RETURNS BOOLEAN INDICATING WHETHER COLLECTION HAS THAT KEY OR NOT
' StatsForDates - GIVEN IVariantArray, RETURNS IVariantArray CONTAINING EARLIEST AND LATEST DATE
' GridHistogram - GIVEN RASTER, LOW VAL, HIGH VAL AND NUMBER BINS, RETURN
' BottleneckAnalysis - RUN BOTTLENECK ANALYSIS AND OPEN DIALOG

Public Sub BottleneckAnalysisDemo(pApp As IApplication)
  On Error GoTo ErrorHandler

  Dim pSpatialReferenceFactory2 As ISpatialReferenceFactory2
16:   Set pSpatialReferenceFactory2 = New SpatialReferenceEnvironment ' The file test.prj contains a spatial reference in its WKT aka string representation.
  Dim pSpRef As ISpatialReference
18:   Set pSpRef = pSpatialReferenceFactory2.CreateSpatialReference(esriSRProjCS_NAD1983UTM_12N)
  
  ' PERFORM ANALYSIS
  Dim pDataArray As esriSystem.IVariantArray
22:   Set pDataArray = Linkages.CorridorSampleData.MakeData
  Dim frmGraph As Linkages.frmWidthGraph
24:   Set frmGraph = New Linkages.frmWidthGraph
  
26:   Set frmGraph.ArcApplication = pApp
27:   Set frmGraph.SpatReference = pSpRef
28:   Set frmGraph.GraphNumbers = pDataArray
  
30:   frmGraph.Frame.Visible = True


33:   Set pSpatialReferenceFactory2 = Nothing
34:   Set pSpRef = Nothing
35:   Set pDataArray = Nothing
36:   Set frmGraph = Nothing
  
  Exit Sub
ErrorHandler:
  HandleError True, "BottleneckAnalysisDemo " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


43:   Set pSpatialReferenceFactory2 = Nothing
44:   Set pSpRef = Nothing
45:   Set pDataArray = Nothing
46:   Set frmGraph = Nothing
  
End Sub

Public Sub BottleneckAnalysis(pParamDetails As esriSystem.IVariantArray, pMxDoc As IMxDocument, pApp As IApplication, _
  pCorridor As IPolygon, pStartBlock As IPolygon, pEndBlock As IPolygon, ProgressForm As Linkages.frmJenProgressPercent)
  On Error GoTo ErrorHandler


  Dim anIndex As Long
  
  Dim pSpRef As ISpatialReference
58:   Set pSpRef = pCorridor.SpatialReference
  
  ' PERFORM ANALYSIS
  Dim pDataArray As esriSystem.IVariantArray
'  Set pDataArray = Linkages.CorridorSampleData.MakeData
  Dim pStartPoly As IPolygon4
  Dim pEndPoly As IPolygon4
  Dim pCorPoly As IPolygon4
66:   Set pStartPoly = pStartBlock
67:   Set pEndPoly = pEndBlock
68:   Set pCorPoly = pCorridor
  
70:   Set pDataArray = RunBottleneck(pStartPoly, pEndPoly, pCorPoly, pMxDoc, pApp, ProgressForm)
'  MsgBox "Data Array is Nothing? " & CStr(pDataArray Is Nothing)
'  MsgBox pDataArray.Count & " items in array..."
  Dim frmGraph As Linkages.frmWidthGraph
74:   Set frmGraph = New Linkages.frmWidthGraph
'  MsgBox "Spatial Reference Name = " & pSpRef.Name
76:   Set frmGraph.ArcApplication = pApp
77:   Set frmGraph.SpatReference = pSpRef
'  frmGraph.GraphNumbers = dblVal
79:   Set frmGraph.GraphNumbers = pDataArray
  
81:   If ProgressForm.chkClose.Value = 1 Then
82:     Unload ProgressForm
83:     Set ProgressForm = Nothing
84:   End If
  
86:   frmGraph.Frame.Visible = True


89:   Set pSpRef = Nothing
90:   Set pDataArray = Nothing
91:   Set pStartPoly = Nothing
92:   Set pEndPoly = Nothing
93:   Set pCorPoly = Nothing
94:   Set frmGraph = Nothing

  Exit Sub
ErrorHandler:
  HandleError True, "BottleneckAnalysis " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


101:   Set pSpRef = Nothing
102:   Set pDataArray = Nothing
103:   Set pStartPoly = Nothing
104:   Set pEndPoly = Nothing
105:   Set pCorPoly = Nothing
106:   Set frmGraph = Nothing

End Sub

Public Function RunBottleneck(pStartPoly As IPolygon4, pEndPoly As IPolygon4, pCorPolygon As IPolygon4, _
    pMxDoc As IMxDocument, pApp As IApplication, frmProgress As Linkages.frmJenProgressPercent) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

  Dim pSpRef As ISpatialReference
115:   Set pSpRef = pCorPolygon.SpatialReference
  
  Dim pTopoOp As ITopologicalOperator
118:   Set pTopoOp = pCorPolygon
  Dim pRelOp As IRelationalOperator
120:   Set pRelOp = pCorPolygon
  
  Dim strMessage As String
  Dim lngCounter As Long
  Dim lngCount As Long
125:   lngCounter = 0
126:   lngCount = 18
  
  Dim theDetailedDescription As String
129:   theDetailedDescription = frmProgress.txtDetails.Text
  
  ' #1
132:   lngCounter = lngCounter + 1
133:   strMessage = "Preparing habitat and corridor polygons (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
136:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
139:   frmProgress.ProgRecCount = lngCount
140:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
141:   theDetailedDescription = theDetailedDescription & strMessage
142:   frmProgress.txtDetails.Text = theDetailedDescription
143:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
144:   frmProgress.Est_Time_Left lngCounter, "Preparing Data...", "Step "
145:   frmProgress.Refresh
146:   frmProgress.icnProgressLine.Refresh
147:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  ' CREATE FINAL START POLYGON REPRESENTING ONLY OUTERMOST RINGS OF ALL SUB-PARTS OF START POLYGON THAT
  ' INTERSECT CORRIDOR POLYGON
  ' GET OUTERMOST COMPONENTS OF START POLYGON THAT INTERSECT CORRIDOR
  Dim pPolyOutStart() As IPolygon 'Declare an array of polygon
  Dim lngNumParts As Long
  Dim lngIndex As Long
156:   lngNumParts = pStartPoly.ExteriorRingCount
  ReDim pPolyOutStart(lngNumParts - 1) 'Redimension the array of polygon with number of exterior ring
158:   pStartPoly.GetConnectedComponents lngNumParts, pPolyOutStart(0) 'Pass the first element of the array
  
  ' GET OUTERMOST RING OF EACH POLYGON IN ARRAY
  Dim pClone As IClone
  Dim pTempStartA As IPolygon4
  Dim pRing() As IRing
  Dim pOuterRing As IRing
  Dim pPath As ISegmentCollection
  Dim pTopoForSimplify As ITopologicalOperator
  Dim pFinalStartPolygon As IPolygon4
168:   Set pFinalStartPolygon = New Polygon
169:   Set pFinalStartPolygon.SpatialReference = pSpRef
  Dim pOutputPolygon As IGeometryCollection
171:   Set pOutputPolygon = pFinalStartPolygon
  Dim pFinalStartLine As IPolyline
173:   Set pFinalStartLine = New Polyline
174:   Set pFinalStartLine.SpatialReference = pSpRef
  Dim pOutputPolyline As IGeometryCollection
176:   Set pOutputPolyline = pFinalStartLine
177:   For lngIndex = 0 To lngNumParts - 1
178:     Set pTempStartA = pPolyOutStart(lngIndex)
179:     Set pTopoForSimplify = pTempStartA
180:     pTopoForSimplify.Simplify
181:     If Not pRelOp.Disjoint(pTempStartA) Then  ' ONLY PICK SUB-POLYGONS THAT INTERSECT CORRIDOR
      ReDim pRing(0)
183:       pTempStartA.QueryExteriorRingsEx 1, pRing(0)   ' ONLY GET OUTERMOST RING OF POLYGON
184:       Set pOuterRing = pRing(0)
185:       Set pClone = pOuterRing
186:       pOutputPolygon.AddGeometry pOuterRing
187:       Set pPath = New Path
188:       pPath.AddSegmentCollection pClone.Clone
189:       pOutputPolyline.AddGeometry pPath
190:     End If
191:   Next
192:   Set pTopoForSimplify = pFinalStartPolygon
193:   pTopoForSimplify.Simplify
194:   Set pTopoForSimplify = pOutputPolyline
195:   pTopoForSimplify.Simplify
  
  Dim pClipStart As IPolygon
198:   Set pClipStart = pTopoOp.Intersect(pFinalStartPolygon, esriGeometry2Dimension)
  'Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pFinalStartPolygon, "delete_corridors"
  
  ' CREATE FINAL END POLYGON REPRESENTING ONLY OUTERMOST RINGS OF ALL SUB-PARTS OF END POLYGON THAT
  ' INTERSECT CORRIDOR POLYGON
  Dim pPolyOutEnd() As IPolygon 'Declare an array of polygon
204:   lngNumParts = pEndPoly.ExteriorRingCount
  ReDim pPolyOutEnd(lngNumParts - 1) 'Redimension the array of polygon with number of exterior ring
206:   pEndPoly.GetConnectedComponents lngNumParts, pPolyOutEnd(0) 'Pass the first element of the array
  
  ' GET OUTERMOST RING OF EACH POLYGON IN ARRAY
  Dim pTempEndA As IPolygon4
  Dim pFinalEndPolygon As IPolygon4
211:   Set pFinalEndPolygon = New Polygon
  Dim pOutputEndPolygon As IGeometryCollection
213:   Set pOutputEndPolygon = pFinalEndPolygon
  Dim pFinalEndPolyline As IPolyline
215:   Set pFinalEndPolyline = New Polyline
  Dim pOutputEndPolyline As IGeometryCollection
217:   Set pOutputEndPolyline = pFinalEndPolyline
218:   For lngIndex = 0 To lngNumParts - 1
219:     Set pTempEndA = pPolyOutEnd(lngIndex)
220:     Set pTopoForSimplify = pTempEndA
221:     pTopoForSimplify.Simplify
222:     If Not pRelOp.Disjoint(pTempEndA) Then  ' ONLY PICK SUB-POLYGONS THAT INTERSECT CORRIDOR
      ReDim pRing(0)
224:       pTempEndA.QueryExteriorRingsEx 1, pRing(0)   ' ONLY GET OUTERMOST RING OF POLYGON
225:       Set pOuterRing = pRing(0)
226:       Set pClone = pOuterRing
227:       pOutputEndPolygon.AddGeometry pOuterRing
228:       Set pPath = New Path
229:       pPath.AddSegmentCollection pClone.Clone
230:       pOutputEndPolyline.AddGeometry pPath
231:     End If
232:   Next
233:   Set pTopoForSimplify = pFinalEndPolygon
234:   pTopoForSimplify.Simplify
235:   Set pTopoForSimplify = pOutputEndPolyline
236:   pTopoForSimplify.Simplify

  Dim pClipEnd As IPolygon
239:   Set pClipEnd = pTopoOp.Intersect(pEndPoly, esriGeometry2Dimension)
  
  
  ' GET SET OF POLYLINES, REPRESENTING PORTION OF FINAL START/END POLYGON EDGES THAT LAY WITHIN CORRIDOR
  Dim pStartPolyline As IPolyline
244:   Set pStartPolyline = pTopoOp.Intersect(pOutputPolyline, esriGeometry1Dimension)
  Dim pEndPolyline As IPolyline
246:   Set pEndPolyline = pTopoOp.Intersect(pOutputEndPolyline, esriGeometry1Dimension)
  
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pClipStart, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pClipEnd, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pStartPolyline, "delete_corridors"
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pEndPolyline, "delete_corridors"
  
'  ' MAKE DISTANCE GRID
'  ' FIRST MAKE POLYON, BASED ON BOUNDARY EXTENT WITH BOUNDARY CLIPPED OUT
  
  ' COMBINE CORRIDOR WITH START AND END HABITAT BLOCK SHELLS (NO INTERIOR HOLES, ONLY PORTIONS THAT INTERSECT CORRIDOR
  Dim pUnionPoly As IPolygon4
259:   Set pTopoOp = pCorPolygon
260:   Set pUnionPoly = pTopoOp.Union(pFinalEndPolygon)
261:   Set pTopoOp = pUnionPoly
  Dim pUnionPoly2 As IPolygon4
263:   Set pUnionPoly2 = pTopoOp.Union(pFinalStartPolygon)
264:   Set pTopoOp = pUnionPoly2
  
  ' CREATE AOI POLYGON
  Dim pAOIPoly As IPolygon4
268:   Set pAOIPoly = New Polygon
269:   Set pAOIPoly.SpatialReference = pSpRef
  Dim pIntermediatePoly As IPointCollection
271:   Set pIntermediatePoly = pAOIPoly
  
  ' CREATE AOI ENVELOPE
  Dim pAOIEnv As IEnvelope
275:   Set pAOIEnv = pCorPolygon.Envelope
276:   pAOIEnv.Expand 1.1, 1.1, True
  
  ' CONVERT AOI ENVELOPE TO AOI POLYGON
  Dim dXmin As Double
  Dim dYmin As Double
  Dim dXmax As Double
  Dim dYmax As Double
283:   pAOIEnv.QueryCoords dXmin, dYmin, dXmax, dYmax
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim pPoint3 As IPoint
  Dim pPoint4 As IPoint
288:   Set pPoint1 = New Point
289:   Set pPoint2 = New Point
290:   Set pPoint3 = New Point
291:   Set pPoint4 = New Point
292:   pPoint1.PutCoords dXmin, dYmin
293:   pPoint2.PutCoords dXmin, dYmax
294:   pPoint3.PutCoords dXmax, dYmax
295:   pPoint4.PutCoords dXmax, dYmin
296:   pIntermediatePoly.AddPoint pPoint1
297:   pIntermediatePoly.AddPoint pPoint2
298:   pIntermediatePoly.AddPoint pPoint3
299:   pIntermediatePoly.AddPoint pPoint4
300:   pIntermediatePoly.AddPoint pPoint1
301:   Set pTopoForSimplify = pAOIPoly
302:   pTopoForSimplify.Simplify
  
  ' CLIP OUT CORRIDOR UNION FROM AOI POLYGON
305:   Set pTopoOp = pAOIPoly
  Dim pNonHabitatPolygon As IPolygon4
307:   Set pNonHabitatPolygon = pTopoOp.Difference(pUnionPoly2)
308:   Set pTopoForSimplify = pNonHabitatPolygon
309:   pTopoForSimplify.Simplify
  Dim pHabitatPolygon As IPolygon4
311:   Set pHabitatPolygon = pTopoOp.Intersect(pUnionPoly2, esriGeometry2Dimension)
312:   Set pTopoForSimplify = pHabitatPolygon
313:   pTopoForSimplify.Simplify

'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pHabitatPolygon, "delete_corridors"
  
  Dim dblMaxDimension As Double
  Dim dblCellSize As Double
  
320:   dblMaxDimension = pAOIEnv.Width
321:   If pAOIEnv.Height > pAOIEnv.Width Then dblMaxDimension = pAOIEnv.Height
  
323:   dblCellSize = dblMaxDimension / 1000
  ' FOR DEBUGGING
'  dblCellSize = dblMaxDimension / 70
                    
  ' #2
328:   lngCounter = lngCounter + 1
329:   strMessage = "Preparing Raster Analysis Environment (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
332:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
335:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
336:   theDetailedDescription = theDetailedDescription & strMessage
337:   frmProgress.txtDetails.Text = theDetailedDescription
338:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
339:   frmProgress.Est_Time_Left lngCounter, "Preparing Analysis Environment...", "Step "
340:   frmProgress.Refresh
341:   frmProgress.icnProgressLine.Refresh
342:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------

  
  ' MAKE CORRIDOR RASTER
  Dim pCorRaster As IRaster
  Dim pEnv As IRasterAnalysisEnvironment  'Create a RasterMakerOp operator
  Dim pRasMakerOp As IRasterMakerOp
350:   Set pRasMakerOp = New RasterMakerOp
  
  'Create an analysis environment object
'  Dim pEnv As IRasterAnalysisEnvironment
354:   Set pEnv = pRasMakerOp
  
  'Set cellsize
357:   pEnv.SetCellSize esriRasterEnvValue, dblCellSize
  
  'Set output extent
360:   pEnv.SetExtent esriRasterEnvValue, pAOIEnv
  
  'Set output spatial reference
363:   Set pEnv.OutSpatialReference = pSpRef
  
  'Create a constant raster
  Dim pBaseRaster As IRaster
367:   Set pBaseRaster = pRasMakerOp.MakeConstant(1, True)
  
'  GridFunctions.ReturnCellSize pBaseRaster
  
  ' CLIP TO CORRIDOR POLYGON
372:   Set pCorRaster = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pHabitatPolygon, True, pAOIEnv, dblCellSize)
 
  ' FOR DEBUGGING
'  Dim pRasterLayer As IRasterLayer
'  Set pRasterLayer = New RasterLayer
'  pRasterLayer.CreateFromRaster pCorRaster
'  pDoc.FocusMap.AddLayer pRasterLayer
  
  ' MAKE RASTER OPERATIONS OBJECTS
  Dim pLogOp As ILogicalOp         ' FOR CHECKING IF NULL
  Dim pDistanceOp As IDistanceOp   ' FOR CALCULATING EUCLIDIAN DISTANCE
  Dim pMathOp As IMathOp           ' FOR MULTIPLYING
  Dim pCondOp As IConditionalOp    ' FOR SETTING TO NULL
  
386:   Set pLogOp = New RasterMathOps
387:   Set pDistanceOp = New RasterDistanceOp
388:   Set pMathOp = New RasterMathOps
389:   Set pCondOp = New RasterConditionalOp
  
  ' #3
392:   lngCounter = lngCounter + 1
393:   strMessage = "Generating grid of corridor region (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
396:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
399:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
400:   theDetailedDescription = theDetailedDescription & strMessage
401:   frmProgress.txtDetails.Text = theDetailedDescription
402:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
403:   frmProgress.Est_Time_Left lngCounter, "Generating Corridor Grid...", "Step "
404:   frmProgress.Refresh
405:   frmProgress.icnProgressLine.Refresh
406:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  ' MAKE RASTER OF NULL PARTS OF pCorRaster
  Dim pOutsideRasterA As IGeoDataset
411:   Set pOutsideRasterA = pLogOp.IsNull(pCorRaster)
  Dim pOutsideRasterB As IGeoDataset
413:   Set pOutsideRasterB = pCondOp.SetNull(pLogOp.NotEqual(pOutsideRasterA, pBaseRaster), pOutsideRasterA)
  
  ' FOR DEBUGGING
'  Dim pOutsideRasterLayer As IRasterLayer
'  Set pOutsideRasterLayer = New RasterLayer
'  pOutsideRasterLayer.CreateFromRaster pOutsideRasterB
'  pDoc.FocusMap.AddLayer pOutsideRasterLayer
  
  ' #4
422:   lngCounter = lngCounter + 1
423:   strMessage = "Calculating distance to corridor edge (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
  
427:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
430:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
431:   theDetailedDescription = theDetailedDescription & strMessage
432:   frmProgress.txtDetails.Text = theDetailedDescription
433:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
434:   frmProgress.Est_Time_Left lngCounter, "Generating Distance Grid...", "Step "
435:   frmProgress.Refresh
436:   frmProgress.icnProgressLine.Refresh
437:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pOutputRaster As IGeoDataset
441:   Set pOutputRaster = pDistanceOp.EucDistance(pOutsideRasterB)
  
  ' FOR DEBUGGING
'  Dim pDistRasterLayerA As IRasterLayer
'  Set pDistRasterLayerA = New RasterLayer
'  pDistRasterLayerA.CreateFromRaster pOutputRaster
'  pDoc.FocusMap.AddLayer pDistRasterLayerA
  
  ' #5
450:   lngCounter = lngCounter + 1
451:   strMessage = "Combining distance and corridor grids (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
454:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
457:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
458:   theDetailedDescription = theDetailedDescription & strMessage
459:   frmProgress.txtDetails.Text = theDetailedDescription
460:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
461:   frmProgress.Est_Time_Left lngCounter, "Combining Grids...", "Step "
462:   frmProgress.Refresh
463:   frmProgress.icnProgressLine.Refresh
464:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pDistanceRaster As IGeoDataset
468:   Set pDistanceRaster = pMathOp.Times(pCorRaster, pOutputRaster)
  
  ' FOR DEBUGGING
'  Dim pDistRasterLayer As IRasterLayer
'  Set pDistRasterLayer = New RasterLayer
'  pDistRasterLayer.CreateFromRaster pDistanceRaster
'  pDoc.FocusMap.AddLayer pDistRasterLayer

  ' MAKE DIRECTION AND FLOW ACCUMULATION GRID FROM DISTANCE GRID
  Dim pDirGrid As IGeoDataset
  Dim pFlowAccGrid As IGeoDataset
  Dim pHydOp As IHydrologyOp
480:   Set pHydOp = New RasterHydrologyOp
  
  ' #6
483:   lngCounter = lngCounter + 1
484:   strMessage = "Generating Flow Direction grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
487:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
490:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
491:   theDetailedDescription = theDetailedDescription & strMessage
492:   frmProgress.txtDetails.Text = theDetailedDescription
493:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
494:   frmProgress.Est_Time_Left lngCounter, "Generating Flow Direction Grid...", "Step "
495:   frmProgress.Refresh
496:   frmProgress.icnProgressLine.Refresh
497:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
500:   Set pDirGrid = pHydOp.FlowDirection(pDistanceRaster, False, False)
  
  ' #7
503:   lngCounter = lngCounter + 1
504:   strMessage = "Generating Flow Accumulation grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
507:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
510:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
511:   theDetailedDescription = theDetailedDescription & strMessage
512:   frmProgress.txtDetails.Text = theDetailedDescription
513:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
514:   frmProgress.Est_Time_Left lngCounter, "Generating Flow Accumulation Grid...", "Step "
515:   frmProgress.Refresh
516:   frmProgress.icnProgressLine.Refresh
517:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
520:   Set pFlowAccGrid = pHydOp.FlowAccumulation(pDirGrid)
  
  ' GET CELLS WITH 0 FLOW ACCUMULATION
  ' #8
524:   lngCounter = lngCounter + 1
525:   strMessage = "Identifying centerline cells in corridor (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
528:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
531:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
532:   theDetailedDescription = theDetailedDescription & strMessage
533:   frmProgress.txtDetails.Text = theDetailedDescription
534:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
535:   frmProgress.Est_Time_Left lngCounter, "Identifying Central Cells...", "Step "
536:   frmProgress.Refresh
537:   frmProgress.icnProgressLine.Refresh
538:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pZeroGrid As IGeoDataset
542:   Set pZeroGrid = pLogOp.EqualTo(pFlowAccGrid, pRasMakerOp.MakeConstant(0, True))
  
  ' #9
545:   lngCounter = lngCounter + 1
546:   strMessage = "Clipping centerline cells (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
549:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
552:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
553:   theDetailedDescription = theDetailedDescription & strMessage
554:   frmProgress.txtDetails.Text = theDetailedDescription
555:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
556:   frmProgress.Est_Time_Left lngCounter, "Clipping Central Cells...", "Step "
557:   frmProgress.Refresh
558:   frmProgress.icnProgressLine.Refresh
559:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pPossibleGrid As IGeoDataset
563:   Set pPossibleGrid = pCondOp.SetNull(pLogOp.NotEqual(pZeroGrid, pBaseRaster), pZeroGrid)
  
  ' FOR DEBUGGING
'  Dim pPossibleGridLayer As IRasterLayer
'  Set pPossibleGridLayer = New RasterLayer
'  pPossibleGridLayer.CreateFromRaster pPossibleGrid
'  pDoc.FocusMap.AddLayer pPossibleGridLayer

  ' MAKE START AND END GRIDS
  Dim pStartGrid As IGeoDataset
  Dim pEndGrid As IGeoDataset
  Dim pExtractionOp As IExtractionOp2
  
  ' #10
577:   lngCounter = lngCounter + 1
578:   strMessage = "Clipping Habitat Block #1 to analysis area (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
581:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
584:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
585:   theDetailedDescription = theDetailedDescription & strMessage
586:   frmProgress.txtDetails.Text = theDetailedDescription
587:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
588:   frmProgress.Est_Time_Left lngCounter, "Clipping Habitat Block 1...", "Step "
589:   frmProgress.Refresh
590:   frmProgress.icnProgressLine.Refresh
591:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
594:   Set pStartGrid = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pFinalStartPolygon, True, pAOIEnv, dblCellSize)
  
  ' #11
597:   lngCounter = lngCounter + 1
598:   strMessage = "Clipping Habitat Block #2 to analysis area (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
601:   Debug.Print strMessage
  
  ' ---- PROGRESS METER STUFF
604:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
605:   theDetailedDescription = theDetailedDescription & strMessage
606:   frmProgress.txtDetails.Text = theDetailedDescription
607:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
608:   frmProgress.Est_Time_Left lngCounter, "Clipping Habitat Block 2...", "Step "
609:   frmProgress.Refresh
610:   frmProgress.icnProgressLine.Refresh
611:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
614:   Set pEndGrid = Linkages.GridFunctions.ClipRasterToPolygon(pBaseRaster, pFinalEndPolygon, True, pAOIEnv, dblCellSize)
  
'  Set pStartGrid = pExtractionOp.Polygon(pBaseRaster, pFinalStartPolygon, True)
'  Set pEndGrid = pExtractionOp.Polygon(pBaseRaster, pFinalEndPolygon, True)
  
  ' FOR DEBUGGING
'  Dim pStartGridLayer As IRasterLayer
'  Set pStartGridLayer = New RasterLayer
'  pStartGridLayer.CreateFromRaster pStartGrid
'  pDoc.FocusMap.AddLayer pStartGridLayer

  ' FOR DEBUGGING
'  Dim pEndGridLayer As IRasterLayer
'  Set pEndGridLayer = New RasterLayer
'  pEndGridLayer.CreateFromRaster pEndGrid
'  pDoc.FocusMap.AddLayer pEndGridLayer
  
  Dim pRasterStats As IRasterStatistics
  Dim pRasterBandCol As IRasterBandCollection
633:   Set pRasterBandCol = pDistanceRaster
  Dim pRasterBand As IRasterBand
635:   Set pRasterBand = pRasterBandCol.Item(0)
636:   Set pRasterStats = pRasterBand.Statistics
  Dim dblMaxDist As Double
638:   dblMaxDist = pRasterStats.Maximum
  
'  Debug.Print "dblMaxDist = " & CStr(dblMaxDist)

  ' #12
643:   lngCounter = lngCounter + 1
644:   strMessage = "Generating Burn Grid of central cells (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
647:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
650:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
651:   theDetailedDescription = theDetailedDescription & strMessage
652:   frmProgress.txtDetails.Text = theDetailedDescription
653:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
654:   frmProgress.Est_Time_Left lngCounter, "Generating Burn Grid...", "Step "
655:   frmProgress.Refresh
656:   frmProgress.icnProgressLine.Refresh
657:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  

  Dim pBurnGrid As IGeoDataset
662:   Set pBurnGrid = pCondOp.Con(pLogOp.EqualTo(pZeroGrid, pBaseRaster), _
        pZeroGrid, pRasMakerOp.MakeConstant(1000000000.001, False))
  
  ' #13
666:   lngCounter = lngCounter + 1
667:   strMessage = "Generating Resistance Grid (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
670:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
673:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
674:   theDetailedDescription = theDetailedDescription & strMessage
675:   frmProgress.txtDetails.Text = theDetailedDescription
676:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
677:   frmProgress.Est_Time_Left lngCounter, "Generating Resistance Grid..", "Step "
678:   frmProgress.Refresh
679:   frmProgress.icnProgressLine.Refresh
680:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pResistanceGrid As IGeoDataset
  Dim pMathSupOp As IMathSupportOp
685:   Set pMathSupOp = New RasterMathSupportOp
  Dim pMaxDistGrid As IGeoDataset
687:   Set pMaxDistGrid = pRasMakerOp.MakeConstant(dblMaxDist, False)
688:   Set pResistanceGrid = pMathSupOp.Plus(pMathSupOp.Times( _
        pMathSupOp.Divide(pMathSupOp.Minus(pMaxDistGrid, pDistanceRaster), pMaxDistGrid), _
        pRasMakerOp.MakeConstant(100000, False)), pBurnGrid)
  
  ' FOR DEBUGGING
'  Dim pResistanceLayer As IRasterLayer
'  Set pResistanceLayer = New RasterLayer
'  pResistanceLayer.CreateFromRaster pResistanceGrid
'  pDoc.FocusMap.AddLayer pResistanceLayer
  
  
'  theLengthFactor = theOriginPoly.Distance(theEndPoly)
'
'  ' MAXIMUM GRID VALUE RANGE = 1.175494351e-38 to 3.402823466e+38
'  theMaxDist = theDistGrid.GetStatistics.Get(1)
'  theBurnGrid = (theZeroGrid=1).Con(theZeroGrid, (1000000000.001).AsGrid)
'  theResistanceGrid = ((((theMaxDist.AsGrid) - theDistGrid) / (theMaxDist.AsGrid)) * 1000) + theBurnGrid
'  theDirFN = FileName.GetCWD.MakeTmp("dir", "")
'  'theAllFN = Filename.GetCWD.MakeTmp("all", "")
'  theCostGrid = theStartGrid.CostDistance(theResistanceGrid, theDirFN, nil, nil)
'  theDirGrid = Grid.Make(Grid.MakeSrcName(theDirFN.GetFullName))
'  'theAllGrid = Grid.Make(Grid.MakeSrcName(theAllFN.GetFullName))
'
  ' MAKE FORWARD AND REVERSE COST AND DIRECTION GRIDS
  Dim pForwardBandCol As IRasterBandCollection
  Dim pRevBandCol As IRasterBandCollection
  Dim pFullForward As IGeoDataset
  Dim pFullBack As IGeoDataset
  
  ' #14
718:   lngCounter = lngCounter + 1
719:   strMessage = "Calc Cost Distance from Habitat Block #1 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
722:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
725:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
726:   theDetailedDescription = theDetailedDescription & strMessage
727:   frmProgress.txtDetails.Text = theDetailedDescription
728:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
729:   frmProgress.Est_Time_Left lngCounter, "Generating Cost Distance Grid #1...", "Step "
730:   frmProgress.Refresh
731:   frmProgress.icnProgressLine.Refresh
732:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
735:   Set pFullForward = pDistanceOp.CostDistanceFull(pStartGrid, pResistanceGrid, True, True, False)
  
  ' #15
738:   lngCounter = lngCounter + 1
739:   strMessage = "Calc Cost Distance from Habitat Block #2 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
742:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
745:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
746:   theDetailedDescription = theDetailedDescription & strMessage
747:   frmProgress.txtDetails.Text = theDetailedDescription
748:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
749:   frmProgress.Est_Time_Left lngCounter, "Generating Cost Distance Grid #2...", "Step "
750:   frmProgress.Refresh
751:   frmProgress.icnProgressLine.Refresh
752:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
755:   Set pFullBack = pDistanceOp.CostDistanceFull(pEndGrid, pResistanceGrid, True, True, False)
  
757:   Set pForwardBandCol = pFullForward
758:   Set pRevBandCol = pFullBack
  
  Dim pForCostBand As IRasterBand
  Dim pForDirBand As IRasterBand
  Dim pBackCostBand As IRasterBand
  Dim pBackDirBand As IRasterBand
  
  Dim pForCostGrid As IRasterBandCollection
  Dim pForDirGrid As IRasterBandCollection
  Dim pBackCostGrid As IRasterBandCollection
  Dim pBackDirGrid As IRasterBandCollection
  
770:   Set pForCostBand = pForwardBandCol.Item(0)
771:   Set pForDirBand = pForwardBandCol.Item(1)
772:   Set pBackCostBand = pRevBandCol.Item(0)
773:   Set pBackDirBand = pRevBandCol.Item(1)
  
775:   Set pForCostGrid = New Raster
776:   pForCostGrid.Add pForCostBand, 0
777:   Set pForDirGrid = New Raster
778:   pForDirGrid.Add pForDirBand, 0
779:   Set pBackCostGrid = New Raster
780:   pBackCostGrid.Add pBackCostBand, 0
781:   Set pBackDirGrid = New Raster
782:   pBackDirGrid.Add pBackDirBand, 0
  
  ' FOR DEBUGGING
'  Dim pForCostLayer As IRasterLayer
'  Set pForCostLayer = New RasterLayer
'  pForCostLayer.CreateFromRaster pForCostGrid
'  pForCostLayer.Name = "pForCostGrid"
'  pDoc.FocusMap.AddLayer pForCostLayer
  ' FOR DEBUGGING
'  Dim pForDirLayer As IRasterLayer
'  Set pForDirLayer = New RasterLayer
'  pForDirLayer.CreateFromRaster pForDirGrid
'  pForDirLayer.Name = "pForDirGrid"
'  pDoc.FocusMap.AddLayer pForDirLayer
  ' FOR DEBUGGING
'  Dim pBackCostLayer As IRasterLayer
'  Set pBackCostLayer = New RasterLayer
'  pBackCostLayer.CreateFromRaster pBackCostGrid
'  pBackCostLayer.Name = "pBackCostGrid"
'  pDoc.FocusMap.AddLayer pBackCostLayer
  ' FOR DEBUGGING
'  Dim pBackDirLayer As IRasterLayer
'  Set pBackDirLayer = New RasterLayer
'  pBackDirLayer.CreateFromRaster pBackDirGrid
'  pBackDirLayer.Name = "pBackDirGrid"
'  pDoc.FocusMap.AddLayer pBackDirLayer
      
  ' #16
810:   lngCounter = lngCounter + 1
811:   strMessage = "Examining potential routes from HB #1 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
814:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
817:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
818:   theDetailedDescription = theDetailedDescription & strMessage
819:   frmProgress.txtDetails.Text = theDetailedDescription
820:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
821:   frmProgress.Est_Time_Left lngCounter, "Examining Potential Routes: #1...", "Step "
822:   frmProgress.Refresh
823:   frmProgress.icnProgressLine.Refresh
824:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  
  Dim pMultPt As IPointCollection
829:   Set pMultPt = New Multipoint
  Dim dblIndex As Double
  Dim pStepPoint As IPoint
832:   Set pStepPoint = New Point
833:   For dblIndex = 0 To pStartPolyline.length Step dblCellSize
834:     pStartPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
835:     Set pClone = pStepPoint
836:     pMultPt.AddPoint pClone.Clone
837:   Next dblIndex
838:   Set pClone = pStartPolyline.ToPoint
839:   pMultPt.AddPoint pClone.Clone
  
  Dim pStartPtCollection As IPointCollection
842:   Set pStartPtCollection = New Multipoint
   
  ' MAKE ARRAY OF CELL VALUES AT POINTS
  Dim pValArray As esriSystem.IVariantArray
  Dim vVal As Variant
  
848:   Set pValArray = Linkages.GridFunctions.CellValues(pMultPt, pZeroGrid)
849:   For lngIndex = 0 To pValArray.Count - 1
850:     vVal = pValArray.Element(lngIndex)
851:     If Not IsNull(vVal) Then
852:       If vVal = 1 Then
'        Set pTestPoint = pMultPt.Point(lngIndex)
854:         pStartPtCollection.AddPoint pMultPt.Point(lngIndex)
'        Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pMultPt.Point(lngIndex), "delete_corridors"
856:       End If
857:     End If
858:   Next lngIndex
  
860:   If pStartPtCollection.PointCount = 0 Then Set pStartPtCollection = pMultPt
  Dim pStartPolylines As IGeometryCollection
862:   Set pStartPolylines = pDistanceOp.CostPathAsPolyline(pStartPtCollection, pBackCostGrid, pBackDirGrid)
  
  ' START EXAMINING ACTUAL LINES
  ' LOOKING FOR BEST LINE BASED ON 4 CRITERIA:
  ' 1) MOST IMPORTANT:  SELECT LINE WITH HIGHEST MINIMUM DISTANCE VALUE
  ' 2) IF > 1, SUB-SELECT LINE WITH LOWEST CUMULATIVE COST
  ' 3) IF STILL > 1, SUB-SELECT LINE WITH HIGHEST MEAN DISTANCE VALUE
  ' 4) IF STILL > 1, SUB-SELECT SHORTEST LINE
  ' 5) IF STILL > 1, THEN ALMOST CERTAINLY SAME LINE.  JUST TAKE THE FIRST ONE.
  
  Dim pFinalLineArray As esriSystem.IArray
873:   Set pFinalLineArray = New esriSystem.Array
  Dim pLngKeys As esriSystem.ILongArray
875:   Set pLngKeys = New esriSystem.LongArray
  
  Dim pLineDataArray As esriSystem.IVariantArray
  Dim pTestPolyline As IPolyline
  Dim pTestMultipoint As IPointCollection
  Dim dblLength As Double
  Dim pTestStatsArray As esriSystem.IVariantArray
  Dim lngIndex2 As Long
  Dim vValue As Variant
  Dim dblValue As Double
  Dim dblMin As Double
  Dim dblMean As Double
  Dim vCost As Variant
  Dim dblCost As Double
  Dim booFoundValue As Boolean
  Dim pFinalDataArray As esriSystem.IVariantArray
  Dim pPointDblArray As esriSystem.IDoubleArray
  Dim lngMeanCounter As Long
  
894:   If pStartPolylines.GeometryCount > 0 Then
895:     For lngIndex = 0 To pStartPolylines.GeometryCount - 1
      
897:       strMessage = "  --> From HB #1:  Calculating Stats for Route #" & CStr(lngIndex + 1) _
        & " of " & CStr(pStartPolylines.GeometryCount) & "..." & vbCrLf
899:       Debug.Print strMessage
      
      ' ---- PROGRESS METER STUFF
902:       frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
903:       theDetailedDescription = theDetailedDescription & strMessage
904:       frmProgress.txtDetails.Text = theDetailedDescription
905:       frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
906:       frmProgress.Est_Time_Left lngCounter, "Examining Potential Routes: #1...", "Step "
907:       frmProgress.Refresh
908:       frmProgress.icnProgressLine.Refresh
909:       DoEvents
      ' ---- END PROGRESS METER STUFF ---------------
      
912:       Set pTestPolyline = pStartPolylines.Geometry(lngIndex)
913:       Set pTopoOp = pTestPolyline
914:       pTopoOp.Simplify
      
916:       dblLength = pTestPolyline.length
      
918:       If dblLength > 0 Then
919:         Set pTestMultipoint = New Multipoint
920:         Set pFinalDataArray = New esriSystem.VarArray
921:         lngMeanCounter = 0
        
        ' GENERATE POINTS ALONG ROUTE
924:         For dblIndex = 0 To dblLength Step dblCellSize
925:           pTestPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
926:           Set pClone = pStepPoint
927:           pTestMultipoint.AddPoint pClone.Clone
928:         Next dblIndex
929:         Set pStepPoint = pTestPolyline.ToPoint
930:         Set pClone = pStepPoint
931:         pTestMultipoint.AddPoint pClone.Clone
        
        ' CALCULATE MAXIMUM COST OF ROUTE
934:         vCost = Linkages.GridFunctions.CellValue(pStepPoint, pForCostGrid)
935:         If IsNull(vCost) Then
936:           dblCost = 2 ^ 30
937:         Else
938:           dblCost = CDbl(vCost)
939:         End If
        
        ' GET CELL VALUES OF ROUTE
942:         Set pTestStatsArray = Linkages.GridFunctions.CellValues(pTestMultipoint, pDistanceRaster)
943:         booFoundValue = False
        
        ' CALCULATE STATISTICS OF CELL VALUES
946:         dblMean = 0
947:         For lngIndex2 = 0 To pTestStatsArray.Count - 1
948:           vValue = pTestStatsArray.Element(lngIndex2)
949:           If Not IsNull(vValue) Then
950:             dblValue = CDbl(vValue) * 2    ' (Width = distance to edge x 2)
951:             If Not booFoundValue Then
952:               dblMin = dblValue
953:             Else
954:               If dblValue < dblMin Then dblMin = dblValue
955:             End If
956:             dblMean = dblMean + dblValue
957:             booFoundValue = True
958:             lngMeanCounter = lngMeanCounter + 1
959:           Else
960:             dblValue = -99
961:           End If
          
          ' MAKE FINAL OUTPUT ARRAY, IN CASE THIS IS THE BEST LINE
964:           Set pPointDblArray = New esriSystem.DoubleArray
965:           Set pStepPoint = pTestMultipoint.Point(lngIndex2)
966:           pPointDblArray.Add pStepPoint.X
967:           pPointDblArray.Add pStepPoint.Y
968:           pPointDblArray.Add dblValue
969:           If lngIndex2 = pTestStatsArray.Count - 1 Then
970:             pPointDblArray.Add pTestPolyline.length
971:           Else
972:             pPointDblArray.Add lngIndex2 * dblCellSize
973:           End If
974:           pFinalDataArray.Add pPointDblArray
975:         Next lngIndex2
        
977:         If booFoundValue Then
          ' ADD DATA TO OVERALL ARRAY, TO ANALYZE LATER
979:           Set pLineDataArray = New esriSystem.VarArray
          
981:           dblMean = dblMean / lngMeanCounter
982:           pLineDataArray.Add pTestPolyline
983:           pLineDataArray.Add dblMin
984:           pLineDataArray.Add dblMean
985:           pLineDataArray.Add dblCost
986:           pLineDataArray.Add pFinalDataArray
987:           pLngKeys.Add lngIndex
988:           pFinalLineArray.Add pLineDataArray
989:         End If
990:       End If
991:     Next lngIndex
992:   End If
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pStartPolylines.Geometry(3), "delete_corridors"
  
  ' #17
997:   lngCounter = lngCounter + 1
998:   strMessage = vbCrLf & "Examining potential routes from HB #2 (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
1001:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
1004:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
1005:   theDetailedDescription = theDetailedDescription & strMessage
1006:   frmProgress.txtDetails.Text = theDetailedDescription
1007:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
1008:   frmProgress.Est_Time_Left lngCounter, "Examining Potential Routes: #2...", "Step "
1009:   frmProgress.Refresh
1010:   frmProgress.icnProgressLine.Refresh
1011:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  Dim pMultPt2 As IPointCollection
1015:   Set pMultPt2 = New Multipoint
  Dim pEndPtColl As IPointCollection
1017:   Set pEndPtColl = New Multipoint
1018:   Set pStepPoint = New Point
1019:   For dblIndex = 0 To pEndPolyline.length Step dblCellSize
1020:     pEndPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
1021:     Set pClone = pStepPoint
1022:     pMultPt2.AddPoint pClone.Clone
1023:   Next dblIndex
1024:   Set pClone = pEndPolyline.ToPoint
1025:   pMultPt2.AddPoint pClone.Clone
  
1027:   Set pValArray = Linkages.GridFunctions.CellValues(pMultPt2, pZeroGrid)
1028:   For lngIndex = 0 To pValArray.Count - 1
1029:     vVal = pValArray.Element(lngIndex)
1030:     If Not IsNull(vVal) Then
1031:       If vVal = 1 Then
1032:         pEndPtColl.AddPoint pMultPt2.Point(lngIndex)
'        Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pMultPt2.Point(lngIndex), "delete_corridors"
1034:       End If
1035:     End If
1036:   Next lngIndex
  
1038:   If pEndPtColl.PointCount = 0 Then Set pEndPtColl = pMultPt2
  Dim pEndPolylines As IGeometryCollection
1040:   Set pEndPolylines = pDistanceOp.CostPathAsPolyline(pEndPtColl, pForCostGrid, pForDirGrid)
  
  ' REMEMBER TO FLIP THESE LINES!
  
1044:   If pEndPolylines.GeometryCount > 0 Then
1045:     For lngIndex = 0 To pEndPolylines.GeometryCount - 1
      
1047:       strMessage = "  --> From HB #2:  Calculating Stats for Route #" & CStr(lngIndex + 1) _
        & " of " & CStr(pEndPolylines.GeometryCount) & "..." & vbCrLf
1049:       Debug.Print strMessage
      
      ' ---- PROGRESS METER STUFF
1052:       frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
1053:       theDetailedDescription = theDetailedDescription & strMessage
1054:       frmProgress.txtDetails.Text = theDetailedDescription
1055:       frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
1056:       frmProgress.Est_Time_Left lngCounter, "Examining Potential Routes: #2...", "Step "
1057:       frmProgress.Refresh
1058:       frmProgress.icnProgressLine.Refresh
1059:       DoEvents
      ' ---- END PROGRESS METER STUFF ---------------
  
1062:       Set pTestPolyline = pEndPolylines.Geometry(lngIndex)
1063:       Set pTopoOp = pTestPolyline
1064:       pTopoOp.Simplify
1065:       pTestPolyline.ReverseOrientation
      
1067:       dblLength = pTestPolyline.length
      
1069:       If dblLength > 0 Then
1070:         Set pTestMultipoint = New Multipoint
1071:         Set pFinalDataArray = New esriSystem.VarArray
1072:         lngMeanCounter = 0
        
        ' GENERATE POINTS ALONG ROUTE
1075:         For dblIndex = 0 To dblLength Step dblCellSize
1076:           pTestPolyline.QueryPoint esriNoExtension, dblIndex, False, pStepPoint
1077:           Set pClone = pStepPoint
1078:           pTestMultipoint.AddPoint pClone.Clone
1079:         Next dblIndex
1080:         Set pStepPoint = pTestPolyline.ToPoint
1081:         Set pClone = pStepPoint
1082:         pTestMultipoint.AddPoint pClone.Clone
        
        ' CALCULATE MAXIMUM COST OF ROUTE
1085:         vCost = Linkages.GridFunctions.CellValue(pTestPolyline.FromPoint, pBackCostGrid)
1086:         If IsNull(vCost) Then
1087:           dblCost = 2 ^ 30
1088:         Else
1089:           dblCost = CDbl(vCost)
1090:         End If
        
        ' GET CELL VALUES OF ROUTE
1093:         Set pTestStatsArray = Linkages.GridFunctions.CellValues(pTestMultipoint, pDistanceRaster)
1094:         booFoundValue = False
        
        ' CALCULATE STATISTICS OF CELL VALUES
1097:         dblMean = 0
1098:         For lngIndex2 = 0 To pTestStatsArray.Count - 1
1099:           vValue = pTestStatsArray.Element(lngIndex2)
1100:           If Not IsNull(vValue) Then
1101:             dblValue = CDbl(vValue) * 2    ' (Width = distance to edge x 2)
1102:             If Not booFoundValue Then
1103:               dblMin = dblValue
1104:             Else
1105:               If dblValue < dblMin Then dblMin = dblValue
1106:             End If
1107:             dblMean = dblMean + dblValue
1108:             booFoundValue = True
1109:             lngMeanCounter = lngMeanCounter + 1
1110:           Else
1111:             dblValue = -99
1112:           End If
          
          ' MAKE FINAL OUTPUT ARRAY, IN CASE THIS IS THE BEST LINE
1115:           Set pPointDblArray = New esriSystem.DoubleArray
1116:           Set pStepPoint = pTestMultipoint.Point(lngIndex2)
1117:           pPointDblArray.Add pStepPoint.X
1118:           pPointDblArray.Add pStepPoint.Y
1119:           pPointDblArray.Add dblValue
1120:           If lngIndex2 = pTestStatsArray.Count - 1 Then
1121:             pPointDblArray.Add pTestPolyline.length
1122:           Else
1123:             pPointDblArray.Add lngIndex2 * dblCellSize
1124:           End If
1125:           pFinalDataArray.Add pPointDblArray
1126:         Next lngIndex2
        
1128:         If booFoundValue Then
          ' ADD DATA TO OVERALL ARRAY, TO ANALYZE LATER
1130:           Set pLineDataArray = New esriSystem.VarArray
          
1132:           dblMean = dblMean / lngMeanCounter
1133:           pLineDataArray.Add pTestPolyline
1134:           pLineDataArray.Add dblMin
1135:           pLineDataArray.Add dblMean
1136:           pLineDataArray.Add dblCost
1137:           pLineDataArray.Add pFinalDataArray
1138:           pLngKeys.Add lngIndex
1139:           pFinalLineArray.Add pLineDataArray
1140:         End If
1141:       End If
1142:     Next lngIndex
1143:   End If
  
  ' BAIL OUT IF DIDN'T FIND ANY POSSIBLE LINES
1146:   If pFinalLineArray.Count = 0 Then
1147:     MsgBox "Unable to calculate connector line!  Reasons unknown." & vbCrLf & "Bailing out...", , "Operation Failed:"
    Exit Function
1149:   End If
  
  ' FIND BEST LINE
  Dim pEligibleIndices As esriSystem.ILongArray
1153:   Set pEligibleIndices = New esriSystem.LongArray
  Dim pTempIndices As esriSystem.ILongArray
  
  ' FIND LINES WITH LARGEST MINIMUM DISTANCES
  Dim dblMinDistValue As Double
1158:   For lngIndex = 0 To pFinalLineArray.Count - 1
1159:     Set pLineDataArray = pFinalLineArray.Element(lngIndex)
1160:     Set pTestPolyline = pLineDataArray.Element(0)
1161:     dblMin = pLineDataArray.Element(1)
1162:     If dblMin > dblMinDistValue Then
1163:       dblMinDistValue = dblMin
1164:       pEligibleIndices.RemoveAll
1165:       pEligibleIndices.Add lngIndex
1166:     ElseIf dblMinDistValue = dblMin Then
1167:       pEligibleIndices.Add lngIndex
1168:     End If
1169:   Next lngIndex
  
  ' IF > 1, SUB-SELECT LINES WITH LOWEST TOTAL COST
  Dim lngFinalIndex As Long
1173:   lngFinalIndex = -999
  Dim dblLowestCost As Double
1175:   If pEligibleIndices.Count > 1 Then
1176:     Set pTempIndices = New esriSystem.LongArray
1177:     For lngIndex = 0 To pEligibleIndices.Count - 1
1178:       lngIndex2 = pEligibleIndices.Element(lngIndex)
1179:       Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
1180:       Set pTestPolyline = pLineDataArray.Element(0)
1181:       dblCost = pLineDataArray.Element(3)
1182:       If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
1183:         dblLowestCost = dblCost
1184:         pTempIndices.RemoveAll
1185:         pTempIndices.Add lngIndex2
1186:       Else
1187:         If dblCost < dblLowestCost Then
1188:           dblLowestCost = dblCost
1189:           pTempIndices.RemoveAll
1190:           pTempIndices.Add lngIndex2
1191:         ElseIf dblCost = dblLowestCost Then
1192:           pTempIndices.Add lngIndex2
1193:         End If
1194:       End If
1195:     Next lngIndex
    
1197:     Set pEligibleIndices = pTempIndices
1198:   Else
1199:     lngFinalIndex = pEligibleIndices.Element(0)
1200:   End If
  
  ' IF > 1, SUB-SELECT LINES WITH HIGHEST MEAN DISTANCE
'  Dim dblHighestMeanDist As Double
'  If pEligibleIndices.Count > 1 Then
'    Set pTempIndices = New esriSystem.LongArray
'    For lngIndex = 0 To pEligibleIndices.Count - 1
'      lngIndex2 = pEligibleIndices.Element(lngIndex)
'      Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
'      Set pTestPolyline = pLineDataArray.Element(0)
'      dblMean = pLineDataArray.Element(2)
'      If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
'        dblHighestMeanDist = dblMean
'        pTempIndices.RemoveAll
'        pTempIndices.Add lngIndex2
'      Else
'        If dblMean > dblHighestMeanDist Then
'          dblHighestMeanDist = dblMean
'          pTempIndices.RemoveAll
'          pTempIndices.Add lngIndex2
'        ElseIf dblCost = dblLowestCost Then
'          pTempIndices.Add lngIndex2
'        End If
'      End If
'    Next lngIndex
'
'    Set pEligibleIndices = pTempIndices
'  Else
'    lngFinalIndex = pEligibleIndices.Element(0)
'  End If
  
  ' IF > 1, SUB-SELECT SHORTEST LINE
  Dim dblShortestLength As Double
1233:   If pEligibleIndices.Count > 1 Then
1234:     Set pTempIndices = New esriSystem.LongArray
1235:     For lngIndex = 0 To pEligibleIndices.Count - 1
1236:       lngIndex2 = pEligibleIndices.Element(lngIndex)
1237:       Set pLineDataArray = pFinalLineArray.Element(lngIndex2)
1238:       Set pTestPolyline = pLineDataArray.Element(0)
1239:       dblLength = pTestPolyline.length
1240:       If lngIndex = 0 Then ' IF THIS IS FIRST VALUE
1241:         dblShortestLength = dblLength
1242:         pTempIndices.RemoveAll
1243:         pTempIndices.Add lngIndex2
1244:       Else
1245:         If dblLength < dblShortestLength Then
1246:           dblShortestLength = dblLength
1247:           pTempIndices.RemoveAll
1248:           pTempIndices.Add lngIndex2
1249:         ElseIf dblCost = dblLowestCost Then
1250:           pTempIndices.Add lngIndex2
1251:         End If
1252:       End If
1253:     Next lngIndex
    
1255:     Set pEligibleIndices = pTempIndices
1256:   Else
1257:     lngFinalIndex = pEligibleIndices.Element(0)
1258:   End If
  
  ' IF > 1, JUST TAKE THE FIRST ONE
1261:   If lngFinalIndex < 0 Then lngFinalIndex = pEligibleIndices.Element(0)
  
  ' GET OUTPUT ARRAY
  Dim pFinalStatsArray As esriSystem.IVariantArray
1265:   Set pLineDataArray = pFinalLineArray.Element(lngFinalIndex)
1266:   Set pFinalStatsArray = pLineDataArray.Element(4)
  
'  Linkages.mygeneraloperations.Graphic_MakeFromGeometry Document, pLineDataArray.Element(0), "delete_corridors"
  
  ' #18
1271:   lngCounter = lngCounter + 1
1272:   strMessage = vbCrLf & "Clearing memory and temporary datasets (Step " & CStr(lngCounter) _
      & " of " & CStr(lngCount) & ")..." & vbCrLf & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
1275:   Debug.Print strMessage

  ' ---- PROGRESS METER STUFF
1278:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
1279:   theDetailedDescription = theDetailedDescription & strMessage
1280:   frmProgress.txtDetails.Text = theDetailedDescription
1281:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
1282:   frmProgress.Est_Time_Left lngCounter, "Clearing temporary datasets...", "Step "
1283:   frmProgress.Refresh
1284:   frmProgress.icnProgressLine.Refresh
1285:   DoEvents
  ' ---- END PROGRESS METER STUFF ---------------
  
  
1289:   Debug.Print "Finished"
  
1291:   Set RunBottleneck = pFinalStatsArray
  


1295:   Set pSpRef = Nothing
1296:   Set pTopoOp = Nothing
1297:   Set pRelOp = Nothing
1298:   If UBound(pPolyOutStart) > 0 Then
1299:     For lngIndex = 0 To UBound(pPolyOutStart)
1300:       Set pPolyOutStart(lngIndex) = Nothing
1301:     Next lngIndex
1302:   End If
1303:   Set pClone = Nothing
1304:   Set pTempStartA = Nothing
1305:   If UBound(pRing) > 0 Then
1306:     For lngIndex = 0 To UBound(pRing)
1307:       Set pRing(lngIndex) = Nothing
1308:     Next lngIndex
1309:   End If
1310:   Set pOuterRing = Nothing
1311:   Set pPath = Nothing
1312:   Set pTopoForSimplify = Nothing
1313:   Set pFinalStartPolygon = Nothing
1314:   Set pOutputPolygon = Nothing
1315:   Set pFinalStartLine = Nothing
1316:   Set pOutputPolyline = Nothing
1317:   Set pClipStart = Nothing
1318:   If UBound(pPolyOutEnd) > 0 Then
1319:     For lngIndex = 0 To UBound(pPolyOutEnd)
1320:       Set pPolyOutEnd(lngIndex) = Nothing
1321:     Next lngIndex
1322:   End If
1323:   Set pTempEndA = Nothing
1324:   Set pFinalEndPolygon = Nothing
1325:   Set pOutputEndPolygon = Nothing
1326:   Set pFinalEndPolyline = Nothing
1327:   Set pOutputEndPolyline = Nothing
1328:   Set pClipEnd = Nothing
1329:   Set pStartPolyline = Nothing
1330:   Set pEndPolyline = Nothing
1331:   Set pUnionPoly = Nothing
1332:   Set pUnionPoly2 = Nothing
1333:   Set pAOIPoly = Nothing
1334:   Set pIntermediatePoly = Nothing
1335:   Set pAOIEnv = Nothing
1336:   Set pPoint1 = Nothing
1337:   Set pPoint2 = Nothing
1338:   Set pPoint3 = Nothing
1339:   Set pPoint4 = Nothing
1340:   Set pNonHabitatPolygon = Nothing
1341:   Set pHabitatPolygon = Nothing
1342:   Set pCorRaster = Nothing
1343:   Set pEnv = Nothing
1344:   Set pRasMakerOp = Nothing
1345:   Set pBaseRaster = Nothing
1346:   Set pLogOp = Nothing
1347:   Set pDistanceOp = Nothing
1348:   Set pMathOp = Nothing
1349:   Set pCondOp = Nothing
1350:   Set pOutsideRasterA = Nothing
1351:   Set pOutsideRasterB = Nothing
1352:   Set pOutputRaster = Nothing
1353:   Set pDistanceRaster = Nothing
1354:   Set pDirGrid = Nothing
1355:   Set pFlowAccGrid = Nothing
1356:   Set pHydOp = Nothing
1357:   Set pZeroGrid = Nothing
1358:   Set pPossibleGrid = Nothing
1359:   Set pStartGrid = Nothing
1360:   Set pEndGrid = Nothing
1361:   Set pExtractionOp = Nothing
1362:   Set pRasterStats = Nothing
1363:   Set pRasterBandCol = Nothing
1364:   Set pRasterBand = Nothing
1365:   Set pBurnGrid = Nothing
1366:   Set pResistanceGrid = Nothing
1367:   Set pMathSupOp = Nothing
1368:   Set pMaxDistGrid = Nothing
1369:   Set pForwardBandCol = Nothing
1370:   Set pRevBandCol = Nothing
1371:   Set pFullForward = Nothing
1372:   Set pFullBack = Nothing
1373:   Set pForCostBand = Nothing
1374:   Set pForDirBand = Nothing
1375:   Set pBackCostBand = Nothing
1376:   Set pBackDirBand = Nothing
1377:   Set pForCostGrid = Nothing
1378:   Set pForDirGrid = Nothing
1379:   Set pBackCostGrid = Nothing
1380:   Set pBackDirGrid = Nothing
1381:   Set pMultPt = Nothing
1382:   Set pStepPoint = Nothing
1383:   Set pStartPtCollection = Nothing
1384:   Set pValArray = Nothing
1385:   Set pStartPolylines = Nothing
1386:   Set pFinalLineArray = Nothing
1387:   Set pLngKeys = Nothing
1388:   Set pLineDataArray = Nothing
1389:   Set pTestPolyline = Nothing
1390:   Set pTestMultipoint = Nothing
1391:   Set pTestStatsArray = Nothing
1392:   Set pFinalDataArray = Nothing
1393:   Set pPointDblArray = Nothing
1394:   Set pMultPt2 = Nothing
1395:   Set pEndPtColl = Nothing
1396:   Set pEndPolylines = Nothing
1397:   Set pEligibleIndices = Nothing
1398:   Set pTempIndices = Nothing
1399:   Set pFinalStatsArray = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "RunBottleneck " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



1407:   Set pSpRef = Nothing
1408:   Set pTopoOp = Nothing
1409:   Set pRelOp = Nothing
1410:   If UBound(pPolyOutStart) > 0 Then
1411:     For lngIndex = 0 To UBound(pPolyOutStart)
1412:       Set pPolyOutStart(lngIndex) = Nothing
1413:     Next lngIndex
1414:   End If
1415:   Set pClone = Nothing
1416:   Set pTempStartA = Nothing
1417:   If UBound(pRing) > 0 Then
1418:     For lngIndex = 0 To UBound(pRing)
1419:       Set pRing(lngIndex) = Nothing
1420:     Next lngIndex
1421:   End If
1422:   Set pOuterRing = Nothing
1423:   Set pPath = Nothing
1424:   Set pTopoForSimplify = Nothing
1425:   Set pFinalStartPolygon = Nothing
1426:   Set pOutputPolygon = Nothing
1427:   Set pFinalStartLine = Nothing
1428:   Set pOutputPolyline = Nothing
1429:   Set pClipStart = Nothing
1430:   If UBound(pPolyOutEnd) > 0 Then
1431:     For lngIndex = 0 To UBound(pPolyOutEnd)
1432:       Set pPolyOutEnd(lngIndex) = Nothing
1433:     Next lngIndex
1434:   End If
1435:   Set pTempEndA = Nothing
1436:   Set pFinalEndPolygon = Nothing
1437:   Set pOutputEndPolygon = Nothing
1438:   Set pFinalEndPolyline = Nothing
1439:   Set pOutputEndPolyline = Nothing
1440:   Set pClipEnd = Nothing
1441:   Set pStartPolyline = Nothing
1442:   Set pEndPolyline = Nothing
1443:   Set pUnionPoly = Nothing
1444:   Set pUnionPoly2 = Nothing
1445:   Set pAOIPoly = Nothing
1446:   Set pIntermediatePoly = Nothing
1447:   Set pAOIEnv = Nothing
1448:   Set pPoint1 = Nothing
1449:   Set pPoint2 = Nothing
1450:   Set pPoint3 = Nothing
1451:   Set pPoint4 = Nothing
1452:   Set pNonHabitatPolygon = Nothing
1453:   Set pHabitatPolygon = Nothing
1454:   Set pCorRaster = Nothing
1455:   Set pEnv = Nothing
1456:   Set pRasMakerOp = Nothing
1457:   Set pBaseRaster = Nothing
1458:   Set pLogOp = Nothing
1459:   Set pDistanceOp = Nothing
1460:   Set pMathOp = Nothing
1461:   Set pCondOp = Nothing
1462:   Set pOutsideRasterA = Nothing
1463:   Set pOutsideRasterB = Nothing
1464:   Set pOutputRaster = Nothing
1465:   Set pDistanceRaster = Nothing
1466:   Set pDirGrid = Nothing
1467:   Set pFlowAccGrid = Nothing
1468:   Set pHydOp = Nothing
1469:   Set pZeroGrid = Nothing
1470:   Set pPossibleGrid = Nothing
1471:   Set pStartGrid = Nothing
1472:   Set pEndGrid = Nothing
1473:   Set pExtractionOp = Nothing
1474:   Set pRasterStats = Nothing
1475:   Set pRasterBandCol = Nothing
1476:   Set pRasterBand = Nothing
1477:   Set pBurnGrid = Nothing
1478:   Set pResistanceGrid = Nothing
1479:   Set pMathSupOp = Nothing
1480:   Set pMaxDistGrid = Nothing
1481:   Set pForwardBandCol = Nothing
1482:   Set pRevBandCol = Nothing
1483:   Set pFullForward = Nothing
1484:   Set pFullBack = Nothing
1485:   Set pForCostBand = Nothing
1486:   Set pForDirBand = Nothing
1487:   Set pBackCostBand = Nothing
1488:   Set pBackDirBand = Nothing
1489:   Set pForCostGrid = Nothing
1490:   Set pForDirGrid = Nothing
1491:   Set pBackCostGrid = Nothing
1492:   Set pBackDirGrid = Nothing
1493:   Set pMultPt = Nothing
1494:   Set pStepPoint = Nothing
1495:   Set pStartPtCollection = Nothing
1496:   Set pValArray = Nothing
1497:   Set pStartPolylines = Nothing
1498:   Set pFinalLineArray = Nothing
1499:   Set pLngKeys = Nothing
1500:   Set pLineDataArray = Nothing
1501:   Set pTestPolyline = Nothing
1502:   Set pTestMultipoint = Nothing
1503:   Set pTestStatsArray = Nothing
1504:   Set pFinalDataArray = Nothing
1505:   Set pPointDblArray = Nothing
1506:   Set pMultPt2 = Nothing
1507:   Set pEndPtColl = Nothing
1508:   Set pEndPolylines = Nothing
1509:   Set pEligibleIndices = Nothing
1510:   Set pTempIndices = Nothing
1511:   Set pFinalStatsArray = Nothing

End Function


Public Function MakeHistogramData(pFieldInfoArray As esriSystem.IStringArray, lngNumBins As Long, _
    dblMinimum As Double, dblMaximum As Double, lngHistArray() As Long, pApp As IApplication, strFolder As String, _
    Optional lngNumDecimalPlaces As Long = -999) As String
  On Error GoTo ErrorHandler
  
  Dim strDecFormatString As String
1522:   strDecFormatString = "0"
1523:   If lngNumDecimalPlaces > 0 Then strDecFormatString = strDecFormatString + "." + String(lngNumDecimalPlaces, "0")
  
  Dim strFieldName As String
  Dim strFileName As String
1527:   strFieldName = pFieldInfoArray.Element(0)
1528:   If Right(strFolder, 1) <> "/" And Right(strFolder, 1) <> "\" Then strFolder = strFolder & "\"
1529:   strFileName = strFolder & strFieldName & "_histogram.dbf"
1530:   strFileName = Linkages.aml_func_mod.MakeUniqueFilename(strFileName)
  
  Dim strReport As String
  Dim dblRunningBinVal As Double
  Dim dblBinInterval As Double
  Dim lngMaxBinCount As Long
  Dim dblBinRatio As Double
  Dim lngBinCount As Long
  
1539:   dblRunningBinVal = dblMinimum
1540:   dblBinInterval = (dblMaximum - dblMinimum) / lngNumBins
  ' FIRST GET MAXIMUM BIN COUNT, THEN SCALE IT TO 40 CHARACTERS
1542:   lngMaxBinCount = 0
1543:   dblBinRatio = 1
  Dim anIndex As Long
1545:   For anIndex = 1 To lngNumBins
1546:     lngBinCount = lngHistArray(anIndex - 1)
1547:     If lngBinCount > lngMaxBinCount Then lngMaxBinCount = lngBinCount
1548:   Next anIndex
1549:   If lngMaxBinCount > 110 Then
1550:     dblBinRatio = 110 / lngMaxBinCount
1551:   End If
  Dim strSub1 As String
  Dim strSub2 As String
    
  Dim pNewFields As IFields
  Dim pNewFieldsEdit As IFieldsEdit
  Dim pNewField As IField
  Dim pNewFieldEdit As IFieldEdit
  
1560:   Set pNewFields = New Fields
1561:   Set pNewFieldsEdit = pNewFields
  ' 1) Unique_ID
  ' 2) Bin_Start
  ' 3) Bin_End
  ' 4) Bin_Count
1566:   pNewFieldsEdit.FieldCount = 4
  
1568:   Set pNewField = New Field
1569:   Set pNewFieldEdit = pNewField
1570:   With pNewFieldEdit
1571:     .Name = "Bin_ID"
1572:     .Precision = 8
1573:     .Type = esriFieldTypeInteger
1574:   End With
1575:   Set pNewFieldsEdit.Field(0) = pNewField
  
1577:   Set pNewField = New Field
1578:   Set pNewFieldEdit = pNewField
1579:   With pNewFieldEdit
1580:     .Name = "Bin_Start"
1581:     .Precision = 16
1582:     .Scale = 8
1583:     .Type = esriFieldTypeDouble
1584:   End With
1585:   Set pNewFieldsEdit.Field(1) = pNewField
  
1587:   Set pNewField = New Field
1588:   Set pNewFieldEdit = pNewField
1589:   With pNewFieldEdit
1590:     .Name = "Bin_End"
1591:     .Precision = 16
1592:     .Scale = 8
1593:     .Type = esriFieldTypeDouble
1594:   End With
1595:   Set pNewFieldsEdit.Field(2) = pNewField
  
1597:   Set pNewField = New Field
1598:   Set pNewFieldEdit = pNewField
1599:   With pNewFieldEdit
1600:     .Name = "Bin_Count"
1601:     .Precision = 16
1602:     .Type = esriFieldTypeInteger
1603:   End With
1604:   Set pNewFieldsEdit.Field(3) = pNewField
  
  Dim pTable As ITable
1607:   Set pTable = Linkages.aml_func_mod.CreatedBASETable(strFileName, pNewFields)
  
  Dim pRow As IRow
  
  ' MAKE REPORT AND FILL TABLE
1612:   strReport = "\b        --- " & strFieldName & " Histogram ---------------------------\b0\par" & vbCrLf
1613:   For anIndex = 1 To lngNumBins
    
1615:     lngBinCount = lngHistArray(anIndex - 1)
1616:     If lngNumDecimalPlaces < 0 Then
1617:       strSub1 = strSub1 & "  Bin " & CStr(anIndex) & "]  Range = " & CStr(dblRunningBinVal) _
          & " to " & CStr(dblRunningBinVal + dblBinInterval) & "     Bin Size = " & _
          Linkages.aml_func_mod.InsertCommas(lngBinCount) & " cases...\par" & vbCrLf
1620:     Else
1621:       strSub1 = strSub1 & "  Bin " & CStr(anIndex) & "]  Range = " & CStr(Format(dblRunningBinVal, strDecFormatString)) _
          & " to " & CStr(Format((dblRunningBinVal + dblBinInterval), strDecFormatString)) & "     Bin Size = " & _
          Linkages.aml_func_mod.InsertCommas(lngBinCount) & " cases...\par" & vbCrLf
1624:     End If
1625:     strSub2 = strSub2 & "  " & CStr(anIndex) & "]\tab " & String(Round(lngBinCount * dblBinRatio), _
        "|") & "\par" & vbCrLf
1627:     If lngNumDecimalPlaces < 0 Then
1628:       strReport = strReport & "   Bin " & CStr(anIndex) & "]  Range = " & CStr(dblRunningBinVal) _
          & " to " & CStr(dblRunningBinVal + dblBinInterval) & "     Bin Size = " & _
          Linkages.aml_func_mod.InsertCommas(lngBinCount) & " cases...\par" & vbCrLf & "        " & _
          String(Round(lngBinCount * dblBinRatio), "|") & "\par" & vbCrLf
1632:     Else
1633:       strReport = strReport & "   Bin " & CStr(anIndex) & "]  Range = " & CStr(Format(dblRunningBinVal, strDecFormatString)) _
          & " to " & CStr(Format((dblRunningBinVal + dblBinInterval), strDecFormatString)) & "     Bin Size = " & _
          Linkages.aml_func_mod.InsertCommas(lngBinCount) & " cases...\par" & vbCrLf & "        " & _
          String(Round(lngBinCount * dblBinRatio), "|") & "\par" & vbCrLf
1637:     End If
1638:     Set pRow = pTable.CreateRow
1639:     pRow.Value(pTable.FindField("Bin_ID")) = anIndex
1640:     pRow.Value(pTable.FindField("Bin_Start")) = dblRunningBinVal
1641:     pRow.Value(pTable.FindField("Bin_End")) = dblRunningBinVal + dblBinInterval
1642:     pRow.Value(pTable.FindField("Bin_Count")) = lngBinCount
1643:     pRow.Store
1644:     dblRunningBinVal = dblRunningBinVal + dblBinInterval
1645:   Next anIndex

  ' MAKE TABLE WINDOW AND ADD IT TO DOCUMENT
  Dim pNewStandaloneTable As IStandaloneTable
1649:   Set pNewStandaloneTable = New StandaloneTable
1650:   Set pNewStandaloneTable.Table = pTable

  Dim pTableWindow2 As ITableWindow2
1653:   Set pTableWindow2 = New TableWindow
  
  Dim lngLeft As Long
  Dim lngTop As Long
  Dim lngRight As Long
  Dim lngBottom As Long
  
1660:   With pTableWindow2
1661:     Set .StandaloneTable = pNewStandaloneTable
1662:     Set .Application = pApp
1663:     .TableSelectionAction = esriSelectFeatures
1664:     .ShowAliasNamesInColumnHeadings = True
1665:     .ShowSelected = False
1666:     .Show True
1667:   End With
  
  Dim pMxDoc As IMxDocument
1670:   Set pMxDoc = pApp.Document
  Dim pStandaloneTableCollection As IStandaloneTableCollection
1672:   Set pStandaloneTableCollection = pMxDoc.FocusMap
1673:   pStandaloneTableCollection.AddStandaloneTable pNewStandaloneTable
  
1675:   pMxDoc.UpdateContents
  
'  MakeHistogramData = "        [Histogram data table saved to " & strfilename & "]" & vbCrLf & _
'      strReport & "        --- " & strFieldName & " Histogram Reorganized ----------" & vbCrLf & strSub1 & strSub2
1679:   MakeHistogramData = "        --- [Histogram data table saved to " & _
      Linkages.aml_func_mod.SubstituteString(strFileName, "\", "\\") & "]\par" & vbCrLf & _
      "\b        --- " & strFieldName & " Histogram ----------\b0\par" & vbCrLf & strSub1 & strSub2
  

1684:   Set pNewFields = Nothing
1685:   Set pNewFieldsEdit = Nothing
1686:   Set pNewField = Nothing
1687:   Set pNewFieldEdit = Nothing
1688:   Set pTable = Nothing
1689:   Set pRow = Nothing
1690:   Set pNewStandaloneTable = Nothing
1691:   Set pTableWindow2 = Nothing
1692:   Set pMxDoc = Nothing
1693:   Set pStandaloneTableCollection = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "MakeHistogramData " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


1700:   Set pNewFields = Nothing
1701:   Set pNewFieldsEdit = Nothing
1702:   Set pNewField = Nothing
1703:   Set pNewFieldEdit = Nothing
1704:   Set pTable = Nothing
1705:   Set pRow = Nothing
1706:   Set pNewStandaloneTable = Nothing
1707:   Set pTableWindow2 = Nothing
1708:   Set pMxDoc = Nothing
1709:   Set pStandaloneTableCollection = Nothing

End Function

Public Function GridHistogram(pRasterLayer As IRasterLayer, dblLowVal As Double, dblHighVal As Double, _
    lngNumBins As Long, pApp As IApplication) As Long()
  On Error GoTo ErrorHandler

  
  Dim pRaster As IRaster
1719:   Set pRaster = pRasterLayer.Raster
  
  Dim lngHistCount() As Long
  ReDim lngHistCount(lngNumBins - 1)
  
  Dim dblInterval As Double
1725:   dblInterval = (dblHighVal - dblLowVal) / lngNumBins
  
  'Get rasters
  Dim pRasOut As IRaster
  
  Dim pMapAlgebraOp As IMapAlgebraOp
1731:   Set pMapAlgebraOp = New RasterMapAlgebraOp
1732:   pMapAlgebraOp.BindRaster pRaster, "pRaster"
  
  Dim lngIndex As Long
  Dim dblLow As Double
  Dim dblHigh As Double
  
1738:   dblLow = dblLowVal
1739:   dblHigh = dblLowVal + dblInterval
  
  Dim pRasterBandCollection As IRasterBandCollection
  Dim pRasterBand As IRasterBand
  Dim pTable As ITable
  Dim lngTotalCount As Long
  Dim lngValField As Long
  Dim lngCountField As Long
  Dim pCursor As ICursor
  Dim pRow As IRow
  Dim lngVal As Long
  Dim lngCount As Long
  
1752:   lngTotalCount = 0
  
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
1756:   Set psbar = pApp.StatusBar
  Dim pPro As IStepProgressor
1758:   Set pPro = psbar.ProgressBar
1759:   pPro.position = 1
1760:   Screen.MousePointer = vbHourglass
1761:   psbar.ShowProgressBar "Generating Histogram for [" & pRasterLayer.Name & "]", 1, _
          lngNumBins, 1, True
          
1764:   For lngIndex = 0 To lngNumBins - 1
1765:     If lngIndex = lngNumBins - 1 Then
1766:       Set pRasOut = pMapAlgebraOp.Execute("([pRaster] >= " & CStr(dblLow) & ") AND ([pRaster] <= " & CStr(dblHigh) & ")")
1767:     Else
1768:       Set pRasOut = pMapAlgebraOp.Execute("([pRaster] >= " & CStr(dblLow) & ") AND ([pRaster] < " & CStr(dblHigh) & ")")
1769:     End If
    
1771:     Set pRasterBandCollection = pRasOut
1772:     Set pRasterBand = pRasterBandCollection.Item(0)
1773:     Set pTable = pRasterBand.AttributeTable
    
1775:     lngValField = pTable.FindField("Value")
1776:     lngCountField = pTable.FindField("Count")
    
1778:     Set pCursor = pTable.Search(Nothing, True)
1779:     Set pRow = pCursor.NextRow
1780:     lngCount = 0
    
1782:     Do Until pRow Is Nothing
1783:       lngVal = pRow.Value(lngValField)
1784:       If lngVal = 1 Then                ' MEANING THE EXPRESSION ABOVE EVALUATES TO "TRUE"
1785:         lngCount = pRow.Value(lngCountField)
1786:       End If
1787:       Set pRow = pCursor.NextRow
1788:     Loop
    
1790:     lngHistCount(lngIndex) = lngCount
1791:     lngTotalCount = lngTotalCount + lngCount
    
1793:     dblLow = dblLow + dblInterval
1794:     dblHigh = dblLow + dblInterval
1795:     psbar.StepProgressBar
1796:   Next lngIndex
  
1798:   Screen.MousePointer = vbDefault
1799:   psbar.HideProgressBar
1800:   GridHistogram = lngHistCount
  

1803:   Set pRaster = Nothing
1804:   Set pRasOut = Nothing
1805:   Set pMapAlgebraOp = Nothing
1806:   Set pRasterBandCollection = Nothing
1807:   Set pRasterBand = Nothing
1808:   Set pTable = Nothing
1809:   Set pCursor = Nothing
1810:   Set pRow = Nothing
1811:   Set psbar = Nothing
1812:   Set pPro = Nothing


  Exit Function
ErrorHandler:
  HandleError True, "GridHistogram " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



1821:   Set pRaster = Nothing
1822:   Set pRasOut = Nothing
1823:   Set pMapAlgebraOp = Nothing
1824:   Set pRasterBandCollection = Nothing
1825:   Set pRasterBand = Nothing
1826:   Set pTable = Nothing
1827:   Set pCursor = Nothing
1828:   Set pRow = Nothing
1829:   Set psbar = Nothing
1830:   Set pPro = Nothing


End Function

Public Function StatsForDates(ByVal pDateArray As esriSystem.IVariantArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler


  Dim pReturn As esriSystem.IVariantArray
1840:   Set pReturn = New esriSystem.VarArray
  
  Dim dateArray() As Date
  ReDim dateArray(pDateArray.Count - 1)
  Dim anIndex As Long
1845:   For anIndex = 0 To pDateArray.Count - 1
1846:     dateArray(anIndex) = pDateArray.Element(anIndex)
1847:   Next anIndex
  
1849:   Call Linkages.QuickSort.DatesAscending(dateArray, 0, UBound(dateArray))
  
  Dim dateStart As Date
  Dim dateEnd As Date
1853:   dateStart = dateArray(0)
1854:   dateEnd = dateArray(UBound(dateArray))
  
1856:   pReturn.Add dateStart
1857:   pReturn.Add dateEnd
  
1859:   Set StatsForDates = pReturn

1861:   Set pReturn = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "StatsForDates " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

1867:   Set pReturn = Nothing
  
End Function


Public Function StatsForStrings(ByVal pStringArray As esriSystem.IStringArray, _
      Optional ByVal pValArray As esriSystem.IDoubleArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

      
  ' RETURN VARIANT ARRAY WILL BE FILLED WITH SUB VARIANT ARRAYS.
  ' EACH SUB-VARIANT ARRAY WILL REFLECT ONE UNIQUE STRING VALUE FROM INPUT pStringArray
  ' EACH SUB-VARIANT ARRAY WILL CONTAIN [STRING VALUE, COUNT, PROPORTION, SIZE]
  ' IMPORTANT:  IF OPTIONAL WEIGHTING ARRAY IS INCLUDED, THEN PROPORTION WILL BE BASED ON SIZE-WEIGHTED COUNTS.  OTHERWISE
  '             PROPORTION IS BASED ONLY ON COUNTS.
  ' IMPORTANT:  IF DRAWN FROM POLYLINE OR POLYGON FEATURE CLASS, THEN CHOOSING PROPORTION FORCES SIZE-WEIGHTED ANALYSIS.
  
  Dim lngIndex As Long
  Dim strVal As String
  Dim dblSize As Double
  Dim strUniqueStrings() As String
  ReDim strUniqueStrings(pStringArray.Count)
  Dim lngUniqueCounter As Long
1890:   lngUniqueCounter = -1
  Dim booHasKey As Boolean
  
  Dim booUseSize As Boolean
1894:   booUseSize = (Not pValArray Is Nothing)
  
  Dim lngCount As Long
  Dim dblProportion As Double
  Dim dblTotalSize As Double
1899:   dblTotalSize = 0
  Dim lngTotalCount As Long
1901:   lngTotalCount = 0
  Dim dblRunningSize As Double
1903:   dblRunningSize = 0
  Dim lngRunningCount As Long
1905:   lngRunningCount = 0
  Dim dblTotalUniqueSize As Double
1907:   dblTotalUniqueSize = 0
  Dim lngTotalUniqueCount As Long
1909:   lngTotalUniqueCount = 0
  
  Dim pReturnArray As esriSystem.IVariantArray
1912:   Set pReturnArray = New esriSystem.VarArray
  Dim pSubArray As esriSystem.IVariantArray
  
  Dim pCollection As New Collection
  Dim pCountCollection As New Collection
  
  ' IF pStringArray IS EMPTY, THEN RETURN EMPTY pReturnArray
  
1920:   If pStringArray.Count = 1 Then
1921:     Set pSubArray = New esriSystem.VarArray
1922:     pSubArray.Add pStringArray.Element(0)
1923:     lngCount = 1
1924:     pSubArray.Add lngCount
1925:     dblProportion = 1
1926:     pSubArray.Add dblProportion
1927:     If booUseSize Then
1928:       dblTotalSize = pValArray.Element(0)
1929:     Else
1930:       dblTotalSize = 1
1931:     End If
1932:     pSubArray.Add dblTotalSize
1933:     pReturnArray.Add pSubArray
1934:   ElseIf pStringArray.Count > 1 Then
1935:     For lngIndex = 0 To pStringArray.Count - 1
1936:       strVal = pStringArray.Element(lngIndex)
1937:       If booUseSize Then
1938:         dblSize = pValArray.Element(lngIndex)
1939:       Else
1940:         dblSize = 1
1941:       End If
      
1943:       booHasKey = CheckCollectionForKey(pCollection, strVal)
1944:       If booHasKey Then
        ' UPDATE SIZE COLLECTION
1946:         dblRunningSize = pCollection.Item(strVal)
1947:         dblRunningSize = dblRunningSize + dblSize
1948:         pCollection.Remove strVal
1949:         pCollection.Add dblRunningSize, strVal
        ' UPDATE COUNT COLLECTION
1951:         lngRunningCount = pCountCollection.Item(strVal)
1952:         lngRunningCount = lngRunningCount + 1
1953:         pCountCollection.Remove strVal
1954:         pCountCollection.Add lngRunningCount, strVal
1955:       Else
1956:         pCollection.Add dblSize, strVal
1957:         pCountCollection.Add 1, strVal
1958:         lngUniqueCounter = lngUniqueCounter + 1
1959:         strUniqueStrings(lngUniqueCounter) = strVal
1960:       End If
1961:     Next lngIndex
    ReDim Preserve strUniqueStrings(lngUniqueCounter)
    
1964:     dblTotalSize = 0
1965:     For lngIndex = 0 To lngUniqueCounter
1966:       strVal = strUniqueStrings(lngIndex)
1967:       dblTotalSize = dblTotalSize + pCollection(strVal)
1968:     Next lngIndex
    
1970:     Call Linkages.QuickSort.StringsAscending(strUniqueStrings, 0, lngUniqueCounter)
1971:     For lngIndex = 0 To lngUniqueCounter
1972:       Set pSubArray = New esriSystem.VarArray
      
1974:       strVal = strUniqueStrings(lngIndex)
1975:       dblTotalUniqueSize = pCollection.Item(strVal)
1976:       lngTotalUniqueCount = pCountCollection.Item(strVal)
1977:       dblProportion = dblTotalUniqueSize / dblTotalSize
      
1979:       pSubArray.Add strVal
1980:       pSubArray.Add lngTotalUniqueCount
1981:       pSubArray.Add dblProportion
1982:       pSubArray.Add dblTotalUniqueSize
1983:       pReturnArray.Add pSubArray
1984:     Next lngIndex
1985:   End If

1987:   Set StatsForStrings = pReturnArray


1990:   Set pReturnArray = Nothing
1991:   Set pSubArray = Nothing
1992:   Set pCollection = Nothing
1993:   Set pCountCollection = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "StatsForStrings " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



2001:   Set pReturnArray = Nothing
2002:   Set pSubArray = Nothing
2003:   Set pCollection = Nothing
2004:   Set pCountCollection = Nothing

End Function

Public Function StatsPropsForNumbers(ByVal pNumberArray As esriSystem.IDoubleArray, _
      Optional ByVal pValArray As esriSystem.IDoubleArray) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

      
  ' RETURN VARIANT ARRAY WILL BE FILLED WITH SUB VARIANT ARRAYS.
  ' EACH SUB-VARIANT ARRAY WILL REFLECT ONE UNIQUE VALUE FROM INPUT pNumberArray
  ' EACH SUB-VARIANT ARRAY WILL CONTAIN [VALUE, COUNT, PROPORTION, SIZE]
  ' IMPORTANT:  IF OPTIONAL WEIGHTING ARRAY IS INCLUDED, THEN PROPORTION WILL BE BASED ON SIZE-WEIGHTED COUNTS.  OTHERWISE
  '             PROPORTION IS BASED ONLY ON COUNTS.
  ' IMPORTANT:  IF DRAWN FROM POLYLINE OR POLYGON FEATURE CLASS, THEN CHOOSING PROPORTION FORCES SIZE-WEIGHTED ANALYSIS.
  
  Dim lngIndex As Long
  Dim dblVal As Double
  Dim dblSize As Double
  Dim dblUniqueNumbers() As Double
  ReDim dblUniqueNumbers(pNumberArray.Count)
  Dim lngUniqueCounter As Long
2026:   lngUniqueCounter = -1
  Dim booHasKey As Boolean
  
  Dim booUseSize As Boolean
2030:   booUseSize = (Not pValArray Is Nothing)
  
  Dim lngCount As Long
  Dim dblProportion As Double
  Dim dblTotalSize As Double
2035:   dblTotalSize = 0
  Dim lngTotalCount As Long
2037:   lngTotalCount = 0
  Dim dblRunningSize As Double
2039:   dblRunningSize = 0
  Dim lngRunningCount As Long
2041:   lngRunningCount = 0
  Dim dblTotalUniqueSize As Double
2043:   dblTotalUniqueSize = 0
  Dim lngTotalUniqueCount As Long
2045:   lngTotalUniqueCount = 0
  
  Dim strVal As String
  
  Dim pReturnArray As esriSystem.IVariantArray
2050:   Set pReturnArray = New esriSystem.VarArray
  Dim pSubArray As esriSystem.IVariantArray
  
  Dim pCollection As New Collection
  Dim pCountCollection As New Collection
  
 ' MsgBox "CorridorAnalysisFunctions.StatsPropsForNumbers" & vbCrLf & "booUseSize = " & CStr(booUseSize)
  
  ' IF pDoubleArray IS EMPTY, THEN RETURN EMPTY pReturnArray
  
2060:   If pNumberArray.Count = 1 Then
2061:     Set pSubArray = New esriSystem.VarArray
2062:     pSubArray.Add pNumberArray.Element(0)
2063:     lngCount = 1
2064:     pSubArray.Add lngCount
2065:     dblProportion = 1
2066:     pSubArray.Add dblProportion
2067:     If booUseSize Then
2068:       dblTotalSize = pValArray.Element(0)
2069:     Else
2070:       dblTotalSize = 1
2071:     End If
2072:     pSubArray.Add dblTotalSize
2073:     pReturnArray.Add pSubArray
2074:   ElseIf pNumberArray.Count > 1 Then
2075:     For lngIndex = 0 To pNumberArray.Count - 1
2076:       dblVal = pNumberArray.Element(lngIndex)
2077:       If booUseSize Then
2078:         dblSize = pValArray.Element(lngIndex)
2079:       Else
2080:         dblSize = 1
2081:       End If
      
2083:       strVal = CStr(dblVal)
      
2085:       booHasKey = CheckCollectionForKey(pCollection, strVal)
2086:       If booHasKey Then
        ' UPDATE SIZE COLLECTION
2088:         dblRunningSize = pCollection.Item(strVal)
2089:         dblRunningSize = dblRunningSize + dblVal
2090:         pCollection.Remove strVal
2091:         pCollection.Add dblRunningSize, strVal
        ' UPDATE COUNT COLLECTION
2093:         lngRunningCount = pCountCollection.Item(strVal)
2094:         lngRunningCount = lngRunningCount + dblSize
2095:         pCountCollection.Remove strVal
2096:         pCountCollection.Add lngRunningCount, strVal
2097:       Else
2098:         pCollection.Add dblSize, strVal
2099:         pCountCollection.Add dblSize, strVal
2100:         lngUniqueCounter = lngUniqueCounter + 1
2101:         dblUniqueNumbers(lngUniqueCounter) = dblVal
2102:       End If
2103:     Next lngIndex
    ReDim Preserve dblUniqueNumbers(lngUniqueCounter)
    
2106:     dblTotalSize = 0
2107:     For lngIndex = 0 To lngUniqueCounter
2108:       dblVal = dblUniqueNumbers(lngIndex)
2109:       dblTotalSize = dblTotalSize + pCollection(CStr(dblVal))
2110:     Next lngIndex
    
2112:     Call Linkages.QuickSort.DoubleAscending(dblUniqueNumbers, 0, lngUniqueCounter)
2113:     For lngIndex = 0 To lngUniqueCounter
2114:       Set pSubArray = New esriSystem.VarArray
      
2116:       dblVal = dblUniqueNumbers(lngIndex)
2117:       dblTotalUniqueSize = pCollection.Item(CStr(dblVal))
2118:       lngTotalUniqueCount = pCountCollection.Item(CStr(dblVal))
2119:       dblProportion = dblTotalUniqueSize / dblTotalSize
      
2121:       pSubArray.Add dblVal
2122:       pSubArray.Add lngTotalUniqueCount
2123:       pSubArray.Add dblProportion
2124:       pSubArray.Add dblTotalUniqueSize
2125:       pReturnArray.Add pSubArray
2126:     Next lngIndex
2127:   End If

2129:   Set StatsPropsForNumbers = pReturnArray


2132:   Set pReturnArray = Nothing
2133:   Set pSubArray = Nothing
2134:   Set pCollection = Nothing
2135:   Set pCountCollection = Nothing


  Exit Function
ErrorHandler:
  HandleError True, "StatsPropsForNumbers " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


2143:   Set pReturnArray = Nothing
2144:   Set pSubArray = Nothing
2145:   Set pCollection = Nothing
2146:   Set pCountCollection = Nothing

End Function


Public Function CheckCollectionForKey(colCollection As Collection, strKey As String) As Boolean
  On Error GoTo ErrorHandler
  
2154:   CheckCollectionForKey = True
  Dim varTest As Variant
2156:   varTest = colCollection.Item(strKey)
  
  Exit Function
ErrorHandler:
2160:   CheckCollectionForKey = False

End Function

Public Function ApplyCorridorSymbol(pPolygonElement As IElement) As IFillShapeElement
  On Error GoTo ErrorHandler


  Dim pFillShapeElement As IFillShapeElement
2169:   Set pFillShapeElement = pPolygonElement

  Dim pColor As IColor
2172:   Set pColor = Linkages.MyGeneralOperations.MakeColorRGB(0, 200, 100)

  Dim pCartoLine As ICartographicLineSymbol
2175:   Set pCartoLine = New CartographicLineSymbol
2176:   With pCartoLine
2177:     .Cap = esriLCSButt
2178:     .Join = esriLJSBevel
2179:     .Color = pColor
2180:     .Width = 1
2181:   End With

  Dim pLineFill As ILineFillSymbol
2184:   Set pLineFill = New LineFillSymbol
2185:   With pLineFill
2186:     .Angle = -30
2187:     .Separation = 3
2188:     .Offset = 5
2189:   End With
2190:   Set pLineFill.LineSymbol = pCartoLine

  Dim pLineSymbol As ISimpleLineSymbol
2193:   Set pLineSymbol = New SimpleLineSymbol

  Dim pLineColor As IColor
2196:   Set pLineColor = Linkages.MyGeneralOperations.MakeColorRGB(0, 250, 100)
2197:   pLineSymbol.Color = pLineColor
2198:   pLineSymbol.Width = 2
2199:   pLineSymbol.Style = esriSLSSolid
2200:   pLineFill.Outline = pLineSymbol
2201:   pFillShapeElement.Symbol = pLineFill

2203:   Set ApplyCorridorSymbol = pFillShapeElement


2206:   Set pFillShapeElement = Nothing
2207:   Set pColor = Nothing
2208:   Set pCartoLine = Nothing
2209:   Set pLineFill = Nothing
2210:   Set pLineSymbol = Nothing
2211:   Set pLineColor = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "ApplyCorridorSymbol " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


2218:   Set pFillShapeElement = Nothing
2219:   Set pColor = Nothing
2220:   Set pCartoLine = Nothing
2221:   Set pLineFill = Nothing
2222:   Set pLineSymbol = Nothing
2223:   Set pLineColor = Nothing

End Function


Public Sub ImplementKruskall(pMxDoc As IMxDocument, pStartPolygon As IPolygon, pEndPolygon As IPolygon, _
    pStartArray As esriSystem.IArray, pCorPolygon As IPolygon, ByRef frmProgressDialog As Object, _
    pExtensionConfig As IExtensionConfig, pParamDetails As esriSystem.IVariantArray, _
    pThisExt As Linkages.clsExtension)
  On Error GoTo ErrorHandler

  Dim pApp As IApplication
  Dim pDoc As IDocument
2236:   Set pDoc = pMxDoc
2237:   Set pApp = pDoc.Parent
  
'  Screen.MousePointer = vbHourglass
  
  ' pStartArray IS ARRAY OF ALL PATCHES THAT INTERSECT CORRIDOR; UNCLIPPED
  
  Dim frmProgress As Linkages.frmJenProgressPercent
2244:   Set frmProgress = frmProgressDialog
  
  ' ---- PROGRESS METER STUFF
  Dim theProgressID As String
  Dim theProgressSpecific As String
  Dim theDetailedDescription As String
2250:   theProgressSpecific = "Preliminary analysis..."
2251:   theDetailedDescription = frmProgress.txtDetails.Text
2252:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2255:   frmProgress.txtDetails.Text = theDetailedDescription
2256:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

2258:   frmProgress.Est_Time_Left 0, "Clipping to Corridor...", "Clipping to Corridor..."
2259:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2260:   frmProgress.Refresh
2261:   frmProgress.icnProgressLine.Refresh
2262:   DoEvents
2263:   If BailOutOfProgress(frmProgress, pExtensionConfig) Then
    Exit Sub
2265:   End If
  ' ---- END PROGRESS METER STUFF --------------------------------
  
  Dim varStartTime As Variant
2269:   varStartTime = Timer
  
  ' START AND END POLYGONS ARE INSERTED IN 'OK' FUNCTION OF FORM
'  pStartArray.Insert 0, pStartPolygon
'  pStartArray.Add pEndPolygon
  
  Dim pClone As IClone
  
  ' CLIP PATCH POLYGONS TO CORRIDOR; MIGHT END UP WITH MORE PATCH POLYGONS THAN BEFORE IF SOME PATCHES ARE SPLIT
  ' ALSO JOIN ADJACENT POLYGONS
  Dim pClipCollection As Collection
2280:   Set pClipCollection = ClipPolysToCorridor(pStartArray, pCorPolygon, pApp)
2281:   If Not pClipCollection.Item(1) Then          ' HERE IT RETURNS AN ERROR OF SOMETHING WAS NOT VALID
    Dim strResponse As String
2283:     strResponse = pClipCollection.Item(2)
2284:     Debug.Print strResponse
    Exit Sub
2286:   End If
  
  Dim pArray As IArray
2289:   Set pArray = pClipCollection.Item(5)
  
'  MsgBox strResponse & vbCrLf & "Array Count = " & CStr(pArray.Count)
'  Exit Sub
  
  Dim strStartNode As String
2295:       strStartNode = "0"
  Dim strEndNode As String
2297:       strEndNode = CStr(pArray.Count - 1)
  
  ' FOR DEBUGGING
'  Dim pCheckPoly As IPolygon
'  Dim pCheckIndex As Long
'  For pCheckIndex = 0 To pArray.Count - 1 '
'    Set pCheckPoly = pArray.Element(pCheckIndex)
'    MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pCheckPoly, "test_order"
'  Next pCheckIndex
  
  Dim lngArrayMaxIndex As Long
2308:   lngArrayMaxIndex = pArray.Count - 1
  
  Dim pColArray As IVariantArray
2311:       Set pColArray = New VarArray
  Dim pPolyline As IPolyline
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim strID As String
  Dim dblDist As Double
  Dim dblBear As Double
  
  Dim pShape1 As IPolygon
  Dim pShape2 As IPolygon
  Dim lngCheckIndex As Long
  
  Dim booShouldReject As Boolean
2324:   DoEvents
  
    ' ---- PROGRESS METER STUFF
2327:   theProgressSpecific = "Calculating Distance Matrix..."
2328:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2331:   frmProgress.txtDetails.Text = theDetailedDescription
2332:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

2334:   frmProgress.Est_Time_Left 0, "Preliminary Analysis...", "Calculating Distance Matrix..."
2335:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2336:   frmProgress.Refresh
2337:   frmProgress.icnProgressLine.Refresh
2338:   If BailOutOfProgress(frmProgress, pExtensionConfig) Then
    Exit Sub
2340:   End If
  ' ---- END PROGRESS METER STUFF --------------------------------
  
  
  Dim pCollection As Collection
2345:       Set pCollection = MyGeometricOperations.CalcDistMatrix(pArray, True, False, pApp)
'  Debug.Print pArray.Count
'  Debug.Print "Collection Count = " & pCollection.Count
  
  ' FOR DEBUGGING BELOW
    ' REMEMBER THAT STARTING SHAPE CAN ONLY GO OUT, AND ENDING SHAPE CAN ONLY COME IN
'  For lngIndex = 0 To lngArrayMaxIndex          ' CAN'T GO BACKWARDS FROM ENDING POLYGON
'    For lngIndex2 = 0 To lngArrayMaxIndex         ' CAN'T GO BACKWARDS TO STARTING POLYGON
'      strID = lngIndex & "_" & lngIndex2
'      Set pColArray = pCollection.Item(strID)
'      Set pPolyline = pColArray.Element(3)
'      Debug.Print strID & ",    Distance = " & _
'        pColArray.Element(2) & ",     Polyline Distance = " & pPolyline.Length
'    Next lngIndex2
'  Next lngIndex

  Dim pFinalCollection As New Collection
  Dim pIDArray As IStringArray
2363:       Set pIDArray = New strArray
  Dim pDoneCollection As CollectionMod
2365:       Set pDoneCollection = New CollectionMod
  Dim pCounter As Long
2367:   pCounter = -1
  Dim pSubArray As IVariantArray
  
   ' FOR DEBUGGING
2371:   MyGeneralOperations.DeleteGraphicsByName pMxDoc, "test_matrix"
  
  ' REMEMBER THAT CORRIDOR MIGHT BE MULTI-STRAND.  THEREFORE ONLY CONSIDER PAIRS OF PATCH POLYGONS
  ' THAT ARE IN THE SAME STRAND.
  Dim pIntPolygon As IPolygon4
2376:       Set pIntPolygon = pCorPolygon
  Dim pRelOp As IRelationalOperator
  Dim pGeometryCollection As IGeometryCollection
2379:       Set pGeometryCollection = pIntPolygon.ConnectedComponentBag
  Dim aSubPolyIndex As Long
  Dim pCorStrandPolygon As IPolygon4
  Dim pSubStartPoly As IPolygon
  Dim pSubEndPoly As IPolygon
  Dim IsMultiStrand As Boolean
2385:       IsMultiStrand = pGeometryCollection.GeometryCount > 1
  Dim IsInSameStrand As Boolean
  
    ' ---- PROGRESS METER STUFF
2389:   theProgressSpecific = "Preliminary analysis of matrix..."
2390:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2393:   frmProgress.txtDetails.Text = theDetailedDescription
2394:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

2396:   frmProgress.Est_Time_Left 0, "Preliminary Analysis...", "Preliminary analysis of matrix..."
2397:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2398:   If BailOutOfProgress(frmProgress, pExtensionConfig) Then
    Exit Sub
2400:   End If
2401:   frmProgress.Refresh
2402:   frmProgress.icnProgressLine.Refresh
2403:   DoEvents
  ' ---- END PROGRESS METER STUFF --------------------------------
  
  ' REMEMBER THAT STARTING SHAPE CAN ONLY GO OUT, AND ENDING SHAPE CAN ONLY COME IN
2407:   For lngIndex = 0 To (lngArrayMaxIndex - 1)      ' CAN'T GO BACKWARDS FROM ENDING POLYGON
2408:     For lngIndex2 = 1 To lngArrayMaxIndex         ' CAN'T GO BACKWARDS TO STARTING POLYGON
2409:       If lngIndex <> lngIndex2 Then               ' DON'T CONSIDER DISTANCE FROM A SHAPE TO ITSELF
        
        ' IF NECESSARY, CHECK TO MAKE SURE 2ND POLYGON INTERSECTS SAME CORRIDOR STRAND AS FIRST POLYGON
2412:         If IsMultiStrand Then
2413:           Set pSubStartPoly = pArray.Element(lngIndex)
2414:           Set pSubEndPoly = pArray.Element(lngIndex2)
2415:           For aSubPolyIndex = 0 To pGeometryCollection.GeometryCount - 1
2416:             Set pCorStrandPolygon = pGeometryCollection.Geometry(aSubPolyIndex)
2417:             Set pRelOp = pCorStrandPolygon
2418:             IsInSameStrand = (Not pRelOp.Disjoint(pSubStartPoly)) And (Not pRelOp.Disjoint(pSubEndPoly))
2419:             If IsInSameStrand Then Exit For
2420:           Next aSubPolyIndex
2421:         Else
2422:           IsInSameStrand = True
2423:         End If
        
2425:         If IsInSameStrand Then
2426:           strID = lngIndex & "_" & lngIndex2
2427:           Set pColArray = pCollection.Item(strID)
        '  pColArray.Add (lngIndex)
2429:           pFinalCollection.Add pColArray, strID
2430:           pIDArray.Add strID
          
2432:           pCounter = pCounter + 1
2433:           Set pSubArray = New VarArray
2434:           pSubArray.Add pCounter
2435:           pSubArray.Add False
2436:           pDoneCollection.AddObject pSubArray, strID, True
2437:         End If
2438:       End If
2439:     Next lngIndex2
2440:   Next lngIndex
  ' NOW HAVE A COLLECTION OF VARIANT ARRAYS, WHERE VARIANT ARRAY CONTAINS {POLYLINE, DISTANCE}
  ' ALSO HAVE AN ARRAY OF STRING ID VALUES FOR ALL OBJECTS IN COLLECTION
  
'  For lngIndex = 0 To pIDArray.Count - 1
'    Set pColArray = pFinalCollection.Item(pIDArray.Element(lngIndex))
'    Debug.Print pIDArray.Element(lngIndex) & ",    Distance = " & _
'        pColArray.Element(2)
'  Next lngIndex
  
  ' FIRST CHECK FOR LINES THAT CROSS EDGE; NEED TO GENERATE GRID PATH FOR THEM
  Dim lngFullIndex As Long
  Dim booHasBeenDone As Boolean
  Dim booCrossesEdge As Boolean
  Dim lngFinalIndex As Long
2455:   lngFinalIndex = -1
  Dim pGridLine As IPolyline
  Dim pGridPoly1 As IPolygon
  Dim pGridPoly2 As IPolygon
  Dim pCorRaster As IRaster
  Dim pEnv As IRasterAnalysisEnvironment
  Dim strRevID As String, pReverseLine As IPolyline, pNewColArray As IVariantArray
  Dim lngRevIndex As Long
  Dim booHasRevDir As Boolean
  
  Dim pRemoveKeyArray As IStringArray
2466:   Set pRemoveKeyArray = New strArray
  
'  Dim pArray2 As esriSystem.IArray2
  
    ' ---- PROGRESS METER STUFF
2471:   frmProgress.ProgRecCount = pIDArray.Count
2472:   theProgressSpecific = "Checking for Boundary Intersections..."
2473:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2476:   frmProgress.txtDetails.Text = theDetailedDescription
2477:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

2479:   frmProgress.Est_Time_Left 0, "Boundary Intersections...", "Working on Analysis #"
2480:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2481:   frmProgress.Refresh
2482:   frmProgress.icnProgressLine.Refresh
2483:   DoEvents
2484:   If BailOutOfProgress(frmProgress, pExtensionConfig) Then
    Exit Sub
2486:   End If
  ' ---- END PROGRESS METER STUFF --------------------------------

  Dim psbar As IStatusBar
2490:   Set psbar = pApp.StatusBar
  Dim pPro As IStepProgressor
2492:   Set pPro = psbar.ProgressBar
  Dim lngCounter As Long
2494:   lngCounter = 0
  Dim lngTotalCount As Long
2496:   lngTotalCount = pIDArray.Count - 1
  Dim strTotalCount As String
2498:   strTotalCount = CStr(pIDArray.Count)
2499:   pPro.position = 1
2500:   psbar.ShowProgressBar "Refining Distance Matrix:  Step 1 of " & strTotalCount & "...", 1, _
          lngTotalCount + 1, 1, True
  
2503:   For lngFullIndex = 0 To pIDArray.Count - 1
2504:     strID = pIDArray.Element(lngFullIndex)
2505:     Set pSubArray = pDoneCollection.GetObject(strID)
2506:     booHasBeenDone = pSubArray.Element(1)
2507:     Set pColArray = pFinalCollection.Item(strID)
2508:     lngIndex = pColArray.Element(0)
2509:     lngIndex2 = pColArray.Element(1)
    
2511:     lngCounter = lngCounter + 1
2512:     pPro.Message = "Refining Distance Matrix:  Step " & CStr(lngCounter) & " of " & strTotalCount & " ( ID = " & strID & ")..."
2513:     psbar.StepProgressBar
    
    ' ---- PROGRESS METER STUFF
2516:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2517:     theProgressID = CStr(lngIndex + 1) & " and " & CStr(lngIndex2 + 1)
2518:     theProgressSpecific = "Checking connector between patch " & theProgressID & "..."
2519:     theDetailedDescription = theDetailedDescription & _
      " --> " & theProgressSpecific & vbCrLf
2521:     frmProgress.txtDetails.Text = theDetailedDescription
2522:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2523:     frmProgress.Est_Time_Left lngFullIndex, "Boundary Intersections...", "Working on Analysis #"
2524:     frmProgress.Refresh
2525:     frmProgress.icnProgressLine.Refresh
2526:     DoEvents
    ' ---- END PROGRESS METER STUFF ---------------
    
2529:     If Not booHasBeenDone Then

 '     ThisDocument.Graphic_MakeFromGeometry pMxDoc, pArray.Element(0), "DeleteMatrix"
 '     ThisDocument.Graphic_MakeFromGeometry pMxDoc, pArray.Element(1), "DeleteMatrix"
      
2534:       dblDist = pColArray.Element(2)
2535:       Set pPolyline = pColArray.Element(3)
2536:       booCrossesEdge = CheckIntersectBoundary(pPolyline, pCorPolygon)
      
      ' CREATE NEW ARRAY FOR REVERSE DIRECTION, SO ONLY HAVE TO EXAMINE THIS ROUTE ONCE
      ' ONLY DO THIS IF THERE ACTUALLY IS A REVERSE DIRECTION UNDER CONSIDERATION
      
2541:       strRevID = CStr(lngIndex2) & "_" & CStr(lngIndex)
2542:       booHasRevDir = pDoneCollection.HasKey(strRevID)
      
2544:       If booHasRevDir Then
2545:         Set pNewColArray = New VarArray
2546:         pNewColArray.Add lngIndex2
2547:         pNewColArray.Add lngIndex
2548:         pNewColArray.Add dblDist
2549:         Set pSubArray = pDoneCollection.GetObject(strRevID)
2550:         lngRevIndex = pSubArray.Element(0)
2551:       End If
      
2553:       If booCrossesEdge Then

'        Debug.Print "LINE CROSSED!!!! BAD BAD BAD!!!  Line connecting polygon " & CStr(lngIndex) & _
'            " and " & CStr(lngIndex2) & "... uh oh!"
'        Debug.Print "Creating grid-based line..."
2558:         Set pGridPoly1 = pArray.Element(lngIndex)
2559:         Set pGridPoly2 = pArray.Element(lngIndex2)
  '      Set pGridPoly1 = pArray.Element(2)
  '      Set pGridPoly2 = pArray.Element(4)
2562:         If pCorRaster Is Nothing Then
          
          ' ---- PROGRESS METER STUFF
2565:           frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2566:           theDetailedDescription = theDetailedDescription & _
            "          Setting up raster analysis environment." & vbCrLf
2568:           frmProgress.txtDetails.Text = theDetailedDescription
2569:           frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2570:           frmProgress.Est_Time_Left lngFullIndex, "Boundary Intersections...", "Setting Up Analysis Environment..."
2571:           frmProgress.Refresh
2572:           frmProgress.icnProgressLine.Refresh
2573:           DoEvents
          ' ---- END PROGRESS METER STUFF ---------------
        
2576:           Set pCorRaster = GenerateCorridorRaster(pCorPolygon, pEnv, pThisExt)
        
2578:         End If
        ' ---- PROGRESS METER STUFF
2580:         frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2581:         theDetailedDescription = theDetailedDescription & _
          "          Patch " & theProgressID & " Connector crosses outside corridor." & vbCrLf & _
          "          *** Creating grid-based connector..." & vbCrLf
2584:         frmProgress.txtDetails.Text = theDetailedDescription
2585:         frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2586:         frmProgress.Est_Time_Left lngFullIndex + 1, "Boundary Intersections...", "Working on Analysis #"
2587:         frmProgress.Refresh
2588:         frmProgress.icnProgressLine.Refresh
2589:         DoEvents
        ' ---- END PROGRESS METER STUFF ---------------
        
  '      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pGridPoly1, "DeleteMatrix"
  '      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pGridPoly2, "DeleteMatrix"
  '      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pCorPolygon, "DeleteMatrix"
2595:         Set pGridLine = GridFunctions.CalcGridLine(pGridPoly1, pGridPoly2, pCorPolygon, pCorRaster, pEnv)
  '       Set pGridLine = pPolyline
  '      Dim zzzEnv As IEnvelope
  '      pEnv.GetExtent 3, zzzEnv
  '      ThisDocument.Graphic_MakeFromGeometry pMxDoc, zzzEnv, "DeleteMatrix"
        
        ' IT IS POSSIBLE THAT pGridLine WILL BE EMPTY IF THERE WAS NO WAY TO GENERATE A LEAST-COST PATH
        ' IF SO, REMOVE SEGMENT FROM CONSIDERATION
2603:         If pGridLine.IsEmpty Then
2604:           pRemoveKeyArray.Add strID
2605:           pRemoveKeyArray.Add strRevID
2606:         ElseIf pGridLine.length = 0 Then
2607:           pRemoveKeyArray.Add strID
2608:           pRemoveKeyArray.Add strRevID
2609:         Else
2610:           Set pGridLine = AttachPolylineToPatches(pGridLine, pGridPoly1, pGridPoly2)
2611:         End If
        
2613:         pColArray.Remove 3
2614:         pColArray.Insert 3, pGridLine
        
        'ADD TO ARRAY FOR REVERSE DIRECTION
2617:         If booHasRevDir Then
2618:           Set pClone = pGridLine
2619:           Set pReverseLine = pClone.Clone
2620:           pReverseLine.ReverseOrientation
2621:           pNewColArray.Add pReverseLine
2622:         End If
2623:       Else
 '       Debug.Print "Line connecting polygon " & CStr(lngIndex) & _
 '           " and " & CStr(lngIndex2) & " does not intersect boundary..."

        ' ---- PROGRESS METER STUFF
2628:         frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2629:         theDetailedDescription = theDetailedDescription & _
          "          Patch " & theProgressID & " Connector does not cross corridor boundary." & vbCrLf
2631:         frmProgress.txtDetails.Text = theDetailedDescription
2632:         frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2633:         frmProgress.Est_Time_Left lngFullIndex, "Boundary Intersections...", "Working on Analysis #"
2634:         frmProgress.Refresh
2635:         frmProgress.icnProgressLine.Refresh
2636:         DoEvents
        ' ---- END PROGRESS METER STUFF ---------------
        
        'ADD TO ARRAY FOR REVERSE DIRECTION
2640:         If booHasRevDir Then
2641:           Set pClone = pPolyline
2642:           Set pReverseLine = pClone.Clone
2643:           pReverseLine.ReverseOrientation
2644:           pNewColArray.Add pReverseLine
2645:         End If
2646:       End If
      
2648:       If booHasRevDir Then
        ' REPLACE REVERSE LINE IN ORIGINAL FINAL COLLECTION
2650:         pFinalCollection.Remove strRevID
2651:         pFinalCollection.Add pNewColArray, strRevID
      
        ' MARK THIS REVERSE LINE AS HAVING BEEN DONE
2654:         Set pSubArray = pDoneCollection.GetObject(strRevID)
2655:         pSubArray.Remove 1
2656:         pSubArray.Add True
2657:         pDoneCollection.AddObject pSubArray, strRevID, True
2658:       End If
      
      
  '    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pColArray.Element(3), "DeleteMatrix"
  '    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pArray.Element(lngIndex), "DeleteMatrix"
  '    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pArray.Element(lngIndex2), "DeleteMatrix"
  '    pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing
  '    ThisDocument.DeleteGraphicsByName pMxDoc, "DeleteMatrix"
2666:     Else

        ' ---- PROGRESS METER STUFF
2669:         frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2670:         theDetailedDescription = theDetailedDescription & _
          "          Patch " & theProgressID & " Connector has already been analyzed." & vbCrLf
2672:         frmProgress.txtDetails.Text = theDetailedDescription
2673:         frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2674:         frmProgress.Est_Time_Left lngFullIndex, "Boundary Intersections...", "Working on Analysis #"
        ' ---- END PROGRESS METER STUFF ---------------
        
 '     Debug.Print "Line connecting polygon " & CStr(lngIndex) & _
 '           " and " & CStr(lngIndex2) & " has been analyzed previously..."
2679:     End If
    
    ' ---- PROGRESS METER STUFF
2682:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2683:     theDetailedDescription = theDetailedDescription & _
      "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2685:     frmProgress.txtDetails.Text = theDetailedDescription
2686:     frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
2687:     frmProgress.Est_Time_Left lngFullIndex, "Boundary Intersections...", "Working on Analysis #"
2688:     frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2689:     If BailOutOfProgress(frmProgress, pExtensionConfig) Then
      Exit Sub
2691:     End If
    ' ---- END PROGRESS METER STUFF --------------------------------
    
2694:   Next lngFullIndex
  
2696:   pPro.position = 1
2697:   psbar.HideProgressBar
  
  ' FINALLY, ONLY CONSIDER LINES THAT DO NOT INTERSECT OTHER SHAPES
  ' ALSO REMOVE ANY SEGMENTS LISTED IN pRemoveKeyArray

  Dim pFinalIDArray As IStringArray
2703:       Set pFinalIDArray = New strArray
  Dim dblDistArray() As Double
      ReDim dblDistArray(pFinalCollection.Count)
  Dim pDistCollection As CollectionMod
2707:       Set pDistCollection = New CollectionMod
  Dim strFormatString As String
2709:       strFormatString = "0.000000000000"
  Dim strDistString As String
  Dim pFinalCollection3 As New CollectionMod
  Dim booIsInRemoveList As Boolean
  Dim lngRemoveKeyIndex As Long
  
2715:   For lngFullIndex = 0 To pIDArray.Count - 1
2716:     strID = pIDArray.Element(lngFullIndex)
2717:     Set pColArray = pFinalCollection.Item(strID)
2718:     lngIndex = pColArray.Element(0)
2719:     lngIndex2 = pColArray.Element(1)
2720:     dblDist = pColArray.Element(2)
2721:     booShouldReject = False
2722:     Set pPolyline = pColArray.Element(3)
    
    ' RECALCULATE DISTANCE HERE BECAUSE IT HAS LIKELY CHANGED
2725:     dblDist = pPolyline.length
    
    ' FOR DEBUGGING
'    Linkages.MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pPolyline, "pre_delete_" & strID
    
    ' CHECK TO SEE IF KEY IS IN REMOVE LIST
2731:     booIsInRemoveList = False
2732:     For lngRemoveKeyIndex = 0 To pRemoveKeyArray.Count - 1
2733:       If pRemoveKeyArray.Element(lngRemoveKeyIndex) = strID Then
2734:         booIsInRemoveList = True
2735:         Exit For
2736:       End If
2737:     Next lngRemoveKeyIndex
    
2739:     If Not booIsInRemoveList Then
      ' CHECK IF CONNECTOR LINE INTERSECTS ANY SHAPES OTHER THAN THE TWO IN QUESTION
2741:       For lngCheckIndex = 0 To lngArrayMaxIndex
2742:         If (lngCheckIndex <> lngIndex) And (lngCheckIndex <> lngIndex2) Then
2743:           Set pShape1 = pArray.Element(lngCheckIndex)
2744:           Set pRelOp = pPolyline
2745:           If Not pRelOp.Disjoint(pShape1) Then    ' THIS CHECKS FOR INTERSECTION
2746:             booShouldReject = True
2747:             Exit For
2748:           End If
2749:         End If
2750:       Next lngCheckIndex
    
2752:       If Not booShouldReject Then
2753:         lngFinalIndex = lngFinalIndex + 1
2754:         strDistString = Format(dblDist, strFormatString)
        
2756:         Do While pDistCollection.HasKey(strDistString)
   '     Do While Not IsNull(pDistCollection.Item(strDistString))
2758:           dblDist = dblDist * 1.00000000001
2759:           strDistString = Format(dblDist, strFormatString)
2760:         Loop
        
2762:         dblDistArray(lngFinalIndex) = dblDist
2763:         pDistCollection.AddVariable strID, strDistString, False
2764:         pFinalIDArray.Add strID
  '      Debug.Print "Adding " & strID & " to pFinalCollection3..."
2766:         pFinalCollection3.AddObject pColArray, strID, False
        
  '      Debug.Print "Connecting " & CStr(lngIndex) & " to " & CStr(lngIndex2) & ":  Distance = " & _
                  CStr(dblDist) & ".     EMPTY POLYLINE = " & CStr(pPolyline.IsEmpty)
        
 '       MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pPolyline, "test_matrix"
2772:       End If
2773:     End If
2774:   Next lngFullIndex
  ReDim Preserve dblDistArray(lngFinalIndex)
'  Debug.Print pFinalIDArray.Count
  
  Dim dblSortArray() As Double
  ReDim dblSortArray(lngFullIndex)
2780:   dblSortArray = dblDistArray

2782:   Call QuickSort.DoubleAscending(dblSortArray, 0, lngFinalIndex)
  

'   NET DICTIONARY -------------------------------
'   DICTIONARY STRUCTURED AS:  CollectionMod Object
'                   KEY = NODE ID STRING
'               ELEMENT = CollectionMod OF POSSIBLE MOVEMENT OPTIONS:  WATCH FOR 1-WAY SITUATIONS
'                         SUCH AS START NODE CAN ONLY GO OUT, END NODE ONLY HAS INCOMING
'                                 KEY = NODE ID STRING WHERE CONNECTOR GOES TO
'                             ELEMENT = IVarArray, {CONNECTOR LINE, CONNECTOR LENGTH}
    
    ' ---- PROGRESS METER STUFF
2794:   theProgressSpecific = "Identifying Best Path..."
2795:   theDetailedDescription = theDetailedDescription & _
    " --> " & theProgressSpecific & vbCrLf & _
    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf & vbCrLf
2798:   frmProgress.txtDetails.Text = theDetailedDescription
2799:   frmProgress.txtDetails.SelStart = Len(theDetailedDescription)

2801:   frmProgress.Est_Time_Left lngFullIndex, "Final Analysis...", "Preliminary analysis of matrix..."
2802:   frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
2803:   If BailOutOfProgress(frmProgress, pExtensionConfig) Then
    Exit Sub
2805:   End If
2806:   frmProgress.Refresh
2807:   frmProgress.icnProgressLine.Refresh
  ' ---- END PROGRESS METER STUFF --------------------------------
  
  Dim theNetDictionary As New CollectionMod
  Dim theNodeDictionary As CollectionMod
  Dim theNodeElement As IVariantArray
  
  Dim lngFrom As Long
  Dim lngTo As Long
  Dim strFrom As String
  Dim strTo As String
    
  Dim pKeys() As String
2820:   pKeys = pFinalCollection3.ReturnKeys
  Dim pVariant As Variant
  
2823:   For lngFullIndex = 0 To lngFinalIndex
2824:     strID = pFinalIDArray.Element(lngFullIndex)
2825:     Set pColArray = pFinalCollection3.GetObject(strID)
2826:     lngFrom = pColArray.Element(0)
2827:     lngTo = pColArray.Element(1)
2828:     strFrom = CStr(lngFrom)
2829:     strTo = CStr(lngTo)
2830:     dblDist = pColArray.Element(2)
2831:     Set pPolyline = pColArray.Element(3)
    
    ' FOR DEBUGGING
'    Linkages.MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pPolyline, "post_delete_" & strID & "_dist_" & CStr(Format(dblDist, "0"))
    
2836:     If Not theNetDictionary.HasKey(strFrom) Then
2837:       Set theNodeDictionary = New CollectionMod
2838:       Set theNodeElement = New VarArray
      
2840:       theNodeElement.Add pPolyline
2841:       theNodeElement.Add dblDist
      
2843:       theNodeDictionary.SetOrReplace theNodeElement, strTo
2844:       theNetDictionary.SetOrReplace theNodeDictionary, strFrom
2845:     Else
2846:       Set theNodeDictionary = theNetDictionary.GetObject(strFrom)
2847:       If theNodeDictionary.HasKey(strTo) Then
2848:         Set theNodeElement = theNodeDictionary.GetObject(strTo)
2849:         theNodeElement.Add pPolyline
2850:         theNodeElement.Add dblDist
2851:       Else
2852:         Set theNodeElement = New VarArray
2853:         theNodeElement.Add pPolyline
2854:         theNodeElement.Add dblDist
        
2856:         theNodeDictionary.SetOrReplace theNodeElement, strTo
2857:       End If
2858:     End If
  
2860:   Next lngFullIndex
'theDictionaryOfConnectors = Dictionary.Make(theLineDistances.Count)
'

  Dim theDictionaryOfConnectors As New CollectionMod
  Dim anIndex As Long
'for each aLineDistance in theLineDistances
'  theData = theDictionaryOfDistances.Get(aLineDistance)
'  thePointA = theData.Get(0)
'  thePointB = theData.Get(1)
'  theLine = theData.Get(2)
'
'  theColor = Color.GetBlue
'
'  if ((theDictionaryOfConnectors.Get(thePointA) <> nil) AND
'      (theDictionaryOfConnectors.Get(thePointB) <> nil)) then
'      IsAcyclic = av.Run("Jennessent.CheckForAcyclicity", {thePointA, thePointB, theDictionaryOfConnectors})
'      if (IsAcyclic.Not) then
''        theColor = Color.GetGreen
''        theGraphic = GraphicShape.Make(theLine)
''        theGraphic.GetSymbol.SetColor(theColor)
''        theGraphic.SetName("Temp")
''        theGraphics.Add(theGraphic)
'        continue
'      end
'  end


  Dim lngHighIndex As Long
  Dim pIndexArray As IStringArray
  Dim pConnectionResult As IUnknown
  
  Dim IsAcyclic As Boolean
  Dim pRouteArray As esriSystem.IArray
  
  ' Dim strDebugReport As String
  
2897:   For anIndex = 0 To UBound(dblSortArray)
2898:     dblDist = dblSortArray(anIndex)
2899:     strID = pDistCollection.GetVariable(Format(dblDist, strFormatString))
2900:     Set pColArray = pFinalCollection3.GetObject(strID)
      
2902:     lngFrom = pColArray.Element(0)
2903:     lngTo = pColArray.Element(1)
2904:     strFrom = CStr(lngFrom)
2905:     strTo = CStr(lngTo)
2906:     Set pPolyline = pColArray.Element(3)
    
2908:     IsAcyclic = True
2909:     If theDictionaryOfConnectors.HasKey(strFrom) And theDictionaryOfConnectors.HasKey(strTo) Then
      ' CHECK FOR ACYCLICITY
2911:       IsAcyclic = CheckForAcyclicity(strFrom, strTo, theDictionaryOfConnectors)
      
2913:     End If
    
'    strDebugReport = strDebugReport & "Testing ID Val [" & strID & "]:  IsAcyclic = " & CStr(IsAcyclic) & vbCrLf & _
        "  --> Length = " & CStr(Format(pPolyline.length, "0")) & vbCrLf & _
        "  --> From = " & strFrom & vbCrLf & _
        "  --> To = " & strTo & vbCrLf & vbCrLf
    
'  theCurrentA = theDictionaryOfConnectors.Get(thePointA)
'  if (theCurrentA = nil) then
'    theDictionaryOfConnectors.Set(thePointA, {thePointB})
'  else
'    theCurrentA.Add(thePointB)
'  end
'  theCurrentB = theDictionaryOfConnectors.Get(thePointB)
'  if (theCurrentB = nil) then
'    theDictionaryOfConnectors.Set(thePointB, {thePointA})
'  else
'    theCurrentB.Add(thePointA)
'  end
'
2933:     If IsAcyclic Then
    
2935:       If Not theDictionaryOfConnectors.HasKey(strFrom) Then
2936:         Set pIndexArray = New strArray
2937:         pIndexArray.Add (strTo)
2938:         theDictionaryOfConnectors.AddObject pIndexArray, strFrom, False
2939:       Else
2940:         Set pIndexArray = theDictionaryOfConnectors.GetObject(strFrom)
2941:         pIndexArray.Add (strTo)
2942:       End If
      
2944:       If Not theDictionaryOfConnectors.HasKey(strTo) Then
2945:         Set pIndexArray = New strArray
2946:         pIndexArray.Add (strFrom)
2947:         theDictionaryOfConnectors.AddObject pIndexArray, strTo, False
2948:       Else
2949:         Set pIndexArray = theDictionaryOfConnectors.GetObject(strTo)
2950:         pIndexArray.Add (strFrom)
2951:       End If
      
      ' CHECK FOR CONNECTION
2954:       Set pRouteArray = CheckForConnection(strStartNode, strEndNode, theDictionaryOfConnectors, theNetDictionary)
      
'      strDebugReport = strDebugReport & "****  Connection Found = " & UCase(CStr(Not pRouteArray Is Nothing)) & vbCrLf & vbCrLf
      
2958:       If Not pRouteArray Is Nothing Then Exit For

'  theGraphic = GraphicShape.Make(theLine)
'  theGraphic.GetSymbol.SetColor(theColor)
'  theGraphic.SetName("Temp")
'  theGraphics.Add(theGraphic)
'
'  x = 1
'
'  theResult = av.Run("Jennessent.CheckForConnection", {theOriginPoint, theEndPoint, theDictionaryOfConnectors, theNetDictionary})
'  if (theResult <> nil) then
'    thePath = theResult
'    break
'  end
'
'end
2974:     End If
  
2976:   Next anIndex
  

  
  
  Dim pElement As ILineElement
  Dim pPolyElement As IFillShapeElement
  Dim pLineSymbol As ISimpleLineSymbol
2984:   Set pLineSymbol = New SimpleLineSymbol
  Dim pColor As IColor
2986:   Set pColor = MyGeneralOperations.MakeColorRGB(0, 100, 0)
  Dim pInternalColor As IColor
2988:   Set pInternalColor = MyGeneralOperations.MakeColorRGB(240, 255, 240)
2989:   pLineSymbol.Color = pColor
2990:   pLineSymbol.Width = 1.5
  
  Dim pCartoLine As ICartographicLineSymbol
2993:   Set pCartoLine = New CartographicLineSymbol
2994:   With pCartoLine
2995:     .Width = 1.5
2996:     .Color = pColor
2997:   End With
 
 'Create a Simple Fill
  Dim pSmplFill As ISimpleFillSymbol
3001:   Set pSmplFill = New SimpleFillSymbol

 'Create the Fill Symbol and set the properties
  Dim pFillSym As IFillSymbol
3005:   Set pFillSym = pSmplFill
3006:   With pFillSym
3007:     .Color = pInternalColor
3008:     .Outline = pCartoLine
3009:   End With
 
  
  Dim pGContainer As IGraphicsContainer
3013:   Set pGContainer = pMxDoc.FocusMap
  Dim pPolygon As IPolygon
  Dim anIndex2 As Long
  
  ' FOR GRADIENT FILL
  Dim pTempArray As IVariantArray
  Dim pGradStartPoint As IPoint
  Dim pGradEndPoint As IPoint
  Dim pPolyline2 As IPolyline
  Dim pStartPoint As IPoint
  Dim pEndPoint As IPoint
3024:   Set pStartPoint = New Point
3025:   Set pEndPoint = New Point
  Dim pStartPoint2 As IPoint
  Dim pEndPoint2 As IPoint
3028:   Set pStartPoint2 = New Point
3029:   Set pEndPoint2 = New Point
  Dim pProxOp As IProximityOperator
  Dim dblStartDist As Double
  Dim dblEndDist As Double
  Dim dblBearing As Double
  Dim pGradFillSym As IGradientFillSymbol
  Dim pAlgColRamp As IAlgorithmicColorRamp
3036:   Set pAlgColRamp = New AlgorithmicColorRamp
   '** Set the Start and End Colors
3038:   pAlgColRamp.ToColor = pColor
3039:   pAlgColRamp.FromColor = pInternalColor
3040:   pAlgColRamp.Algorithm = esriHSVAlgorithm
3041:   pAlgColRamp.SIZE = 30
3042:   pAlgColRamp.CreateRamp (True)

  Dim pLineArray As IVariantArray
  Dim booWasSuccessful As Boolean
  
3047:   If pRouteArray Is Nothing Then
3048:     MsgBox "Unable to determine route!  Bailing out..."
3049:   Else
    Dim dblFinalDistances() As Double
    ReDim dblFinalDistances(pRouteArray.Count - 1)
    
3053:     For anIndex = 0 To pRouteArray.Count - 1
3054:       Set pLineArray = pRouteArray.Element(anIndex)
'      Debug.Print "Moving from " & pLineArray.Element(2) & " to "; pLineArray.Element(3) _
          & ";    Distance = " & pLineArray.Element(1)
      
3058:       Set pPolyline = pLineArray.Element(0)
3059:       dblFinalDistances(anIndex) = pPolyline.length
      
3061:       Set pElement = MyGeneralOperations.Graphic_ReturnElementFromGeometry(pMxDoc, pPolyline, "Route_Graphics", False)
3062:       pElement.Symbol = pLineSymbol
            
      ' ADD GRAPHIC TO GRAPHICS CONTAINER
3065:       pGContainer.AddElement pElement, 0
      
3067:       Set pGradFillSym = New GradientFillSymbol
      
3069:       pPolyline.QueryFromPoint pStartPoint
3070:       pPolyline.QueryToPoint pEndPoint
      
      ' GET POLYGONS
3073:       If anIndex = 0 Then
3074:         Set pPolygon = pArray.Element(0)
3075:         Set pPolyElement = MyGeneralOperations.Graphic_ReturnElementFromGeometry(pMxDoc, pPolygon, "Route_Graphics", False)
        
        ' FOR GRADIENT FILL
3078:         Set pProxOp = pStartPoint
3079:         dblStartDist = pProxOp.ReturnDistance(pPolygon)
3080:         Set pProxOp = pEndPoint
3081:         dblEndDist = pProxOp.ReturnDistance(pPolygon)
3082:         If dblStartDist < dblEndDist Then
3083:           dblBearing = MyGeometricOperations.CalcBearing(pStartPoint, pEndPoint)
3084:         Else
3085:           dblBearing = MyGeometricOperations.CalcBearing(pEndPoint, pStartPoint)
3086:         End If
3087:         dblBearing = 90 - dblBearing
3088:         If dblBearing < -180 Then dblBearing = dblBearing + 360
3089:         pGradFillSym.GradientAngle = dblBearing
3090:         pGradFillSym.Outline = pCartoLine
3091:         pGradFillSym.ColorRamp = pAlgColRamp
3092:         pGradFillSym.IntervalCount = 30
3093:         pGradFillSym.Style = esriGFSLinear
3094:         pGradFillSym.GradientPercentage = 1
        
'        Debug.Print dblStartDist & ", " & dblEndDist & ", " & dblBearing
        
3098:         pPolyElement.Symbol = pGradFillSym
3099:         pGContainer.AddElement pPolyElement, 0
3100:       End If
      
      ' FOR GRADIENT FILL
3103:       Set pProxOp = pStartPoint
3104:       dblStartDist = pProxOp.ReturnDistance(pPolygon)
3105:       Set pProxOp = pEndPoint
3106:       dblEndDist = pProxOp.ReturnDistance(pPolygon)
3107:       If anIndex <> pRouteArray.Count - 1 Then    ' THEN NOT LAST POLYGON, AND CAN SHADE BETWEEN CONNECTORS
3108:         If dblStartDist < dblEndDist Then
3109:           Set pGradStartPoint = pStartPoint
3110:         Else
3111:           Set pGradStartPoint = pEndPoint
3112:         End If
3113:         Set pTempArray = pRouteArray.Element(anIndex + 1)
3114:         Set pPolyline2 = pTempArray.Element(0)
3115:         pPolyline2.QueryFromPoint pStartPoint2
3116:         pPolyline2.QueryToPoint pEndPoint2
3117:         Set pProxOp = pStartPoint
3118:         dblStartDist = pProxOp.ReturnDistance(pPolygon)
3119:         Set pProxOp = pEndPoint
3120:         dblEndDist = pProxOp.ReturnDistance(pPolygon)
3121:         If dblStartDist < dblEndDist Then
3122:           Set pGradEndPoint = pStartPoint2
3123:         Else
3124:           Set pGradEndPoint = pEndPoint2
3125:         End If
3126:         dblBearing = MyGeometricOperations.CalcBearing(pGradStartPoint, pGradEndPoint)
3127:       Else
3128:         If dblStartDist < dblEndDist Then
3129:           dblBearing = MyGeometricOperations.CalcBearing(pStartPoint, pEndPoint)
3130:         Else
3131:           dblBearing = MyGeometricOperations.CalcBearing(pEndPoint, pStartPoint)
3132:         End If
3133:       End If
3134:       dblBearing = 90 - dblBearing
3135:       If dblBearing < -180 Then dblBearing = dblBearing + 360
3136:       pGradFillSym.GradientAngle = dblBearing
3137:       pGradFillSym.Outline = pCartoLine
3138:       pGradFillSym.ColorRamp = pAlgColRamp
3139:       pGradFillSym.IntervalCount = 145
3140:       pGradFillSym.Style = esriGFSLinear
3141:       pGradFillSym.GradientPercentage = 1
      
3143:       anIndex2 = CLng(pLineArray.Element(3))
3144:       Set pPolygon = pArray.Element(anIndex2)
3145:       Set pPolyElement = MyGeneralOperations.Graphic_ReturnElementFromGeometry(pMxDoc, pPolygon, "Route_Graphics", False)
3146:       pPolyElement.Symbol = pGradFillSym
3147:       pGContainer.AddElement pPolyElement, 0
3148:     Next anIndex
    
3150:     booWasSuccessful = True
3151:   End If
  
3153:   pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing

  ' PROGRESS METER STUFF
  
  Dim theDateString As String
  Dim theElapsedTimeString As String
  
3160:   theDateString = Format(Now, "long date") & ";  " & Format(Now, "long time")
3161:   Screen.MousePointer = vbDefault
  Dim theTimeBegan As Date
  Dim theTimeEnd As Date
3164:   theTimeBegan = frmProgress.ProgBeginTime
3165:   theTimeEnd = Now

3167:   theElapsedTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)

  Dim theReport As String
3170:   theReport = "Operation completed at " & theDateString & vbCrLf & _
      "---------------------------------------------------------" & vbCrLf & theElapsedTimeString
  
  ' FOR DEBUGGING
'  theReport = theReport & vbCrLf & "-------------------------------------------------" & vbCrLf & strDebugReport
  
  Dim strCurrentReport As String
3177:   strCurrentReport = frmProgress.txtDetails.Text
3178:   frmProgress.txtDetails.Text = strCurrentReport & vbCrLf & theReport
  
  Dim strTimeElapsedRTF As String
3181:   strTimeElapsedRTF = MyGeneralOperations.ReturnTimeElapsedRTF(theTimeBegan, theTimeEnd, 8)
  
3183:   If booWasSuccessful Then
    ' MAKE OUTPUT REPORT
3185:     Call MakePatchReport(dblFinalDistances, pMxDoc, strTimeElapsedRTF, pParamDetails)
3186:   End If
  
'  For lngFullIndex = 0 To lngFinalIndex
'    Debug.Print "Distance = " & pSortArray(lngFullIndex) & ",  " & "Node ID Value for " & _
'        Format(pSortArray(lngFullIndex), strFormatString) & " = " & _
'        pDistCollection.GetVariable(Format(pSortArray(lngFullIndex), strFormatString))
'  Next lngFullIndex
  
' Debug.Print "Calculation Time = " & (Timer - varStartTime) & " seconds..."


3197:   Set pApp = Nothing
3198:   Set pDoc = Nothing
3199:   Set frmProgress = Nothing
3200:   Set pClone = Nothing
3201:   Set pClipCollection = Nothing
3202:   Set pArray = Nothing
3203:   Set pColArray = Nothing
3204:   Set pPolyline = Nothing
3205:   Set pShape1 = Nothing
3206:   Set pShape2 = Nothing
3207:   Set pCollection = Nothing
3208:   Set pFinalCollection = Nothing
3209:   Set pIDArray = Nothing
3210:   Set pDoneCollection = Nothing
3211:   Set pSubArray = Nothing
3212:   Set pIntPolygon = Nothing
3213:   Set pRelOp = Nothing
3214:   Set pGeometryCollection = Nothing
3215:   Set pCorStrandPolygon = Nothing
3216:   Set pSubStartPoly = Nothing
3217:   Set pSubEndPoly = Nothing
3218:   Set pGridLine = Nothing
3219:   Set pGridPoly1 = Nothing
3220:   Set pGridPoly2 = Nothing
3221:   Set pCorRaster = Nothing
3222:   Set pEnv = Nothing
3223:   Set pRemoveKeyArray = Nothing
3224:   Set psbar = Nothing
3225:   Set pPro = Nothing
3226:   Set pFinalIDArray = Nothing
3227:   Set pDistCollection = Nothing
3228:   Set pFinalCollection3 = Nothing
3229:   Set theNetDictionary = Nothing
3230:   Set theNodeDictionary = Nothing
3231:   Set theNodeElement = Nothing
3232:   Set theDictionaryOfConnectors = Nothing
3233:   Set pIndexArray = Nothing
3234:   Set pConnectionResult = Nothing
3235:   Set pRouteArray = Nothing
3236:   Set pElement = Nothing
3237:   Set pPolyElement = Nothing
3238:   Set pLineSymbol = Nothing
3239:   Set pColor = Nothing
3240:   Set pInternalColor = Nothing
3241:   Set pCartoLine = Nothing
3242:   Set pSmplFill = Nothing
3243:   Set pFillSym = Nothing
3244:   Set pGContainer = Nothing
3245:   Set pPolygon = Nothing
3246:   Set pTempArray = Nothing
3247:   Set pGradStartPoint = Nothing
3248:   Set pGradEndPoint = Nothing
3249:   Set pPolyline2 = Nothing
3250:   Set pStartPoint = Nothing
3251:   Set pEndPoint = Nothing
3252:   Set pStartPoint2 = Nothing
3253:   Set pEndPoint2 = Nothing
3254:   Set pProxOp = Nothing
3255:   Set pGradFillSym = Nothing
3256:   Set pAlgColRamp = Nothing
3257:   Set pLineArray = Nothing

  Exit Sub
ErrorHandler:
  HandleError True, "ImplementKruskall " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



3265:   Set pApp = Nothing
3266:   Set pDoc = Nothing
3267:   Set frmProgress = Nothing
3268:   Set pClone = Nothing
3269:   Set pClipCollection = Nothing
3270:   Set pArray = Nothing
3271:   Set pColArray = Nothing
3272:   Set pPolyline = Nothing
3273:   Set pShape1 = Nothing
3274:   Set pShape2 = Nothing
3275:   Set pCollection = Nothing
3276:   Set pFinalCollection = Nothing
3277:   Set pIDArray = Nothing
3278:   Set pDoneCollection = Nothing
3279:   Set pSubArray = Nothing
3280:   Set pIntPolygon = Nothing
3281:   Set pRelOp = Nothing
3282:   Set pGeometryCollection = Nothing
3283:   Set pCorStrandPolygon = Nothing
3284:   Set pSubStartPoly = Nothing
3285:   Set pSubEndPoly = Nothing
3286:   Set pGridLine = Nothing
3287:   Set pGridPoly1 = Nothing
3288:   Set pGridPoly2 = Nothing
3289:   Set pCorRaster = Nothing
3290:   Set pEnv = Nothing
3291:   Set pRemoveKeyArray = Nothing
3292:   Set psbar = Nothing
3293:   Set pPro = Nothing
3294:   Set pFinalIDArray = Nothing
3295:   Set pDistCollection = Nothing
3296:   Set pFinalCollection3 = Nothing
3297:   Set theNetDictionary = Nothing
3298:   Set theNodeDictionary = Nothing
3299:   Set theNodeElement = Nothing
3300:   Set theDictionaryOfConnectors = Nothing
3301:   Set pIndexArray = Nothing
3302:   Set pConnectionResult = Nothing
3303:   Set pRouteArray = Nothing
3304:   Set pElement = Nothing
3305:   Set pPolyElement = Nothing
3306:   Set pLineSymbol = Nothing
3307:   Set pColor = Nothing
3308:   Set pInternalColor = Nothing
3309:   Set pCartoLine = Nothing
3310:   Set pSmplFill = Nothing
3311:   Set pFillSym = Nothing
3312:   Set pGContainer = Nothing
3313:   Set pPolygon = Nothing
3314:   Set pTempArray = Nothing
3315:   Set pGradStartPoint = Nothing
3316:   Set pGradEndPoint = Nothing
3317:   Set pPolyline2 = Nothing
3318:   Set pStartPoint = Nothing
3319:   Set pEndPoint = Nothing
3320:   Set pStartPoint2 = Nothing
3321:   Set pEndPoint2 = Nothing
3322:   Set pProxOp = Nothing
3323:   Set pGradFillSym = Nothing
3324:   Set pAlgColRamp = Nothing
3325:   Set pLineArray = Nothing

End Sub

Public Sub MakePatchReport(dblDistances() As Double, pMxDoc As IMxDocument, strElapsedTime As String, _
    pParamDetails As esriSystem.IVariantArray)
  On Error GoTo ErrorHandler

  Dim pApp As IApplication
  Dim pDoc As IDocument
3335:   Set pDoc = pMxDoc
3336:   Set pApp = pDoc.Parent
  
3338:   Call QuickSort.DoubleDescending(dblDistances, 0, UBound(dblDistances))
  
  Dim strReport As String
  Dim strHab1 As String
  Dim strHab2 As String
  Dim strCor As String
  Dim booUsePatches As Boolean
  Dim strPatch As String
  Dim booUseAllPatches As Boolean
  Dim strPatchField As String
  Dim strOperator As String
  Dim strValue As String
  
3351:   strHab1 = pParamDetails.Element(0)
3352:   strHab2 = pParamDetails.Element(1)
3353:   strCor = pParamDetails.Element(2)
3354:   booUsePatches = pParamDetails.Element(3)
3355:   strPatch = pParamDetails.Element(4)
3356:   booUseAllPatches = pParamDetails.Element(5)
3357:   strPatchField = pParamDetails.Element(6)
3358:   strOperator = pParamDetails.Element(7)
3359:   strValue = pParamDetails.Element(8)
  
  ' MAKE NEW TABLE
  Dim pNewField As IField
  Dim pNewFieldEdit As IFieldEdit
  Dim pNewFields As IFields
  Dim pNewFieldsEdit As IFieldsEdit
3366:   Set pNewFields = New Fields
3367:   Set pNewFieldsEdit = pNewFields
3368:   pNewFieldsEdit.FieldCount = 2
  
  ' MAKE UNIQUE ID FIELD
3371:   Set pNewField = New Field
3372:   Set pNewFieldEdit = pNewField
3373:   pNewFieldEdit.Name = "Unique_ID"
3374:   pNewFieldEdit.Type = esriFieldTypeInteger
3375:   pNewFieldEdit.Precision = 8
3376:   Set pNewFieldsEdit.Field(0) = pNewField
    
  ' MAKE LENGTH FIELD
3379:   Set pNewField = New Field
3380:   Set pNewFieldEdit = pNewField
3381:   With pNewFieldEdit
3382:     .Type = esriFieldTypeDouble
3383:     .Name = "Seg_Length"
3384:     .Precision = 14
3385:     .Scale = 8
3386:   End With
3387:   Set pNewFieldsEdit.Field(1) = pNewField
  
  ' WORKSPACE
  ' FIRST SEE IF IT HAS BEEN SAVED TO EXTENSION PROPERTIES.  THIS PROPERTY WILL BE EMPTY THE FIRST TIME THE DIALOG
  '      IS OPENED, BUT EACH TIME THEREAFTER IT WILL HAVE A VALUE.
  ' IF NOT IN EXTENSION PROPERTY, THEN CHECK ArcGIS LAST SAVE TO LOCATION
  ' IF THIS DOESN'T WORK, USE MxDoc PATH NAME.
  Dim strDirPath As String
  Dim strUserName As String
  Dim strFName As String
  
  Dim newUid As New uID
  Dim pExtConfig As IExtensionConfig
3400:   newUid.Value = "Linkages.clsExtension"
3401:   Set pExtConfig = pApp.FindExtensionByCLSID(newUid)
  Dim ext As Linkages.clsExtension
3403:   Set ext = pExtConfig
  
3405:   strDirPath = ext.ClipDirectoryPath
3406:   If Not Linkages.aml_func_mod.ExistFileDir(strDirPath) Then
3407:     strDirPath = Linkages.aml_func_mod.ReturnArcGISGeneralDir(enumLastSaveToLocation)
3408:   End If
3409:   If Not Linkages.aml_func_mod.ExistFileDir(strDirPath) Then
3410:     strDirPath = Linkages.aml_func_mod.GetFullFileString(Linkages.aml_func_mod.GetMxDocPath(pApp))
3411:     strDirPath = Linkages.aml_func_mod.ReturnDir(strDirPath)
3412:   End If

3414:   If Right(strDirPath, 1) <> "\" And Right(strDirPath, 1) <> "/" Then
3415:     strDirPath = strDirPath & "\"
3416:   End If
3417:   strFName = strDirPath & "segment_lengths.dbf"

3419:   strFName = Linkages.aml_func_mod.MakeUniqueFilename(strFName)
  Dim pTable As ITable
3421:   Set pTable = Linkages.aml_func_mod.CreatedBASETable(strFName, pNewFields)
  Dim pCursor As ICursor
  Dim pRowBuffer As IRowBuffer
3424:   Set pCursor = pTable.Insert(True)
3425:   Set pRowBuffer = pTable.CreateRowBuffer
  
3427:   strReport = "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fswiss\fcharset0 Arial;}}" & vbCrLf & _
              "{\*\generator Msftedit 5.41.15.1507;}\viewkind4\uc1\pard\qc\b\f0\fs16 Report of Patch Analysis:\b0\par" & vbCrLf & _
              "\b ---------------------------------------------------------\b0\par" & vbCrLf & _
              "\pard\b Habitat Block #1 \b0 = " & strHab1 & "\par" & vbCrLf & _
              "\b Habitat Block #2 \b0 = " & strHab2 & "\par" & vbCrLf & _
              "\b Corridor Polygon \b0 = " & strCor & "\par" & vbCrLf
3433:   If Not booUsePatches Then
3434:     strReport = strReport & "No patch layer used in analysis...\par" & vbCrLf
3435:   Else
3436:     strReport = strReport & "\b Patch Layer \b0 = " & strPatch & "\par" & vbCrLf
3437:     If booUseAllPatches Then
3438:       strReport = strReport & "  ** All patch polygons considered in analysis...\par" & vbCrLf
3439:     Else
3440:       strReport = strReport & "  ** Patch Query String:  " & strPatchField & " " & _
          Linkages.aml_func_mod.BasicTrimAvenue(strOperator, " ", " ") & " " & _
          Linkages.aml_func_mod.BasicTrimAvenue(strValue, " ", " ") & "\par" & vbCrLf
3443:     End If
3444:   End If
3445:   strReport = strReport & _
              "\pard\qc\b ---------------------------------------------------------\b0\par" & vbCrLf & _
              "\pard\b " & CStr(UBound(dblDistances) + 1) & " segments required to move from one " & _
              "habitat block to the other.\b0\par" & vbCrLf & _
              "Segment lengths listed in decreasing order:\par" & vbCrLf
  
  Dim lngIndex As Long
3452:   For lngIndex = 0 To UBound(dblDistances)
3453:     strReport = strReport & "     " & CStr(lngIndex + 1) & "] " & _
        CStr(Format(dblDistances(lngIndex), "0.0000000")) & "\par" & vbCrLf
3455:     pRowBuffer.Value(pRowBuffer.Fields.FindField("Unique_ID")) = (lngIndex + 1)
3456:     pRowBuffer.Value(pRowBuffer.Fields.FindField("Seg_Length")) = dblDistances(lngIndex)
3457:     pCursor.InsertRow pRowBuffer
3458:   Next lngIndex
3459:   strReport = strReport & "\par" & vbCrLf & _
              "\b Table of Segment Lengths saved to:\b0\par" & vbCrLf & _
              Linkages.aml_func_mod.SubstituteString(strFName, "\", "\\") & "\par" & vbCrLf & _
              "\par" & vbCrLf & _
              "\b Note: \b0  You can use the ""Create New Shapefile"" function to convert " & _
              "your graphic patch polygons and segment polylines to new polyline and polygon shapefiles.  " & _
              "Polygons and polylines produced by this analysis will be named ""Route_Graphics"" in the " & _
              "new shapefile attribute tables.\par" & vbCrLf & _
              "\pard\qc\b ---------------------------------------------------------\b0\par" & vbCrLf & strElapsedTime _
              & "\par" & vbCrLf & "}"
  
3470:   pCursor.Flush
  
  ' MAKE TABLE AND ADD TO MAP DOCUMENT
  Dim pNewStandaloneTable As IStandaloneTable
  Dim pTableWindow2 As ITableWindow2
  Dim pStandaloneTableCollection As IStandaloneTableCollection
  
3477:   Set pNewStandaloneTable = New StandaloneTable
3478:   Set pNewStandaloneTable.Table = pTable
  
3480:   Set pTableWindow2 = New TableWindow

3482:   With pTableWindow2
3483:     Set .StandaloneTable = pNewStandaloneTable
3484:     Set .Application = pApp
3485:     .TableSelectionAction = esriSelectFeatures
3486:     .ShowAliasNamesInColumnHeadings = True
3487:     .ShowSelected = False
3488:     .Show True
3489:   End With
3490:   Set pStandaloneTableCollection = pMxDoc.FocusMap
3491:   pStandaloneTableCollection.AddStandaloneTable pNewStandaloneTable
  
  
  Dim frmReportForm As New frmReport_modal
3495:   frmReportForm.txtReport.TextRTF = strReport
3496:   frmReportForm.Show vbModal
  
3498:   pMxDoc.UpdateContents
  


3502:   Set pApp = Nothing
3503:   Set pDoc = Nothing
3504:   Set pNewField = Nothing
3505:   Set pNewFieldEdit = Nothing
3506:   Set pNewFields = Nothing
3507:   Set pNewFieldsEdit = Nothing
3508:   Set newUid = Nothing
3509:   Set pExtConfig = Nothing
3510:   Set ext = Nothing
3511:   Set pTable = Nothing
3512:   Set pCursor = Nothing
3513:   Set pRowBuffer = Nothing
3514:   Set pNewStandaloneTable = Nothing
3515:   Set pTableWindow2 = Nothing
3516:   Set pStandaloneTableCollection = Nothing
3517:   Set frmReportForm = Nothing

  Exit Sub
ErrorHandler:
  HandleError True, "MakePatchReport " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


3524:   Set pApp = Nothing
3525:   Set pDoc = Nothing
3526:   Set pNewField = Nothing
3527:   Set pNewFieldEdit = Nothing
3528:   Set pNewFields = Nothing
3529:   Set pNewFieldsEdit = Nothing
3530:   Set newUid = Nothing
3531:   Set pExtConfig = Nothing
3532:   Set ext = Nothing
3533:   Set pTable = Nothing
3534:   Set pCursor = Nothing
3535:   Set pRowBuffer = Nothing
3536:   Set pNewStandaloneTable = Nothing
3537:   Set pTableWindow2 = Nothing
3538:   Set pStandaloneTableCollection = Nothing
3539:   Set frmReportForm = Nothing

End Sub

Public Function CheckForConnection(strStartNode As String, strEndNode As String, _
    theDictionaryOfConnectors As CollectionMod, theNetDictionary As CollectionMod) As esriSystem.IArray
  On Error GoTo ErrorHandler

    
'   NET DICTIONARY -------------------------------
'   DICTIONARY STRUCTURED AS:  CollectionMod Object
'                   KEY = NODE ID STRING
'               ELEMENT = CollectionMod OF POSSIBLE MOVEMENT OPTIONS:  WATCH FOR 1-WAY SITUATIONS
'                         SUCH AS START NODE CAN ONLY GO OUT, END NODE ONLY HAS INCOMING
'                                 KEY = NODE ID STRING WHERE CONNECTOR GOES TO
'                             ELEMENT = IVarArray, {CONNECTOR LINE, CONNECTOR LENGTH}

'   theDictionaryOfConnectors ---------------------------
'   DICTIONARY STRUCTURED AS:  CollectionMod Object
'                   KEY = NODE ID STRING
'               ELEMENT = IStringArray OF POSSIBLE CONNECTION NODE ID STRINGS

Dim theReturnDictionary As esriSystem.IArray

' IF START OR END NODES HAVE NEVER BEEN RECORDED IN theDictionaryOfConnectors THEN THERE CANNOT BE A PATH
3564: If (Not theDictionaryOfConnectors.HasKey(strStartNode)) Or (Not theDictionaryOfConnectors.HasKey(strEndNode)) Then
3565:   Set CheckForConnection = Nothing
  Exit Function
3567: End If

Dim theDoneDictionary As New CollectionMod
3570: theDoneDictionary.SetOrReplace True, strStartNode

Dim pPathArray As IStringArray
3573: Set pPathArray = New strArray
3574: pPathArray.Add strStartNode

Dim thePathDictionary As New CollectionMod
3577: thePathDictionary.AddObject pPathArray, strStartNode, False

Dim IsConnected As Boolean
3580: IsConnected = False

Dim theListOfPossibleConnectors As IStringArray
3583: Set theListOfPossibleConnectors = theDictionaryOfConnectors.GetObject(strStartNode)

Dim strConnector As String
Dim lngIndex As Long
3587: For lngIndex = 0 To theListOfPossibleConnectors.Count - 1
3588:   strConnector = theListOfPossibleConnectors.Element(lngIndex)
3589:   Set pPathArray = New strArray
3590:   pPathArray.Add strStartNode
3591:   pPathArray.Add strConnector
3592:   thePathDictionary.AddObject pPathArray, strConnector, True
3593: Next lngIndex

Dim pNewList As IStringArray
Dim theCheckPoint As String
Dim pFinalList As IStringArray
Dim pNewPoints As IStringArray

Dim lngNodeIndex As Long
Dim strNode As String
Dim theNodeDictionary As CollectionMod
Dim strNextNode As String
Dim pPathData As IVariantArray
Dim theCurrentList As IStringArray
Dim strNewPoint As String
Dim strSaveList As IStringArray
Dim pReturnPathData As IVariantArray
Dim anIndex As Long
Dim lngIndex2 As Long

3612: Set theReturnDictionary = Nothing

3614: Do While Not IsConnected And (theListOfPossibleConnectors.Count > 0)
3615:   Set pNewList = New strArray
3616:   For anIndex = 0 To theListOfPossibleConnectors.Count - 1
3617:     theCheckPoint = theListOfPossibleConnectors.Element(anIndex)
3618:     If theCheckPoint = strEndNode Then
3619:       IsConnected = True
3620:       Set pFinalList = thePathDictionary.GetObject(strEndNode)
3621:       Set theReturnDictionary = New esriSystem.Array
      
3623:       For lngNodeIndex = 0 To pFinalList.Count - 2
3624:         strNode = pFinalList.Element(lngNodeIndex)
3625:         Set theNodeDictionary = theNetDictionary.GetObject(strNode)
3626:         strNextNode = pFinalList.Element(lngNodeIndex + 1)
        
3628:         Set pPathData = theNodeDictionary.GetObject(strNextNode)
3629:         Set pReturnPathData = New VarArray
3630:         pReturnPathData.Add pPathData.Element(0)
3631:         pReturnPathData.Add pPathData.Element(1)
3632:         pReturnPathData.Add strNode
3633:         pReturnPathData.Add strNextNode
        
3635:         theReturnDictionary.Add pReturnPathData      ' CONTAINS (0) POLYLINE; (1) LENGTH
        
3637:       Next lngNodeIndex
      
3639:       Set CheckForConnection = theReturnDictionary
      Exit Function     ' <------------------------------------------ EXIT FUNCTION WITH A FULL ROUTE AT THIS POINT
3641:     ElseIf Not theDoneDictionary.HasKey(theCheckPoint) Then         ' DON'T WANT IT DOUBLING BACK
3642:       theDoneDictionary.SetOrReplace True, theCheckPoint
3643:       Set theCurrentList = thePathDictionary.GetObject(theCheckPoint)
3644:       If theDictionaryOfConnectors.HasKey(theCheckPoint) Then
3645:         Set pNewPoints = theDictionaryOfConnectors.GetObject(theCheckPoint)
3646:         For lngIndex = 0 To pNewPoints.Count - 1
3647:           strNewPoint = pNewPoints.Element(lngIndex)
3648:           Set strSaveList = New strArray
3649:           For lngIndex2 = 0 To theCurrentList.Count - 1
3650:             strSaveList.Add theCurrentList.Element(lngIndex2)
3651:           Next lngIndex2
3652:           strSaveList.Add strNewPoint
3653:           thePathDictionary.AddObject strSaveList, strNewPoint, True
          
3655:           pNewList.Add strNewPoint
3656:         Next lngIndex
3657:       End If
3658:     End If
3659:   Next anIndex
  
3661:   Set theListOfPossibleConnectors = pNewList
3662: Loop

3664: Set CheckForConnection = theReturnDictionary


3667:   Set theReturnDictionary = Nothing
3668:   Set theDoneDictionary = Nothing
3669:   Set pPathArray = Nothing
3670:   Set thePathDictionary = Nothing
3671:   Set theListOfPossibleConnectors = Nothing
3672:   Set pNewList = Nothing
3673:   Set pFinalList = Nothing
3674:   Set pNewPoints = Nothing
3675:   Set theNodeDictionary = Nothing
3676:   Set pPathData = Nothing
3677:   Set theCurrentList = Nothing
3678:   Set strSaveList = Nothing
3679:   Set pReturnPathData = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "CheckForConnection " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



3687:   Set theReturnDictionary = Nothing
3688:   Set theDoneDictionary = Nothing
3689:   Set pPathArray = Nothing
3690:   Set thePathDictionary = Nothing
3691:   Set theListOfPossibleConnectors = Nothing
3692:   Set pNewList = Nothing
3693:   Set pFinalList = Nothing
3694:   Set pNewPoints = Nothing
3695:   Set theNodeDictionary = Nothing
3696:   Set pPathData = Nothing
3697:   Set theCurrentList = Nothing
3698:   Set strSaveList = Nothing
3699:   Set pReturnPathData = Nothing

End Function

Public Function CheckForAcyclicity(ByRef strNodeA As String, strNodeB As String, theDictionaryOfConnectors As CollectionMod) As Boolean
  On Error GoTo ErrorHandler


' Jennessent.CheckForAcyclicity

' TREE IS ACYCLIC IF POINT A AND POINT B ARE ALREADY CONNECTED

Dim pListOfPossibleConnectors As IStringArray
3712: Set pListOfPossibleConnectors = theDictionaryOfConnectors.GetObject(strNodeA)

Dim IsAcyclic As Boolean
3715: IsAcyclic = True
Dim theCheckPoint As String

Dim theDoneDictionary As CollectionMod
3719: Set theDoneDictionary = New CollectionMod
3720: theDoneDictionary.AddVariable True, strNodeA, False

Dim pNewList As IStringArray
Dim lngIndex As Long
Dim lngCheckIndex As Long
Dim pNewPoints As IStringArray
Dim lngIndex2 As Long

3728: Do While IsAcyclic And (pListOfPossibleConnectors.Count > 0)
3729:   Set pNewList = New strArray
3730:   For lngIndex = 0 To (pListOfPossibleConnectors.Count - 1)
3731:     theCheckPoint = pListOfPossibleConnectors.Element(lngIndex)
    
3733:     If theCheckPoint = strNodeB Then
3734:       IsAcyclic = False
3735:       Exit For

3737:     ElseIf Not theDoneDictionary.HasKey(theCheckPoint) Then  ' DON'T WANT TO HAVE IT DOUBLING BACK
3738:       theDoneDictionary.SetOrReplace True, theCheckPoint
3739:       Set pNewPoints = theDictionaryOfConnectors.GetObject(theCheckPoint)
3740:       If Not pNewPoints Is Nothing Then
3741:         For lngIndex2 = 0 To pNewPoints.Count - 1
3742:           pNewList.Add pNewPoints.Element(lngIndex2)
3743:         Next lngIndex2
3744:       End If
3745:     End If
3746:   Next lngIndex

3748:   Set pListOfPossibleConnectors = pNewList
3749: Loop

3751: CheckForAcyclicity = IsAcyclic



3755:   Set pListOfPossibleConnectors = Nothing
3756:   Set theDoneDictionary = Nothing
3757:   Set pNewList = Nothing
3758:   Set pNewPoints = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "CheckForAcyclicity " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


3765:   Set pListOfPossibleConnectors = Nothing
3766:   Set theDoneDictionary = Nothing
3767:   Set pNewList = Nothing
3768:   Set pNewPoints = Nothing

End Function
Public Function ClipPolysToCorridor(ByVal pArray As IArray, pCorPolygon As IPolygon, pApp As IApplication) As Collection
  On Error GoTo ErrorHandler
  
  Dim pMxDoc As IMxDocument
3775:   Set pMxDoc = pApp.Document
      
  Dim pResponseCollection As New Collection
  Dim StartIntersects As Boolean
  Dim EndIntersects As Boolean
  Dim pReturnArray As IArray
  Dim pWorked As Boolean
  Dim strMessage As String
  
  Dim pRelOp As IRelationalOperator
  Dim pTopOp As ITopologicalOperator
  Dim pTopOp2 As ITopologicalOperator
  Dim pTopOp4 As ITopologicalOperator2
  Dim pProxOp As IProximityOperator
  Dim pSpatialRef As ISpatialReference
  
  Dim pTopoOpSimp As ITopologicalOperator
  
  Dim anIndex As Long
  Dim pPolygon As IPolygon
  Dim pStartPolygon As IPolygon
  Dim pEndPolygon As IPolygon
  Dim pIntPolygon As IPolygon
  Dim pCheckCountPoly As IPolygon4
  Dim pGeometryBag As IGeometryCollection
  Dim pSubPoly As IPolygon4
  Dim anIndex2 As Long
  
3803:   Set pReturnArray = New esriSystem.Array
3804:   Set pRelOp = pCorPolygon
3805:   Set pTopOp = pCorPolygon
  
  Dim pCombinedArray As esriSystem.IArray
  Dim pCombinedArrayInfo As esriSystem.IVariantArray
  Dim booAllIntersects As Boolean
    
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
3813:   Set psbar = pApp.StatusBar
  Dim pPro As IStepProgressor
3815:   Set pPro = psbar.ProgressBar
  Dim lngCounter As Long
3817:   lngCounter = 0
  Dim lngTotalCount As Long
3819:   lngTotalCount = ((pArray.Count - 2) * 3)
  Dim strTotalCount As String
3821:   strTotalCount = "approximately " & CStr(lngTotalCount)
3822:   pPro.position = 1
3823:   psbar.ShowProgressBar "Clipping patch polygons to corridor:  Step 1 of " & strTotalCount & "...", 1, _
          lngTotalCount, 1, True
  
  Dim pRelOp3 As IRelationalOperator
3827:   If pArray.Count = 1 Then
3828:     strMessage = strMessage + "Origin polygon not separate from Destination polygon!  No corridor necessary!..." + vbCrLf
3829:   Else
  
      ' EXTRACT START AND END POLYGONS
3832:     Set pStartPolygon = pArray.Element(0)
3833:     pArray.Remove (0)
3834:     Set pEndPolygon = pArray.Element(pArray.Count - 1)
3835:     pArray.Remove (pArray.Count - 1)
    
    ' FIRST SEE IF START AND END POLYGONS INTERSECT, OR IF THEY BOTH INTERSECT CORRIDOR
    
3839:     pPro.Message = "Confirming Habitat Block 1 intersects Corridor..."
'    Set pStartPolygon = pArray.Element(0)
3841:     StartIntersects = Not pRelOp.Disjoint(pStartPolygon)
3842:     If Not StartIntersects Then strMessage = strMessage + "Origin polygon does not intersect corridor..." + vbCrLf
    
3844:     pPro.Message = "Confirming Habitat Block 2 intersects Corridor..."
'    Set pEndPolygon = pArray.Element(pArray.Count - 1)
3846:     EndIntersects = Not pRelOp.Disjoint(pEndPolygon)
3847:     If Not EndIntersects Then strMessage = strMessage + "Destination polygon does not intersect corridor..." + vbCrLf

3849:     Set pRelOp3 = pStartPolygon
3850:     If Not pRelOp3.Disjoint(pEndPolygon) Then
3851:         strMessage = strMessage + "Origin polygon intersects with Destination polygon!  No corridor necessary!..." + vbCrLf
3852:     End If
3853:   End If
  
  Dim pRelOp2 As IRelationalOperator
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollectionCurves As ISegmentCollection
  Dim pSegmentCurve As ISegment
  Dim pGeometryTypeA As esriGeometryType
  Dim pNewPoints As IPointCollection
  Dim pNewPolygon As IPointCollection
  Dim pRecreatedPolygon As IPolygon
  Dim pRevisedArray As esriSystem.IArray
  
3865:   If Not (StartIntersects And EndIntersects) Then
3866:     pWorked = False
3867:     Set pReturnArray = New esriSystem.Array
3868:     pReturnArray.Add pStartPolygon
3869:     pReturnArray.Add pEndPolygon
    
3871:   Else
    
3873:     pWorked = False
3874:     strMessage = ""
    
    ' CLIP START AND END POLYGONS TO CORRIDOR
3877:     Set pStartPolygon = pTopOp.Intersect(pStartPolygon, esriGeometry2Dimension)
3878:     Set pEndPolygon = pTopOp.Intersect(pEndPolygon, esriGeometry2Dimension)
'    Module4.DeleteGraphicsByName Document, "test_order"
'    Module4.Graphic_MakeFromGeometry Document, pCorPolygon, "test_order"
'    Module4.Graphic_MakeFromGeometry Document, pStartPolygon, "test_order"
'    Module4.Graphic_MakeFromGeometry Document, pEndPolygon, "test_order"
    
3884:     If pArray.Count = 0 Then
'      strMessage = "Array contained no polygons!"
'      StartIntersects = False
'      EndIntersects = False
3888:       Set pReturnArray = New esriSystem.Array
3889:       pReturnArray.Add pStartPolygon
3890:       pReturnArray.Add pEndPolygon
3891:       pWorked = True
3892:     Else
      ' CONVERT ANY ODD SHAPES TO NORMAL POLYGONS BEFORE DOING ANYTHING ELSE
      ' FORCE OBJECT TO BE POLYGON IF THE FIRST SEGMENT IS A CURVE
      
3896:       Set pRevisedArray = New esriSystem.Array
      
   '   MsgBox "Before:  " & pArray.Count
      
3900:       For anIndex = 0 To pArray.Count - 1
        
3902:         lngCounter = lngCounter + 1
3903:         pPro.Message = "Clipping patch polygons to corridor:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
3904:         psbar.StepProgressBar
        
3906:         Set pPolygon = pArray.Element(anIndex)
        
3908:         Set pSegmentCollectionCurves = pPolygon
3909:         Set pSegmentCurve = pSegmentCollectionCurves.Segment(0)
3910:         pGeometryTypeA = pSegmentCurve.GeometryType
        
3912:         booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
          (pGeometryTypeA = esriGeometryCircularArc) Or _
          (pGeometryTypeA = esriGeometryEllipticArc)
        
3916:         If booWorkingWithCurves Then
3917:           Set pNewPoints = GridFunctions.EllipticArcToPolygon2(pSegmentCollectionCurves, 100)
3918:           Set pNewPolygon = New Polygon
3919:           pNewPolygon.SetPointCollection pNewPoints
3920:           Set pRecreatedPolygon = pNewPolygon
3921:           Set pTopOp4 = pRecreatedPolygon
3922:           pTopOp4.IsKnownSimple = False
3923:           pTopOp4.Simplify
3924:           pRevisedArray.Add pRecreatedPolygon
  '        ThisDocument.Graphic_MakeFromGeometry pMxDoc, pNewPolygon, "test_order"
3926:         Else
3927:           pRevisedArray.Add pPolygon
  '        ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
3929:         End If
3930:       Next anIndex
3931:       Set pArray = pRevisedArray
      ' DONE CONVERTING CIRCLES, ELLIPTIC ARCS AND BEZIER CURVES TO POLYGONS
          
'      Linkages.MyGeneralOperations.DeleteGraphicsByName pMxDoc, "test_order"
'      For anIndex = 0 To pArray.Count - 1
'        Set pPolygon = pArray.Element(anIndex)
'        Linkages.MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
'        MsgBox "Adding Array Index " & anIndex
'      Next anIndex
  
      ' INTERSECT POLYGONS WITH CORRIDOR, AND SEPARATE SUB-POLYGONS
3942:       For anIndex = 0 To pArray.Count - 1
3943:         lngCounter = lngCounter + 1
3944:         pPro.Message = "Checking for Multiple Strands:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
3945:         psbar.StepProgressBar
        
3947:         Set pPolygon = pArray.Element(anIndex)
3948:         Set pSpatialRef = pPolygon.SpatialReference
3949:         Set pIntPolygon = pTopOp.Intersect(pPolygon, esriGeometry2Dimension)
3950:         Set pTopoOpSimp = pIntPolygon
        
3952:         pTopoOpSimp.Simplify
        
3954:         Set pCheckCountPoly = pIntPolygon
        ' SPLIT INTO MULTIPLE POLYGONS IF NECESSARY; ONLY IF NOT FIRST OR LAST POLYGON
        
'        If anIndex = 0 Or anIndex = pArray.Count - 1 Then   ' START AND END POLYGONS ARE REINSERTED BACK INTO ARRAY BELOW
'          pReturnArray.Add pIntPolygon
'        Else
3960:         Set pGeometryBag = pCheckCountPoly.ConnectedComponentBag
         
3962:         For anIndex2 = 0 To pGeometryBag.GeometryCount - 1
3963:           Set pSubPoly = pGeometryBag.Geometry(anIndex2)
           
3965:           If Not pSubPoly.IsEmpty Then
3966:             Set pSubPoly.SpatialReference = pSpatialRef
3967:             pReturnArray.Add pSubPoly
3968:           End If
3969:         Next anIndex2
'        End If
3971:       Next anIndex
3972:       pWorked = True
      
'      MsgBox "From ClipPolysToCorridor:  Just before sending to CombineAdjacentPolygons..." & vbCrLf & _
          "pReturnArray.Count = " & CStr(pReturnArray.Count)
      
      ' COMBINE ADJACENT POLYGONS
'      MsgBox "Before Combine: " & pReturnArray.Count + 2
3979:       Set pCombinedArrayInfo = CombineAdjacentPolygons(pReturnArray, _
        pStartPolygon, pEndPolygon, pPro, psbar, lngCounter, strTotalCount)
3981:       booAllIntersects = pCombinedArrayInfo.Element(0)
3982:       Set pCombinedArray = pCombinedArrayInfo.Element(1)
'      MsgBox "After Combine: " & pCombinedArray.Count
      
3985:       Set pReturnArray = pCombinedArray
      
'      MsgBox "From ClipPolysToCorridor:  Just after sending to CombineAdjacentPolygons..." & vbCrLf & _
'          "pReturnArray.Count = " & CStr(pReturnArray.Count)
          
3990:       If booAllIntersects Then
3991:         strMessage = strMessage + "Continuous strand of patch connecting habitat blocks!  No corridor necessary!..." + vbCrLf
3992:       End If
3993:     End If   ' END CHECKING IF REMAINING ARRAY CONTAINS ANY POLYGONS
3994:   End If  ' END CHECKING IF BOTH START AND END POLYGONS INTERSECT WITH CORRIDOR

3996:   pResponseCollection.Add pWorked
3997:   pResponseCollection.Add strMessage
3998:   pResponseCollection.Add StartIntersects
3999:   pResponseCollection.Add EndIntersects
4000:   pResponseCollection.Add pReturnArray
  
4002:   pPro.position = 1
4003:   psbar.HideProgressBar
  
4005:   Set ClipPolysToCorridor = pResponseCollection



4009:   Set pMxDoc = Nothing
4010:   Set pResponseCollection = Nothing
4011:   Set pReturnArray = Nothing
4012:   Set pRelOp = Nothing
4013:   Set pTopOp = Nothing
4014:   Set pTopOp2 = Nothing
4015:   Set pTopOp4 = Nothing
4016:   Set pProxOp = Nothing
4017:   Set pSpatialRef = Nothing
4018:   Set pTopoOpSimp = Nothing
4019:   Set pPolygon = Nothing
4020:   Set pStartPolygon = Nothing
4021:   Set pEndPolygon = Nothing
4022:   Set pIntPolygon = Nothing
4023:   Set pCheckCountPoly = Nothing
4024:   Set pGeometryBag = Nothing
4025:   Set pSubPoly = Nothing
4026:   Set pCombinedArray = Nothing
4027:   Set pCombinedArrayInfo = Nothing
4028:   Set psbar = Nothing
4029:   Set pPro = Nothing
4030:   Set pRelOp3 = Nothing
4031:   Set pRelOp2 = Nothing
4032:   Set pSegmentCollectionCurves = Nothing
4033:   Set pSegmentCurve = Nothing
4034:   Set pNewPoints = Nothing
4035:   Set pNewPolygon = Nothing
4036:   Set pRecreatedPolygon = Nothing
4037:   Set pRevisedArray = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "ClipPolysToCorridor " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



4045:   Set pMxDoc = Nothing
4046:   Set pResponseCollection = Nothing
4047:   Set pReturnArray = Nothing
4048:   Set pRelOp = Nothing
4049:   Set pTopOp = Nothing
4050:   Set pTopOp2 = Nothing
4051:   Set pTopOp4 = Nothing
4052:   Set pProxOp = Nothing
4053:   Set pSpatialRef = Nothing
4054:   Set pTopoOpSimp = Nothing
4055:   Set pPolygon = Nothing
4056:   Set pStartPolygon = Nothing
4057:   Set pEndPolygon = Nothing
4058:   Set pIntPolygon = Nothing
4059:   Set pCheckCountPoly = Nothing
4060:   Set pGeometryBag = Nothing
4061:   Set pSubPoly = Nothing
4062:   Set pCombinedArray = Nothing
4063:   Set pCombinedArrayInfo = Nothing
4064:   Set psbar = Nothing
4065:   Set pPro = Nothing
4066:   Set pRelOp3 = Nothing
4067:   Set pRelOp2 = Nothing
4068:   Set pSegmentCollectionCurves = Nothing
4069:   Set pSegmentCurve = Nothing
4070:   Set pNewPoints = Nothing
4071:   Set pNewPolygon = Nothing
4072:   Set pRecreatedPolygon = Nothing
4073:   Set pRevisedArray = Nothing

End Function

Public Function CombineAdjacentPolygons( _
    pArray As esriSystem.IArray, _
    pStartPolygon As IPolygon, _
    pEndPolygon As IPolygon, _
    pPro As IStepProgressor, _
    psbar As IStatusBar, _
    lngCounter As Long, _
    strTotalCount As String) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler


  Dim pNewArray As esriSystem.IArray
  Dim lngIndex As Long
  Dim pPolygon As IPolygon
  Dim pRelOp As IRelationalOperator
  Dim pPolygon2 As IPolygon
  Dim pTopoOp As ITopologicalOperator
  Dim booFoundAdjacent As Boolean
  
4096:   Set pNewArray = New esriSystem.Array
  
4098:   Do Until pArray.Count = 0
    
4100:     lngCounter = lngCounter + 1
4101:     pPro.Message = "Checking for Multiple Strands:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
4102:     psbar.StepProgressBar
  
    ' PULL OUT FIRST POLYGON IN LIST
4105:     Set pPolygon = pArray.Element(0)
4106:     pArray.Remove (0)
4107:     Set pRelOp = pPolygon
4108:     Set pTopoOp = pPolygon
4109:     booFoundAdjacent = True
    
    ' COMPARE FIRST POLYGON WITH EACH OTHER POLYGON IN LIST.
    ' IF ONE IS FOUND TO BE ADJACENT, UNION THEM AND START OVER
4113:     Do Until booFoundAdjacent = False
4114:       booFoundAdjacent = False
      
      ' ONLY GO THROUGH REST OF LIST IF THERE ACTUALLY IS MORE TO THE LIST
4117:       If pArray.Count > 0 Then
4118:         For lngIndex = 0 To pArray.Count - 1
4119:           Set pPolygon2 = pArray.Element(lngIndex)
          ' IF SECOND POLYGON INTERSECTS, UNION IT AND START LIST OVER
4121:           If Not pRelOp.Disjoint(pPolygon2) Then
4122:             Set pPolygon = pTopoOp.Union(pPolygon2)
4123:             Set pRelOp = pPolygon
4124:             Set pTopoOp = pPolygon
4125:             booFoundAdjacent = True
4126:             pArray.Remove (lngIndex)
4127:             Exit For
4128:           End If
4129:         Next lngIndex
4130:       End If
4131:     Loop
4132:     pNewArray.Add pPolygon
4133:   Loop
  
  ' GET START POLYGON AND COMBINE ANY INTERSECTING POLYGONS
  Dim pUsedIndexArray As esriSystem.ILongArray
4137:   Set pUsedIndexArray = New esriSystem.LongArray
4138:   If pNewArray.Count > 0 Then
4139:     Set pRelOp = pStartPolygon
4140:     Set pTopoOp = pStartPolygon
4141:     For lngIndex = 0 To pNewArray.Count - 1
4142:       Set pPolygon2 = pNewArray.Element(lngIndex)
4143:       If Not pRelOp.Disjoint(pPolygon2) Then
4144:         pUsedIndexArray.Add (lngIndex)
4145:         Set pStartPolygon = pTopoOp.Union(pPolygon2)
4146:         Set pRelOp = pStartPolygon
4147:         Set pTopoOp = pStartPolygon
4148:       End If
4149:     Next lngIndex
4150:   End If
  
  ' GET END POLYGON AND COMBINE ANY INTERSECTING POLYGONS
4153:   If pNewArray.Count > 0 Then
4154:     Set pRelOp = pEndPolygon
4155:     Set pTopoOp = pEndPolygon
4156:     For lngIndex = 0 To pNewArray.Count - 1
4157:       Set pPolygon2 = pNewArray.Element(lngIndex)
4158:       If Not pRelOp.Disjoint(pPolygon2) Then
4159:         pUsedIndexArray.Add (lngIndex)
4160:         Set pEndPolygon = pTopoOp.Union(pPolygon2)
4161:         Set pRelOp = pEndPolygon
4162:         Set pTopoOp = pEndPolygon
4163:       End If
4164:     Next lngIndex
4165:   End If
  
  ' CHECK IF START AND END POLYGONS INTERSECT EACH OTHER NOW
  Dim booIntersects As Boolean
4169:   Set pRelOp = pStartPolygon
4170:   booIntersects = Not pRelOp.Disjoint(pEndPolygon)
  
  ' INSERT START AND END POLYGONS
  Dim pFinalArray As esriSystem.IArray
4174:   Set pFinalArray = New esriSystem.Array
4175:   pFinalArray.Add pStartPolygon
  Dim lngCheckIndex As Long
  Dim booSkipIndex As Boolean
4178:   For lngIndex = 0 To pNewArray.Count - 1
4179:     booSkipIndex = False
4180:     If pUsedIndexArray.Count > 0 Then
4181:       For lngCheckIndex = 0 To pUsedIndexArray.Count - 1
4182:         If pUsedIndexArray.Element(lngCheckIndex) = lngIndex Then
4183:           booSkipIndex = True
4184:           Exit For
4185:         End If
4186:       Next lngCheckIndex
4187:     End If
4188:     If Not booSkipIndex Then pFinalArray.Add pNewArray.Element(lngIndex)
4189:   Next lngIndex
4190:   pFinalArray.Add pEndPolygon
  
  Dim pReturnArray As esriSystem.IVariantArray
4193:   Set pReturnArray = New esriSystem.VarArray
4194:   pReturnArray.Add booIntersects
4195:   pReturnArray.Add pFinalArray
  
4197:   Set CombineAdjacentPolygons = pReturnArray
      

4200:   Set pNewArray = Nothing
4201:   Set pPolygon = Nothing
4202:   Set pRelOp = Nothing
4203:   Set pPolygon2 = Nothing
4204:   Set pTopoOp = Nothing
4205:   Set pUsedIndexArray = Nothing
4206:   Set pFinalArray = Nothing
4207:   Set pReturnArray = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "CombineAdjacentPolygons " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


4214:   Set pNewArray = Nothing
4215:   Set pPolygon = Nothing
4216:   Set pRelOp = Nothing
4217:   Set pPolygon2 = Nothing
4218:   Set pTopoOp = Nothing
4219:   Set pUsedIndexArray = Nothing
4220:   Set pFinalArray = Nothing
4221:   Set pReturnArray = Nothing

End Function



Public Function ClipPolysToCorridor_ORIG(ByVal pArray As IArray, pCorPolygon As IPolygon, pApp As IApplication) As Collection
  On Error GoTo ErrorHandler


'      Dim pMxDoc As IMxDocument
'      Set pMxDoc = ThisDocument
      
      
  Dim pResponseCollection As New Collection
  Dim StartIntersects As Boolean
  Dim EndIntersects As Boolean
  Dim pReturnArray As IArray
  Dim pWorked As Boolean
  Dim strMessage As String
  
  Dim pRelOp As IRelationalOperator
  Dim pTopOp As ITopologicalOperator
  Dim pTopOp2 As ITopologicalOperator
  Dim pTopOp4 As ITopologicalOperator2
  Dim pProxOp As IProximityOperator
  Dim pSpatialRef As ISpatialReference
  
  Dim anIndex As Long
  Dim pPolygon As IPolygon
  Dim pStartPolygon As IPolygon
  Dim pEndPolygon As IPolygon
  Dim pIntPolygon As IPolygon
  Dim pCheckCountPoly As IPolygon4
  Dim pGeometryBag As IGeometryCollection
  Dim pSubPoly As IPolygon4
  Dim anIndex2 As Long
  
4259:   Set pReturnArray = New esriSystem.Array
4260:   Set pRelOp = pCorPolygon
4261:   Set pTopOp = pCorPolygon
  
  Dim pRelOp2 As IRelationalOperator
  
4265:   pWorked = False
4266:   strMessage = ""
  
    ' PROGRESS BAR STUFF
  Dim psbar As IStatusBar
  Dim pPro As IStepProgressor
  Dim lngCounter As Long
  Dim lngTotalCount As Long
  Dim strTotalCount As String
  
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollectionCurves As ISegmentCollection
  Dim pSegmentCurve As ISegment
  Dim pGeometryTypeA As esriGeometryType
  Dim pNewPoints As IPointCollection
  Dim pNewPolygon As IPointCollection
  Dim pRecreatedPolygon As IPolygon
  Dim pRevisedArray As esriSystem.IArray
  
  Dim booFoundAdjacent As Boolean
  Dim pCheckAdjacentArray As esriSystem.IArray
  Dim pCombineArray As esriSystem.ILongArray
  Dim pNewArray As esriSystem.IArray
  Dim pDoneIndicesArray As esriSystem.ILongArray
  Dim pCombinePolygon As IPolygon
  Dim booIncludesEnd As Boolean
  Dim lngCombineIndex As Long
  Dim booAlreadyBeenCombined As Boolean
  Dim lngCheckDoneIndex As Long
  Dim pEndCombinedPolygon As IPolygon
  Dim pRelOp3 As IRelationalOperator
    
4297:   If pArray.Count = 0 Then
4298:     strMessage = "Array contained no polygons!"
4299:     StartIntersects = False
4300:     EndIntersects = False
4301:   Else
    ' CONVERT ANY ODD SHAPES TO NORMAL POLYGONS BEFORE DOING ANYTHING ELSE
    ' FORCE OBJECT TO BE POLYGON IF THE FIRST SEGMENT IS A CURVE
    
    
4306:     Set psbar = pApp.StatusBar
4307:     Set pPro = psbar.ProgressBar
4308:     lngCounter = 0
4309:     lngTotalCount = (pArray.Count) ^ 2 + (pArray.Count * 2)
4310:     strTotalCount = CStr(lngTotalCount)
4311:     pPro.position = 1
4312:     psbar.ShowProgressBar "Clipping patch polygons to corridor:  Step 1 of " & strTotalCount & "...", 1, _
            lngTotalCount, 1, True
4314:     Set pRevisedArray = New esriSystem.Array
    
4316:     For anIndex = 0 To pArray.Count - 1
      
4318:       lngCounter = lngCounter + 1
4319:       pPro.Message = "Clipping patch polygons to corridor:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
4320:       psbar.StepProgressBar
      
4322:       Set pPolygon = pArray.Element(anIndex)
      
4324:       Set pSegmentCollectionCurves = pPolygon
4325:       Set pSegmentCurve = pSegmentCollectionCurves.Segment(0)
4326:       pGeometryTypeA = pSegmentCurve.GeometryType
      
4328:       booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
        (pGeometryTypeA = esriGeometryCircularArc) Or _
        (pGeometryTypeA = esriGeometryEllipticArc)
      
4332:       If booWorkingWithCurves Then
4333:         Set pNewPoints = GridFunctions.EllipticArcToPolygon2(pSegmentCollectionCurves, 100)
4334:         Set pNewPolygon = New Polygon
4335:         pNewPolygon.SetPointCollection pNewPoints
4336:         Set pRecreatedPolygon = pNewPolygon
4337:         Set pTopOp4 = pRecreatedPolygon
4338:         pTopOp4.IsKnownSimple = False
4339:         pTopOp4.Simplify
4340:         pRevisedArray.Add pRecreatedPolygon
'        ThisDocument.Graphic_MakeFromGeometry pMxDoc, pNewPolygon, "test_order"
4342:       Else
4343:         pRevisedArray.Add pPolygon
'        ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
4345:       End If
4346:     Next anIndex
4347:     Set pArray = pRevisedArray
    ' DONE CONVERTING CIRCLES, ELLIPTIC ARCS AND BEZIER CURVES TO POLYGONS
        
    ' NEXT, COMBINE ADJACENT POLYGONS
    
4352:     Set pCheckAdjacentArray = New esriSystem.Array
4353:     Set pEndCombinedPolygon = pArray.Element(pArray.Count - 1)
    
'    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pEndCombinedPolygon, "test_order"
    
4357:     booFoundAdjacent = False
    
    
'    ThisDocument.DeleteGraphicsByName pMxDoc, "test_order"
'    For anIndex = 0 To pArray.Count - 1
'      Set pPolygon = pArray.Element(anIndex)
'      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
'    Next anIndex
    
4366:     For anIndex = 0 To pArray.Count - 1
4367:       Set pCombineArray = New LongArray
'      Set pProxOp = pArray.Element(anIndex)
4369:       Set pRelOp2 = pArray.Element(anIndex)
4370:       For anIndex2 = 0 To pArray.Count - 1
            
4372:         lngCounter = lngCounter + 1
4373:         pPro.Message = "Clipping patch polygons to corridor:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
4374:         psbar.StepProgressBar
        
4376:         Set pPolygon = pArray.Element(anIndex2)
4377:         If Not pRelOp2.Disjoint(pPolygon) Then
'        If pProxOp.ReturnDistance(pPolygon) = 0 Then
4379:           pCombineArray.Add (anIndex2)    ' BUILD LIST OF ALL POLYGON INDICES THAT INTERSECT CURRENT POLYGON
4380:           booFoundAdjacent = True
4381:         End If
4382:       Next anIndex2
4383:       pCheckAdjacentArray.Add pCombineArray
4384:     Next anIndex
    
'    ThisDocument.DeleteGraphicsByName pMxDoc, "test_order"
'    For anIndex = 0 To pArray.Count - 1
'      Set pPolygon = pArray.Element(anIndex)
'      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
'    Next anIndex

4392:     booFoundAdjacent = False
'    MyGeneralOperations.DeleteGraphicsByName pMxDoc, "test_order"
4394:     For anIndex = 0 To pArray.Count - 1
4395:       Set pCombineArray = pCheckAdjacentArray.Element(anIndex)
4396:       If pCombineArray.Count > 1 Then
4397:         booFoundAdjacent = True
4398:         Exit For
4399:       End If
4400:     Next anIndex
    
4402:     If booFoundAdjacent Then
4403:       Set pNewArray = New esriSystem.Array
4404:       Set pDoneIndicesArray = New LongArray
4405:       For anIndex = 0 To pCheckAdjacentArray.Count - 1
        
4407:         Set pCombineArray = pCheckAdjacentArray.Element(anIndex)
        
        ' CHECK IF THIS INDEX HAS ALREADY BEEN DONE
4410:         booAlreadyBeenCombined = False
4411:         If pDoneIndicesArray.Count > 0 Then
4412:           For lngCheckDoneIndex = 0 To pDoneIndicesArray.Count - 1
4413:             If pDoneIndicesArray.Element(lngCheckDoneIndex) = anIndex Then
4414:               booAlreadyBeenCombined = True
4415:               Exit For
4416:             End If
4417:           Next lngCheckDoneIndex
4418:         End If
        
4420:         If Not booAlreadyBeenCombined Then
4421:           lngCombineIndex = pCombineArray.Element(0)
4422:           Set pCombinePolygon = pArray.Element(lngCombineIndex)
          
4424:           booIncludesEnd = (anIndex = pCheckAdjacentArray.Count - 1)
          
 '         Set pCombineArray = pCheckAdjacentArray.Element(anIndex)
4427:           If pCombineArray.Count > 1 Then
            
4429:             booIncludesEnd = (pCombineArray.Element(pCombineArray.Count - 1) = (pArray.Count - 1))
            
4431:             Set pTopOp2 = pCombinePolygon
4432:             For anIndex2 = 1 To pCombineArray.Count - 1
4433:               lngCombineIndex = pCombineArray.Element(anIndex2)
4434:               Set pCombinePolygon = pTopOp2.Union(pArray.Element(lngCombineIndex))
4435:               pDoneIndicesArray.Add (lngCombineIndex)
4436:             Next anIndex2
'            pDoneIndicesArray.Add (anIndex)
4438:             If booIncludesEnd Then
4439:               Set pEndCombinedPolygon = pCombinePolygon
4440:             End If
4441:           End If
4442:           If Not booIncludesEnd Then
            
4444:             Set pTopOp2 = pCombinePolygon
4445:             pTopOp2.Simplify
4446:             pNewArray.Add pCombinePolygon
'            ThisDocument.Graphic_MakeFromGeometry pMxDoc, pCombinePolygon, "test_order"
4448:           End If
4449:           pDoneIndicesArray.Add (anIndex)
4450:         End If
4451:       Next anIndex
      
4453:       Set pTopOp2 = pEndCombinedPolygon
4454:       pTopOp2.Simplify
      
4456:       pNewArray.Add pEndCombinedPolygon
      
4458:       Set pArray = pNewArray
4459:     End If
    
'    ThisDocument.DeleteGraphicsByName pMxDoc, "test_order"
'    For anIndex = 0 To pArray.Count - 1
'      Set pPolygon = pArray.Element(anIndex)
'      ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPolygon, "test_order"
'    Next anIndex

    ' DONE COMBINING ADJACENT POLYGONS ------------------------------
    
4469:     pPro.Message = "Confirming Habitat Block 1 intersects Corridor..."
4470:     Set pStartPolygon = pArray.Element(0)
4471:     StartIntersects = Not pRelOp.Disjoint(pStartPolygon)
4472:     If Not StartIntersects Then strMessage = strMessage + "Origin polygon does not intersect corridor..." + vbCrLf
    
4474:     pPro.Message = "Confirming Habitat Block 2 intersects Corridor..."
4475:     Set pEndPolygon = pArray.Element(pArray.Count - 1)
4476:     EndIntersects = Not pRelOp.Disjoint(pEndPolygon)
4477:     If Not EndIntersects Then strMessage = strMessage + "Destination polygon does not intersect corridor..." + vbCrLf
    
4479:     If pArray.Count = 1 Then
4480:       strMessage = strMessage + "Origin polygon not separate from Destination polygon!  No corridor necessary!..." + vbCrLf
4481:     Else
4482:       Set pRelOp3 = pStartPolygon
4483:       If Not pRelOp3.Disjoint(pEndPolygon) Then
4484:           strMessage = strMessage + "Origin polygon intersects with Destination polygon!  No corridor necessary!..." + vbCrLf
4485:       End If
4486:     End If
    
4488:     If StartIntersects And EndIntersects Then
4489:       For anIndex = 0 To pArray.Count - 1
        
4491:         lngCounter = lngCounter + 1
4492:         pPro.Message = "Checking for Multiple Strands:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
4493:         psbar.StepProgressBar
        
4495:         Set pPolygon = pArray.Element(anIndex)
4496:         Set pSpatialRef = pPolygon.SpatialReference
4497:         Set pIntPolygon = pTopOp.Intersect(pPolygon, esriGeometry2Dimension)
        
4499:         Set pCheckCountPoly = pIntPolygon
        ' SPLIT INTO MULTIPLE POLYGONS IF NECESSARY; ONLY IF NOT FIRST OR LAST POLYGON
 '       Set pTopOp2 = pPolygon
        
4503:         If anIndex = 0 Or anIndex = pArray.Count - 1 Then
4504:           pReturnArray.Add pIntPolygon
4505:         Else
4506:          Set pGeometryBag = pCheckCountPoly.ConnectedComponentBag
4507:          For anIndex2 = 0 To pGeometryBag.GeometryCount - 1
4508:            Set pSubPoly = pGeometryBag.Geometry(anIndex2)
        
4510:            If Not pSubPoly.IsEmpty Then
4511:              Set pSubPoly.SpatialReference = pSpatialRef
4512:              pReturnArray.Add pSubPoly
4513:            End If
4514:          Next anIndex2
4515:         End If
4516:       Next anIndex
4517:       pWorked = True
4518:     End If
4519:   End If
  
4521:   pResponseCollection.Add pWorked
4522:   pResponseCollection.Add strMessage
4523:   pResponseCollection.Add StartIntersects
4524:   pResponseCollection.Add EndIntersects
4525:   pResponseCollection.Add pReturnArray
  
4527:   pPro.position = 1
4528:   psbar.HideProgressBar
  
4530:   Set ClipPolysToCorridor_ORIG = pResponseCollection


4533:   Set pResponseCollection = Nothing
4534:   Set pReturnArray = Nothing
4535:   Set pRelOp = Nothing
4536:   Set pTopOp = Nothing
4537:   Set pTopOp2 = Nothing
4538:   Set pTopOp4 = Nothing
4539:   Set pProxOp = Nothing
4540:   Set pSpatialRef = Nothing
4541:   Set pPolygon = Nothing
4542:   Set pStartPolygon = Nothing
4543:   Set pEndPolygon = Nothing
4544:   Set pIntPolygon = Nothing
4545:   Set pCheckCountPoly = Nothing
4546:   Set pGeometryBag = Nothing
4547:   Set pSubPoly = Nothing
4548:   Set pRelOp2 = Nothing
4549:   Set psbar = Nothing
4550:   Set pPro = Nothing
4551:   Set pSegmentCollectionCurves = Nothing
4552:   Set pSegmentCurve = Nothing
4553:   Set pNewPoints = Nothing
4554:   Set pNewPolygon = Nothing
4555:   Set pRecreatedPolygon = Nothing
4556:   Set pRevisedArray = Nothing
4557:   Set pCheckAdjacentArray = Nothing
4558:   Set pCombineArray = Nothing
4559:   Set pNewArray = Nothing
4560:   Set pDoneIndicesArray = Nothing
4561:   Set pCombinePolygon = Nothing
4562:   Set pEndCombinedPolygon = Nothing
4563:   Set pRelOp3 = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "ClipPolysToCorridor_ORIG " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



4571:   Set pResponseCollection = Nothing
4572:   Set pReturnArray = Nothing
4573:   Set pRelOp = Nothing
4574:   Set pTopOp = Nothing
4575:   Set pTopOp2 = Nothing
4576:   Set pTopOp4 = Nothing
4577:   Set pProxOp = Nothing
4578:   Set pSpatialRef = Nothing
4579:   Set pPolygon = Nothing
4580:   Set pStartPolygon = Nothing
4581:   Set pEndPolygon = Nothing
4582:   Set pIntPolygon = Nothing
4583:   Set pCheckCountPoly = Nothing
4584:   Set pGeometryBag = Nothing
4585:   Set pSubPoly = Nothing
4586:   Set pRelOp2 = Nothing
4587:   Set psbar = Nothing
4588:   Set pPro = Nothing
4589:   Set pSegmentCollectionCurves = Nothing
4590:   Set pSegmentCurve = Nothing
4591:   Set pNewPoints = Nothing
4592:   Set pNewPolygon = Nothing
4593:   Set pRecreatedPolygon = Nothing
4594:   Set pRevisedArray = Nothing
4595:   Set pCheckAdjacentArray = Nothing
4596:   Set pCombineArray = Nothing
4597:   Set pNewArray = Nothing
4598:   Set pDoneIndicesArray = Nothing
4599:   Set pCombinePolygon = Nothing
4600:   Set pEndCombinedPolygon = Nothing
4601:   Set pRelOp3 = Nothing

End Function

Public Function CheckIntersectBoundary(pPolyline As IPolyline, pCorPolygon As IPolygon) As Boolean
  On Error GoTo ErrorHandler


  Dim pBoundaryLine As IPolyline
  Dim pTopoOp As ITopologicalOperator
  Dim pRelOp As IRelationalOperator
4612:   Set pTopoOp = pCorPolygon
  
4614:   Set pBoundaryLine = pTopoOp.Boundary
  
4616:   Set pRelOp = pBoundaryLine
4617:   CheckIntersectBoundary = Not pRelOp.Disjoint(pPolyline)


4620:   Set pBoundaryLine = Nothing
4621:   Set pTopoOp = Nothing
4622:   Set pRelOp = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "CheckIntersectBoundary " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



4630:   Set pBoundaryLine = Nothing
4631:   Set pTopoOp = Nothing
4632:   Set pRelOp = Nothing

End Function

Public Function GenerateCorridorRaster(pCorPolygon As IPolygon, ByRef pEnv As IRasterAnalysisEnvironment, _
  pThisExt As Linkages.clsExtension) As IRaster
  On Error GoTo erh
  
  Dim pEnvelope As IEnvelope
'  Set pEnvelope = pStartPolygon.Envelope
'  pEnvelope.Union pEndPolygon.Envelope

4644:   Set pEnvelope = pCorPolygon.Envelope
4645:   pEnvelope.Expand 1.05, 1.05, True
  
  Dim lngRastResolution As Long
4648:   lngRastResolution = pThisExt.KruskallPixelResolution
4649:   If lngRastResolution < 50 Then lngRastResolution = 600
  
  ' CREATE A RASTER ENVIRONMENT OBJECT
  Dim dblCellSize As Double
  Dim dblWidth As Double
  Dim dblHeight As Double
4655:   dblWidth = pEnvelope.Width
4656:   dblHeight = pEnvelope.Height
4657:   If dblWidth > dblHeight Then
4658:     dblCellSize = dblWidth / lngRastResolution
4659:   Else
4660:     dblCellSize = dblHeight / lngRastResolution
4661:   End If
  
  'Create a RasterMakerOp operator
  Dim pRasMakerOp As IRasterMakerOp
4665:   Set pRasMakerOp = New RasterMakerOp
  
  'Create an analysis environment object
'  Dim pEnv As IRasterAnalysisEnvironment
4669:   Set pEnv = pRasMakerOp
  
  'Set cellsize
4672:   pEnv.SetCellSize esriRasterEnvValue, dblCellSize
  
  'Set output extent
4675:   pEnv.SetExtent esriRasterEnvValue, pEnvelope
  
  'Set output spatial reference
  Dim pSpRef As ISpatialReference
4679:   Set pSpRef = pCorPolygon.SpatialReference
4680:   Set pEnv.OutSpatialReference = pSpRef
  
  'Create a constant raster
  Dim pBaseRaster As IRaster
4684:   Set pBaseRaster = pRasMakerOp.MakeConstant(1, True)
  
4686:   DoEvents
  
4688:   GridFunctions.ReturnCellSize pBaseRaster
  
  ' CLIP TO CORRIDOR POLYGON
  Dim pCorRaster As IRaster
4692:   Set pCorRaster = GridFunctions.ClipRasterToPolygon(pBaseRaster, pCorPolygon, True, pEnvelope, dblCellSize)
  
4694:   Set GenerateCorridorRaster = pCorRaster
  

4697:   Set pEnvelope = Nothing
4698:   Set pRasMakerOp = Nothing
4699:   Set pSpRef = Nothing
4700:   Set pBaseRaster = Nothing
4701:   Set pCorRaster = Nothing

  Exit Function
erh:
4705:   MsgBox "Failed in GenerateCorridorRaster: " & err.Description


4708:   Set pEnvelope = Nothing
4709:   Set pRasMakerOp = Nothing
4710:   Set pSpRef = Nothing
4711:   Set pBaseRaster = Nothing
4712:   Set pCorRaster = Nothing

End Function



Public Function AttachPolylineToPatches(pPolyline As IPolyline, pStartPatch As IPolygon, pEndPatch As IPolygon) As IPolyline
  On Error GoTo ErrorHandler


  Dim pTopoOp As ITopologicalOperator
4723:   Set pTopoOp = pPolyline
  
  Dim pNewPolyline As IPolyline
4726:   Set pNewPolyline = pTopoOp.Difference(pStartPatch)
  
4728:   Set pTopoOp = pNewPolyline
4729:   Set pNewPolyline = pTopoOp.Difference(pEndPatch)
  
  Dim pGeoCollection As IGeometryCollection
4732:   Set pGeoCollection = pNewPolyline
  
  ' IF TRIMMING OFF THE EDGES RESULTED IN A MULTIPART POLYLINE, THEN IT IS TOO DIFFICULT TO FIGURE OUT THE
  ' BEST WAY TO ATTACH THIS POLYLINE.  JUST RETURN THE ORIGINAL POLYLINE
  
4737:   If pGeoCollection.GeometryCount > 1 Or pNewPolyline.IsEmpty Then
4738:     Set AttachPolylineToPatches = pPolyline
    Exit Function
4740:   End If
  
  Dim pPointStart As IPoint
4743:       Set pPointStart = New Point
  Dim pPointEnd As IPoint
4745:       Set pPointEnd = New Point
  
4747:   pNewPolyline.QueryPoint esriNoExtension, 0, True, pPointStart
4748:   pNewPolyline.QueryPoint esriNoExtension, 100, True, pPointEnd
  
  ' FLIP IF NECESSARY
  Dim pProxOp As IProximityOperator
4752:   Set pProxOp = pStartPatch
  Dim booShouldFlip As Boolean
4754:   booShouldFlip = False
  Dim pTempPoint As IPoint
4756:   If pProxOp.ReturnDistance(pPointStart) > pProxOp.ReturnDistance(pPointEnd) Then
4757:     Set pTempPoint = pPointStart
4758:     Set pPointStart = pPointEnd
4759:     Set pPointEnd = pTempPoint
4760:     booShouldFlip = True
'    pNewPolyline.ReverseOrientation
4762:   End If
  
  ' FIND NEAREST POINT ON START PATCH TO START POINT
  Dim pArray As esriSystem.IArray
4766:   Set pProxOp = pNewPolyline
  Dim pEndPatchPoint As IPoint
  Dim pStartPatchPoint As IPoint
  Dim pPointCollection As IPointCollection
4770:   Set pPointCollection = pNewPolyline
    
4772:   If booShouldFlip Then
4773:     If pProxOp.ReturnDistance(pStartPatch) > 0 Then
4774:       Set pArray = MyGeometricOperations.CalcClosestPoints(pPointEnd, pStartPatch)
4775:       Set pStartPatchPoint = pArray.Element(2)
4776:       pPointCollection.AddPoint pStartPatchPoint, 0
4777:     End If
    ' FIND NEAREST POINT ON END PATCH TO END POINT
4779:     If pProxOp.ReturnDistance(pEndPatch) > 0 Then
4780:       Set pArray = MyGeometricOperations.CalcClosestPoints(pPointStart, pEndPatch)
4781:       Set pEndPatchPoint = pArray.Element(2)
4782:       pPointCollection.AddPoint pEndPatchPoint
4783:     End If
  
4785:   Else
4786:     If pProxOp.ReturnDistance(pStartPatch) > 0 Then
4787:       Set pArray = MyGeometricOperations.CalcClosestPoints(pPointStart, pStartPatch)
4788:       Set pStartPatchPoint = pArray.Element(2)
4789:       pPointCollection.AddPoint pStartPatchPoint, 0
4790:     End If
    ' FIND NEAREST POINT ON END PATCH TO END POINT
4792:     If pProxOp.ReturnDistance(pEndPatch) > 0 Then
4793:       Set pArray = MyGeometricOperations.CalcClosestPoints(pPointEnd, pEndPatch)
4794:       Set pEndPatchPoint = pArray.Element(2)
4795:       pPointCollection.AddPoint pEndPatchPoint
4796:     End If
4797:   End If
  
4799:   Set pNewPolyline.SpatialReference = pPolyline.SpatialReference
  
4801:   Set AttachPolylineToPatches = pNewPolyline


4804:   Set pTopoOp = Nothing
4805:   Set pNewPolyline = Nothing
4806:   Set pGeoCollection = Nothing
4807:   Set pPointStart = Nothing
4808:   Set pPointEnd = Nothing
4809:   Set pProxOp = Nothing
4810:   Set pTempPoint = Nothing
4811:   Set pArray = Nothing
4812:   Set pEndPatchPoint = Nothing
4813:   Set pStartPatchPoint = Nothing
4814:   Set pPointCollection = Nothing

  Exit Function
ErrorHandler:
  HandleError True, "AttachPolylineToPatches " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


4821:   Set pTopoOp = Nothing
4822:   Set pNewPolyline = Nothing
4823:   Set pGeoCollection = Nothing
4824:   Set pPointStart = Nothing
4825:   Set pPointEnd = Nothing
4826:   Set pProxOp = Nothing
4827:   Set pTempPoint = Nothing
4828:   Set pArray = Nothing
4829:   Set pEndPatchPoint = Nothing
4830:   Set pStartPatchPoint = Nothing
4831:   Set pPointCollection = Nothing
  
End Function


Public Function CleanPolyline(ByVal pPolyline As IPolyline, pBoundary As IPolyline) As IPolyline
  On Error GoTo ErrorHandler


  Dim pPointCol As IPointCollection
4841:   Set pPointCol = pPolyline
  
  Dim pNewPointCol As IPointCollection
4844:   Set pNewPointCol = New Polyline
  
  Dim pSegment As IPointCollection
  Dim pStartPoint As IPoint
  Dim pEndPoint As IPoint
  Dim pClone As IClone
  
  Dim pCount As Long
4852:   pCount = pPointCol.PointCount
  
4854:   Set pStartPoint = pPointCol.Point(0)
4855:   Set pClone = pStartPoint
4856:   pNewPointCol.AddPoint pClone.Clone
  
  Dim anIndex1 As Long
  Dim anIndex2 As Long
  Dim anIndexStep As Long
  
  Dim pRelOp As IRelationalOperator
  
4864:   anIndex1 = 1
4865:   anIndexStep = 0
  
4867:   Do Until anIndex1 = pPointCol.PointCount - 1
4868:     Set pEndPoint = pPointCol.Point(anIndex1)
4869:     Set pSegment = New Polyline
4870:     pSegment.AddPoint pStartPoint
4871:     pSegment.AddPoint pEndPoint
4872:     Set pRelOp = pSegment
4873:     If pRelOp.Disjoint(pBoundary) Then
4874:       anIndex1 = anIndex1 + 1
4875:     Else
4876:       anIndex2 = anIndex1 - 1
4877:       Do Until anIndex2 <= anIndexStep
4878:         Set pStartPoint = pPointCol.Point(anIndex2)
4879:         Set pSegment = New Polyline
4880:         pSegment.AddPoint pStartPoint
4881:         pSegment.AddPoint pEndPoint
4882:         Set pRelOp = pSegment
4883:         If pRelOp.Disjoint(pBoundary) Then
4884:           anIndex2 = anIndex2 - 1
4885:         Else
4886:           Set pClone = pStartPoint
4887:           pNewPointCol.AddPoint pClone.Clone
4888:           anIndexStep = anIndex2 + 1
4889:           Set pStartPoint = pPointCol.Point(anIndexStep)
4890:           Set pClone = pStartPoint
4891:           pNewPointCol.AddPoint pClone.Clone
4892:           anIndex1 = anIndexStep + 1
4893:           Exit Do
4894:         End If
4895:       Loop
4896:     End If
4897:   Loop
4898:   Set pClone = pPointCol.Point(pPointCol.PointCount - 1)
4899:   pNewPointCol.AddPoint pClone.Clone
4900:   Set CleanPolyline = pNewPointCol
4901:   Set CleanPolyline.SpatialReference = pPolyline.SpatialReference


4904:   Set pPointCol = Nothing
4905:   Set pNewPointCol = Nothing
4906:   Set pSegment = Nothing
4907:   Set pStartPoint = Nothing
4908:   Set pEndPoint = Nothing
4909:   Set pClone = Nothing
4910:   Set pRelOp = Nothing


  Exit Function
ErrorHandler:
  HandleError True, "CleanPolyline " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4



4919:   Set pPointCol = Nothing
4920:   Set pNewPointCol = Nothing
4921:   Set pSegment = Nothing
4922:   Set pStartPoint = Nothing
4923:   Set pEndPoint = Nothing
4924:   Set pClone = Nothing
4925:   Set pRelOp = Nothing


End Function


Public Function BailOutOfProgress(frmProgressDialog As Object, pExtensionConfig As IExtensionConfig) As Boolean
  On Error GoTo ErrorHandler


  Dim pExt As Linkages.clsExtension
4936:   Set pExt = pExtensionConfig
    
  Dim frmProgress As Linkages.frmJenProgressPercent
4939:   Set frmProgress = frmProgressDialog
  
4941:   pExt.ProgressDialogAutoClose = (frmProgress.chkClose.Value = 1)
4942:   pExt.ProgressDialogSetExpanded = (frmProgress.SetExpanded)
  
'  MsgBox "Checking Autoclose Stuff: " & vbCrLf & _
      "Check-Box checked? " & CStr(frmProgress.chkClose.Value = 1) & vbCrLf & _
      "Extension Property Checked? " & CStr(pExt.ProgressDialogAutoClose)
      
  
  ' PROGRESS METER STUFF --------------------------------------------------------
4950:   BailOutOfProgress = Not frmProgress.ShouldContinue
4951:   If BailOutOfProgress Then
    Dim theDateString As String
    Dim theElapsedTimeString As String
    
4955:     theDateString = Format(Now, "long date") & ";  " & Format(Now, "long time")
4956:     Screen.MousePointer = vbDefault
    Dim theTimeBegan As Date
    Dim theTimeEnd As Date
4959:     theTimeBegan = frmProgress.ProgBeginTime
4960:     theTimeEnd = Now

4962:     theElapsedTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)
  
    Dim theReport As String
4965:     theReport = "Operation cancelled prematurely at " & theDateString & vbCrLf & _
        "---------------------------------------------------------" & vbCrLf & theElapsedTimeString
    
    Dim strCurrentReport As String
4969:     strCurrentReport = frmProgress.txtDetails.Text
4970:     frmProgress.txtDetails.Text = strCurrentReport & vbCrLf & theReport
    
4972:     If frmProgress.chkClose.Value = 1 Then
    '  Unload frmProgress
    '  Set frmProgress = Nothing
4975:       frmProgress.Frame.Visible = False
4976:     End If
    
    Exit Function
4979:   End If

4981:   Set pExt = Nothing
4982:   Set frmProgress = Nothing
  
  Exit Function
ErrorHandler:
  HandleError True, "BailOutOfProgress " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

4988:   Set pExt = Nothing
4989:   Set frmProgress = Nothing
  
End Function
'
'
'
'
'Screen.MousePointer = vbHourglass
'
'
'
'
'  ' PROGRESS METER STUFF ------------------------------------------------------------------
'  Dim frmProgress As New frmJenProgressPercent
'  frmProgress.ShouldContinue = True
'  frmProgress.ProgBeginTime = Now
'  frmProgress.ProgRecCount = theFinalList.Count - 2
'  frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
'  frmProgress.lblBeginTime.Caption = "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd")
'
'  theDetailedDescription = "Pass 2:  Generating Hexagons..." & vbCrLf & _
'      "Began Job: " & Format(theTimeBegan, "ttttt,  ddddd") & vbCrLf & _
'      "-------------------------------------------" & vbCrLf
'  frmProgress.txtDetails.Text = theDetailedDescription
'
'  frmProgress.Frame.Caption = "Current Status:"
'  frmProgress.Frame.Visible = True
'
'  theProgressTimeCheck = CDate(50000)
'  Dim theDescription As String
'  theDescription = "Pass 2:  Generating Hexagons..."
'  ' PROGRESS METER STUFF ------------------------------------------------------------------
'
'
'
'
'
'
'
'
'
'
'    Do While theHorizIndex < theMiddleCount
'
'      ' PROGRESS METER STUFF --------------------------------------------------------
'      If frmProgress.ShouldContinue = False Then
'        theDateString = Format(Now, "long date") & ";  " & Format(Now, "long time")
'        Screen.MousePointer = vbDefault
'
'        theTimeEnd = Now
'
'        theElapsedTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)
'
'        theReport = theReport & "Operation cancelled prematurely at " & theDateString & vbCrLf & _
'            "---------------------------------------------------------" & vbCrLf & theElapsedTimeString
'        Unload frmProgress
'        Set frmProgress = Nothing
'
'        CreateHexagonShapefile = theReport
'        Exit Function
'      End If
'
'      If Abs(DateDiff("s", theProgressTimeCheck, Now)) >= 1 Then    ' UPDATE STUFF AFTER EVERY SECOND
'        theProgressTimeCheck = Now
'        theProgressSpecific = "Working on Row #" & aml_func_mod.InsertCommas(theCounter)
'        theDetailedDescription = theDetailedDescription & _
'          " --> " & theProgressSpecific & vbCrLf & _
'          "          " & aml_func_mod.InsertCommas(theHexagonCount) & " preliminary hexagons created so far..." & vbCrLf & _
'          "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
'        frmProgress.txtDetails.Text = theDetailedDescription
'        frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
'
'        frmProgress.Est_Time_Left theCounter, "Generating Hexagons...", "Working on Row #"
'        frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
'
'      End If
'      ' END PROGRESS METER STUFF --------------------------------------------------------
'
'
'
'
'
'
'
'
'  theProgressTimeCheck = Now
'  theProgressSpecific = "Saving Final Shapefile..."
'  theDetailedDescription = theDetailedDescription & _
'    " --> " & theProgressSpecific & vbCrLf & _
'    "          " & aml_func_mod.InsertCommas(theHexagonCount) & " preliminary hexagons created so far..." & vbCrLf & _
'    "          [time stamp " & Format(Now, "ttttt,  ddddd") & "]" & vbCrLf
'  frmProgress.txtDetails.Text = theDetailedDescription
'  frmProgress.txtDetails.SelStart = Len(theDetailedDescription)
'
'  frmProgress.Est_Time_Left theCounter, "Saving Shapefile...", "Working on Row #"
'  frmProgress.lblCurrentTime.Caption = Format(Now, "ttttt")
'
'  pFeatCur.Flush
'
'  Screen.MousePointer = vbDefault
'
'  Unload frmProgress
'  Set frmProgress = Nothing
'  theTimeEnd = Now
'
'  strTimeString = MyGeneralOperations.ReturnTimeElapsed(theTimeBegan, theTimeEnd)
'
